package MiscTests;

import java.io.*;
import java.util.*;

import java.nio.*;
import java.nio.channels.*;

// a document list.
// loaded into compressed buffer.
// 
public class DocList {

    // byte versions set to null if modify int version.
    // ok to have both alive at once.

    int m_size = 0;

    ByteBuffer m_byteCat = null;  // compressed cat ids - RLE
    IntBuffer m_intCat = null;

    ByteBuffer m_byteNode = null;  // compressed node ids - diff from prev
    IntBuffer m_intNode = null;


    public DocList() {
    }

    public DocList(ByteBuffer cats, ByteBuffer nodes) {
	m_byteCat = cats;
	m_byteNode = nodes; // don't know size yet...
    }
     
    public final static void writeString(ByteBuffer buf, String s) throws IOException {
	int i = s.length();
        final int end = i;
	//	System.out.println("writing a string length "+length);
	//writeVInt(buf, length);
        while ((i & ~0x7F) != 0) {
	    byte b = (byte)((i & 0x7f) | 0x80);
	    buf.put(b);	    
	    i >>>= 7;
	}
	buf.put((byte)i);        
	
        i=0;
        char truc[]=s.toCharArray();
	while( i < end) {
	    final int code = (int)truc[i];
	    if ((code & 0x7E)!=0)
		buf.put((byte)code);
	    else if (((code >= 0x80) && (code <= 0x7FF)) || code == 0) {
		buf.put((byte)(0xC0 | (code >> 6)));
		buf.put((byte)(0x80 | (code & 0x3F)));
	    } else {
		buf.put((byte)(0xE0 | (code >>> 12)));
		buf.put((byte)(0x80 | ((code >> 6) & 0x3F)));
		buf.put((byte)(0x80 | (code & 0x3F)));
	    }
            i++;
	}        
    }

    public final static void writeChars(ByteBuffer buf, String s, int start, int length)
	throws IOException {
	// System.out.println("writing a substring "+s.substring(start, start+length));
	final int end = start + length;
	for (int i = start; i < end; i++) {
	    final int code = (int)s.charAt(i);
	    if (code >= 0x01 && code <= 0x7F)
		buf.put((byte)code);
	    else if (((code >= 0x80) && (code <= 0x7FF)) || code == 0) {
		buf.put((byte)(0xC0 | (code >> 6)));
		buf.put((byte)(0x80 | (code & 0x3F)));
	    } else {
		buf.put((byte)(0xE0 | (code >>> 12)));
		buf.put((byte)(0x80 | ((code >> 6) & 0x3F)));
		buf.put((byte)(0x80 | (code & 0x3F)));
	    }
	}
    }
    
    public final static void readChars(ByteBuffer src, char[] buffer, int start, int length)
    {
	final int end = start + length;
	for (int i = start; i < end; i++) {
	    byte b = src.get();
	    if ((b & 0x80) == 0)
		buffer[i] = (char)(b & 0x7F);
	    else if ((b & 0xE0) != 0xE0) {
		buffer[i] = (char)(((b & 0x1F) << 6)
				   | (src.get() & 0x3F));
	    } else
		buffer[i] = (char)(((b & 0x0F) << 12)
				   | ((src.get() & 0x3F) << 6)
				   |  (src.get() & 0x3F));
	}
    }

    // read string into buf
    public final static void readString(ByteBuffer buf, char[] chars, String myStr) {
	//int length = readVInt(buf);
        byte b = buf.get();
	// System.out.println("Read vint Read byte "+b+" from "+buf);
	int length = b & 0x7F;
	for (int shift = 7; (b & 0x80) != 0; shift += 7) {
	    b = buf.get();
	    length |= (b & 0x7F) << shift;
	}
	//return i;        
	if (chars == null || length > chars.length)
	    chars = new char[length];
	//readChars(buf, chars, 0, length);
        final int end = length;
	for (int i = 0; i < end; i++) {
	    b = buf.get();
	    if ((b & 0x80) == 0)
		chars[i] = (char)(b & 0x7F);
	    else if ((b & 0xE0) != 0xE0) {
		chars[i] = (char)(((b & 0x1F) << 6)
				   | (buf.get() & 0x3F));
	    } else
		chars[i] = (char)(((b & 0x0F) << 12)
				   | ((buf.get() & 0x3F) << 6)
				   |  (buf.get() & 0x3F));
        
	}        
        if(myStr==null||myStr.length()<length)
            myStr=new String(chars, 0, length);
        else
            myStr.copyValueOf(chars,0, length);
    }


    // reads a vint from buf
    public final static int readVInt(ByteBuffer buf) {
	byte b = buf.get();
	// System.out.println("Read vint Read byte "+b+" from "+buf);
	int i = b & 0x7F;
	for (int shift = 7; (b & 0x80) != 0; shift += 7) {
	    b = buf.get();
	    i |= (b & 0x7F) << shift;
	}
	return i;
    }
    public final static long readVLong(ByteBuffer buf) {
	byte b = buf.get();
	// System.out.println("Read vint Read byte "+b+" from "+buf);
	long i = b & 0x7F;
	for (int shift = 7; (b & 0x80) != 0; shift += 7) {
	    b = buf.get();
	    i |= (b & 0x7F) << shift;
	}
	return i;
    }

    int getSize() {
	return m_size;
    }

    public final static void writeVInt(ByteBuffer buf, int i) {	
	while ((i & ~0x7F) != 0) {
	    byte b = (byte)((i & 0x7f) | 0x80);
	    buf.put(b);	    
	    i >>>= 7;
	}
	buf.put((byte)i);
    }

    public final static void writeVLong(ByteBuffer buf, long i) {	
	while ((i & ~0x7F) != 0) {
	    byte b = (byte)((i & 0x7f) | 0x80);
	    buf.put(b);	    
	    i >>>= 7;
	}
	buf.put((byte)i);
    }
    
    ByteBuffer getByteCat() {
	if (m_byteCat==null) {
	    if (m_intCat==null) {
		m_intCat = getIntCat();
	    }
	    m_byteCat = compressRLEDiff( m_intCat );
	    // m_byteCat = compressDiff( m_intCat );
	}
	return m_byteCat;
    }
    
    ByteBuffer getByteNode() {
	if (m_byteNode==null) {
	    if (m_intNode==null) {
		m_intNode = getIntNode();
	    }
	    m_byteNode = compressDiff( m_intNode );
	}
	return m_byteNode;
    }
	
    IntBuffer getIntCat() {
	if (m_intCat != null) {
	}
	else if (m_byteCat != null) {
	    m_intCat = uncompressRLEDiff( m_byteCat );
	    // m_intCat = uncompressDiff( m_byteCat );
	}
	else {
	    m_intCat = IntBuffer.allocate(32);
	    m_intCat.position(0);
	}
	return m_intCat;
    }
    
    IntBuffer getIntNode() {
	if (m_intNode != null) {
	}
	else if (m_byteNode != null) {
	    m_intNode = uncompressDiff( m_byteNode );
	}
	else {
	    m_intNode = IntBuffer.allocate(32);
	    m_intNode.position(0);
	}
	return m_intNode;
    }

    // uncompress a byte buffer using diff. encoding.
    final IntBuffer uncompressDiff(ByteBuffer buf) {
	// copy from m_compressed into m_uncompressed.
	buf.position(0);
	int len = readVInt(buf);

	// System.out.println("Uncompressing Diff size is "+m_size);

	IntBuffer ret = IntBuffer.allocate(len + 3);
	ret.position(0);
	
	int prev = 0;
	for (int i=0; i<len; i++) {
	    int next = readVInt(buf) + prev;
	    ret.put(next);
	    prev = next;
	}	
	return ret;
    }

    // uncompress a byte buffer using RLE and diff encoding.
    // (good for catids which will probably appear > once)
    final IntBuffer uncompressRLEDiff(ByteBuffer buf) {
	// copy from m_compressed into m_uncompressed.
	buf.position(0);
	int len = readVInt(buf); 
	m_size = len;
	//	System.out.println("Uncompressing RLE size is "+m_size);
	IntBuffer ret = IntBuffer.allocate(len); // no space for expansion
	ret.position(0);
	int prev = 0;
	for (int i=0; i<len; i++) {
	    int count = readVInt(buf);
	    int next = readVInt(buf) + prev;
	    i+= count-1;// needs a sanity check.
	    //    System.out.println("Uncompressing RLE "+count+" "+next+" to "+ret);
	    if (i >= m_size){
		System.out.println("Error in RLE - overrun");
		break;
	    }
	    for (int j=0; j<count; j++) {
		ret.put(next);		
	    }
	    prev = next;
	}	
	return ret;
    }

    static ByteBuffer m_scratch = null;
    
    // return initialised scratch space.
    static ByteBuffer getScratch(int size) {
	if (size < 5000) { size = 5000; }
	if (m_scratch == null) {
	    m_scratch = ByteBuffer.allocate(size);
	    m_scratch.position(0);
	    return m_scratch;
	}
	if (size >= m_scratch.capacity()) {
	    m_scratch = ByteBuffer.allocate(size);
	}
	m_scratch.position(0);
	return m_scratch;
    }

    final ByteBuffer compressDiff(IntBuffer buf) {
	int count = buf.position();
	int [] ary = buf.array();

	// need to sort.
	Arrays.sort(buf.array(), 0, count);
	
	// count duplicates (important to get this right)
	// *** no good - as there are two parallel arrays.
	// *** need some other way to check if a document has been mentioned
	// *** perhaps split the arrays into "sorted" and "added"
	// --- and do nasty linear search in added.
	/*
	int duplicates = 0;
	for (int i=1; i<ary.length; i++) {
	    if (ary[i-1]==ary[i]) {
		duplicates++;
	    }
	}
	*/

	// worst case : 5 per int. + 1
	// but we don't really want to have to allocate a new buffer here.
	int worstCase = (count << 2) + count + 5;

	m_scratch = getScratch( worstCase );

	writeVInt(m_scratch, count);
	

	// System.out.println("compress diff about to put in the bytes...");
	int prev = 0;
	for (int i=0; i<count; i++) {
	    int curr = ary[i];
	    writeVInt(m_scratch, curr-prev);
	    prev = curr;
	}

	// copy to new buffer
	ByteBuffer ret = ByteBuffer.allocate(m_scratch.position());
	ret.put(m_scratch.array(), 0, m_scratch.position());
	return ret;
    }


    final ByteBuffer compressRLEDiff(IntBuffer buf) {
	int count = buf.position();
	int [] ary = buf.array();
	// need to sort.
	Arrays.sort(buf.array(), 0, count);

	// could remove duplicates at this point...
	// to save bothering on doc add time.

	// System.out.println("Doing RLE compression... "+count+" elements");

	// worst case : 10 per int. + 5
	// but we don't really want to have to allocate a new buffer here.
	int worstCase = (count * 10) + 5;
	m_scratch = getScratch(worstCase);
	m_scratch.position(0);

	// System.out.println("Doing RLE compression... worst case "+worstCase+" bytes");
	// System.out.println("RLE Compressing - count is "+count);

	writeVInt(m_scratch, count);

	if (count > 0) {
	    int prev = 0;
	    int curr = ary[0]; // value to be written next...

	    int runCount = 1;

	    for (int i=1; i<count; i++) {	    
		int next = ary[i];		    

		if (curr != next) {
		    writeVInt(m_scratch, runCount);
		    writeVInt(m_scratch, curr-prev);
		    // System.out.println("RLE compress written "+runCount+" "+(curr-prev)+" curr "+curr+" prev "+prev);
		    prev = curr;
		    curr = next;		   
		    runCount = 1;			
		}
		else {
		    runCount ++;
		}
	    }
	    // end case
	    writeVInt(m_scratch, runCount);
	    writeVInt(m_scratch, curr-prev);				    
	    // System.out.println("RLE compress written "+runCount+" "+(curr-prev)+" curr "+curr+" prev "+prev);
	}

	// copy to new buffer
	ByteBuffer ret = ByteBuffer.allocate(m_scratch.position());
	ret.put(m_scratch.array(), 0, m_scratch.position());
	return ret;
    }

    // compress if needed.
    final void compress() {
	if (m_byteCat==null) {
	    if (m_intCat == null) {
		// make an empty one...
		getIntCat();
		getIntNode();
	    }

	    // System.out.println("Compressing "+m_intCat+" and "+m_byteCat);
	    m_byteNode = compressDiff( m_intNode );
	    m_byteCat = compressRLEDiff( m_intCat );
	}
    }

    // always frees memory of uncompressed arrays.
    // doesn't check if the compressed version is there.
    final void free() {
	m_intCat = null;
	m_intNode = null;
    }

    IntBuffer addInt(IntBuffer buf, int val) {
	if (buf.remaining() > 0) {
	    // System.out.println("Adding "+val+" to "+buf);
	    return buf.put(val);
	}
	else {
	    // System.out.println("Resizing buffer... from "+buf.capacity());
	    IntBuffer newBuf = IntBuffer.allocate( buf.capacity() * 2  + 10);
	    newBuf.position(0);
	    newBuf = newBuf.put(buf.array(), 0, buf.position()).put(val);
	    return newBuf;
	}
    }

    
    // linear search... could replace with bin search (see Arrays....)
    // ** potential bottleneck later on ***
    boolean containsDocument(int catid, int nodeid) {
	int[] cats = getIntCat().array();
	int[] nodes = getIntNode().array();
	boolean found = false;
	for (int i=0; i<m_size && !found; i++) {
	    found = cats[i]==catid && nodes[i]==nodeid;
	}
	return found;
    }

    // add a document to the lists
    // could add to a "pending" queue and only check presence
    // on sort -> unique.
    void addDocument(int catid, int nodeid) {
	// not checking for duplicates now - could do at sort time instead.
	//	System.out.println("Adding a document "+catid+" "+nodeid);

	m_intCat = getIntCat();
	m_intNode = getIntNode();

	m_intCat = addInt(m_intCat, catid);
	m_intNode = addInt(m_intNode, nodeid);
	m_size ++;
	m_byteCat = null;
	m_byteNode = null;
	//	System.out.println("Adding a document "+catid+" "+nodeid+" size is now "+m_size);
    }
}
