package guikit;

import java.io.*;
import java.io.IOException;
import javax.swing.*; 
import javax.swing.text.JTextComponent;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;
import java.util.*;


import neuroml.*;
import neuroml.util.*;
import neuroml.core.*;
import neuroml.model.network.*;
//JAG
import java.net.*;

public class BasicApp extends JFrame implements ActionListener {

    /** Environment for editors
     */
    public Environment env;
    

    /** The top level object being edited 
     */
    public Object topObject = null;
    
    /** The namespace for the edited model
     */
    public Namespace ns = null;

    /** Current filename
     */
    public String filename="test.xml";
    
    /** The menu bar
     */
    public JMenuBar menuBar;    
    public JMenu fileMenu;
    public JMenu editMenu;

    /** The object browser dialog
     */
    BrowseObjectsPanel objBrowser;

    /** The modules browser dialog
     */
    BrowseModulesPanel modBrowser;

    /** The about dialog
     */
    AboutPanel aboutPanel;

    /** Resources used for images
     */
    
    Vector ModuleVector=new Vector(1,1);
    
    private static ResourceBundle resources;
    static {
        try {
            resources = ResourceBundle.getBundle("guikit.resources/BasicApp",  Locale.getDefault());
        } catch (MissingResourceException mre) {
            System.err.println("guikit.resources/BasicApp.properties not found");
        }
    }
    ImageUtils imgLoader;


    /** Add a menu item to a menu 
     * e.g. addMenu(menuBar,"Test", "This is a test", KeyEvent.VK_A)
     */
    
    //JAG
    String ModuleDir=null;
    
    static int i=0;
    
    public JMenu addMenu( JMenuBar parent, String name, String desc, int m) {
	JMenu menu = new JMenu(name);
	if (m!=-1)  menu.setMnemonic(m);
        menu.getAccessibleContext().setAccessibleDescription(desc); 
	parent.add(menu);
	menu.addActionListener(this);
	return menu;
    }

    public JMenuItem addMenuItem( JMenu parent, String name, String desc, int m) {
	JMenuItem mi = new JMenuItem(name);
	if (m!=-1) 
	    mi.setAccelerator(KeyStroke.getKeyStroke(m, ActionEvent.ALT_MASK)); 
        mi.getAccessibleContext().setAccessibleDescription(desc); 
	mi.addActionListener(this);
	parent.add(mi);
	return mi;
    }

    /** Create the main menu
     * Override this for customised menu
     */
    public void createMenu() {
        menuBar = new JMenuBar();
        setJMenuBar(menuBar);
	fileMenu = addMenu( menuBar, "File", "File", KeyEvent.VK_F);
	addMenuItem(fileMenu, "Open", "Open NeuroML Model", -1);
	addMenuItem(fileMenu, "Edit Model", "Edit model", -1);
	addMenuItem(fileMenu, "Save", "Save Model", -1);

	fileMenu.addSeparator();
	addMenuItem(fileMenu, "Exit", "Exit", -1);
	
	editMenu = addMenu( menuBar, "Edit", "Edit", KeyEvent.VK_E);
	addMenuItem(editMenu, "Browse Objects", "Object browser", -1);
	addMenuItem(editMenu, "Modules", "Module browser", -1);
	menuBar.add(Box.createHorizontalGlue());
	JMenu helpMenu = addMenu(menuBar,"Help", "Help about Basic Application", KeyEvent.VK_H);
	JMenuItem about = addMenuItem(helpMenu, "About", "About this program", -1);

    }     
    
    
    /** Popup a file chooser 
     */
    public File fileChooser(String ext,String hint) {
	try {
	    if(ModuleDir==null) ModuleDir = System.getProperty("user.dir");
            if(ModuleDir==null) ModuleDir = "./";
	} catch (java.security.AccessControlException ae){
	    ModuleDir = "./";
	}
	JFileChooser chooser = new JFileChooser(ModuleDir);  
	//ExampleFileFilter filter = new ExampleFileFilter( "xml", "NeuroML model files");
        ExampleFileFilter filter = new ExampleFileFilter(ext,hint);
	chooser.addChoosableFileFilter(filter);	
	int returnVal = chooser.showOpenDialog(this); 
	if (returnVal == JFileChooser.APPROVE_OPTION){
            ModuleDir=chooser.getSelectedFile().getParent();
           // newModule = chooser.getSelectedFile().toURL(); 
            //chooser.getSelectedFile().
	    //return chooser.getSelectedFile().getPath();
            return chooser.getSelectedFile();
	} 
	return null;
    }

    /** Top level edit object method - 
     * Replace namespace with new object.
     * Derived versions can do as they please.
     */
    public void editObject( Namespace ns, Object o ) {
	topObject = o;
	this.ns = ns;
	EditObjectPanel eop = new EditObjectPanel( ns,
						   topObject, 
						   o.getClass().getName(),
						   false );
	eop.popup();
    }

    /** Add the object to existing model
     */
    public void importObject( Namespace ns, Object o ) {
	System.out.println("Import object not yet done - ");
	// *** Need to merge namespaces ***
    }
    
    /** Respond to menu choices
     */
    public void actionPerformed(ActionEvent e) {
	update(getGraphics());
	if (e.getActionCommand().equals("Save")) {
            File machin=fileChooser("xml","NeuroML file");
            String fname=(machin!=null)?machin.getPath():null;
	    if (fname!=null) {
                System.out.println("Saving "+fname);
                NeuromlSys.toFile(topObject,fname);//filename
                System.out.println("Writen model to "+fname);//filename
            }
        } else if (e.getActionCommand().equals("Edit Model")) {
	    editObject( ns, topObject );
        } else if (e.getActionCommand().equals("Exit")) {
            System.exit(0);
        } else if (e.getActionCommand().equals("Browse Objects")) {
	    if (objBrowser==null) {
		objBrowser = new BrowseObjectsPanel( ns );
	    } else {
		objBrowser.popup();
	    }        
	} else if (e.getActionCommand().equals("Modules")) {
            //JAG: no need
	  /* if (modBrowser==null) {
		modBrowser = new BrowseModulesPanel( env );
	    } else {
		modBrowser.popup();
	    }*/
            loadModule();
        } else if (e.getActionCommand().equals("Open")) { 
	    File machin=fileChooser("xml","NeuroML file");
            String fname=(machin!=null)?machin.getPath():null;
	    if (fname!=null) {
		System.out.println("Opening "+fname);
		Object o = NeuromlSys.loadXML(fname);
		System.out.println("Loaded "+o);
		if (o!=null) {
		    if (o instanceof NeuromlPkg){
                        Namespace truv=NeuromlSys.getNamespace(fname);
                        truv.setModuleRef(((NeuromlPkg)o).ImplementionModules.getElements());
                        ((NeuromlPkg)o).ImplementionModules.setElements(truv.getModuleRef());
			Environment env = new Environment(truv , (NeuromlPkg)o, imgLoader );
			SpecialEditors.setEnvironment(env);		
		    }				    
		    editObject( NeuromlSys.getNamespace(fname), o );
		    filename = fname;
		}
	    }
		  	   
	} else  if (e.getActionCommand().equals("About")) { 
 	    if (aboutPanel==null) {               
                
		Image aboutImage = imgLoader.getImage("aboutImage");
		AboutPanel ap = new AboutPanel("About", aboutImage, "(c) 2001 University of Edinburgh");
		ap.popup(); 
	    }
	} else { 
            System.out.println("Unknown command");
        }
    }
    
    /** Initialise application
     */
    public BasicApp(String frameName) {
        super(frameName);   
       	
	imgLoader = new ImageUtils(this,resources);

	SpecialEditors.put("guikit.RGBColour", "guikit.ColourEditor" );
	//SpecialEditors.put("neuroml.model.network.Network", "guikit.SubnetEditor" );

	ns = NeuromlSys.makeNamespace("editor2"); 
        //ns.getModule(NeuromlPkg.getClassList());
        //ns.getModules(NeuromlPkg.getClassList());
	NeuromlPkg np = new NeuromlPkg();
        
	//np.setName("New Model");
        //JAG
        np.setName("editor2");
        
	topObject = np;
	NeuromlSys.addReferences(np, "editor2");
        
        np.ImplementionModules.setElements(ns.getModuleRef());
        
	createMenu();

	// Set environment for sub editors.
	env = new Environment( ns, np, imgLoader );
	SpecialEditors.setEnvironment(env);

	JPanel tmp = new JPanel();
	tmp.setBackground(Color.blue);
	tmp.setLayout( new BorderLayout(5,5) ); 
	getContentPane().add(tmp);
	tmp.setPreferredSize(new Dimension(300,100));
    }
   
    /** Sample main program
     */
    public static void main(String[] args) {
        SpecialEditors.put("neuroml.util.RefInterfaces","guikit.EditRefInterfaces");
	System.out.println("Starting Basic Applet");
        JFrame frame = new BasicApp("Basic neuroml editor applet");
        
	frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) { System.exit(0);  }
	    });  	
        frame.pack();
        frame.setVisible(true);
    } 
    
    
    //JAG
    public void loadModule()
    {
        URL newModule=null;

        try{
            File machin=fileChooser("jar","NeuroML modules");
            newModule=(machin!=null)?machin.toURL():null;
        }
        catch(Exception e){
            return;
        }
        
        if(newModule==null) return;
	if (newModule!=null) {
            System.out.println("Loading jar file "+newModule);
	// *** Try to get the module ***
	// *** Just the default one... ***
            Object m = ModuleLoader.getObject( newModule.toString(),"");
            String beanName = ModuleLoader.getObjectName();
            String imgName  = ModuleLoader.getManifestValue("Icon");
            String docName  = ModuleLoader.getManifestValue("Doc");
            String modClass = ModuleLoader.getManifestValue("ModuleClass");
            Module NewModule=new Module(beanName);
            NewModule.location=newModule.toString();

            URL imgUrl = null;
            
            if (imgName!=null){
                imgUrl = ModuleLoader.getResource( NewModule.location, imgName);
		System.out.println("imgUrl "+imgUrl);
                System.out.println("Got resource "+imgName+" url "+imgUrl+" object "+m);
            }
            URL docUrl = null;
            if (docName!=null)
                docUrl = ModuleLoader.getResource( NewModule.location, docName);
            
            NewModule.setModuleObject(m);//like Fred
            
            NewModule.className=modClass;
            NewModule.imageUrl=(imgUrl==null)?new String(""):imgUrl.toString();
            NewModule.docUrl=(docName==null)?new String(""):docName.toString();
            
            if (beanName==null) beanName = new String(NewModule.location);
	    NewModule.setName( beanName );
            
 
            if(topObject instanceof NeuromlPkg)
            {
                NeuromlPkg o=(NeuromlPkg)topObject;
                o.modules.addElement(NewModule);
                
                JarClassLoader truc=NeuromlSys.classLoader;
                if(truc!=null)
                {
                    String[] strings=null;
                    try{
                        strings=truc.readJar(NewModule.location);
                    }
                    catch(Exception e){
                        System.out.println("Can not read "+ beanName +"?...");
                    }
                    if(strings!=null){
                        Namespace truv=NeuromlSys.getNamespace(o.getName());
                        truv.initialiseDerivedFrom(strings);
                        truv.addRef(NewModule.getName(),NewModule);
                    }else{
                        System.out.println("Hmmmm... no classes for module "+ beanName +"?...");
                    }
                }
                else System.out.println("NeuromlSys.classLoader is null!");
            }
        }
    }
    
    public static ImageIcon getScaledIcon(URL url) {	
	ImageIcon ii1 = new ImageIcon(url);
	ImageIcon ii = null;
	if (ii1.getIconHeight() > 32) {
	    ii = new ImageIcon( ii1.getImage().getScaledInstance( -1,32,Image.SCALE_DEFAULT));
	} else {
	    ii = ii1;
	}
	return ii;
    }
}

