package guikit;

import java.lang.reflect.*;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.net.*;
import java.io.*;
import java.text.NumberFormat;

import neuroml.*;
import neuroml.core.*;
import neuroml.util.*;
import neuroml.environment.*;
import neuroml.model.cell.*;

/** View all available modules
 */

public class BrowseModulesPanel implements ActionListener, ListSelectionListener   {
 
    Namespace ns;
    String ModuleDir=null;

    boolean debug = true;

    JFrame dialog;
    DefaultListModel classesList;
    DefaultListModel objectsList;
    
    JList jclassesList, jobjectsList;
    
    Environment env;
    
    String defaultImgUrl = new String();

    /** Build the panel
     */
    public BrowseModulesPanel(Environment env) {
	this.env = env;
	this.ns  = env.getNamespace();

	// Make a panel
	dialog = new JFrame("Browse modules");
	dialog.getContentPane().setLayout( new BoxLayout(dialog.getContentPane(), BoxLayout.Y_AXIS) );

	JPanel left = new JPanel();
	left.setLayout( new BoxLayout(left, BoxLayout.Y_AXIS) );

	JPanel right = new JPanel();
	right.setLayout( new BoxLayout(right, BoxLayout.Y_AXIS) );

	//	lists.add(left);
	//	lists.add(right);

	// List of classes + objects
	classesList = new DefaultListModel();
	jclassesList = new JList(classesList);
	jclassesList.addListSelectionListener( this );
	JScrollPane sl = new JScrollPane(jclassesList);
  	objectsList = new DefaultListModel();
	jobjectsList = new JList(objectsList);
	
	// Display modules as ImageIcons...
	ComboBoxRenderer renderer= new ComboBoxRenderer();
	// renderer.setPreferredSize(new Dimension(200, 130));
        jobjectsList.setCellRenderer(renderer);

	jobjectsList.addListSelectionListener( this );

	JScrollPane sr = new JScrollPane(jobjectsList);

	sl.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
	sl.setMinimumSize( new Dimension(50,30) );
	sl.setPreferredSize( new Dimension(50,70) );
	sr.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
	sr.setMinimumSize( new Dimension(50,30) );
	sr.setPreferredSize( new Dimension(50,70) );
  
	left.add( new JLabel("Module Flavours"));
	left.add(sl);
	right.add( new JLabel("Modules"));
	right.add(sr);

	JPanel tmp = new JPanel();
	tmp.setLayout( new BoxLayout(tmp, BoxLayout.X_AXIS) );
	JSplitPane lists = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, true, left, right);
	tmp.add(lists); 
	dialog.getContentPane().add(tmp);
	  
	// Add the buttons...
	JPanel buts = new JPanel();
	buts.setLayout( new BoxLayout(buts, BoxLayout.X_AXIS) );
	buts.add( EditPanel.makeButton("Load New", "load", this));
	right.add(buts);

	dialog.pack();
	popup();
    }      

    /** Make a list of all module types 
     */
    public void popup() {
	Vector v = ns.getDerivedFrom("neuroml.core.Module");
	classesList.removeAllElements(); // primitive way to update classes...
	for (int i=0; i<v.size(); i++) {
	    String k = (String)v.elementAt(i);
	    classesList.addElement(chopName(k));
	} 
	objectsList.removeAllElements();
	dialog.setVisible(true);
    }
  
    /** Listen to buttons
     */
    public void actionPerformed(java.awt.event.ActionEvent e) {
	if (e.getActionCommand().equals("load")) {
	    URL newModule = fileChooser();
	    if (newModule!=null) {
		
		System.out.println("Loading jar file "+newModule);
		// *** Try to get the module ***
		// *** Just the default one... ***
		Object m = ModuleLoader.getObject( newModule.toString(), "" );
		String beanName = ModuleLoader.getObjectName();
		String imgName = ModuleLoader.getManifestValue("Icon");
		String docName = ModuleLoader.getManifestValue("Doc");
		String modClass = ModuleLoader.getManifestValue("ModuleClass");
		URL imgUrl = null;
		if (imgName!=null){
		    imgUrl = ModuleLoader.getResource( newModule.toString(), imgName);
		    System.out.println("imgUrl "+imgUrl);
System.out.println("Got resource "+imgName+" url "+imgUrl+" object "+m);
		}
		URL docUrl = null;
		if (docName!=null)
		    docUrl = ModuleLoader.getResource( newModule.toString(), docName);
		//System.out.println("Got resource "+imgName+" url "+imgUrl+" "+docUrl+" object "+m);
		if (modClass!=null) {
		    changeCurrentClass( modClass );
		}

		Module newm = (Module)NeuromlSys.makeNeuromlObject(currentClass);
		if (beanName==null) beanName = new String("noname");
		newm.setName( beanName );


		if (imgUrl!=null) {
		    newm.imageUrl = imgUrl.toString();
		
		} else newm.imageUrl = new String("");

		if (docUrl!=null) {
		    newm.docUrl   = docUrl.toString();
		} else newm.docUrl   = new String(""); 
		newm.location = newModule.toString();
		newm.setModuleObject( m ); // Built object... don't want to write this

		// big trouble if no image in the module...
		ImageIcon ii = null;
		if (imgUrl!=null) {
		    ii = getScaledIcon(imgUrl);
		
	
		    ii.setDescription(beanName);
		    objectsList.addElement(ii); 
		}

		// Add it to the top object's list...
		NeuromlPkg top = env.getTopObject();
		top.modules.addElement( newm );
		// Notify any lists.............
		// .... update any references....
		String ref = "modules."+beanName;
		env.getNamespace().addRef( "modules."+beanName, newm ); 


		// OK, now the tricky bit...
		// Make an appropriate cell - do lookup on "NeuronModule" ==> "ModuleNeuron"
		ModuleNeuron mn = new ModuleNeuron();
//MODULE--JAG		mn.module.module.setTargetName( ref );
		// find somewhere to put this new ModuleNeuron...
		// do a lookup on "ModuleNeuron" --> make a package if necessary
		env.addNewObject( "ModuleNeuron", "models", mn, Namespace.getShortClassName( beanName ) );
	    }
	} else {
	    System.out.println("unknown command "+e);
	}
    }
    
    public  static ImageIcon getScaledIcon(String url) {
	try { 
	    URL u = new URL(url);
	    return getScaledIcon(u);
	} catch (Exception e) {
	    return null;
	}
    }
    public static ImageIcon getScaledIcon(URL url) {	
	ImageIcon ii1 = new ImageIcon(url);
	ImageIcon ii = null;
	if (ii1.getIconHeight() > 32) {
	    ii = new ImageIcon( ii1.getImage().getScaledInstance( -1,32,Image.SCALE_DEFAULT));
	} else {
	    ii = ii1;
	}
	return ii;
    }
   
    /** Popup a file chooser 
     */
    public URL fileChooser() {
        
	if(this.ModuleDir==null) ModuleDir = System.getProperty("user.dir");
	JFileChooser chooser = new JFileChooser(ModuleDir); 
	ExampleFileFilter filter = new ExampleFileFilter( "jar", "NeuroML modules");
	chooser.addChoosableFileFilter(filter);	
	int returnVal = chooser.showOpenDialog(dialog); 
	if (returnVal == JFileChooser.APPROVE_OPTION) { 
	    URL ret = null;
	    try { 
                ret = chooser.getSelectedFile().toURL(); 
                ModuleDir=chooser.getSelectedFile().getParent();
            } catch (Exception e ) { 
		System.out.println(e); 
	    }
	    return ret;
	} 
	return null;
    }

    /** Chop name  "a.c.b.d.myname"  -> "myname"
     */
    public String chopName(String cname) {
 	if (cname.indexOf(".")!=-1) {
	    cname = cname.substring(cname.lastIndexOf(".")+1);
	} 
	return cname;
    }
    
 
    String currentClass="Module";
    
    /** Change class
     */
    void changeCurrentClass( String c ) {
	currentClass = c;
System.out.println("changeCurrentClass objectsList "+objectsList);
	objectsList.removeAllElements();
System.out.println("changeCurrentClass ns "+ns);
	Hashtable refs = ns.getReferences(); 
	Hashtable objs = null;
	if (refs!=null) {
	     objs = (Hashtable) refs.get(c);
	} else System.out.println("refs is null");
System.out.println("changeCurrentClass objs "+objs);
	if (objs!=null) {
	    for (Enumeration enum_ = objs.keys() ; enum_.hasMoreElements() ;) {
		Object e = enum_.nextElement();
		Object val = objs.get(e);
		if (val instanceof Module) {
		    Module m = (Module)val;
System.out.println("changeCurrentClass imageUrl "+m.imageUrl);
		    ImageIcon k = getScaledIcon(m.imageUrl);
System.out.println("changeCurrentClass after imageUrl ");
		    k.setDescription((String)e);
		    objectsList.addElement(k);
		} else if (e instanceof String) {
		    objectsList.addElement(new ImageIcon((String)e));
		} else {
		    System.out.println(c+" key not imageIcon "+e);
		}
	    }
	}
	jclassesList.setSelectedIndex( classesList.indexOf(currentClass) );
    }

    /** Listen to selections 
     */
    public void valueChanged(ListSelectionEvent e) {
	if (e.getValueIsAdjusting())
	    return;
	
	JList theList = (JList)e.getSource();
	if (theList.isSelectionEmpty()) {
	} else {
	    int index = theList.getSelectedIndex();
	    if (theList == jclassesList) {
		currentClass=(String)classesList.elementAt(index);
		System.out.println("You chose Class "+currentClass);
		changeCurrentClass( currentClass );
	    } else {
		System.out.println("You chose Object "+objectsList.elementAt(index));
		// Edit it...	
		ImageIcon ii = (ImageIcon) objectsList.elementAt(index);
		String name = ii.getDescription();
		Ref tmp = new Ref(currentClass);
		tmp.setTargetName(name);
		Object o = ns.deRef( tmp );
		if (o!=null) {
 		    ObjectNode on = new ObjectNode( o );
		    on.setPath(name);
		    // default type is problematic here
		    // Need to get it from the containing list
		    // How???
		    EditObjectPanel subp = new EditObjectPanel(ns, on, currentClass, false);	
		    subp.popup();
		} else {
		    System.out.println("Couldn't deref "+name);
		}
	    }
	}
    }


    /** A Special renderer for icon + label
     */
    class ComboBoxRenderer extends JLabel implements ListCellRenderer {
        public ComboBoxRenderer() {
            setOpaque(true);
            setHorizontalAlignment(CENTER);
            setVerticalAlignment(CENTER);
        }
        public Component getListCellRendererComponent( JList list,
						      Object value,
						      int index,
						      boolean isSelected,
						      boolean cellHasFocus) {
            if (isSelected) {
                setBackground(list.getSelectionBackground());
                setForeground(list.getSelectionForeground());
            } else {
                setBackground(list.getBackground());
                setForeground(list.getForeground());
            }
            ImageIcon icon = (ImageIcon)value;
            setText(icon.getDescription());
            setIcon(icon);
            return this;
        }
    }

    
    
    
    
    
    static public void loadModule()
    {
        String ModuleDir=null;
        JFrame dialog=null;
        URL newModule=null;
        
        if(ModuleDir==null) ModuleDir = System.getProperty("user.dir");
	JFileChooser chooser = new JFileChooser(ModuleDir); 
	ExampleFileFilter filter = new ExampleFileFilter( "jar", "NeuroML modules");
	chooser.addChoosableFileFilter(filter);	
	int returnVal = chooser.showOpenDialog(dialog); 
	if (returnVal == JFileChooser.APPROVE_OPTION) { 
	    newModule = null;
	    try { 
                newModule = chooser.getSelectedFile().toURL(); 
                ModuleDir=chooser.getSelectedFile().getParent();
            } catch (Exception e ) { 
		System.out.println(e); 
	    }
	} 
        if(newModule==null) return;
	    if (newModule!=null) {		
		System.out.println("Loading jar file "+newModule);
		// *** Try to get the module ***
		// *** Just the default one... ***
		Object m = ModuleLoader.getObject( newModule.toString(), "" );
		String beanName = ModuleLoader.getObjectName();
		String imgName = ModuleLoader.getManifestValue("Icon");
		String docName = ModuleLoader.getManifestValue("Doc");
		String modClass = ModuleLoader.getManifestValue("ModuleClass");
		URL imgUrl = null;
		if (imgName!=null){
		    imgUrl = ModuleLoader.getResource( newModule.toString(), imgName);
		    System.out.println("imgUrl "+imgUrl);
                    System.out.println("Got resource "+imgName+" url "+imgUrl+" object "+m);
		}
		URL docUrl = null;
		if (docName!=null)
		    docUrl = ModuleLoader.getResource( newModule.toString(), docName);
		//System.out.println("Got resource "+imgName+" url "+imgUrl+" "+docUrl+" object "+m);
                
                //JAG:needed?
                
		/*if (modClass!=null) {
		    changeCurrentClass( modClass );
		}

		Module newm = (Module)NeuromlSys.makeNeuromlObject(currentClass);
		if (beanName==null) beanName = new String("noname");
		newm.setName( beanName );


		if (imgUrl!=null) {
		    newm.imageUrl = imgUrl.toString();
		
		} else newm.imageUrl = new String("");

		if (docUrl!=null) {
		    newm.docUrl   = docUrl.toString();
		} else newm.docUrl   = new String(""); 
		newm.location = newModule.toString();
		newm.setModuleObject( m ); // Built object... don't want to write this

		// big trouble if no image in the module...
		ImageIcon ii = null;
		if (imgUrl!=null) {
		    ii = getScaledIcon(imgUrl);
		
	
		    ii.setDescription(beanName);
		    objectsList.addElement(ii); 
		}

		// Add it to the top object's list...
		NeuromlPkg top = env.getTopObject();
		top.modules.addElement( newm );
		// Notify any lists.............
		// .... update any references....
		String ref = "modules."+beanName;
		env.getNamespace().addRef( "modules."+beanName, newm ); 


		// OK, now the tricky bit...
		// Make an appropriate cell - do lookup on "NeuronModule" ==> "ModuleNeuron"
		ModuleNeuron mn = new ModuleNeuron();
		mn.module.module.setTargetName( ref );
		// find somewhere to put this new ModuleNeuron...
		// do a lookup on "ModuleNeuron" --> make a package if necessary
		env.addNewObject( "ModuleNeuron", "models", mn, Namespace.getShortClassName( beanName ) );*/
	    }
    }
    
 
}
