package guikit;

import java.lang.reflect.*;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.NumberFormat;

import neuroml.core.*;
import neuroml.util.*;
 
/** View all referenced objects
 */
public class BrowseObjectsPanel implements ActionListener, ListSelectionListener   {
 
    Namespace ns;

    boolean debug = true;

    JFrame dialog;
    DefaultListModel classesList;
    DefaultListModel objectsList;
    
    JList jclassesList, jobjectsList;

    /** Build the panel
     */
    public BrowseObjectsPanel(Namespace ns) {
	this.ns = ns;

	// Make a panel
	dialog = new JFrame("Browse objects");
	dialog.getContentPane().setLayout( new BoxLayout(dialog.getContentPane(), BoxLayout.Y_AXIS) );


	JPanel left = new JPanel();
	left.setLayout( new BoxLayout(left, BoxLayout.Y_AXIS) );

	JPanel right = new JPanel();
	right.setLayout( new BoxLayout(right, BoxLayout.Y_AXIS) );

	//	lists.add(left);
	//	lists.add(right);

	// List of classes + objects
	classesList = new DefaultListModel();
	jclassesList = new JList(classesList);
	jclassesList.addListSelectionListener( this );
	JScrollPane sl = new JScrollPane(jclassesList);
  	objectsList = new DefaultListModel();
	jobjectsList = new JList(objectsList);
	jobjectsList.addListSelectionListener( this );

	JScrollPane sr = new JScrollPane(jobjectsList);

	sl.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
	sl.setMinimumSize( new Dimension(50,30) );
	sl.setPreferredSize( new Dimension(200,200) );
	sr.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
	sr.setMinimumSize( new Dimension(50,30) );
	sr.setPreferredSize( new Dimension(400,200) );
  
	left.add( new JLabel("Classes"));
	left.add(sl);
	right.add( new JLabel("Objects"));
	right.add(sr);

	JPanel tmp = new JPanel();
	tmp.setLayout( new BoxLayout(tmp, BoxLayout.X_AXIS) );
	JSplitPane lists = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, true, left, right);
	tmp.add(lists); // get around layout bug?
	dialog.getContentPane().add(tmp);
	  
	// Add the buttons...
	JPanel buts = new JPanel();
	buts.setLayout( new BoxLayout(buts, BoxLayout.X_AXIS) );
	buts.add( EditPanel.makeButton("Add", "add", this));
	buts.add( EditPanel.makeButton("Refresh", "refresh", this));
	buts.add( EditPanel.makeButton("Edit", "edit", this));
	buts.add( EditPanel.makeButton("Remove", "remove", this));
	dialog.getContentPane().add(buts);

	dialog.pack();
	popup();
    }      

    public void popup() {
	refreshClasses();
	objectsList.removeAllElements();
	dialog.setVisible(true);
    }

    public void refreshClasses(){
 	classesList.removeAllElements(); // primitive way to update classes...
	Hashtable refs = ns.getReferences();
	boolean foundOne = false;
	for (Enumeration enum_ = refs.keys() ; enum_.hasMoreElements() ;) {
	    String k = (String)enum_.nextElement();
	    classesList.addElement(k);
	} 
   }
  
    /** Listen to buttons
     */
    public void actionPerformed(java.awt.event.ActionEvent e) {
	if (e.getActionCommand().equals("refresh")) {
	    refreshClasses();
	} 
    }


    /** Chop name  "a.c.b.d.myname"  -> "myname"
     */
    public String chopName(String cname) {
 	if (cname.indexOf(".")!=-1) {
	    cname = cname.substring(cname.lastIndexOf(".")+1);
	} 
	return cname;
    }
    
    String currentClass="NamedObject";

    /** Listen to selections 
     */
    public void valueChanged(ListSelectionEvent e) {
	if (e.getValueIsAdjusting())
	    return;
	
	JList theList = (JList)e.getSource();
	if (theList.isSelectionEmpty()) {
	} else {
	    int index = theList.getSelectedIndex();
	    if (theList == jclassesList) {
		currentClass=(String)classesList.elementAt(index);
		System.out.println("You chose Class "+currentClass);
		// Get a list of objects...
		objectsList.removeAllElements();
		Hashtable refs = ns.getReferences();
		String cname = (String)classesList.elementAt(index);
		Hashtable objs = (Hashtable) refs.get(cname);
		if (objs!=null) {
		    for (Enumeration enum_ = objs.keys() ; enum_.hasMoreElements() ;) {
			String k = (String)enum_.nextElement();
			objectsList.addElement(k);
		    }
		}
	
	    } else {
		System.out.println("You chose Object "+objectsList.elementAt(index));
		// Edit it...
		String name = (String)(objectsList.elementAt(index));
		Ref tmp = new Ref(currentClass);
		tmp.setTargetName(name);
		Object o = ns.deRef( tmp );
		if (o!=null) {
		    String cname = chopName(name);
		    ObjectNode on = new ObjectNode( o );
		    on.setPath(name);
		    // default type is problematic here
		    // Need to get it from the containing list
		    // How???
		    EditObjectPanel subp = new EditObjectPanel(ns, on, currentClass, false);	
		    subp.popup();
		} else {
		    System.out.println("Couldn't deref "+name);
		}
	    }
	}
	
    }
}
