package guikit;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.NumberFormat;


/** Top level class for field editors
 */
public class DisplayPanel extends JPanel {
    String title;
    DecimalField paramValueField;
    JTextField paramTextField; 
    ConverterRangeModel sliderModel;
    JSlider slider;
    JCheckBox checkbutton1,checkbutton2;

    public DisplayPanel(){
	setLayout(new BorderLayout());
    }

    public void refresh(){
	removeAll();	
    }

    /**
     * Display double number parameter
     *
     */
    public void displayDouble(String disptitle, double value){
	title = disptitle;
        JLabel paramLabel = new JLabel(title);
        paramValueField = new DecimalField(value, 10);

	paramValueField.getDocument().addDocumentListener( 
		     new DisplayListener(this,title,paramValueField,"Digit") );

	JPanel parameterPane = createPanel();

        parameterPane.add(paramLabel);
	parameterPane.add(paramValueField);
        add(parameterPane); 
    }

    /**
     * Display int number parameter
     *
     */
    public void displayInteger(String disptitle, int value){
	title = disptitle;
	JLabel paramLabel = new JLabel(title);

	//Add the text field.  It initially displays "0" and needs
        //to be at least 10 columns wide. 
	
        paramValueField = new DecimalField((double)value,10);
	paramValueField.addActionListener(new DisplayListener(this,title,paramValueField,"Digit"));
	paramValueField.getDocument().addDocumentListener(new DisplayListener(this,title,paramValueField,"Digit"));

	JPanel parameterPane = createPanel();

        parameterPane.add(paramLabel);
	parameterPane.add(paramValueField);
        add(parameterPane); 
    }

    /**
     * Display Slider for double number parameter
     *
     * min and max : range of the slide
     */
    public JSlider displaySliderDouble(String disptitle, double value, double min, double max){
	title = disptitle;
        JLabel paramLabel = new JLabel(title);
	JLabel paramLabelEmpty = new JLabel("");

	paramValueField = new DecimalField(value, 10);

	sliderModel = new ConverterRangeModel(); // define here the range
	sliderModel.setDoubleValue(value);
	Double dmin = new Double(min);
	Double dmax = new Double(max);
	sliderModel.setMaximum(dmax.intValue());
	sliderModel.setMinimum(dmin.intValue());
	//Add the slider.
        slider = new JSlider(sliderModel);

        sliderModel.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                setParam(title,sliderModel.getDoubleValue());
		paramValueField.setValue(sliderModel.getDoubleValue());
            }
        });
	//slider.setMajorTickSpacing(1000);
	//slider.setMinorTickSpacing(1000);
	//slider.setPaintTicks(true);
	//slider.setPaintLabels(true);
	slider.setBorder(BorderFactory.createEmptyBorder(0,0,10,0));

	paramValueField.addActionListener(new DisplayListener(this,title,paramValueField,"Digit",slider));
	//paramValueField.getDocument().addDocumentListener(new DisplayListener(this,title,paramValueField,"None",slider));

	JPanel parameterPane = createPanel();
	parameterPane.setLayout(new GridLayout(2,2));
        parameterPane.add(paramLabel);
	parameterPane.add(paramValueField);
	parameterPane.add(paramLabelEmpty);
	parameterPane.add(slider);
        add(parameterPane); 

	return slider;
    }

    /**
     * Display a slider for int number parameter
     *
     */
    public JSlider displaySliderInteger(String disptitle, int value){
	title = disptitle;
	JLabel paramLabel = new JLabel(title);
	JLabel paramLabelEmpty = new JLabel("");

	// to be better defined or set somewhere else
	int min = 0;
	int max = 10000;


	paramValueField = new DecimalField((double)value,10);
	
	//Add the slider.

        slider = new JSlider(min,max,value);
        slider.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
		setParam(title,slider.getValue());
		paramValueField.setValue(slider.getValue());
            }
        });
	//slider.setMajorTickSpacing(10);
	//slider.setMinorTickSpacing(10);
	//slider.setPaintTicks(true);
	//slider.setPaintLabels(true);
	slider.setBorder(BorderFactory.createEmptyBorder(0,0,10,0));
	paramValueField.addActionListener(new DisplayListener(this,title,paramValueField,"Digit",slider));
	//paramValueField.getDocument().addDocumentListener(new DisplayListener(this,title,paramValueField,"None",slider));

	JPanel parameterPane = createPanel();
	parameterPane.setLayout(new GridLayout(2,2));
        parameterPane.add(paramLabel);
	parameterPane.add(paramValueField);
	parameterPane.add(paramLabelEmpty);
	parameterPane.add(slider);
        add(parameterPane); 

	return slider;
    }

    /**
     * Display string parameter
     *
     */
    public void displayString(String disptitle, Object value){
	 displayString(disptitle,value,0,"");
    }
    public void displayString(String disptitle, Object value, int scrollOffset){
	displayString(disptitle,value,scrollOffset,"");
    }
    public void displayString(String disptitle, Object value,int scrollOffset, String option){
	title = disptitle;
        JLabel paramLabel = new JLabel(title);

	//Add the text field.  It initially displays "0" and needs
        //to be at least 10 columns wide.
	String val = (String)value;
	if(val==null) val = new String("");
        paramTextField = new JTextField(val,val.length());
	//paramTextField.setMaximumSize(new Dimension(500,25));
	//if(option.equals("direct")){
	paramTextField.getDocument().addDocumentListener(new DisplayListener(this,title,paramTextField,"Text"));
	//System.out.println("display string with offset = "+scrollOffset);
	//paramTextField.setScrollOffset(scrollOffset);
	Rectangle rscroll = new Rectangle(paramTextField.getColumns()-scrollOffset,1);
	//paramTextField.scrollRectToVisible(rscroll);

	    //}
	//else
	paramTextField.addActionListener(new DisplayListener(this,title,paramTextField,"Text"));
	//paramTextField.getDocument().addDocumentListener(new DisplayListener(this,title,paramTextField,"Text"));
	
	JPanel parameterPane = createPanel();
	//parameterPane.setLayout(new BoxLayout(parameterPane,BoxLayout.X_AXIS));
	//parameterPane.setMaximumSize(new Dimension(500,1000));
        parameterPane.add(paramLabel);
	parameterPane.add(paramTextField);
        add(parameterPane); 
        //parameterPane.setAlignmentY(TOP_ALIGNMENT);
    }

    /**
     * Display text
     *
     */
    public JTextArea displayText(String disptitle, String value){
	return displayText( disptitle,  value,"");
    }
    public JTextArea displayText(String disptitle, String value, String option){
	title = disptitle;
        JLabel paramLabel = new JLabel(title);
	
	JTextArea textArea = new JTextArea(value);
	//textArea.setFont(new Font("Serif", Font.ITALIC, 16));
	textArea.setLineWrap(true);
	textArea.setWrapStyleWord(true);
	textArea.setEditable(true);
	
	JScrollPane areaScrollPane = new JScrollPane(textArea);
	if(option.equals("small"))
	    areaScrollPane.setPreferredSize(new Dimension(300, 50));
	else
	    areaScrollPane.setPreferredSize(new Dimension(300, 200));
	add(areaScrollPane);
	// returned to get the modified text elsewhere
	return textArea;
    }
    
    /**
     * Display  location : string + button likn to file chooser
     *
     */
    public void displayLocation(String disptitle, String value){
	displayLocation(disptitle,value,0);
    }

    public void displayLocation(String disptitle, String value, int scrollOffset){

	title = disptitle;
	
	//Add the text field.  It initially displays "0" and needs
        //to be at least 10 columns wide.
	String val = (String)value;
	if(val==null) val = new String("");
        paramTextField = new JTextField(val,val.length());
	//paramTextField.setMaximumSize(new Dimension(500,25));
	paramTextField.addActionListener(new DisplayListener(this,title,paramTextField,"Text"));
	paramTextField.getDocument().addDocumentListener(new DisplayListener(this,title,paramTextField,"Text"));
	paramTextField.setScrollOffset(scrollOffset);
	JButton button = null;

	// Browse button
	button = new JButton(" Set "+title);
        //button.setToolTipText(help);
        button.addActionListener(new DisplayListener(this,"Set"+title,paramTextField,"Browse"));
	//button.addDocumentListener(new DisplayListener(this,"Set"+title,paramTextField,"Browse"));
	button.setFocusPainted(false);
	JPanel parameterPane = createWidePanel();

	parameterPane.add(button);
	parameterPane.add(paramTextField);
	
	
        add(parameterPane); 
        //parameterPane.setAlignmentY(TOP_ALIGNMENT);
    }


    /**
     * Display  combobox
     *
     */
    public JComboBox displayComboBox(String disptitle,String initVal,Hashtable list){
	return displayComboBox(disptitle,initVal,list,"");
    }

    /**
     * Display  combobox
     *
     */
    public JComboBox displayComboBox(String disptitle,String initVal,Hashtable list, String width){
	// index of the initial value
	int index = 0;
	String[] possibilities = new String[list.size()];
	
	int i = 0;
	for (Enumeration enum_ = list.keys() ; enum_.hasMoreElements() ;) {
	    String elem = (String)enum_.nextElement();
	    possibilities[i] = new String(elem);
	    if(elem.endsWith(initVal))index=i;
	    i++;
	}
	
	JComboBox comboBox = new JComboBox(possibilities);
	comboBox.setSelectedIndex(index);
	comboBox.addActionListener(new DisplayListener(this,disptitle,"ComboBox"));
	JPanel parameterPane = null;
	if (width.equals("wide")){
	     parameterPane = createWidePanel();
	     parameterPane.setLayout(new BoxLayout(parameterPane,BoxLayout.X_AXIS));
	     parameterPane.setMaximumSize(new Dimension(500,1000));
	     comboBox.setMaximumSize(new Dimension(500,1000));
	     comboBox.setMaximumRowCount(500);
	} else
	   parameterPane = createPanel();
	

	JLabel paramLabel = new JLabel(disptitle);
	paramLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
	parameterPane.add(paramLabel);
	parameterPane.add(comboBox);
        add(parameterPane);

	return comboBox;
    }


    /**
     * Display  checkboxes
     *
     */
    public void displayTwoExclusiveCheckButtons(String disptitle,String disptitle2,boolean button1active){
	
	checkbutton1 = new JCheckBox(disptitle);
	checkbutton2 = new JCheckBox(disptitle2);  
	if(button1active)
              checkbutton1.setSelected(true);
	else checkbutton2.setSelected(true);
	
            
            
	checkbutton1.addItemListener(new DisplayListener(this,disptitle,"CheckBox"));
	checkbutton2.addItemListener(new DisplayListener(this,disptitle2,"CheckBox"));

	JPanel parameterPane = createPanel();

	parameterPane.add(checkbutton1);
	parameterPane.add(checkbutton2);
        add(parameterPane); 
    }

    /**
     * update mutual exclusion on check button
     */
    public void updateExclusionBox1(boolean buttonActive){
	if(buttonActive){
	    if(!checkbutton1.isSelected())
		checkbutton1.setSelected(true);
	}
	else {
	    if(checkbutton1.isSelected())
		checkbutton1.setSelected(false);	  
	}
    }

    /**
     * update mutual exclusion on check button
     */
    public void updateExclusionBox2(boolean buttonActive){
	if(buttonActive){
	    if(checkbutton2.isSelected())
		checkbutton2.setSelected(false);
	}
	else {
	   if(!checkbutton2.isSelected())
	       checkbutton2.setSelected(true);
	}
    }

    /**
     * Display  button
     *
     */
    public void displayButton(String disptitle){
	displayButton(disptitle,"left");
    }

    public void displayButton(String disptitle, String pos){
	title = disptitle;
	JLabel paramLabel = new JLabel("");
	JButton button = null;

	// Browse button
	button = new JButton(title);
        //button.setToolTipText(help);
        button.addActionListener(new DisplayListener(this,title,paramTextField,"Button"));
	//button.addDocumentListener(new DisplayListener(this,title,paramTextField,"Text"));
	button.setFocusPainted(false);
	JPanel parameterPane = createPanel();

	if(pos.equals("right"))
	    parameterPane.add(paramLabel);
	parameterPane.add(button);
	if(pos.equals("left"))
	   parameterPane.add(paramLabel);
        add(parameterPane); 
        //parameterPane.setAlignmentY(TOP_ALIGNMENT);
    }

    /**
     * Display  2 buttons
     *
     */
    public void displayTwoButton(String disptitle, String disptitle2){
	
	JButton button = null;
	JButton button2 = null;
	
	button = new JButton(disptitle);
	button2 = new JButton(disptitle2);
        //button.setToolTipText(help);
        button.addActionListener(new DisplayListener(this,disptitle,paramTextField,"Button"));
	button2.addActionListener(new DisplayListener(this,disptitle2,paramTextField,"Button"));
	//button.addDocumentListener(new DisplayListener(this,title,paramTextField,"Text"));
	button.setFocusPainted(false);
	button2.setFocusPainted(false);

	JPanel parameterPane = createPanel();
		    
	parameterPane.add(button);
	parameterPane.add(button2);
        add(parameterPane); 
        //parameterPane.setAlignmentY(TOP_ALIGNMENT);
    }

    /**
     * Display 2 buttons for a list, the index help to find which element 
     * of the list is concerned
     * note : this method deosn't add the pane by itself but return it instead
     *
     */
    public JPanel displayList(String nametitle, String dispname,
			      String disptitle2,String disptitle3, int index, String option){
	return displayList( nametitle,0,  dispname,  disptitle2, disptitle3,  index,  option);
    }

    public JPanel displayList(String nametitle, int scrollOffset, 
			      String dispname, String disptitle2,String disptitle3, int index, String option){
	
	//JLabel label = null;
	JButton button2 = null;
	JButton button3 = null;
	//if(option.equals("indexed"))
	//    label = new JLabel(" "+disptitle+" "+index);
	//else
	//    label = new JLabel(" "+disptitle);
	
	//super.getPreferredSize().width,
	// super.getPreferredSize().height);
	button2 = new JButton(disptitle2);
	button3 = new JButton(disptitle3);

	if(dispname==null) dispname = new String("");
        paramTextField = new JTextField(dispname,dispname.length());
	//paramTextField.addActionListener(new DisplayListener(this,title,paramTextField,"Text"));
	paramTextField.getDocument().addDocumentListener(
               new DisplayListener(this,nametitle,paramTextField,"List",index));
	paramTextField.setScrollOffset(scrollOffset);
	//Dimension dispDim = new Dimension(100,25);
	//label.setPreferredSize(dispDim);
	//button2.setPreferredSize(dispDim);
	//button3.setPreferredSize(dispDim);
	//label.setMaximumSize(dispDim);
	//button2.setMaximumSize(dispDim);
	//button3.setMaximumSize(dispDim);
        //button.setToolTipText(help);
	button2.addActionListener(new DisplayListener(this,disptitle2,"List",index));
	button3.addActionListener(new DisplayListener(this,disptitle3,"List",index));

	button2.setFocusPainted(false);
	button3.setFocusPainted(false);
	JPanel parameterPane = createWidePanel();
	//JPanel parameterPane = createPanel();

	parameterPane.setBorder(BorderFactory.createEmptyBorder(
                                                0,0,0,5));
	parameterPane.setLayout(new GridLayout(1,3));
	//parameterPane.add(label);
	parameterPane.add(paramTextField);
	parameterPane.add(button2);
	parameterPane.add(button3);
	//parameterPane.setPreferredSize(dispDim);
	//parameterPane.setMaximumSize(dispDim);
        return parameterPane; 
        //parameterPane.setAlignmentY(TOP_ALIGNMENT);
    }

    /**
     * Display a title
     *
     */
    public void displayTitle(String disptitle){
	title = disptitle;
        JLabel paramLabel = new JLabel(title);
	
	JPanel parameterPane = createPanel();
	
        parameterPane.add(paramLabel);
        add(parameterPane);  
        //parameterPane.setAlignmentY(TOP_ALIGNMENT);
    }	

    /**
     * Display two titles
     *
     */
    public void displayTwoTitles(String disptitle,String disptitle2){
	title = disptitle;
        JLabel paramLabel = new JLabel(disptitle);
	JLabel paramLabel2 = new JLabel(disptitle2);
	
	JPanel parameterPane = createPanel();
	
        parameterPane.add(paramLabel);
	parameterPane.add(paramLabel2);
        add(parameterPane);  
        //parameterPane.setAlignmentY(TOP_ALIGNMENT);
    }
  
    /**
     * create the JPanel
     *
     */
    public JPanel createPanel(){
	//Make the name/value group a fixed size.
        JPanel parameterPane = new JPanel() {
            public Dimension getMinimumSize() {
                return getPreferredSize(); 
            }
            public Dimension getPreferredSize() {
                return new Dimension(300,super.getPreferredSize().height); 
		//super.getPreferredSize().width,
		// super.getPreferredSize().height);
            }
            public Dimension getMaximumSize() {
                //return getPreferredSize();
		//return super.getMaximumSize();
		return new Dimension(super.getMaximumSize().width,super.getPreferredSize().height);
            }
        };
	parameterPane.setBorder(BorderFactory.createEmptyBorder(
                                                0,0,0,5));
	//parameterPane.setLayout(new GridLayout(1,2));
	parameterPane.setLayout(new BoxLayout(parameterPane, 
					      BoxLayout.X_AXIS));
	

	return parameterPane;
    }

    /**
     * create the JPanel
     *
     */
    public JPanel createWidePanel(){
	//Make the name/value group a fixed size.
        JPanel parameterPane = new JPanel() {
            public Dimension getMinimumSize() {
                return getPreferredSize(); 
            }
            public Dimension getPreferredSize() {
                return new Dimension(500,super.getPreferredSize().height); 
		//super.getPreferredSize().width,
		// super.getPreferredSize().height);
            }
            public Dimension getMaximumSize() {
                //return getPreferredSize();
		//return super.getMaximumSize();
		return new Dimension(super.getMaximumSize().width,super.getPreferredSize().height);
		//return new Dimension(getPreferredSize().width,super.getPreferredSize().height);
		//return super.getMaximumSize();
            }
        };
	parameterPane.setBorder(BorderFactory.createEmptyBorder(
                                                0,0,0,5));
		//parameterPane.setLayout(new GridLayout(1,2));
	parameterPane.setLayout(new BoxLayout(parameterPane, 
						      BoxLayout.X_AXIS));
	return parameterPane;
    }

    /**
     * create a very wide JPanel
     *
     */
    public JPanel createVeryWidePanel(){
	//Make the name/value group a fixed size.
        JPanel parameterPane = new JPanel() {
            public Dimension getMinimumSize() {
                return getPreferredSize(); 
            }
            public Dimension getPreferredSize() {
                return new Dimension(600,super.getPreferredSize().height); 
		//super.getPreferredSize().width,
		// super.getPreferredSize().height);
            }
            public Dimension getMaximumSize() {
                //return getPreferredSize();
		return new Dimension(super.getMaximumSize().width,super.getPreferredSize().height);
		//return super.getMaximumSize();
            }
        };
	parameterPane.setBorder(BorderFactory.createEmptyBorder(
                                                0,0,0,5));
	//parameterPane.setLayout(new GridLayout(1,2));
	parameterPane.setLayout(new BoxLayout(parameterPane, 
						      BoxLayout.X_AXIS));
	return parameterPane;
    }


    void setParam(String title, String value){
	System.out.println("bad setParam String");
    }
    void setParam(String title, double value){
	System.out.println("bad setParam double");
    }
    void setParam(String title, int value){
	System.out.println("bad setParam int");
    }
    void setParam(String title, String name, int offset){
	System.out.println("bad setParam String  int");
    }
    void setParam(String title, String name,int offset, int value){
	System.out.println("bad setParam String  int");
    }
    void confirm(){
	System.out.println("wrong confirm() ");
    }
}
