package guikit;

import java.lang.reflect.*;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.NumberFormat;
import java.net.*;

import neuroml.util.*;
import neuroml.core.*;
import neuroml.sim.*;


/** Object editor
 */
public class EditObjectPanel extends EditPanel implements ActionListener, ElementChangedListener {

    Object value;
    JFrame dialog;
    GraphPanel ParentPanel;
   // JDialog dialog;
    Namespace ns;
    Vector subpanels = new Vector(); // sub panels
    Vector subobjects = new Vector(); // sub object nodes
    JComboBox classComboBox; // for choosing subclasses...
    String baseClass; // Expected base class
    JPanel topPanel;
    boolean inline, inlist, subinline;
   
    ElementChangedListener notifyParent = null;
    
    boolean debug = false;

    String objName="";
 
    Editor specialEditor = null;
   

    /** Build the panel
     * @param ns The namespace for references
     * @param node The object node (field of parent)
     * @param baseClass The expected class of this object
     * @param inline Display contents in parent's JPanel
     * @param inlist A member of a list
     * @param subinline Display subobjects as popups or inline
     */
    public EditObjectPanel(Namespace ns, Object node, String baseClass, boolean inline, boolean inlist, 
			   boolean subinline) {
	this(  ns, new ObjectNode( node ), baseClass, inline,inlist,subinline ); 
    }
    public EditObjectPanel(Namespace ns, Object node, String baseClass, boolean inline, boolean inlist, 
			   boolean subinline, String label) {
	this(  ns, new ObjectNode( node ), baseClass, inline,inlist,subinline,label ); 
    }
    public EditObjectPanel(Namespace ns, ObjectNode node, String baseClass, boolean inline, boolean inlist, GraphPanel ParentPanel) {
        this( ns, node, baseClass, inline,inlist ); 
        this.ParentPanel=ParentPanel;
    }
    public EditObjectPanel(Namespace ns, Object node, String baseClass, boolean inline, boolean inlist) {
	this( ns, new ObjectNode( node ), baseClass, inline,inlist ); 
    }
    public EditObjectPanel(Namespace ns, Object node, String baseClass, boolean inline) {
	this( ns, new ObjectNode( node ),baseClass, inline );
    }
    public EditObjectPanel(Namespace ns,  ObjectNode node, String baseClass, boolean inline) {
	this( ns, node,baseClass, inline, false );	
    }    
    public EditObjectPanel(Namespace ns, ObjectNode node, String baseClass, 
			   boolean inline, boolean inlist) {
	this(  ns, node,baseClass, inline,inlist, true);
    }
    public EditObjectPanel(Namespace ns, ObjectNode node, String baseClass, 
			   boolean inline, boolean inlist, boolean subinline) {
	this(  ns, node,baseClass, inline,inlist, true, null);

    }
    public EditObjectPanel(Namespace ns, ObjectNode node, String baseClass, 
			   boolean inline, boolean inlist, boolean subinline, String label) {
	super(node);
	//System.out.println("EditObject panel path="+node.getPath()+" inline "+inline);
	this.ns = ns;
	this.baseClass = baseClass;
	value = node.getContent();
	objName = getObjectName();

	this.inline = inline;
	this.inlist = inlist;
	this.subinline = subinline;
 
	// Is there a special editor for this object?
	// *** Note - there will be a problem with derived classes...
	// *** explicitly register the special editor for them too???? ****
	specialEditor = SpecialEditors.get( value.getClass().getName() );
	//System.out.println("EditObject specialEditor "+specialEditor);

	if (specialEditor != null) {
	    inline = false; // make a button
	}

	if (inline) {
	    // Make a border...
	    setBorder( BorderFactory.createTitledBorder(node.getName()) );
                //setLayout( new BoxLayout(this, BoxLayout.Y_AXIS) );
                setLayout( new BoxLayout(this, BoxLayout.Y_AXIS) );
            //else
             //   setLayout( new BoxLayout(this, BoxLayout.X_AXIS) );
            
	    topPanel = this;
            
	    addFields( this, baseClass, inline, inlist );
	} else {
	    // Make a popup dialog     	    
	    // Make a frame, with OK/Cancel
 	    if (specialEditor==null) {
		add(makeButton("Edit "+objName, "edit", this));
		if (label == null) label = "Edit "+objName;
		dialog = new JFrame(label);	  
                //dialog = new JDialog(label);	  
		updateDialog();
	    } else {
		add(makeButton("Special Edit "+objName, "specialedit", this));
	    }
	}
    }    


    public Dimension getMinimumSize() {	
	return super.getMinimumSize();
    }
    public Dimension getPreferredSize() {
	return super.getPreferredSize();
    }
    public Dimension getMaximumSize() {
	return super.getMaximumSize();
	//return new Dimension(super.getMaximumSize().width,super.getPreferredSize().height);
    }
    


    /** Add notification of parent */
    public void addElementChangedListener( ElementChangedListener p ) {
	notifyParent = p;
    }

    /** Return name of this object */
    String getObjectName() {
	return (value instanceof Nameable) ? ((Nameable)value).getName() : node.getName();
    }

    /** Put new content in dialog
     */
    public void updateDialog() {
	if (debug) System.out.println("updating dialog");
	dialog.getContentPane().setLayout( new BoxLayout(dialog.getContentPane(),BoxLayout.Y_AXIS));

	JPanel fields = new JPanel() ;
        /*{
		public Dimension getMinimumSize() {	
		    return super.getMinimumSize();
		}
		public Dimension getPreferredSize() {
		    return super.getPreferredSize();
		}
		public Dimension getMaximumSize() {
		    return super.getMaximumSize();
		}
	    };*/
	fields.setLayout( new BoxLayout(fields, BoxLayout.Y_AXIS) );
        //fields.setLayout( new GridLayout(0,2) );
            
	topPanel = fields;
	inline = true;
	addFields( fields, baseClass, inline, inlist );
	JPanel buttons = new JPanel();
	buttons.setLayout( new BoxLayout(buttons, BoxLayout.X_AXIS) );
//	buttons.add(makeButton("OK", "ok", this));
        String stuff=new String();
        if(node!=null)
            if(node.getParent()!=null) stuff=this.node.getParent().getClass().getName();
        if(stuff.compareTo(new String("neuroml.core.Set"))==0) buttons.add(makeButton("Remove", "remove", this));
//        System.out.println(stuff);
//	buttons.add(makeButton("Cancel", "cancel", this));
        buttons.add(makeButton("Save "+Namespace.getShortClassName(node.getName()), "save", this));
        
        JPanel pan=new JPanel();
        pan.add(fields);
       
        int v=ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED;
        int h=ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED;     
         JScrollPane jsp = new JScrollPane(pan,v,h);
        dialog.getContentPane().add(jsp);
        dialog.getContentPane().add(buttons); 
        
	dialog.pack();
        
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

        if(dialog.getHeight()>screenSize.height)
            dialog.setSize(dialog.getWidth()+h, screenSize.height-40);
	if (debug) System.out.println("done updating dialog");
    }

    public void popup() {
	if (dialog!=null) dialog.setVisible(true);
    }
    

    public void cancel() {
	dialog.setVisible(false);
	super.cancel();
    }
    
    public void remove() {
        if(ParentPanel!=null)
        {
        	ParentPanel.RemoveElement(node);
                if (dialog!=null)
                    dialog.setVisible(false);
        }
        else System.out.println("Not yet implemented in these circumstances!");
    }
    
    /** Respond to buttons
     */
    public void actionPerformed(java.awt.event.ActionEvent e) {
	update(getGraphics());
	if (e.getActionCommand().equals("ok")) {
	    dialog.setVisible(false);
	    confirm();
	} else if (e.getActionCommand().equals("edit")) { // For "Edit xxx" button
	    dialog.pack();
            dialog.setVisible(true);      
	} else if (e.getActionCommand().equals("specialedit")) { // For special editor
	    specialEditor.editObject( value, getNode().path, this ); 
	} else if (e.getActionCommand().equals("cancel")) {          	    
	    cancel();
	} else if (e.getActionCommand().equals("remove")) {          	    
	    remove();            
        } else if (e.getActionCommand().equals("save")) {          	    
	    save(); 
        }  else {
	    /** Change the class of this object */
	    JComboBox cb = (JComboBox)e.getSource();	    
	    String selectName = (String)cb.getSelectedItem();	    
	    objName=getObjectName();
	    if (debug) System.out.println("Changing class to "+selectName);
	    
	    // Remove reference to old object...
	    // and all referenced objects...
            String str=node.getPath();
            String truc=str.substring(0,str.lastIndexOf("."));
	    ns.removeRef(node.getPath(),value);

	    // make a new object of this class
	    // rebuild the panel...
	    Object newo = NeuromlSys.makeNeuromlObject( selectName );

	    // if object is parameter and so not a neuroml object :
	    if (newo==null)
		 newo = NeuromlSys.makeObjectFromJar( selectName );

	    if (debug) System.out.println("made "+newo);

	    topPanel.removeAll();
	    if (debug) System.out.println("Node is "+getNode());
	    // set name...
            if (newo instanceof Nameable) ((Nameable)newo).setName(objName);
            this.SwitchObjects(value,newo); 

            Object Obj=getNode().getParent();
            if(Obj!=null)
            {
                if(Obj instanceof neuroml.core.Set&&value instanceof NamedObject)
                {
                    ((neuroml.core.Set)Obj).removeElement(value);
                    ((neuroml.core.Set)Obj).addElement(newo);
                }
            }          
                     
            getNode().setContent(newo);
            if(value instanceof NamedObject){
                getNode().setPath(truc+"."+((NamedObject)value).getName());
                ns.addRef(getNode().getPath(), newo);
            }
            if (notifyParent!=null) 
            {	    
                  notifyParent.elementChanged(value,newo);// notifyParent.elementChanged( value,value );
            }
            value=newo;  
	    addFields( topPanel, baseClass, inline, inlist );
	    if (dialog!=null) { dialog.pack(); dialog.setVisible(true); }
	    topPanel.validate();
	    revalidate();
	    if (debug) System.out.println("done popup");

        }
    }

    /** Look for a (probably static) field called <fieldname>Hints
     */
    DisplayHints getDisplayHints( Field[] flds, Object parent, String fieldname ) {
	String hintName = fieldname+"Hints";
 
	DisplayHints ret = null;
	for (int i=0; i<flds.length && ret==null; i++) {
	    if (flds[i].getName().equals(hintName)) {
		try {
		    //		    System.out.println("Found the hints for "+fieldname);
		    Object h = flds[i].get(parent);
		    if (h instanceof DisplayHints) {
			ret = (DisplayHints) h;
			// System.out.println("Hints: "+ret);			
		    }
		} catch (Exception e) {}
	    }
	}	
	return ret;
    }

    void addFields( JPanel fieldPan, String baseClass, boolean inline, boolean inlist ) {
	// For all fields...
	Class c = value.getClass();
	Field[] flds = c.getFields();
 
	if (inlist) { // allow changing of name --- this should allow nonnamable items in list
	    if (value instanceof Nameable) {
		NameNode on = new NameNode(value,((Nameable)value).getName());
		EditStringPanel namep = new EditStringPanel( on );
		namep.setElementChangedListener(this);
		addSubPanel(fieldPan,namep);
	    }
	}

	// Get a list of possble subclasses...
	// What's the root class for this object? The type of the parent...
	Class expClass = null;
	String thisClassName = c.getName();
        //System.out.println("thisClassName "+thisClassName);
        //System.out.println("baseClass "+baseClass);

	// if still null, try from extra jar classes
	/*if (baseClass != null) { 
	     expClass =  NeuromlSys.getClassFromJar( baseClass );
	 }
	*/
	if (baseClass != null) {
	//if ((baseClass != null)&&(expClass == null)) {

	    expClass =  NeuromlSys.getClassFromShortName( baseClass );
	} else {	    
	    // Expected class??
	    if (inlist) {
		// - if list : list.getDefaultClassName()	
		Object p = getNode().getParent();
		if (p instanceof neuroml.core.List) {
		    neuroml.core.List lp = (neuroml.core.List)p;
		    expClass = NeuromlSys.getClassFromShortName( lp.getElementType() );
		} else if (p instanceof neuroml.core.Set) {
		    neuroml.core.Set lp = (neuroml.core.Set)p;
		    expClass = NeuromlSys.getClassFromShortName( lp.getElementType() );
		}
	    } else {
		Field parentField = getNode().getField();
		if (parentField != null) {
		    expClass = parentField.getType();
		}
	    }	
	}
        //System.out.println("expclass "+expClass);

	// if still null, try from extra jar classes
	if (expClass == null) { 
	    expClass =  NeuromlSys.getClassFromJar( baseClass );
	}
	if (expClass != null) { // we know what we're expecting...
	    // Now get a list... 

	    if (debug) System.out.println("  This object "+value+" could be a "+expClass.getName());

	    //ns.printDerivedFrom();
	    //System.out.println("namespace is then "+ns);
	      

	    Vector derivedFrom = ns.getDerivedFrom( expClass.getName() );

	    if (debug) {
		for (int i=0; i<derivedFrom.size(); i++) {
		    System.out.println("  or a "+derivedFrom.elementAt(i));
		}	    
	    }

	    // OK, make a "change Class" selection box...
	    if (derivedFrom.size() > 0) {
		int currIdx = derivedFrom.indexOf(thisClassName)+1;
		int sz = derivedFrom.size()+1;
		String[] choices = new String[sz];
		choices[0] = expClass.getName(); // first choice is base class
		for (int j=1; j<sz; j++) choices[j]=(String)(derivedFrom.elementAt(j-1));
		classComboBox = new JComboBox(choices);
		classComboBox.setSelectedIndex(currIdx);
		classComboBox.addActionListener(this);
		fieldPan.add(classComboBox);
	    }
	}

	for (int i=0; i<flds.length; i++) {
	    try {
		Object fob = flds[i].get(value);
		int mod  = flds[i].getModifiers();
		if (Modifier.isStatic(mod)) {
		    // System.out.println("Field "+flds[i]+" is STATIC");
		} else if (Modifier.isPublic(mod)) {		
		    if (debug) System.out.println("Adding field "+flds[i].getName());
		    if (fob!=null) {
			ObjectNode on = new ObjectNode( value, flds[i].getName(),this );
			// Any display hints???
			DisplayHints hint = getDisplayHints( flds, value, flds[i].getName());
			on.setDisplayHints(hint);
			on.setPath(node.getPath(),flds[i].getName());
			if (fob instanceof Integer ) {
			    addSubPanel( fieldPan, new EditIntegerPanel( on ) );
			} else if (fob instanceof Short ) {
			    addSubPanel( fieldPan, new EditShortPanel( on ) );
			} else if (fob instanceof Double ) {
			    addSubPanel( fieldPan, new EditDoublePanel( on ) );
			} else if (fob instanceof DValue ) {
			    addSubPanel( fieldPan, new EditDValuePanel( ns,on ) );
                        } else if (fob instanceof Boolean ) {
			    addSubPanel( fieldPan, new EditBooleanPanel( on ) );                            
			} else if (fob instanceof String ) {
			    addSubPanel( fieldPan, new EditStringPanel( on ) );
			} else if (fob instanceof URL ) {
			    addSubPanel( fieldPan, new EditURLPanel( on ) );
			} else if (fob instanceof neuroml.core.NeuromlArray ) {
			    addSubPanel( fieldPan, new EditArrayPanel(ns, on ) );
                         } else if (fob instanceof neuroml.core.Set ) {
			    addSubPanel( fieldPan, new EditSetPanel(ns, on ) );
			} else if (fob instanceof neuroml.core.List ) {
			    addSubPanel( fieldPan, new EditListPanel(ns, on ) );
			} else if (fob instanceof neuroml.core.TreeRef ) {
			    EditTreeRefPanel erp = new EditTreeRefPanel(ns, on );
			    erp.addElementChangedListener(this);
			    addSubPanel( fieldPan, erp );
                        } else if (fob instanceof neuroml.core.Ref ) {
			    EditRefPanel erp = new EditRefPanel(ns, on );
			    erp.addElementChangedListener(this);
			    addSubPanel( fieldPan, erp );
			} else if (fob instanceof neuroml.core.ModuleRef ) {
			    addSubPanel( fieldPan, new EditModuleRefPanel(ns, on ) );
			} else if (fob instanceof neuroml.core.Enum ) {
			    addSubPanel( fieldPan, new EditEnumPanel(ns, on ) );
			} else if (fob != null) {

			    EditObjectPanel subp = new EditObjectPanel(ns, on, flds[i].getType().getName(),subinline);	
			    addSubPanel( fieldPan, subp );
			    subp.addElementChangedListener( this ); // Listen to changes
			}		
		    }
		} 
	    } catch ( Exception e) {
		System.out.println(" EditObjectPanel : "+e);
	    }
	} 
	if (flds.length==0) { // no fields 
            System.out.println(" EditObjectPanel : flds.length==0 "+flds.length);
	    LabelPanel subp = new LabelPanel("None ");	
	    addSubPanel( fieldPan, subp );
	}
	//if (dialog != null)
	//   dialog.pack();
    }      
    // Add a sub panel
    void addSubPanel( JPanel p, EditPanel ep ) {
	subpanels.addElement( ep );

	p.add(ep);

        if (ep.getNode()!=null)
	    subobjects.addElement( ep.getNode().getContent() );

    }
  
    /** Listen to text field changes
     */
    public void elementChanged(Object oldn, Object newn) {
	if (oldn == value) { // element has changed
	    value = newn;
	} else {
	    // Check subobjects...
	    if (subobjects.indexOf(oldn)!=-1) {
		// a sub object has changed...
		System.out.println("subobject "+newn+" has changed");
	    }
	    if (newn instanceof String) {
		//	System.out.println("Element changed from "+
		// node.getParentPath()+"."+oldn+" to "+node.getParentPath()+"."+newn);
		ns.rename( node.getParentPath()+"."+oldn, node.getParentPath()+"."+newn);
		node.changeFieldName((String)newn);
	    }
	}
	node.setContent(value); 
	
	/** tell parent that things have changed... */
	if (notifyParent!=null) {	    
            //JAG
	    notifyParent.elementChanged(value,newn);// notifyParent.elementChanged( value,value );
	}
	if (dialog!=null) { 
	    dialog.validate();
	    //dialog.pack();
	    //	    System.out.println("Repacking dialog now");
	}
    }

    /** Listen to text field changes
     */
    public void confirmElementChanged(Object oldn, Object newn) {
	// confirmation of changes, could be nice to implement it
    }

    // Should we be duplicating the passed object, just in case?
    public void confirm() {
	for (int i=0; i<subpanels.size(); i++) {
	    ((EditPanel)(subpanels.elementAt(i))).confirm();
	}
	node.setContent(value);

	/** tell parent that things have definitively changed... */
	if (notifyParent!=null) {	    
	    notifyParent.confirmElementChanged( value,value );
	}

	/** Update the references ... 
	 * This should be done on the fly by EditSetPanel!!!
	 */
	if (node.getPath().equals(""))
	    ns.addReferences( "","", value );
	else
	    ns.addReferences( node.getPath()+".", node.getPath()+".", value );

    }

    // Update display from values...
    public void updateDisplay() {
	updateDisplay(null);
    }
    public void updateDisplay(Object newob) {
	if (newob !=null) value = newob;
	Class c = value.getClass();
	Field[] flds = c.getFields();
	int j=0;
	for (int i=0; i<flds.length; i++) {
	    int mod  = flds[i].getModifiers();
	    if (Modifier.isStatic(mod)) {
	    } else if (Modifier.isPublic(mod)) {	
		try {
		    Object fob = flds[i].get(value);
		    if (fob !=null) {
			((EditPanel)(subpanels.elementAt(j))).updateDisplay(fob);		
			j++;
			if (debug) System.out.println("updating Object "+flds[i].getName());
		    }
		} catch (Exception e) {
		    System.out.println(" EditObjectPanel : "+e);
		}
	    }
	}

    }
    
    private void save()
    {
        String fname = fileChooser();
	    if (fname!=null) {
                System.out.println("Saving "+fname);
                NeuromlSys.toFile( value, fname );//filename
                System.out.println("Writen model to "+fname);//filename
            }
    }
    
     private String fileChooser() {
	String cwd;
	try {
	    cwd = System.getProperty("user.dir");
	} catch (java.security.AccessControlException ae){
	    cwd = "./";
	}
	JFileChooser chooser = new JFileChooser(cwd);  
	ExampleFileFilter filter = new ExampleFileFilter( "xml", "NeuroML model files");
	chooser.addChoosableFileFilter(filter);	
	int returnVal = chooser.showOpenDialog(this); 
	if (returnVal == JFileChooser.APPROVE_OPTION) { 
	    return chooser.getSelectedFile().getPath();
	} 
	return null;
    }
     
     private void SwitchObjects(Object oldObj,Object newObj) 
     {
            Field[] flds_old=oldObj.getClass().getFields();
            Field[] flds_new=newObj.getClass().getFields();
            Object fob;
            Object CurrenField;
            for (int i=0; i<flds_new.length; i++) {
                try {
                    fob  = flds_new[i].get(oldObj);
                    int mod  = flds_new[i].getModifiers();
                    if (Modifier.isStatic(mod)) {
                    } else  {		//if (Modifier.isPublic(mod) ||Modifier.isProtected(mod) )
                        if (fob!=null) {
                            flds_new[i].set(newObj,fob);                            
                        }//fobnull
                    }//if Modifier
                }
                catch ( Exception e) {
                    System.out.println(" EditObjectPanel : "+e);
                }
            } //for	    
     }
     
     protected JComboBox addComboBox(String List[],int index,ActionListener l)
     {
         

         //This is because the super class - interface is protected:(need to be changed if becomes public)
       
        String[] possibilities;
	
	if (List!=null) {
	    possibilities = new String[List.length+1];
	    possibilities[0] = "<not set>";
	    	    
	    for (int i=0;i<List.length;i++) {
		String elem = List[i];
		possibilities[i+1] = new String(elem);
            }   
	} else {
	    possibilities = new String[1];
	    possibilities[0] = "<no objects of this type to reference>";
	}
	
	// Make a combo box
        JComboBox comboBox = new JComboBox(possibilities);
        comboBox.setSelectedIndex(index);
        comboBox.addActionListener(l);
        return comboBox;
     }    
     
     public void updateSize(JPanel topPanel)
     {
         Component stuff=topPanel;
         boolean found=false;
         while(!found&&(stuff=stuff.getParent())!=null)
         {
             if(stuff instanceof JFrame)
             {
                    found=true;                    
             }
         }
         if(stuff==null) return;
         /*
          * NOT THE DIALOG FROM THIS EDITOBJECTPANELTHE ANCESTOR
         */
         JFrame dialog=(JFrame)stuff;
         //Component[] content=ParentFrame.getContentPane().getComponents();
         
        //int v=ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED;
        //int h=ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED;     
        //JScrollPane jsp = new JScrollPane(topPanel,v,h);
        //dialog.getContentPane().removeAll();
        //dialog.getContentPane().add(jsp);
        //for(int i=1;i<content.length;i++)
         //   dialog.getContentPane().add(content[i]);
            
        
	dialog.pack();
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

        if(dialog.getHeight()>screenSize.height)
            dialog.setSize(dialog.getWidth()+ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED, screenSize.height-40);
	if (debug) System.out.println("done updating dialog");
     }
}

