package guikit;

import java.lang.reflect.*;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.NumberFormat;

import neuroml.core.*;
import neuroml.util.*;
import neuroml.sim.*;
 
/** Set editor
 */
public class EditSetPanel extends EditPanel implements ActionListener, ElementChangedListener {

    /** The set / list
     */
    protected Object theparent; 


    /** If it's a set...
     */
    neuroml.core.Set theset;

    /** The actual elements of the set/list
     */
    protected Vector elements; 

    /** Base class for new objects
     */
    protected String baseClass; 

    /** Namespace
     */
    protected Namespace ns;

    /** The JList
     */
    JList jlist;

    /** The list of strings of element names
     */
    DefaultListModel dlm;

    /** Empty constructor
     */
    public EditSetPanel(ObjectNode node) { 	
	super(node);
    }

    /** Build the set panel
     */
    public EditSetPanel(Namespace ns, ObjectNode node) {
	super(node);
	//System.out.println("EditSet panel path="+node.getPath());
	this.ns = ns;    
	theset = (neuroml.core.Set)node.getContent();
	baseClass = theset.getType();
	elements = theset.getElements();
	makePanel(elements);
	theparent = theset; 
    }

    /** Return a name for an element
     */
    String getElementName(Object el, int i) {
	String name;
	if (el instanceof Nameable) {
	    name = ((Nameable)el).getName();
	} else {
	    name = el.toString();
	}
	if (name==null||name.equals("")) { name=el.getClass().getName() + "_" +i; }  
	return name;
    }

    /** Make a panel
     */
    protected String NameBorder(){
        return node.getName();
    }
    
    void makePanel(Vector elements) {       
	// Make a border...
	setBorder( BorderFactory.createTitledBorder(NameBorder()) );
	setLayout( new BoxLayout(this, BoxLayout.Y_AXIS) );

	// Add contents to the list... just name for now.
	// Could extend it to add value for simple elements.
	dlm = new DefaultListModel();
	jlist = new JList(dlm);

	String[] names = new String[elements.size()];
	for (int i=0; i<elements.size(); i++) {
	    Object ob = elements.elementAt(i);
	    String name = getElementName(ob,i);
	    dlm.addElement(name);
	}

	// Add the scroll pane...
	JScrollPane scroll = new JScrollPane(jlist);
        scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        SetDefaultSize(scroll);        
//	scroll.setMinimumSize( new Dimension(50,30) );
//	scroll.setPreferredSize( new Dimension(50,70) );
        add(scroll);  

	// Add the buttons...
	JPanel buts = new JPanel();
	buts.setLayout( new BoxLayout(buts, BoxLayout.X_AXIS) );
	if(!(getNode().getContent() instanceof neuroml.core.PermanentSet)) buts.add( makeButton("Add", "add", this));
	buts.add( makeButton("Edit", "edit", this));
	if(!(getNode().getContent() instanceof neuroml.core.PermanentSet)) buts.add( makeButton("Remove", "remove", this));
        buts.add( makeButton("Up", "up", this));
        buts.add( makeButton("Down", "down", this));
	add(buts);
    }
   
    /** Edit an object
     */
    public void popupEditor(Namespace ns, ObjectNode ne, String base) {
	Editor specialEditor = SpecialEditors.get( ne.getContent().getClass().getName() );

	if (specialEditor==null) {
	    EditObjectPanel eop = new EditObjectPanel( ns, ne, 
						       baseClass,
						       false,true );
	    eop.popup();
	} else {
	    specialEditor.editObject( ne.getContent(), ne.getPath(), this );
	}
    }
     protected void popupEditor(int i,Object o) {
        
     }
			    
    
    
    /** Listen to buttons
     */
    public void actionPerformed(java.awt.event.ActionEvent e) {
        if (e.getActionCommand().equals("add")) {
	    Object o = NeuromlSys.makeNeuromlObject(baseClass);
	    String newname = baseClass+"_"+dlm.size();
	    dlm.addElement(newname);
	    elements.addElement(o);
	    if (o instanceof Nameable) { ((Nameable)o).setName( newname ); }
	    if (o==null) {
		System.out.println("Couldn't make instance of "+baseClass);
	    } else {
		ObjectNode n = new ObjectNode(theparent, o, this);
		n.setPath( node.getPath(),newname ); 
		//System.out.println("Edit Set Add to ns "+ns);
		ns.addRef(n.getPath(), o); 
		popupEditor( ns,n,baseClass);
	    }
        } else if (e.getActionCommand().equals("remove")) {
	    int i = jlist.getSelectedIndex();
	    if (i!=-1) {
                //JAG: need to remove the reference too...
                Object o = elements.elementAt(i);
                if (o instanceof Nameable) 
                {
                    
                    String newname=((Nameable)o).getName();
                    ObjectNode n = new ObjectNode(theparent, o, this);
                    n.setPath( node.getPath(),newname ); 
                    ns.removeRef(n.getPath(),o);
                    
                }
		dlm.removeElementAt(i);
		elements.removeElementAt(i);
                popupEditor(i,o);
	    }
        } else if (e.getActionCommand().equals("edit")) {          
	    int i = jlist.getSelectedIndex();
	    if (i!=-1) {
		Object o = elements.elementAt(i);
		ObjectNode ne = new ObjectNode(theparent, o, this);
		if (o instanceof Nameable) { 
		    ne.setPath( node.getPath(),((Nameable)o).getName() );
		} else {
		    ne.setPath( node.getPath(),baseClass+"_"+i );
		}
		popupEditor( ns,ne,baseClass);
	    }
        } else if (e.getActionCommand().equals("up")) {          
	    int i = jlist.getSelectedIndex();
	    if (i!=-1&&i!=0) {
		Object o = elements.elementAt(i);
		 elements.remove(i);
                 elements.add(i-1,o);
                 o=dlm.getElementAt(i);
                 dlm.remove(i);
                 dlm.add(i-1,o);
	    }
        } else if (e.getActionCommand().equals("down")) {          
	    int i = jlist.getSelectedIndex();
	    if (i!=-1&&i!=dlm.size()-1) {
		Object o = elements.elementAt(i);
		elements.remove(i);
                elements.add(i+1,o);
                o=dlm.getElementAt(i);
                dlm.remove(i);
                dlm.add(i+1,o);
	    }          
        } else { 
            System.out.println("Unknown command");
        }
    }

    /** Update real object 
     */
    public void confirm() {
	theset.setElements( elements ); 
	node.setContent(theset);
    }  

    /** An element has changed
     * - update parent list 
     */
    public void elementChanged( Object oldobject, Object newobject ) {
	int i = elements.indexOf(oldobject);
   //     if (i==-1)
   //         i = elements.indexOf(newobject);//already changed somehow

	if (i==-1) {
            System.out.println("Error - couldn't find "+oldobject+" in list"); 
        }
	else {
	    // update dlm -- lookup by oldobject index
	    elements.setElementAt(newobject,i);
	    dlm.setElementAt(getElementName(newobject,i),i);
	}
    }
    /** Listen to text field changes
     */
    public void confirmElementChanged(Object oldn, Object newn) {
	// confirmation of changes, could be nice to implement it
    }
    
    protected void SetDefaultSize(JComponent pane){
        pane.setMinimumSize( new Dimension(50,30) );
	pane.setPreferredSize( new Dimension(50,70) );
    }
}
