/*
 * EditShortPanel.java
 *
 * Created on 03 January 2005, 18:15
 */

package guikit;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.NumberFormat;
/**
 *
 * @author  Jean-Alain Grunchec
 */
public class EditShortPanel extends EditPanel implements DocumentListener {
    short value;       // Value
    JSlider slider;
    int smin, smax;
    JLabel paramLabel;
    DecimalField paramValueField;
    Font font;
    
    
    /** Build the panel
     */
    public EditShortPanel(ObjectNode node) {
	super(node);
        
	font =  GuiKit.getDefaultFont();
	smin = 0; 
	Short val = (Short)node.getContent();
	value = val.shortValue();
	rescale(value);
	slider = makeSlider(node.getName(),value,smin,smax);
    }
    
    boolean busy = false; // flag for mutex

    double log10(double d) {
	return Math.log(d)/Math.log(10);
    }

    void rescale( double v ) {
	DisplayHints h = node.getDisplayHints();
	if (h!=null) {	    
	    smin = (short)h.getMin();
	    smax = (short)h.getMax();	
	    if ((v > smax) && h.isRangeFixed()) {
		System.out.println("Error - range out of bounds "+v);
	    }
	}	
	if (v > smax) {
	    if (v < 1) v = 10; // avoid log(0)
	    double lg = log10(v);	
	    smax = (short)Math.pow(10,Math.floor(lg-0.01)+1);	
	}
    }
    public short valToSlider( double v ) {
	short sv = (short)(100.0 * (v-smin)/smax);
	if (sv < 0) sv = 0;
	if (sv > 100) sv = 100; 
	return sv;
    }
    public int sliderToVal( int v ) {
	return (int)(smin + v * (smax-smin) / 100.0);
    }
    
    /** Sets min/man labels on display */
    void setMinMaxDisplay(double min, double max) {
	if (slider!=null) {
	    Hashtable labelTable = new Hashtable();
	    JLabel l1 = new JLabel(""+min); l1.setFont(font);
	    JLabel l2 = new JLabel(""+max); l2.setFont(font);
	    labelTable.put( new Integer( 0 ), l1 );
	    labelTable.put( new Integer( 100 ), l2);
	    slider.setLabelTable( labelTable );	
	}
    }

    /**
     * Display a slider for int number parameter
     *
     */
    public JSlider makeSlider(String title, short value, int min, int max) {
	paramLabel = new JLabel(title);
	paramLabel.setPreferredSize( new Dimension(80, 32));
	paramLabel.setMinimumSize( new Dimension(10, 32));// For resizing
	paramLabel.setFont( font ); // test
	paramValueField = new DecimalField((double)value,10); // For typing in value
	paramValueField.getDocument().addDocumentListener( this );

        paramValueField.setPreferredSize( new Dimension(40, 32));
	paramValueField.setMinimumSize( new Dimension(10, 32));// For resizing
        
        slider = new JSlider(0,100,valToSlider(value));
        
	slider.setFont( font ); 
	//	slider.setMajorTickSpacing(10);
	// 	slider.setPaintTicks(true);
        slider.setPaintLabels(true);
        slider.setPreferredSize(new Dimension(100, 32));
        slider.setMinimumSize( new Dimension(10, 32));
//        System.out.println("IN short stuff...........................................1");
	setMinMaxDisplay(min,max);
//        System.out.println("IN short stuff...........................................2");
	slider.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
		if (!busy) {
		    paramValueField.setValue(sliderToVal(slider.getValue()));
		    confirm();
		}
            }
	    });
//        System.out.println("IN short stuff...........................................3");
	// slider.setPreferredSize( new Dimension(-1, 16));
	slider.setBorder(BorderFactory.createEmptyBorder(0,0,0,0));
	GridBagLayout gb = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	setLayout(gb);
	c.fill = GridBagConstraints.HORIZONTAL;
	c.gridx = 0;
	c.weightx = 5.0;
	//	c.gridwidth = 10;
	add(paramLabel);      gb.setConstraints(paramLabel,c);
	c.weightx = 10.0;
	c.gridx = 1;
	add(slider);          gb.setConstraints(slider,c);
	//	c.gridwidth = GridBagConstraints.REMAINDER;
	c.weightx = 5.0;
	c.gridx = 2;
 	add(paramValueField); gb.setConstraints(paramValueField,c);
	return slider;
    }      

    // Listen to text field changes
    public void insertUpdate(DocumentEvent e) {
	updateDisplayValue(e);
    }

    public void removeUpdate(DocumentEvent e) {
	updateDisplayValue(e);
    }

    public void changedUpdate(DocumentEvent e) {
    }

    public void updateDisplayValue(DocumentEvent e) {
	busy = true;
	value = (short)paramValueField.getValue();
	if (value >= smax) {
	    rescale(value);
	    setMinMaxDisplay(smin,smax);
	}
	confirm();
	slider.setValue(valToSlider(value));	
	busy = false;
    }

    /** Update display from values...
     */
    public void updateDisplay(Object newob) {
	try {
	    Short val = (Short)newob;
	    value = val.shortValue();
	    paramValueField.setValue( value );
	    updateDisplayValue(null);
	} catch (Exception e ) {
	}
    }

    /** Update real object 
     */
    public void confirm() {
	node.setContent(new Short( value ) );
    }  
    
    
}

