/*
 * EditTreeRefPanel.java
 *
 * Created on 13 January 2004, 17:03
 */

package guikit;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.NumberFormat;

import neuroml.core.*;
import neuroml.util.*;
import neuroml.sim.*;
import neuroml.model.network.*;
import neuroml.model.cell.*;
import neuroml.util.module.*;

/*
 * @author  Jean-Alain Grunchec
 * JAG for networks
 */

public class EditTreeRefPanel extends EditPanel implements TreeSelectionListener  {//ActionListener,
    TreeRef reference;
    JTree TreeNet;
    JTextField jtf;
    JScrollPane jsp; 
    Namespace ns;
    
    
    Attachment myAttachment; //this specialize a bit the EditTreeRefPanel...
    
    JLabel paramLabel;
    static Hashtable editors = new Hashtable();    
    ElementChangedListener notifyParent = null;
    
 /*
 * Say where the changes (selection) are handled. Likely to be an EditObjectPanel for the parent.
 */   
    
    public void addElementChangedListener( ElementChangedListener p ) {
	notifyParent = p;
    }
 
/*
 * put in set editors the name of a class of object and the GUI editor associated for the treeRef.
 * The editor class should inherit from TreeNodeEditor
 */
      
    public static void put(String classname, String eclassname) {
	editors.put(classname, eclassname);
    }
    
    public static TreeNodeEditor get(String classname, Object Element,String Name) {
	String e = (String)editors.get( classname );
	TreeNodeEditor ret = null;
	if (e!=null) {
	    try { 
		Class c = Class.forName( e);
		Object o = c.newInstance();
		if (o instanceof TreeNodeEditor) 
                    ret = (TreeNodeEditor) o;
                ret.setParameter(Element,Name);
	    } catch (Exception ex) {
                System.out.println("Error EditTreeRefPanel No TreeNodeEditor jag");
            }
	}
	return ret;
    }   
    
    /** Creates a new instance of EditTreeRefPanel */
    
    public EditTreeRefPanel(Namespace ns, ObjectNode node) {
	super(node);
	this.ns = ns;
        boolean found=false;
        TreeNodeEditor.setNameSpace(ns);
        Enumeration enum_;
        DefaultMutableTreeNode NodeToSelect=null;
        Hashtable TableNodes=new Hashtable();
        Hashtable TableLeaf=new Hashtable();
        
        int i,lsize,nsize,j,l;
        
        reference = (TreeRef)node.getContent();
        myAttachment = ns.lookAttachment(reference);
        
	String tmp,tname = reference.getTargetName();
//        System.out.println("Target "+tname);
	Vector leaves  = reference.getLeaves();
        Vector nodes = reference.getNodes();
        Vector Elements=reference.getElements();
        
        GridBagLayout gb = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
	setLayout(gb);
 	      
        paramLabel = new JLabel(tname);
	paramLabel.setPreferredSize( new Dimension(100, 32));
	paramLabel.setMinimumSize( new Dimension(10, 32));// For resizing   
        add(paramLabel);
        c.fill = GridBagConstraints.HORIZONTAL;
	c.gridx = 0;
	c.weightx = 5.0;
        gb.setConstraints(paramLabel,c);  

        Vector StackName=new Vector(1,1);
        j=0;
        tmp=new String(tname);
        i=tmp.length();
        while(i>-1)
        {
            StackName.add(j++, tmp=tmp.substring(0,i));
            i=tmp.lastIndexOf(".");
        }
        
        lsize=leaves.size();
        nsize=nodes.size();
        
        DefaultMutableTreeNode top = new DefaultMutableTreeNode(tname);
        TreeNet=new JTree(top);
        
        for(i=0;i<lsize;i++)
        {
            Hashtable temp = (Hashtable)ns.getReferences().get((String)leaves.elementAt(i));
            if (temp!=null) 
            {
                enum_=temp.keys();
                while(enum_.hasMoreElements())
                {
                    String k = (String)enum_.nextElement();
		    Object element = temp.get(k);
                    if(Elements.contains(element))
                    {
                        DefaultMutableTreeNode a = new DefaultMutableTreeNode(k); 
                        top.add(a);
                        if(!found&&tname.compareTo(k)==0)
                        {
                            found=true;
                            NodeToSelect=a;
                        }                      
                    }
                }
            }
        }
        
        for(i=0;i<nsize;i++)
        {
            String OfClass=(String)nodes.elementAt(i);
            Hashtable temp = (Hashtable)ns.getReferences().get(OfClass);
            if (temp!=null) {
                enum_=temp.keys();
                while(enum_.hasMoreElements())
                {
                    String k = (String)enum_.nextElement();
		    Object element = temp.get(k);//useless for now
                    if(Elements.contains(element))
                    {                 
                        TreeNodeEditor a = get(OfClass,element,k);
                        top.add(a);
                        if(!found&&StackName.contains(k))
                        {
                            TreeNode[] arrayNode=a.getPath();
                            TreePath thatTreePath=new TreePath(arrayNode);
                            try{
                                //thatTreePath[0];
                                 //TreeNet.scrollPathToVisible(thatTreePath);
                                 TreeNet.expandPath(thatTreePath);
                            }
                            catch(Exception e){
                                System.out.println("Error EditTreeRefPanel jag: " + thatTreePath);
                            }
                            if(k.compareTo(tname)==0)
                                NodeToSelect=a;
                            else
                                NodeToSelect=a.SelectNode(StackName,TreeNet);
                        }
                    }
                }
            }
        }

        int v=ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED;
        int h=ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED;
        
        TreeNet.addTreeSelectionListener(this);
        
        jsp = new JScrollPane(TreeNet,v,h);
        jsp.setPreferredSize( new Dimension(450, 100));
	jsp.setMinimumSize( new Dimension(300, 40));// For resizing
        add(jsp);      
        //c.fill = GridBagConstraints.HORIZONTAL;
	c.gridx = 0;
	c.weightx = 20.0;
        gb.setConstraints(jsp,c);  
        if(NodeToSelect!=null)
            TreeNet.setSelectionPath(new TreePath(NodeToSelect.getPath()));
    }
    
    //public void actionPerformed(ActionEvent e) {
    //}
    
    public void valueChanged(TreeSelectionEvent e) {
        //TreeRef ThatNode=(TreeRef)node.getContent();
        
        reference.setTargetName(TreeNet.getLastSelectedPathComponent().toString());
        if(myAttachment!=null){
            Object myPop=ns.getPopRef(reference);
            if(myPop!=null){
                Object obj=NeuromlSys.getNewestNamespace().deRef(((Population)myPop).elementClass);
                if (obj instanceof ModuleNeuron) {
                    try {
                        ModuleNeuron mn = (ModuleNeuron) obj;
                        ModuleRef mr = mn.module; 
                        PermanentSet truc=mn.getMeasurementList().Clone();
                        if(truc!=null&&myAttachment.MeasurementList!=null&&truc.size()==myAttachment.MeasurementList.size())
                        {
                            boolean On=true;
                            for(int i=0;i<truc.size()&&On;i++)
                                if(truc.elementAt(i).getClass()!=myAttachment.MeasurementList.elementAt(i).getClass())
                                    On=false;
                            if(!On)
                                myAttachment.MeasurementList=truc;
                        }
                        else
                            myAttachment.MeasurementList=truc;
                    }
                    catch(Exception except){
                    }
                }
            }                
        }
        paramLabel.setText(TreeNet.getLastSelectedPathComponent().toString());
        if(notifyParent!=null) notifyParent.elementChanged(reference,reference);
        //repaint();      
    }
    
}
