package guikit;

import java.lang.reflect.*;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.NumberFormat;
import java.net.*;

import neuroml.sim.ElementChangedListener;

/** String field editor
 */
public class EditURLPanel extends EditPanel implements DocumentListener, ActionListener  {

    URL value;
    JLabel paramLabel;
    JTextField paramValueField;
    ElementChangedListener ecl = null;
    JButton browseButton;

    /** Build the panel
     */
    public EditURLPanel(ObjectNode node) {
	super(node);
	value = (URL)node.getContent();
	paramLabel = new JLabel(node.getName());
	paramLabel.setPreferredSize( new Dimension(100, 32));
	paramLabel.setMinimumSize( new Dimension(10, 16));// For resizing
	//paramLabel.setMaximumSize( new Dimension(10000, 32));// For resizing

	paramLabel.setFont(GuiKit.getDefaultFont());
	paramValueField = new JTextField(value.toString());
	paramValueField.setPreferredSize( new Dimension(150, 32));
	paramValueField.setMinimumSize( new Dimension(10, 16));// For resizing
	//paramValueField.setMaximumSize( new Dimension(10000, 32));// For resizing

	paramValueField.getDocument().addDocumentListener( this ); 
	paramValueField.setFont(GuiKit.getDefaultFont());

	browseButton = makeButton("Browse","Browse",this);


	GridBagLayout gb = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	setLayout(gb);
	//c.fill = GridBagConstraints.BOTH;
	c.fill = GridBagConstraints.HORIZONTAL;

	c.gridwidth = 10;
	c.weightx = 10.0;
	c.weighty = 5.0;
	//add(paramLabel);      gb.setConstraints(paramLabel,c); 



	c.weightx = 20.0;
	c.gridwidth = GridBagConstraints.REMAINDER;

	JPanel parameterPane = new JPanel();
	//parameterPane.setBorder(BorderFactory.createEmptyBorder(0,0,0,5));
	parameterPane.setLayout(new BoxLayout(parameterPane, 
						      BoxLayout.X_AXIS));
 	parameterPane.add(paramLabel); 
 	parameterPane.add(paramValueField); 
	parameterPane.add(browseButton);
	add(parameterPane);

	//gb.setConstraints(paramValueField,c);
	gb.setConstraints(parameterPane,c);
    }      

    public void insertUpdate(DocumentEvent e) {
	updateDisplayValue(e);
    }
    public void removeUpdate(DocumentEvent e) {
	updateDisplayValue(e);
    }
    public void changedUpdate(DocumentEvent e) {
    }
    public void setElementChangedListener( ElementChangedListener ecl ) { this.ecl = ecl; }
    public void updateDisplayValue(DocumentEvent e) {
	URL oldvalue=value;
	try {
	    value = new URL(paramValueField.getText());
	} catch (MalformedURLException eurl){
	    System.out.println("Malformed URL "+eurl);
	}
	confirm();
	if (ecl!=null) {
	    ecl.elementChanged( oldvalue,value );
	}
    }
    public JTextField getTextField() { return paramValueField; }
    public void confirm() {
	try {
	    if (paramValueField.getText().startsWith("file:"))
		node.setContent(new URL(paramValueField.getText()));
	    else
		node.setContent(new URL("file:"+paramValueField.getText()));

	} catch (MalformedURLException eurl){
	    System.out.println("Malformed URL "+eurl);
	}
    }

    public void actionPerformed(ActionEvent e) {
	if (e.getActionCommand().equals("Browse")) {
	    String fname = fileWithPathChooser("Choose NeuroML model to run");
	    try {
		if (fname!=null) {
		    value = new URL("file:"+fname);
		} else value = new URL("file:none");
		paramValueField.setText(value.toString());
	    } catch (MalformedURLException eurl){
		System.out.println("Malformed URL "+eurl);
	    }
	} else 	System.out.println("Unknown command");
    }

    /** Popup a file chooser 
     */
    public String fileWithPathChooser(String title) {
	String cwd = System.getProperty("user.dir");

	JFileChooser chooser = new JFileChooser(cwd);  
	ExampleFileFilter filter = new ExampleFileFilter( "xml", "NeuroML model files");
	chooser.addChoosableFileFilter(filter);	
	chooser.setDialogTitle(title);
	int returnVal = chooser.showOpenDialog(this); 
	if (returnVal == JFileChooser.APPROVE_OPTION) { 
	    String res = new String(chooser.getSelectedFile().getParent()+"/"+chooser.getSelectedFile().getName());
	   
	    return res;
	} 
	return null;
    }
}
