package guikit;
 
import neuroml.sim.Editor;
import java.util.Hashtable;
import java.util.Vector;
import java.lang.reflect.*;
import neuroml.*;
import neuroml.core.*;
import neuroml.util.*;

class PkgField {
    public String pkgName = "";
    public String fieldName = "";
    public String fieldType = "";
    PkgField(String p,String f,String t) { pkgName = p; fieldName = f; fieldType=t;}
}

/** Environment for sub editors
 */
public class Environment {
    Namespace ns;
    NeuromlPkg topObject;
    ImageUtils img; // Really needed here?

    public Environment( Namespace ns, NeuromlPkg top, ImageUtils img ) {
	this.ns = ns;
	this.topObject = top;
	this.img = img;		
	buildAppropriatePackages();
    }
    public Namespace getNamespace() { return ns; }
    public void setNamespace(Namespace ns) { this.ns=ns; }
    public NeuromlPkg getTopObject() { return topObject; }
    public void setTopObject(NeuromlPkg t) { topObject = t; }
    public ImageUtils getImageLoader() { return img; }
    public void setImageLoader(ImageUtils iu) { img = iu; }


    /** Here we need to maintain some information about which editors
     * are currently open... so that when we add a new object, it can
     * appear in all the right places.
     * For example, the MenuPanels of network editors need to get notification
     * of new cells / networks.
     */


    /** Hashtable { type } of most appropriate Pkg
     */
    Hashtable apprPkg = new Hashtable();

    /** This should get called once at the start....
     * e.g. when building list info
     * It contains a list of the appropriate Pkg to build 
     * for a given type.
     */
    void buildAppropriatePackages() {
	
	// Should really do this automatically for "new CellPkg", &c.
	// DerivedFrom...
	// An entry per list...
	apprPkg.put("neuroml.core.Model", 
		    new PkgField("NeuromlPkg","models", "neuroml.core.Model"));
	apprPkg.put("neuroml.core.Module", 
		    new PkgField("NeuromlPkg", "modules","neuroml.core.Module"));
	apprPkg.put("neuroml.model.cell.AbstractNeuron", 
		    new PkgField("CellPkg", "neurons", "neuroml.model.cell.AbstractNeuron"));


	apprPkg.put("neuroml.model.network.Population",
		    new PkgField( "Network", "populations","neuroml.model.network.Population"));  
 
	// Get a list of Pkgs
	// Get a list of their Sets
	// If sth already there, choose more specific
    }
    
    /** Get an appropriate package for a given type (or its superclass) 
     */
    PkgField getAppropriatePkg( String type ) {
	Class c = NeuromlSys.getClassFromShortName( type );
	PkgField ret = null;
	if (c!=null) {
	    do {
		ret = (PkgField)apprPkg.get( c.getName() );
		if (ret==null) {
		    // Try the superclass
		    c = c.getSuperclass();
		    if (c.getName().equals("java.lang.Object")) { 
			c=null; // Give up... could use Robert's "lost+found" list...
		    }
		}
	    } while ((ret==null) && (c!=null));	    
	}
	return ret;
    }


    /** Get the best set to put a new object of type X in given
     * that it's referred to from path P.  If no sets found, 
     * make an appropriate package and put it at the top level.
     * Returns reference to new object
     * Nearly coded properly...
     */
    public String addNewObject( String type, String pathOfReferrer, String baseName ) {
	Object newo = NeuromlSys.makeNeuromlObject( type );
	return addNewObject( type, pathOfReferrer, newo, baseName );
    }


    /** Get the best set to put an existing object of type X in given
     * that it's referred to from path P.  If no sets found, 
     * make an appropriate package and put it at the top level.
     * Returns reference to new object
     * Nearly coded properly...
     */
    public String addNewObject( String type, String pathOfReferrer, Object newo, String baseName ) {
	System.out.println("Add new object "+type+" from "+pathOfReferrer+" not yet debugged");
	Vector lists = ns.getListInfo( type ); 
	ListInfo li=null;
	Set newlists = null;
	String newname = baseName;

	// When a new package gets added, its Sets should also get added...
	String ref=null;
	if (lists==null) {
	    PkgField p = getAppropriatePkg(type); 
	    if (p!=null) {
		Object newp = NeuromlSys.makeNeuromlObject( p.pkgName );
		Pkg newpkg = (Pkg)newp;
		newpkg.setName( p.pkgName+topObject.models.getElements().size() );
		li = new ListInfo( newp, "models", p.fieldName );
		ns.addListInfo( type, li );
		topObject.models.addElement( newp ); 
		try {
		    Field newlistf = newp.getClass().getField(p.fieldName); // This is the Set.
		    newlists = (Set) newlistf.get(newp);		    
		} catch (Exception e) {
		    System.out.println("Env.addNewObject exception "+e);
		}
	    }
	} else {
	    // Find the best of the available lists...
	    int found = 0;
	    int bestmatch = 0;
	    for (int i=0; i<lists.size(); i++) {
		li = (ListInfo) lists.elementAt(i);
		// Path of referrer ignored for now...
		//    if (pathOfReferrer.startsWith(li.getPath())) {
		if (li.getPath().length() > bestmatch) {
		    bestmatch = li.getPath().length();
		    found = i;
		}
	    }
	    li = (ListInfo)lists.elementAt(found);	 
	    Object thePkg = li.getOwner();
	    try {
		Field newlistf = thePkg.getClass().getField(li.getFieldName()); 
		newlists = (Set) newlistf.get(thePkg);		    
	    } catch (Exception e) {
		System.out.println("Env.addNewObject exception "+e);
	    }
	}
	
	// Get a sensible name
	if (newname.length()==0) newname = type;
	newname = newname+newlists.getElements().size();
	if (newo instanceof Nameable) {
	    ((Nameable)newo).setName( newname );
	}
	newlists.addElement( newo ); // Put it in the new list... 
	ref = li.getPath()+"."+li.getFieldName()+"."+newname;
	getNamespace().addRef(ref, newo );     
	return ref;
    }
}
