package guikit;

import java.applet.Applet;
import java.awt.BorderLayout;
import java.awt.event.*;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;
import java.io.*;
import java.net.*;
import java.applet.Applet;
import java.awt.event.*;
import java.awt.image.*;

import java.util.Vector;
import java.util.StringTokenizer;
import java.util.Enumeration;

import javax.swing.*;

import neuroml.util.*;
import neuroml.core.*;
import neuroml.model.cell.*;
import neuroml.model.network.*;

 
/** Top level class for user panels
 * Performs scaling / zooming 
 * (add  MouseWheelListener when 1.4 released)
 */

public class GraphPanel extends JPanel implements MouseListener, MouseMotionListener{
    protected Font font;
    protected BufferedImage bimg = null;
    protected Graphics2D g2 = null;
    protected AffineTransform t = null;
    protected AffineTransform inv = null;

    protected ScrolledPanel sp=null; // set if there's a scrolled panel

    // Scaling - shape of panel in real coords
    protected double x0 = 0.0; // Top left in real coords
    protected double y0 = 0.0;
    protected double xsz = 400;//500 modified with resolution issue
    protected double ysz = 350;//400
    
    // -- shape of panel in pixels
    protected int px0;   // top left in pixels
    protected int py0;
    protected int pxsz;
    protected int pysz;
    protected double platonicXPixelSize;  // Size for placing images
    protected double platonicYPixelSize;  // Size for placing images

    // initial zoom factor
    protected double zoomFactor = 1.0;
    protected double tx=0.0,ty=0.0;    

    public void setScrolledPanel( ScrolledPanel sp ) { this.sp = sp; }
    public ScrolledPanel getScrolledPanel() { return sp; }


    
    public double getZoomFactor() { 
	return zoomFactor;
    }
    public void setZoomFactor(double f) {
	zoomFactor = f;
	// Doesn't actually scale t...
    }
    //Method called by EditObjectPanel to remove graphical component
    public void RemoveElement(ObjectNode ne)
    {
        rerender();
        repaint();
    }
    // Override this for different shape
    protected void setInitialScale() {
	px0 = 0;   // top left in pixels
	py0 = 0;
	pxsz = 500;
	pysz = 400;
	platonicXPixelSize = xsz / (double)pxsz;  // Size for placing images
	platonicYPixelSize = ysz / (double)pysz;  // Size for placing images    
    }
    // Conversion pixel points <-> real numbers
    int pixX( double x ) {
	Point2D mapped = t.transform( new Point2D.Double(x,0),null);
	return (int) mapped.getX(); 
    }    
    int pixY( double y ) {
	Point2D mapped = t.transform( new Point2D.Double(0,y),null);
	return (int) mapped.getY();
    }
    int distX( double x ) { // convert distances
	return (int) (pixX(x) - pixX(0)); 
    }    
    int distY( double y ) { // convert distances
	return (int) (pixY(y) - pixY(0)); 
    }    
    public double realX( int p ) {
        if(inv==null) return (double)p;
	Point2D mapped = inv.transform( new Point2D.Double(p,0),null); 
	return mapped.getX();
    }
    public double realY( int p ) {
        if(inv==null) return (double)p;
	Point2D mapped = inv.transform( new Point2D.Double(0,p),null);
	return mapped.getY(); 
    }
    // Real size of single pixel
    double oneXPixel() { return realX(1) - realX(0); }
    double oneYPixel() { return realY(1) - realY(0); }

    public GraphPanel() {
	setInitialScale();
	setBackground(Color.blue);
	font = new Font("serif", Font.PLAIN, 14);
	addMouseListener(this);
	addMouseMotionListener(this);    
    }

    // Repaint the backing store next time
    boolean needToRender = true;

    public void rerender() {
	needToRender = true;
    }
    
    // Render dragged items into front display
    public void renderDragged( int w, int h, Graphics2D g2) {
    }

    // Paint into the backing store
    public void render( int w, int h, Graphics2D g2) {
	g2.setColor(Color.black);
	Font f = font;
	String msg="Hello Fred";
	int sw = getFontMetrics(f).stringWidth(msg);
	g2.setFont(f);
	g2.drawString(msg, (int) (w/2-sw/2),h/2);
    }

    // Repaint if necessary
    public void paint(Graphics g) {
	Graphics2D gg = (Graphics2D)g;
	Dimension d = getSize();
	if (bimg == null || bimg.getWidth() != d.width || bimg.getHeight() != d.height) {
	    bimg = (BufferedImage) createImage(d.width, d.height);  
	    needToRender = true;
	}
	if (needToRender) {
	    pxsz = d.width;
	    pysz = d.height; 
	    g2 = bimg.createGraphics();
	    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, 
				RenderingHints.VALUE_ANTIALIAS_ON);
	    g2.setRenderingHint(RenderingHints.KEY_INTERPOLATION, 
				RenderingHints.VALUE_INTERPOLATION_BILINEAR);
	    g2.setRenderingHint(RenderingHints.KEY_RENDERING, 
				RenderingHints.VALUE_RENDER_QUALITY);
	    gg.setRenderingHint(RenderingHints.KEY_RENDERING, 
				RenderingHints.VALUE_RENDER_QUALITY);
	    gg.setRenderingHint(RenderingHints.KEY_INTERPOLATION, 
				RenderingHints.VALUE_INTERPOLATION_BILINEAR);
	    gg.setRenderingHint(RenderingHints.KEY_ANTIALIASING, 
				RenderingHints.VALUE_ANTIALIAS_ON);
	    g2.setBackground(getBackground());
	    g2.clearRect(0, 0, d.width, d.height);
            
	    if (t==null) { 
		t = g2.getTransform();
		// Initial scaling provided using zoomFactor
		t.scale(zoomFactor, zoomFactor);
                t.translate(tx,ty);
		g2.setTransform(t);
		//System.out.println("Scaling by "+zoomFactor);
	    } else {
		g2.setTransform(t);
	    }
	    //	    g2.scale(1,1);
	    // ((Graphics2D)g).setTransform(g2.getTransform());
	    inv = g2.getTransform();
            
            //g2.setColor(colour.
            
	    try { inv = inv.createInverse();  } catch (Exception e) {}
	    render(d.width, d.height, g2);
	    needToRender = false;
	}
	g.drawImage(bimg, 0, 0, null);

	/** Render dragged using std xform */
	AffineTransform orig = gg.getTransform();
	gg.setTransform( t );
	renderDragged(d.width, d.height,gg);
	gg.setTransform( orig );
        try{
            Thread.sleep(10);//sleep 1cs
        }
        catch(Exception e){
            
        }
    }
    /*    public Dimension getPreferredSize() {
	  return new Dimension(400,400);
	  }
    */
    /** Respond to mouse events
     */
    public void mouseDragged(MouseEvent e) {    } 
    public void mouseMoved(MouseEvent e) {  }
    public void mousePressed(MouseEvent e) {    }
    public void mouseReleased(MouseEvent e) {    }    
    public void mouseEntered(MouseEvent e) {    }
    public void mouseExited(MouseEvent e) {    }  
    public void mouseClicked(MouseEvent e) {     }
    //    public void mouseWheelMoved(MouseWheelEvent e) {     }
}
 
