package guikit;

import java.lang.reflect.*;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.NumberFormat;
import java.util.Vector;
import java.net.URL;

/** Super class for panel editors
 */
public class HTMLPanel extends JPanel {
    String url; 
    public int x = 250;
    public int y = 250;
    // navigation part
    Vector uRLs = new Vector();
    int uRLindex = 0;
    // wait frame
    JFrame waitFrame;
    // hyperlink 
    HyperlinkListener hyperlinkListener;
    // eventual Navigator (should implement some interface)
    Navigator nav = null;
    /** Build the panel
     */
    public HTMLPanel() {
	super();
	this.url = new String();

    }
    public HTMLPanel(int x, int y) {
	super();
	this.url = new String();
	this.x = x;
	this.y = y;
    }
    /** Build the panel and open an url
     */
    public HTMLPanel(String url) {
	super();
	this.url = url;
	openURL(url);
	registerURL(url);
    }
    /** Build the panel and open an url
     */
    public HTMLPanel(String url,int x, int y) {
	super();
	this.x = x;
	this.y = y;
	this.url = url;
	openURL(url);
	registerURL(url);
    }
    public void setNavigator(Navigator nav){
	this.nav = nav;
    }
    public Dimension getPreferredSize() {
	return new Dimension(x,y);
    }
    /** open an url
     */
    public void openURL(String url){
 
	try{
	    
	    //createWaitURL();
	    waitURL();
	    
	    JEditorPane webpage = new JEditorPane();
	    webpage.setContentType("text/html");
	    webpage.setPage(url);
	    
	    webpage.setEditable(false);
	    hyperlinkListener = new Hyperactive(webpage);
	    webpage.addHyperlinkListener(hyperlinkListener);
	    JScrollPane htmlView = new JScrollPane(webpage);
	    this.setLayout(new GridLayout(1, 1));
	    this.removeAll();
	    this.add(htmlView);
	    webpage.revalidate();
	    this.revalidate();
	    
	    closeWaitURL();
	} catch (Throwable t) {
	    t.printStackTrace();
	    closeWaitURL();
	}
    }

    /** set the home url
     *
     */
    public void setHome(String url){
	if(uRLs.size()>0)
	    uRLs.remove(0);
	uRLs.add(0,url);
    }

    /** open back the precedent url displayed
     * returns true if back is still possible
     */
    public boolean backURL(){
	
	// open new url
	uRLindex--;
	Thread t = new OpenWindow(this,(String)uRLs.elementAt(uRLindex-1));
	t.start();	
	
	if(uRLindex<2){
	  return false;
	}
	return true;
	
    }

    /** after a back, allows to display the following url again
     * returns true if forward is still possible
     */
    public boolean forwardURL(){
	
	// open new url
	uRLindex++;
	Thread t = new OpenWindow(this,(String)uRLs.elementAt(uRLindex-1));
	t.start();
	
	    
	if(uRLindex==uRLs.size()){
	    return false;
	}
	return true;
    }

    /** open home url
     * when used : check returned boolean  to know if we should enable forward
     * but anyway always disable backward
     */
    public boolean homeURL(){

	// open new url
	uRLindex = 1;
	Thread t = new OpenWindow(this,(String)uRLs.elementAt(uRLindex-1));
	t.start();
		
	if(uRLindex==uRLs.size()){
	    return false;
	}
	return true;
    }


    /** register the url in a list
     * when used : check returned boolean  to know if we should enable Back
     * but anyway always disable Forward
     */
    public void registerURL(String url){
	if(uRLindex!=0)
	    if(url.equals((String)uRLs.elementAt(uRLindex-1)))
		return;
	    
	uRLs.add(uRLindex,url);
	uRLindex++;
	if (nav!=null){
	    if(uRLindex>1){
		nav.enableBack(true);
	    }
	    nav.enableForward(false);
	}
    }

    /** display a wait frame . useful when loading an url takes a long time
     */
    public void waitURL(){
     	if(waitFrame!=null)waitFrame.dispose();
	String waitString = new String("Please wait. Loading...");
	waitFrame = new JFrame(waitString);
	JProgressBar pb = new JProgressBar(0,100);
	JLabel waitLabel = new JLabel(waitString);
	JPanel waitPane = new JPanel() {
		public Dimension getMinimumSize() {
		    return getPreferredSize();
		}
		public Dimension getPreferredSize() {
		    return new Dimension(300, 15);
		}
		public Dimension getMaximumSize() {
		    return getPreferredSize();
		}
	    };	
	waitPane.setBorder(BorderFactory.createEmptyBorder( 0,0,0,5));
	waitPane.setLayout(new BoxLayout(waitPane,BoxLayout.X_AXIS));
	waitPane.add(waitLabel);
	waitPane.add(pb);
	waitFrame.addWindowListener(new WindowAdapter() {
		    public void windowClosing(WindowEvent e) {
			//System.exit(0);
			e.getWindow().dispose();
		    }
	    });  
	waitFrame.setContentPane(waitPane);
	GraphicsConfiguration gc = waitFrame.getGraphicsConfiguration();
	Rectangle bounds = gc.getBounds();
	waitFrame.setLocation(((bounds.width-bounds.x)/2)-100,((bounds.height-bounds.y)/2)-50);
	waitFrame.pack();
	waitFrame.setVisible(true);
	waitFrame.repaint();		
    }
    /** close a wait frame
     */
    public void closeWaitURL(){
	if (waitFrame != null)
	waitFrame.dispose();
	//waitFrame.setVisible(false);
    }

    class Hyperactive implements HyperlinkListener {
	//String doneURL= new String("http://anc.ed.ac.uk/~paulro/neosim/xmldone.htm");
	JFrame webFrame;
	JEditorPane webPanel;
	String option = new String();

	public Hyperactive(JEditorPane panel){
	    super();
	    webPanel = panel;
	    option = new String("panel");

	}
	public void hyperlinkUpdate(HyperlinkEvent e) {
	    if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
		URL link = e.getURL();
		JEditorPane pane;

		    // find module name
		String url = link.toString();
		int nameIndex = url.indexOf("#");
		
	
		pane = (JEditorPane) e.getSource();
		try {
		    waitURL();
		      
		    pane.setContentType("text/html");
		    pane.setPage(e.getURL());
		    closeWaitURL();

		    registerURL(url);
		} catch (Throwable t) {
		    //t.printStackTrace();
		    System.out.println("cannot open url : page not found "+e.getURL());
		}
		
	    }
	}
    }

    class OpenWindow extends Thread {
	HTMLPanel panel = null;
	String url = null;


	OpenWindow(HTMLPanel panel, String url ) {
	    this.panel = panel;
	    this.url = url;
	}

	/** open url in htmlPanel
	 */
	public void run() {
	    
	    panel.openURL(url);
	
	}
    }


}
