package guikit;
 
import neuroml.*;
import neuroml.util.*;
import neuroml.core.*;
import neuroml.model.network.*;
import neuroml.sim.run.*;

import javax.swing.*; 
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.net.*;
import java.lang.reflect.*;
import java.util.*;

/** Example extension of the Basic Editor Application
 * -- Adds some extra menus items
 *
 * @author  Fred Howell
 * @date July 2001
 */
public class RunApp extends EmptyApp {
 
    /** Initialise application
     */
    Simulator launcher;
    String currentDir = new String("none");
    String currentJarDir = new String("none");
  
    boolean nullParam = true;
    String args[] = new String[1];

    LabelStatusPanel statusPanel = new LabelStatusPanel();

    public RunApp(String frameName) {
        super(frameName); 
	
  	JMenu runMenu = addMenu( menuBar, "Run", "Run", KeyEvent.VK_R);
	addMenuItem(runMenu, "Choose Simulator", "Choose Simulator", -1);
	addMenuItem(runMenu, "Set Parameters", "Set Parameters", -1);
	addMenuItem(runMenu, "Run", "Run", -1);
	runMenu.addSeparator();
	addMenuItem(runMenu, "Exit", "Exit", -1);

	getContentPane().add(statusPanel, BorderLayout.CENTER);
	
    }
    
    /** Deal with the new menu items...
     */
    public void actionPerformed(ActionEvent e) {
	update(getGraphics());

	if (e.getActionCommand().equals("Run")) {

	    //System.out.println("Opening for Run "+filename);

	    if (launcher!=null){
		//if ( filename!=null){
		    if (nullParam){
			System.out.println("Error : Some parameters of the simulator are not set.");	
		    } else {
		    
			System.out.println("Launching the simulation.");	
			//String args[] = new String[2];
			//args[0] = new String("seq");
			//args[args.length-1] = filename;
			//launcher.launch(args);
			launcher.launch();

		    } 
		    //} else 
		    //   System.out.println("Please parameter the simulator before running a simulation.");	
	    } else {
		System.out.println("Please choose a simulator before running a simulation.");	
	    }
		//}
	} else if (e.getActionCommand().equals("Choose Simulator")) {
	    System.out.println("choose");
	    URL newModule = URLfileChooser();
	    if (newModule!=null) {
		
		System.out.println("Loading jar file "+newModule);
		// *** Try to get the simulator module ***
		launcher = ModuleLoader.getSimulator( newModule.toString(), "" );

		// status validation
		if (launcher == null) {	   
		    statusPanel.currentSimValue = new String("none");
		    statusPanel.refresh();
		    System.out.println("Module is not a XMLReader "+newModule);
		} else {
		    statusPanel.currentSimValue = newModule.toString();
		    statusPanel.paramsSet = false;
		    statusPanel.refresh();
		    System.out.println("Module is a XMLReader "+newModule);

		}
		
	    }
	} else if (e.getActionCommand().equals("Set Parameters")) {
	  
	    /*
	    if (fname==null) {
		// status invalidation
		statusPanel.currentFileValue = new String("none");
		statusPanel.refresh();

	    } else {
		
		System.out.println("Opening for Run "+fname);
		NeuromlSys.loadXML(fname);
		filename = fname;

		// status validation
		statusPanel.currentFileValue = filename;
		statusPanel.refresh();
	    */

		if (launcher!=null){

		    System.out.println("set parameters");
		    
		    Namespace ns = NeuromlSys.makeNamespace("_tmp.xml");
		System.out.println("ns "+ns);
		    String currentClass = new String("neuroml.sim.run.Simulator");
		    FiringEditObjectPanel subp = new FiringEditObjectPanel(this,ns,launcher , currentClass, false);	
		    subp.popup();

		} else {
		    System.out.println("Please choose a simulator before setting its parameters.");	
		}
	
		//}
	} else if (e.getActionCommand().equals("Termination")) {
	    System.out.println("RunApp : popup termination");	
	    Class c = launcher.getClass();
	    Field[] flds = c.getFields();
	    // setup paramater array
	    args = new String[flds.length];
	    String argsNames[] = new String[flds.length];
	    // we have to check if some parameters are not set
	    nullParam = false;
	    try {
		// for all fields
		for (int i=0; i<flds.length; i++) {
		    // get field i
		    
		    Object fob = flds[i].get(launcher);
		    String fieldName = flds[i].getName();
		    if (fieldName== null)
			fieldName = new String("null");
		    argsNames[i] = fieldName;

		    // if field i is a string or URL (should be)
		    if (fob == null){
			nullParam = true;
			args[i] = new String("null");
		    } else if (fob instanceof String ) {
			args[i] = (String)fob;			
		    } else if (fob instanceof URL ) {
			args[i] = ((URL)fob).toString();
			// neuroml parameter
			//if (args[i].endsWith(".xml")){
			//    filename = args[i];
			//}		
		    }
		    else {
			System.out.println("Unknown type for parameter "+fob);
			nullParam = true;
			args[i] = new String("null");
		    }
		}
	    } catch ( Exception e2) {
		System.out.println(" RunApp Run : "+e2);
	    }
	    
	    // params status validation
	    Hashtable paramValues = new Hashtable();
	    for (int i=0; i<(args.length); i++) {
		paramValues.put(argsNames[i],args[i]);
		System.out.println(" RunApp add "+argsNames[i]+" "+args[i]);
	    }
	    statusPanel.currentParamsValue = paramValues;
	    statusPanel.paramsSet = !nullParam;


	    //System.out.println(" debug Filename is "+filename);
	    /*
	    if ((filename==null)||(filename.equals("none"))) {
		// status invalidation
		statusPanel.currentFileValue = new String("none");
	    } else {
		
		// status validation
		statusPanel.currentFileValue = filename;
	    }
	    */
	    // refresh status panel
	    statusPanel.refresh();

	} else if (e.getActionCommand().equals("Exit")) {
            System.exit(0);

	} else {
	    super.actionPerformed(e);
	}
	
    }

    /** Popup a file chooser which returns an URL
     */
    public URL URLfileChooser() {
	String cwd;
	if (currentJarDir.equals("none")){
	    cwd = System.getProperty("user.dir");
	}
	else cwd = new String(currentDir);
	JFileChooser chooser = new JFileChooser(cwd); 
	ExampleFileFilter filter = new ExampleFileFilter( "jar", "NeuroML modules");
	chooser.addChoosableFileFilter(filter);	

	int returnVal = chooser.showOpenDialog(this); 
	if (returnVal == JFileChooser.APPROVE_OPTION) { 
	    URL ret = null;
	    try { 
		ret = chooser.getSelectedFile().toURL();
		currentJarDir = chooser.getSelectedFile().getParent();
	    } 
	    catch (Exception e ) { 
		System.out.println(e); 
	    }
	    return ret;
	} 
	return null;
    }
  /** Popup a file chooser 
     */
    public String fileWithPathChooser(String title) {
	String cwd;
	if (currentDir.equals("none")){
	     cwd = System.getProperty("user.dir");
	}
	else cwd = new String(currentDir);

	JFileChooser chooser = new JFileChooser(cwd);  
	ExampleFileFilter filter = new ExampleFileFilter( "xml", "NeuroML model files");
	chooser.addChoosableFileFilter(filter);	
	chooser.setDialogTitle(title);
	int returnVal = chooser.showOpenDialog(this); 
	if (returnVal == JFileChooser.APPROVE_OPTION) { 
	    String res = new String(chooser.getSelectedFile().getParent()+"/"+chooser.getSelectedFile().getName());
	    currentDir = chooser.getSelectedFile().getParent();
	    return res;
	} 
	return null;
    }
    /** Sample main program
     */
    public static void main(String[] args) {
	System.out.println("Starting Simulator Launcher");
        JFrame frame = new RunApp("Simulator Launcher");
	frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) { System.exit(0);  }
	    });  	
        frame.pack();
        frame.setVisible(true);
    }


    // private class for displaying status
    class LabelStatusPanel extends JPanel {

	public boolean paramsSet = false;

	int paramNumber = 0;

	JLabel currentSim;
	JLabel[] currentParams;

	String currentSimTitle = new String("<html><b><font size=-1 face=\"Arial\" color=\"#222222\"> Selected simulator :</font> ");
	String currentParamsTitle = new String("<html><b><font size=-1 face=\"Arial\" color=\"#222222\"> ");
	String currentParamsTitleMiddle = new String(" :</font> ");


	String titleEnd = new String("</font></html></b>");
	String titleGreen = new String("<font color=green>");
	String titleRed =  "<font color=red>";
	String titleColor = new String("");

	public String currentSimValue = new String("none");
	public Hashtable currentParamsValue = new Hashtable();

	public LabelStatusPanel() {
	    super();
	    build();
	}
	
	public Dimension getMinimumSize() {
	    return getPreferredSize(); 
	}
	public Dimension getPreferredSize() {
	    int heigth;
	    if (paramNumber<5) heigth = 0;
	    else heigth = 20*(paramNumber-5);

	    return new Dimension(500,130+heigth); 

	}
	public Dimension getMaximumSize() {
	    return new Dimension(super.getMaximumSize().width,super.getPreferredSize().height);
	}
	
	private void build(){
	    
	    if (currentSimValue.equals("none"))
		titleColor = titleRed;
	    else titleColor = titleGreen;
	    currentSim = new JLabel(currentSimTitle+titleColor+" "+currentSimValue+titleEnd);

	    setLayout(new BoxLayout(this, 
			BoxLayout.Y_AXIS));
	    add(currentSim);


	    if (!paramsSet)
		titleColor = titleRed;
	    else titleColor = titleGreen;

	    // array of params jlabels
	    currentParams = new JLabel[currentParamsValue.size()];

	    int index = 0;
	    for (Enumeration enum_ = currentParamsValue.keys() ; enum_.hasMoreElements() ;) {
		String pName = (String)enum_.nextElement();
		String pValue = (String)currentParamsValue.get(pName);
		pName = pName.replace('_',' ');
	System.out.println("status panel param "+pName+" "+pValue);

		currentParams[index] = new JLabel(currentParamsTitle+pName+currentParamsTitleMiddle+titleColor+" "+pValue+titleEnd);
	    
		add(currentParams[index]);
		index++;
	    }
	    paramNumber = index;

	    System.out.println("status panel built");

	}

	
	public void refresh(){
	    removeAll();
	    System.out.println("refresh with currentSimValue = "+currentSimValue);
	    build();
	    updateUI();
	    pack(); // for resizing if more params than displayable
	    update(getGraphics());
	}

	
    } // end class LabelStatusPanel


 
}
