/*
 * LearningSynapse.java
 *
 * Created on 18 August 2004, 13:36
 */

package Holger_modules.module.shmemneuron;
import neuroml.model.Synapse_nml.*;

/**
 *
 * @author  Jean-Alain Grunchec
 * See Serge Thill's Thesis 
 */
public final class LearningSynapse implements Synapse {
    protected double t_last_spike=0.0;
    protected double Use;
    protected double Use_plus;
    protected double x,y,z;
    protected double tau_rec,tau_in,tau_facil;
    protected double Ase;
    protected double Learning_step;
    SynapticCurrent Current;
    
    /** Creates a new instance of LearningSynapse */
    public LearningSynapse(SynapticCurrent Current){
        this.Current=Current;
//        this.dt=dt;
    }
    public LearningSynapse(){

    }
    public void setCurrent(SynapticCurrent Current){
        this.Current=Current;
    }
    //public void setDt(double dt){
    //    this.dt=dt;
    //}
    public void advance(double dt)
    {
        double dx,dy,dz,dUse;
        dx=z/tau_rec*dt;
        dy=-y/tau_in*dt;
        dz=(y/tau_in-z/tau_rec)*dt;
        dUse=-Use/tau_facil*dt;
        x+=dx;
        y+=dy;
        z+=dz;
        Use+=dUse;
        Current.addValue(Ase*dy);
    }
    public void handleSpike(double d, double dt)
    {    
        double dx,dy,dz,dUse;
        t_last_spike=d;
        dx=-Use*x*dt;
        dy=Use*x*dt;
        dUse=Use_plus*(1-Use)*dt;
        x+=dx;
        y+=dy;
        Use+=dUse;
        Current.addValue(Ase*dy);
    }
    public void CheckLearning(double d, double dt)
    {    
        if(Learning_step>=d-t_last_spike)
            Use+=Use_plus*(1-Use)*dt;
    }
    public void setParameter(double Excitation_factor,double Synaptic_Weight,
                                double tau_rec,double tau_in,double tau_facil,
                                double Learning_step,
                                double recovered_prop,double active_prop,double inactive_prop){
        Use_plus=Excitation_factor;
        Ase=Synaptic_Weight;                // 1e-9 //-1e-9
        this.tau_rec=tau_rec;               // 1 (1000 ms)
        this.tau_facil=tau_facil;           // 0.010 (10 ms)
        this.tau_in=tau_in;                 // 0.001 (1 ms)
        this.Learning_step=Learning_step;   // 0.02 20 ms
        this.x=recovered_prop;              // 1.00
        this.y=active_prop;                 // 0.00
        this.z=inactive_prop;               // 0.00
    }
    
    public double getSynapseIntensity() {
        return Ase*y;
    }
    
    public double getLearningFactor(){
        return Use;
    }
    
    public double get_X_Concentration(){
        return x;
    }    
    public double get_Y_Concentration(){
        return y;
    }    
    public double get_Z_Concentration(){
        return z;
    }        
    
    
}
