package neosim.iface;

/** This stores the name, parent and ID of a class. 
 * The method <tt>isAssignableFrom()</tt> provides 
 * basic type checking - classes are asssignable from 
 * other identical classes or derived classes.
 */
public interface NeosimClass extends java.io.Serializable {

    /** Return the parent class from which this one is derived
     * @return the parent class, or null if no parent
     */
    public NeosimClass getParent();

    /** Return the integer class ID
     * @return the class id
     */
    public ClassID     getClassID();

    /** Check if this class is assignable from another class.
     * A class is assignable from another if the other
     * class is the same, or derived from this class.
     * e.g. event.isAssignableFrom( myKindOfEvent ) returns true
     * @ec the other class
     * @return true if ec is a sub class of this one
    public boolean     isAssignableFrom( NeosimClass ec );

    /** Return the name of this class 
     */
    public String      getName();

    /** Return the package of this class.
     */
    public String      getPackage();
}
