package neosim.iface;
import java.util.Hashtable;

/** The part of a general projection which runs on
 * each source entity and sends requests to make connections
 * to candidate destination entities.
 */
public interface SourceMethod extends java.io.Serializable {
    /** 
     * THIS METHOD SHOULD BE OVERRIDDEN BY DIFFERENT SOURCE METHODS.
     * The sendRequests method checks the source entity's details 
     * and sends connection requests to individual destination
     * entities (using sendRequest) or to all members of a population
     * (using bcastRequest). 
     * @param srce the source entity
     * @param destPop the destination Population
     * @param dm the destination method to handle requests at the far end
     */
    public void sendRequests( Entity srce, Population destPop, DestMethod dm, boolean BuildingConnection );

    /** 
     * THIS METHOD CAN BE CALLED FROM WITHIN sendRequests() ABOVE.
     * The sendRequest method sends an individual connection request to 
     * a destination entity
     * @param srce the source entity
     * @param deste the handle of the destination entity
     * @param cr the connection request
     */
    void sendRequest(  Entity srce, int deste, ConnectionRequest cr, boolean BuildingConnection );//EntityID deste

    /**
     * THIS METHOD CAN BE CALLED FROM WITHIN sendRequests() ABOVE.
     * The bcastRequest method sends an identical request to
     * all members of a destination population
     * @param srce the source entity
     * @param destpop the population to connect to
     * @param cr the connection request
     */
    void bcastRequest( Entity srce, Population destpop, ConnectionRequest cr,  boolean BuildingConnection );
    /** 
     * This method is called when the method is loaded 
     * from a jar file
     * 
     * @param name name of the method
     * @param params hashtable of the method's parameters
     *
     */
    public void init( String name, Hashtable params);
}
