

package neosim.implementation;
import neosim.iface.*;
import neosim.kernel.*;
import java.util.Vector;
import java.util.Hashtable;
import neosim.neuroml.*;


/** A neosim entity. All user entities derive from this.
 */

public class EntityImpl implements Entity, DefaultHandler {
    static boolean debug = false;
    static EventList last_evl;
    transient private Population p;
    transient private int index;
    transient protected int handle;//EntityID
    transient private EntityClass ec;
    transient private Time localTime;
    transient protected Kernel sk;
    transient protected int NumberConnection=0;
    transient protected InportsTableOut LinkedListCon=new InportsTableOut(); //18/04/05-> added protected
//    transient protected DeadlockTracker tracker=new DeadlockTracker();
    //JAG
    public EntityImpl(){
        System.out.println("THIS SHOULDN'T BE CALLED (except for memory tests)");
    }
    /** Make an entity of population p, class ec
     */
    
    public void compress_final(){
         //System.out.println("Compressing Entity__ "+handle+"("+getClass().getName()+")");
         LinkedListCon.compress_final();   
    }

  /*  public void check_compressed(){
        LinkedListCon.check_compressed();
    } */   
    
    public void compress(){
         LinkedListCon.compress();   
    }
    
    public EntityImpl( EntityClass ec, Population p, int index ) { 
	//	System.out.println("EntityImpl constructor starting");

	this.p = p; this.ec = ec; this.index = index; localTime = new Time(0);
	eventList = new EventListImpl();
        //System.out.println("ARRAYINTERFACES");
        Class array[]=SimSystem.getKernel().getClass().getInterfaces();
        for(int i=0;i<array.length;i++)
            System.out.println(array[i].getName());
	sk = SimSystem.getKernel();

	inPorts        = new Vector();
	outPorts       = new Vector();
	inPortClasses  = new Vector();
	outPortClasses = new Vector();   
	outConnections = new Vector();    
//	inConnections  = new Vector();
        
	if (p==null) { handle = EntityID.SYSTEM_ENTITY; }
	else  { handle = p.getEntityID(index); }

	if (debug) System.out.println("  EntityImpl... Making new Entity "+handle+" pop "+p);
	sk.addEntity( this );


	/** Add ports of class */
	if (ec!=null) {
	    Vector classInPorts = ec.getInPorts();
	    for (int i=0; i< classInPorts.size(); i++) {
		addInPort( (EventClass)classInPorts.elementAt(i));
	    }
	    Vector classOutPorts = ec.getOutPorts();
	    for (int j=0; j< classOutPorts.size(); j++) {
		addOutPort( (EventClass)classOutPorts.elementAt(j));
	    }
	}
    }

    /** User entities can extend this method 
     */
    public void init(Population p, int index) {
    }

    /** 
     * @return the index of this entity in its population/
     */
    public final int getIndex() { return index; }

    /** User entities can either extend this method  - or the advance() and handle() methods
     * It deals with events in el up
     * to time t, and advance the entity's time to at least t and return the time 
     * actually advanced to (which may be later than t, as long as the state can be 
     * rewound back to t and no new events are generated after t). If the 
     * event type of incoming events is not recognised, the
     * method defaultEventHandler() should be called.
     * @param el The (possible empty) event list in time stamp order
     * @param endt The time to advance to
     * @return the time actually advanced to (must be >= t)
     * @see #defaultEventHandler
     */
    public Time handleEvents(EventList el, Time endt) {
	if (debug) System.out.println("Entity "+this+" handleEvents to "+endt);
	boolean passed = false;
        //JAG ADDED SYNCHRONIZED HERE OTHERWISE EXCEPTION WITH E1 first element removed ?..
        //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(el,ecp);
     /*   synchronized(el){
	while (el.hasMoreElements() && !passed) { 
	    Event e = el.top();
	    if (e.getTime().t <= endt.t) {
		e = el.pop();
		advance( e.getTime() );
		if (debug) 	System.out.println("Entity "+this+" about to deliver "+e);
                //JAG
		//e.deliverToEntity( e.getTime(), this, new Input(e.getConnection())); 
                Input Truc=new Input(e.getConnection());
                e.deliverToEntity( e.getTime(), this, Truc); 
	    } else { 
		passed = true; 
	    }
	}
        }*/
        boolean goon=true;
        
	while (goon && !passed) { 
            Event e = null;
            synchronized(el){
                goon=el.hasMoreElements();
                if(goon){
                    e = el.top();
                    if (e.getTime().t <= endt.t) {
                        e = el.pop();
                    } else { 
                        passed = true; 
                    }
                }
            }
            if(goon&&!passed){
		advance( e.getTime() );
		if (debug) 	System.out.println("Entity "+this+" about to deliver "+e);
                //JAG
		//e.deliverToEntity( e.getTime(), this, new Input(e.getConnection())); 
                Input Truc=new Input(e.getConnection());
                e.deliverToEntity( e.getTime(), this, Truc); 
            }
        }        
        //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
	advance( endt );
	return endt;
    }

    /** This method is called when simulation time
     * is rest to zero. User entities can extend this method to perform
     * a local reset.
     */
    public void reset() {
	localTime.set( 0.0 );
        
        //neosim.kernel.Deadlock_Message ecp6=new neosim.kernel.Deadlock_Message();
        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp6);        
	eventList.clear();
        //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp6);
    }

    /** This method is called when the entity is deleted.
     * User entities can extend this to remove any locally
     * allocated storage.
     */
    public void destroy() {
    }
 

    /** This method sends an event on output port c.
     * @param c The output port to send the event to.
     * @param e The event to send. Its timestamp should
     * be the <em>sending time</em>
     */
    final public void sendEvent( short c, Event e ) {//PortID c
	if (debug)  System.out.println("Entity::sendEvent to port "+c);
	sk.sendEvent( handle, c, e );
	if (debug)  System.out.println("Entity::sent event "+c);
    }

    /** This method sends an event <b>to</b> input port p of entity e.
     * @param c The output port to send the event from.
     * @param deste The destination entity
     * @param destp The destination input port
     * @param e The event to send. Its timestamp should
     * be the <em>sending time</em>
     */
    final public void sendEvent(   int deste, short destp, Event e ) {//EntityID deste, PortID destp
	if (debug) {	System.out.println("Entity::sendEvent to input port "+destp+" deste "+deste); }
	sk.sendEvent( handle, PortID.SYSTEM_PORT, deste, destp, e );
	if (debug) {	System.out.println("Entity::sentEvent to input port "+destp); }
    }

    /** Handle all events 
     */
    public void handle(  Time t, Event e, Input i) {
	if (debug) System.out.println("Received default event at time "+t+" from input "+i);	
	defaultEventHandler( e );
    }

    /** Advance Time 
     * This method should be overridden to advance internal state
     */
    public void advance( Time t ) {
	if (debug) System.out.println("Entity "+this+" Advancing time to "+t);
    }


    /** This method deals with built in events, used for querying
     * and updating entity state.
     * @param e The event to handle
     */
    final public void defaultEventHandler( Event e ) {
	sk.defaultEventHandler( this, e );
    }

    /** 
     * @return the population this entity was created in.
     */
    final public Population getPopulation() {
	return p;
    }

    /** @return the class of this entity.
     */
    final public EntityClass getEntityClass() {
	return ec;
    }

    /** @return the handle of this entity.
     */
    final public int getID() {//EntityID
	return handle;
    }

    /** Sets the handle of this entity.
     * @param h the new handle
     */
    final public void setID(int h) {//EntityID
	handle = h;
    }

    final public String getName() {
	if (p==null) return "system"; else return p.getName()+index;
    }

    /**  Return the top level population tree.
     * @param t the time of the request (needed as the population tree
     * changes with time)
     * @return the top level population tree
     *
     */
    final public PopulationTree getPopulationTree(Time t) {
	return sk.getPopulationTree(t);
    }

    /** Return the set of user defined classes.
     * @return the set of user classes.
     *
     */
    final public NeosimClasses  getNeosimClasses() {
	return sk.getNeosimClasses();
    }
 
    /** Look up an EntityClass object given a name,
     * e.g. "CompartmentalEntity"
     * @param name The fully qualified name of the class
     * @return the entity class, if found, or null if not.
     */
    final public EntityClass    lookupEntityClass( String name ) {
	return sk.lookupEntityClass(name);
    }

    /** Look up an Entity handle given a name,
     * e.g. "cortex.layer1.pyramidal1234"
     * @param t The time of the request
     * @param name The fully qualified name of the entity
     * @return the entity handle, if found, or null if not.
     */
    final public int   lookupEntityID( Time t, String name ) { //EntityID
	return sk.lookupEntityID(t,name);
    }

    /** Look up a Population given a name,
     * e.g. "cortex.layer1"
     * @param t The time of the request
     * @param name The fully qualified name of the population
     * @return the population if found, or null if not.
     */
    final public Population     lookupPopulation( Time t, String name ) {
	return sk.lookupPopulation(t,name);
    }
 

    /** Build a population at a given time 
     * @param t the time to build the population (<b> must be >= localTime() + getMinOutputDelay()</b>)
     * @param p the population to build
     * @param c the callback to call when the population has been built.
     */
    final public  void   construct ( Time t, PopulationBuilder p, Callback c) {
	sk.construct( handle, t, p, c );
    }

    /** Destroy a population at a given time. Calls every member's destroy() method.
     * @param t the time to build the population (<b> must be >= localTime() + getMinOutputDelay()</b>)
     * @param p the population to destroy
     * @param c the callback to call when the population has been destroyed	
     */
    final public void         destroy ( Time t, Population p, Callback c ) {
	sk.destroy( handle, t, p, c );
    }

    /** Make connections at a given time
     * @param t the time to make the connections (<b> must be >= localTime() + getMinOutputDelay()</b>)
     * @param p the projection to make
     * @param c the callback to call when the connections have been made
     */
    final public void         connect ( Time t, Projection p, Callback c ) {
	sk.connect( handle, t, p, c );
    }

    /** Delete connections at a given time
     * @param t the time to delete the connections (<b> must be >= localTime() + getMinOutputDelay()</b>)
     * @param p the population to destroy
     * @param c the callback to call when the population has been destroyed	
     */
    final public void         disconnect ( Time t, Projection p, Callback c ) {
	sk.disconnect( handle, t, p, c );
    }

    /** Update another entity at a given time
     * @param t the time to update the entity (<b> must be >= localTime() + getMinOutputDelay()</b>)
     * @param e the handle of the entity to update
     * @param eu the update to perform	
     * @param c the callback to call when the update has been done
     */
    final public void         updateEntity( Time t, int e, EntityUpdate eu, Callback c ) {//EntityID e
	sk.updateEntity( handle, t, e, eu, c );
    }

    /** Query another entity at a given time
     * @param t the time to query the entity (<b> must be >= localTime() + getMinOutputDelay()</b>)
     * @param e the handle of the entity to query
     * @param eq the query to perform	
     * @param c the callback to call with the results of the query
     */
    final public void         queryEntity( Time t, int e, EntityQuery eq, Callback c ) {//EntityID e
	sk.queryEntity( handle, t, e, eq, c );
    }
    
    /** Update a population of entities at a given time
     * @param t the time to update the entity (<b> must be >= localTime() + getMinOutputDelay()</b>)
     * @param p the population to update
     * @param eu the update to perform	
     * @param c the callback to call when the update has been done
     */
    final public void         updatePopulation( Time t, Population p, EntityUpdate eu, Callback c ) {
	sk.updatePopulation( handle, t, p, eu, c );
    }

    /** Query a population of entities at a given time
     * @param t the time to query the population(<b> must be >= localTime() + getMinOutputDelay()</b>)
     * @param p the population to query
     * @param eq the query to perform	
     * @param c the callback to call with the results of the query. The results will
     * be in the form of a vector of objects, indexed by entties' index in the population.
     */
    final public void         queryPopulation( Time t, Population p, EntityQuery eq, Callback c ) {
	sk.queryPopulation( handle, t, p, eq, c );
    }


    final public void queryOutputConnections( Time t, short p, ConnectionQuery cq, Callback cb ) {//PortID p
	sk.queryOutputConnections( handle, t, handle, p, cq, cb );
    }



    /** Vector of input port ids */
    transient private Vector inPorts;

    /** Vector of output port ids */
    transient private Vector outPorts;

    /** Vector of input port event classes */
    transient private Vector inPortClasses;

    /** Vector of output port event classes */
    transient private Vector outPortClasses; 
 
    /** Vector of vectors of OutPortTable indexed by portid */
    transient private Vector outConnections;



    /** Add an output port 
     * @param ec the class of events the port generates
     * @return the id of the output port (from 1..#output ports).
     */
    final public short  addOutPort(EventClass ec) {//PortID
 	Vector vec = new Vector(); vec.addElement(ec);
	return addOutPort(vec);
    }

    /** Add an output port 
     * @param ec the class of events the port generates
     * @return the id of the output port (from 1..#output ports).
     */
    final public short addOutPort(Vector vec) {//PortID
	outPortClasses.addElement( vec );
	outConnections.addElement( new OutPortTable() );
	return (short)outPortClasses.size();//new PortID(
    }

    /** Add an input port 
     * @param ec the class of events the port accepts
     * @return the id of the input port (from 1..#input ports).
     */
    final public short  addInPort (EventClass ec) {//PortID
	Vector vec = new Vector(); vec.addElement(ec);
	return addInPort(vec);
    }

    /** Add an input port 
     * @param ec the class of events the port accepts
     * @return the id of the input port (from 1..#input ports).
     */
    final public short  addInPort (Vector vec) {//PortID
	inPortClasses.addElement( vec );
	return (short)inPortClasses.size();//new PortID(
    }

    /** Return the classes of events accepted by an input port
     * @param pid the input port id
     * @return the classes of events
     */
    final public Vector   getClassesOfInPort(short pid) {//PortID pid
	return (Vector) inPortClasses.elementAt(pid-1);
    }

    /** Return the classes of events produced by an output port
     * @param pid the output port id
     * @return the classes of events
     */
    final public Vector  getClassesOfOutPort(short pid) {//PortID pid
	return (Vector) outPortClasses.elementAt(pid-1);
    }

    /** Return the number of input connections on a port
     * @param inport the input port id
     * @return the number of connections
     */
    final public int  getNumInConnections(short inport) {//PortID inport
        return NumberConnection;//LinkedListCon.NumberPerPort(inport, (short)-1);//port does not matter
	//return sk.getNumInConnections(handle, inport);
    }

    /** Return the number of output connections from a port
     * @param outport the output port id
     * @return  the number of connections
     */
    final public int  getNumOutConnections(short outport) {//PortID outport
	OutPortTable op = (OutPortTable)(outConnections.elementAt(outport-1));
	return op.numConnections();
    }

    final public OutPortTable getOutConnection( short outport ) {//PortID outport
	return (OutPortTable)(outConnections.elementAt(outport-1));
    }

    final public int getNumOutPorts() {
	return outConnections.size();
    }

    final public int getNumInPorts() {
	return inPortClasses.size();
    }

    /** Return the input connections to a port
     * @param outport the output port id
     * @return  the number of connections
     */
    final public int[][] getInConnections(short inport) {//PortID inport
	/*System.out.println("Warning: EntityImpl.getInConnections ignores port #");
        Vector v=new Vector();

        
	return inConnections; // ignores port # - could filter*/
        return LinkedListCon.getInConnection(inport);
	//	return sk.getInConnections(handle, inport);
    }
    
    /** Add an output connection from an output port to a given dest
     * @param outport the output port id
     * @param delay the delay on the connection
     * @param dest the destination entity
     * @param destinport the destination port id 
     */
    final public void  addOutConnection(short outport, long delay, //PortID outport
					int dest, short destinport ) {//EntityID dest//PortID destinport
	// get processor of connection
	int proc = sk.getEntityProcessor( dest );
	if (minOutputDelay.t > delay) { minOutputDelay.t = delay; }
//The following stuff is done for Entities connected to probes
	if (outport> getNumOutPorts()) {
            if (ec!=null){
                while(outport > getNumOutPorts()) {
                    if(this.ec.getOutPorts().size()<=outport) 
                        ((NeosimEntityClassImpl)ec).addOutPort(SpikeEvent.inst.getEventClass());                    
                    addOutPort((EventClass)ec.getOutPorts().elementAt(getNumOutPorts()));
                }
            }
	    //System.out.println(getName()+" error - adding  out connection from non existent port "+outport.id);
	} // else {
	    OutPortTable opt = (OutPortTable)( outConnections.elementAt(outport -1) );
	    opt.add( proc );
//	}
    }
    
    final public void  removeOutConnection(short outport, long delay, //PortID outport
					int dest, short destinport ) {////EntityID dest//PortID destinport
	// get processor of connection
	int proc = sk.getEntityProcessor( dest );
	

	if (outport > getNumOutPorts()) {
	    System.out.println(getName()+" error - removing  out connection from non existent port "+outport);
	} else {
	    OutPortTable opt = (OutPortTable)( outConnections.elementAt(outport -1) );
	    opt.remove( proc );
	}
    }


    final public void  deleteOutConnection(short outport, int dest, ////EntityID dest//PortID outport
					   short destinport ) {//PortID destinport
	// update min o/p delay ... not possible yet, as not storing delays here :(
	// To do this, need to recalc output delays. Population operation?
	int proc = sk.getEntityProcessor( dest );
	OutPortTable opt = (OutPortTable)(outConnections.elementAt(outport -1));
	opt.remove( proc );
    }

    /** Add an input connection
     * @param c the input connection to add
     * @return  the connection id
     */
    final public int addInConnection(Connection c){
      //  nbconns1++;
        //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(c,ecp); 
        int i=addInConnection(c.getSrcEntityID(),c.getSrcPortID(),c.getDstPortID());
        //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
        return i;
    }
//    public static int nbconns1=0;
//    public static int nbconns2=0;
    
    public synchronized int addInConnection(int srcID, short srcPort, short destPort) {//ConnectionID return
//        nbconns2++;
	// Add to local vector of incoming connections.
	//inConnections.addElement(c);
        //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp);   
        this.LinkedListCon.addOutConnection(srcPort, destPort, srcID);
        //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
	 //new ConnectionID( NumberConnection++ );//inConnections.size() ConnectionID cid = new ConnectionID
	//c.setConnectionID(cid);
        ++NumberConnection;
        notify();//18/04/05 (added for the netvis entity to be sure that the right number of connections is there);
	return NumberConnection;
    }
/** Add an input connection
     * @param c the input connection to add
     * @return  the connection id
     */
    
    /** Delete an input connection
     * @param cid the input connection to delete
     */
    final public void  deleteInConnection(short SrcPortID, short destPortID, int srcID) {//ConnectionID cid
        //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message(); 
        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp);             
        LinkedListCon.deleteInConnection(SrcPortID,destPortID, srcID);
         //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);     
        NumberConnection--;        //Added 18/04/05
    }
    final public void  deleteInConnection(Connection c) {
	deleteInConnection(c.getSrcPortID(),c.getDstPortID(),c.getSrcEntityID());
 }
    /** Set additional connection parameters at src end 
     * Called by ParamConnectionSpec.setSrcParams
     * User entities should override this to set 
     * their connection params
     * @param params the table of name/value (strings)
     */
    public void setSrcConnectionParams( Connection c,  short srcpid, Hashtable params) {// PortID srcpid
    }

    /** Set additional connection parameters at dst end 
     * Called by ParamConnectionSpec.setDstParams
     * User entities should override this to set 
     * their connection params
     * @param params the table of name/value (strings)
     */
    public void setDstConnectionParams( Connection c, int destcid, Hashtable params ) {//ConnectionID destcid
    }


    /** Delete an input port   
     * @param inport the input port id
     */
    final public void  deleteInPort(short inport) {//PortID inport
	// not yet implemented
    }

    /** Delete an output port   
     * @param inport the input port id
     */
    final public void deleteOutPort(short outport) {//PortID outport
	// not yet implemented
    }

    transient private Time minOutputDelay = new Time(1000.0); // *** Should be infinity ***
    transient private Time rightWindowTime = new Time(0);
    transient private Time provLocalTime = new Time(0); // Provisional local time
    transient private int  zeroed = 0;

    /** Zero delay on system output port */
    final public void zeroOutputDelay() {
	zeroed ++;
    }

    /** Set output delay to its previous value */
    public final void         resetOutputDelay() {
	zeroed --;
    }

    /** Get the minimum output delay */
    public final Time         getMinOutputDelay() {
	if (zeroed>0) { return Time.ZERO; } else { return minOutputDelay; }
    }

    /** Set the minimum output delay */
    public final void         setMinOutputDelay(Time t) {
	minOutputDelay.t = t.t;
    }

    /** Get the current local time */
    public final Time         getLocalTime() {
	return localTime;
    }

    /** Set the current local time */
    public final void        setLocalTime(Time t) {
	localTime.t = t.t;
    }

    /** Set the provisional local time */
    public void        setProvLocalTime(Time t) {
	provLocalTime.t = t.t;
    }

    /** Get the right window time */
    public final Time         getRightWindowTime() {
	rightWindowTime.t = localTime.t + minOutputDelay.t;
	return rightWindowTime;
    }


    /** ---- internal Package level methods, called from Basic kernel ---- */
    transient EventList eventList;

    /** Get EventList 
     */
    public final EventList getEventList() { return eventList; }

    /** Post event, connection id is coded in event. */
    public final void postEvent(Event e) { 
	//	trace(" received event at "+e.getTime().s());
        if (e.getTime().t < localTime.t) {
            trace("C Causality violation: ET:"+e.getTime().s()+" CLOCK:"+localTime.s());
        }
        //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(e,ecp);          
        eventList.add(e); 
        //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);      
    } 
    
    public final void trace( String msg ) {
	sk.trace(getName()+" at "+getLocalTime().s()+":"+msg);
    }

    /** Compare two "Entity" objects based on Right Window Times
     */
    public final int compareTo( Object o ) throws ClassCastException {
 	Entity m = (Entity)o;
	Time t1 = getRightWindowTime();
	Time t2 = m.getRightWindowTime();
	if (t1.t > t2.t) return -1;
	if (t1.t < t2.t) return 1;

	/** Entities with identical timestamps are also ordered on
	 * their IDs - this is required for placing entities in a set
	 * (see java.util.TreeSet, neosim.kernel.Entqueue)
	 */
	if (getID()>m.getID()) return -1;
	if (getID()<m.getID()) return 1;

	//	System.out.println("*** Entity "+getName()+" compareto "+m.getName()+" called ***");
	return 0;
    }
    
    public void addSpike(int index,long time){
        System.out.println("EntityImpl.addSpike should NOT be called");

    }
    public InportsTableOut getList(){
        return this.LinkedListCon;
    }
}
