package neosim.implementation;

import neosim.iface.*;

/** This stores the name, parent and ID of a class. 
 * The method <tt>isAssignableFrom()</tt> provides 
 * basic type checking - classes are asssignable from 
 * other identical classes or derived classes.
 */
public class NeosimClassImpl implements NeosimClass {
    NeosimClass parent;
    String name, pkg;
    ClassID id;

    public NeosimClassImpl() {
      name = "none"; parent = null; pkg="none"; id = new ClassID(0);
    }
     
    public NeosimClassImpl( NeosimClass parent, String name, String pkg, ClassID id ) {
	this.name = name; this.pkg = pkg; this.parent = parent; this.id = id;
    }
    
    /** Return the parent class from which this one is derived
     * @return the parent class, or null if no parent
     */
    public final NeosimClass getParent() { return parent; }

    /** Return the integer class ID
     * @return the class id
     */
    public final ClassID     getClassID() { return id; }

    /** Check if this class is assignable from another class.
     * A class is assignable from another if the other
     * class is the same, or derived from this class.
     * e.g. event.isAssignableFrom( myKindOfEvent ) returns true
     * @ec the other class
     * @return true if ec is a sub class of this one
     */
    public final boolean     isAssignableFrom( NeosimClass ec ) {
        boolean ans = false;
        do {
	    if (ec.getClassID() == id) { ans = true; }
	    ec = ec.getParent();
	} while (ans==false && ec != null);
	return ans;
    }

    /** Return the name of this class 
     */
    public final String      getName() { return name; }

    /** Return the package of this class.
     */
    public final String      getPackage() { return pkg; }
}
