/** Basic implementation of a SimKernel.
 */

package neosim.kernel;

import neosim.iface.*;
import neosim.implementation.*;
import neosim.perf.*;
  
import java.util.Vector;
import java.util.Random;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.StringTokenizer;

 

/** Basic Simulation Kernel.
 * Specific implementations on parallel platforms may override/extend this.
 * @author Fred Howell
 * @date Nov 2000 fixed a queryEntity bug
 */
public class BasicKernel implements RmtApp, SimKernel { //Kernel

    private boolean debug = false;
    private boolean debug_doit = false;
    /** Buffered link for performing local work */
    protected CommandQueue workq;

    /** Buffered link for performing commands */
    protected CommandQueue cq;
     
    /** Receive semaphore */
    protected Semaphore wflag = new Semaphore(1);
    protected Semaphore rflag = new Semaphore(0);
    protected Object rbuf;
    protected int rsrc;
    
    /** Barrier semaphore */
    protected Semaphore bsem = new Semaphore(0);

    /** Query semaphore */
    protected Vector inpQueues = new Vector();

    /** FIXME **/
    protected int allgatherid = 1000; // ID Tag for separating main commands
    
    /** Sleep semaphore */
    protected Semaphore ssem = new Semaphore(0);
    
    /** Add semaphore */
    protected Semaphore asem = new Semaphore(1);

    /** Vector of all processors */
    protected ProcManager allprocs;

    /** Input connections - vector of InPortsTable, for delivering */
//    protected Vector inports;
    protected InPortsTable inports;//=new InPortsTable(this)

    /** Input connections - hash table, for querying, hashed by deste/portid, elements are vectors. */
//    protected Hashtable inConnections; JAG030205

    protected int id;
    protected int qid; /* ID for queries */
    protected CallbackQueue callbackQueue;

    protected RmtProcMgr pmgr;
    
    protected Vector otherprocs;
    protected Vector othercqs;  

  //  InPortTableOneOne[]  inports_1_1;

   /* public InPortTableOneOne[] getInPortTableOneOne(){
        return inports_1_1;
    }*/
//    transient neosim.kernel.DeadlockTracker _Tracker=new neosim.kernel.DeadlockTracker();
    
    
    public BasicKernel() {
        if(neosim.util.loaders.nmllauncher.NeosimNeuroMLLauncher.Player!=null){
                neosim.util.loaders.nmllauncher.NeosimNeuroMLLauncher.Player.println("");
                neosim.util.loaders.nmllauncher.NeosimNeuroMLLauncher.Player.println("Start new kernel "+this.getClass().getName());     
        }
	//otherprocs = new Vector(10);    // vector of RmtApps
	//allprocs   = new ProcManager(); // vector of RmtProcs
	//inports    = new Vector();      // vector of InPorts
	//inConnections = new Hashtable();
	//othercqs   = new Vector();      // vector of cqs
	//workq      = new CommandQueue(this); // Async queue for running blockable jobs here
	//workq.setQName("WORK");
	//workq.start();

	/* Initialise callback queue */
	//callbackQueue = new CallbackQueue();


	kernelInit(); // Initialise implementation.Kernel stuff
    }

    public void terminate() {

	getSimSystem().terminate();
	workq.terminate();
	for (int p=0; p<othercqs.size(); p++) {
	    CommandQueue ocq = ((CommandQueue)othercqs.elementAt(p));
	    ocq.terminate();
	}
	cq.terminate();
        for(int i=0;i<allprocs.size();i++){
            Object obj=allprocs.elementAt(i);
            if(obj!=null &&obj instanceof RmtProc)
                try{
                    ((RmtProc)obj).poison();
                }
                catch(Exception e){
                    e.printStackTrace();
                }
        }
        //JAG DANGEROUS OPTIMIZATION
        
        Thread truc[]=new Thread[Thread.activeCount()];
        Thread.enumerate(truc);
        
        for(int i=0;i<truc.length;i++){
            try{
                if(truc[i].getClass()==CommandQueue.class)
                    ((CommandQueue)truc[i]).terminate();
            }
            catch(Exception e){
                //Has been terminated before going in the termination orders
            }
        }
        //System.out.println("NbThreads: "+truc.length);
        //System.out.println("2********");       
    }

    public final SimSystem getSimSystem() { return new SimSystem(); }
    public final SystemEntity getSysEntity() { return SimSystem.getSysEntity(); }

    /** Add an entity (called from EntityImpl constructor) 
     */
    public final void addEntity( EntityImpl ne ) {
	SimSystem.add( ne );
    }

    public final int getID()        { return id; }
    public final int getRank()      { return id; }
    public final int getGroupsize() { return otherprocs.size(); }
    public final int getNumEnts()   { return SimSystem.getNumEntities(); }
    
    /** Return minimum output delay across all processors 
     */
    public final Time getMinDelay() {
	return SimSystem.calcMinOutputDelay();
    }

    /** Like gather, but all call it *** in same order ***. 
     */
    public Vector allgather(Object o)  throws ResetException {
	int qid =  allgatherid++;
	Vector qbuf = new Vector();

//JAG DANGEROUS OPTIMIZATION        
       // QueryResponse qr = new QueryResponse(getID(), qid, o);
        QueryResponse qr ;
        if(o==null){
            qr = new QueryResponseNull(getID(), qid, o);
        }
        else{
            Class cls=o.getClass();
            if(cls==neosim.iface.Time.class){
                qr = new QueryResponseTime(getID(), qid, o);
                ((QueryResponseTime)qr).index=1;
            }
            else if(cls==Integer.class)
                qr = new QueryResponseInteger(getID(), qid, o);
            else
                qr = new QueryResponse(getID(), qid, o);
        }
        
        
        // bcast command
	for (int p=0; p<otherprocs.size(); p++) {
            
	    sendCommand(p, qr);
	}
	// Wait on all results to come in
	for (int i=0; i<otherprocs.size(); i++) {
	    InputQueue iq = (InputQueue)(inpQueues.elementAt(i));
            //System.out.println("A. Pop "+i);
            //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
            //neosim.kernel.DeadlockTracker.LockDeadlockTracker(iq,ecp);
	    Object ro = iq.popSync(qid); /* Includes semaphore wait */
            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
            //System.out.println("Z. Pop "+i);
	    qbuf.addElement( ro );
	}	
	return qbuf;
    }

    /** allgatherv - Like allgather, but passed with a vector size nprocs, element i goes to 
     * processor i
     */
    public final Vector allgatherv(Vector o)  throws ResetException {
	int qid = allgatherid++;
	Vector qbuf = new Vector();
	//	System.out.println("Allgatherv: size = "+o.size()+ " iq "+inpQueues.size()+" qb "+qbuf.size());

       	// bcast command
	for (int p=0; p<otherprocs.size(); p++) {
            
//JAG DANGEROUS OPTIMIZATION             
	    //QueryResponse qr = new QueryResponse(getID(), qid, o.elementAt(p));
            Object muche=o.elementAt(p);
            
            QueryResponse qr ;
            if(muche==null){
                qr = new QueryResponseNull(getID(), qid, muche);
            }
            else{            
                Class cls=muche.getClass();
                if(cls==neosim.iface.Time.class){
                    qr = new QueryResponseTime(getID(), qid, muche);
                    ((QueryResponseTime)qr).index=2;
                }
                else if(cls==Integer.class)
                    qr = new QueryResponseInteger(getID(), qid, muche);
                else
                    qr = new QueryResponse(getID(), qid, muche);   
            }
            
	    sendCommand(p, qr);
	}
	// Wait on all results to come in
	for (int i=0; i<otherprocs.size(); i++) {
	    InputQueue iq = (InputQueue)(inpQueues.elementAt(i));
            //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
            //neosim.kernel.DeadlockTracker.LockDeadlockTracker(iq,ecp);            
	    Object ro = iq.popSync(qid); /* Includes semaphore wait */
            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
	    qbuf.addElement( ro );
	}	
	return qbuf;
    }

    /** Get a unique query identifier from 0-9999 
     */
    public final int getQid() { 
	qid = (qid+1) % 10000;
	return qid;
    }


    /** Called by callback handler when it's finished
     */
    public final void callbackDone( int q ) {
        //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp);        
	callbackQueue.removeCallback(q);
        //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);                
    }

    /** Gather query responses 
     * Needed for SysEnt command that separates QueryRequests from responses.
     */
    public final Vector gatherQresponses(int qid) {
	Vector tmp = new Vector(); tmp.setSize(otherprocs.size());
	// Wait on all results to come in
	for (int i=0; i<otherprocs.size(); i++) {
	    InputQueue iq = (InputQueue)(inpQueues.elementAt(i));
            //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
            //neosim.kernel.DeadlockTracker.LockDeadlockTracker(iq,ecp);
	    Object ro = iq.popSync(qid); /* Includes semaphore wait */
            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
	    tmp.setElementAt( ro, i );
	}	
	return tmp;
    }

    /** Remote answer to query request 
     */
    public final void qresponseRmt(int id,int qid,Object res) {
	//	System.out.println("Response arrived for query "+qid+" from "+id+
	//			   " obj "+res);

	InputQueue iq = (InputQueue)(inpQueues.elementAt(id));
	//	System.out.println("iq "+iq);
        //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(iq,ecp);   
	iq.addSync(res,qid); // Assumes message order preserved pt-pt
        //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
    }

    /** Return result of remote query (waits for results to arrive)
     * Returns a vector of the remote answers
     * Assumes a QueryCommand has been broadcast already
     */
    public final Vector waitForAnswers(int qid) {
	//	System.out.println("Wait for answers started "+qid);
	int i;
	Vector retv = new Vector();
	for (i=0; i<otherprocs.size(); i++) {
	    InputQueue iq = (InputQueue)(inpQueues.elementAt(i));
            //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
            //neosim.kernel.DeadlockTracker.LockDeadlockTracker(iq,ecp);            
	    Object ro = iq.popSync(qid); 
            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
	    retv.addElement( ro );
	    //System.out.println("Wait for answers got 1, id="+qid);
	}	
	return retv;
    }


    /** Return a single result of remote query  
     */
    public final Object waitForAnswer(int qid, int proc) {
	InputQueue iq = (InputQueue)(inpQueues.elementAt(proc));
        
        //CANT'have a tracker here :(
	return iq.popSync(qid);
    }


    /** Barrier 
     */
    public void barrier() {
    }

    /** Flush output queues 
     */
    public final void flush(boolean FlushMeasurement) {	
	//	System.out.println("About to flush");
        exchangeSpikes();//JAN05
       // System.out.println("Flush "+SimSystem.getSimTime().t);
	for (int p=0; p<othercqs.size(); p++) {
	    CommandQueue ocq = ((CommandQueue)othercqs.elementAt(p));
	    if (ocq!=null) {
		try {
		    ocq.flush(FlushMeasurement);
		} catch (Exception e) { 
		    System.out.println(" ... flush "+p+" exception "+e);
		}
	    }
	}
	//	System.out.println("Done flush");
    }
    
    /** Set this processor as sleeping */
    public final void setSleeping() {
	try {
	    if (debug) System.out.println("  zzz  sleeping...");
	    ssem.semWait(this);
	    if (debug) System.out.println("  zzz  woken up...");
	} catch (Exception ex) {
	    System.out.println("Failed to do sleeping cmd "+ex.getMessage());
	}
    }

    /** Remote entity has started running */
    public final void runningRmt(int rid, Time rwt) {
        try {
            RmtProc proc = allprocs.getProcessor(rid);
            if (proc==null) { System.out.println("allprocs.proc"+rid+" is null"); }
            else {
                proc.setState( SimProcessor.RUNNING );
                proc.setRightWindowTime(rwt);
                //neosim.kernel.Deadlock_Message ecp5=new neosim.kernel.Deadlock_Message();
                //neosim.kernel.DeadlockTracker.LockDeadlockTracker(proc,ecp5);                 
                allprocs.calcMinTime();
                //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp5);
            }
        } catch (Exception e) { System.out.println("RunningRmt exc:"+e); }
    }
     
    /** Set this processor as running */
    public final void setRunning(Time rwt) { 
        //JAG DANGEROUS OPTIMIZATION
	//sendCommand(new RunningCommand(id,rwt));
        runningRmt(id,rwt); 
    }
    
    /** Set this processor as stopped  */
    public final void setStopped() {
	sendCommand(new StoppedCommand(id));
    }

    /** Get current min time   */
    public final Time getMinTime() {
	return allprocs.getMinTime();
    }

    /** Remote entity has sent a message */
    public final void sendRmt(int rid, Object o) {
	wflag.semWait(this);
	rbuf = o;
	rsrc = rid;
	rflag.semSignal();
    }

    /** Remote entity is asking for some info */
    public final Object queryRmt(CommandObj co) {
	return co.doquery(this);
    }

    
    /** Remote entity has stopped running 
     */
    public final void stoppedRmt(int rid) {
	try {
	    if (debug)  System.out.println("Stopped Rmt arrived from "+rid);
	    // *** WAS:	    allprocs.getProcessor(rid).setState(SimProcessor.STOPPED);
            //neosim.kernel.Deadlock_Message ecp5=new neosim.kernel.Deadlock_Message();
            //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp5);               
	    allprocs.setStopped( allprocs.getProcessor(rid) );
            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp5);
	    //  allprocs.calcMinTime();
	    if (debug)  System.out.println("Stopped Rmt dealt with");
	} catch (Exception e) { System.out.println("stoppedRmt exc:"+e); }
	ssem.semSignal();
    }

    /** Someone has joined */
    public void addRmt(int rid, String n, RmtApp obj) {
	if (debug) 	System.out.println(">>> AddRmt::- "+rid);
	asem.semWait(this);
	//	System.out.println("    Name="+n);
	//	getState(rid).name = n;
	//	System.out.println("XYZ About to add rmt, sz="+otherprocs.size());
	if (otherprocs.size() < rid+1) {
	    otherprocs.setSize(rid+1);
	    othercqs.setSize(rid+1);
	    allprocs.setSize(rid+1);
	    //inports.setSize(rid+1);

	    // Set input buf for collective comms. - vector of input queues.
	    inpQueues.setSize(rid+1);
	}
	//inports.setElementAt(new InPortsTable(this), rid);
	allprocs.setElementAt(new SimProcessor(rid), rid);
	otherprocs.setElementAt(obj,rid);
	inpQueues.setElementAt(new InputQueue(),rid);
	CommandQueue ncq = new CommandQueue(obj);
	ncq.setQName("OCQ"+rid);
	othercqs.setElementAt(ncq,rid);
        System.out.println("Test othercqs: ID: "+this.id+" othercqs new size : "+othercqs.size()+" class of the RmtApp: "+obj.getClass().getName());
	ncq.start();
	if (debug) 	System.out.println("<<< Done Add"+rid+"  allprocs size is now "+allprocs.size());
	
	asem.semSignal();
    }

    /** Some processor has finished its tasks
     */
    public void doneRmt(int rid) {
	// 	System.out.println("<<< Processor "+rid+" waiting for the end");
    }

    /** Some processor has gone 
     */
    public void removeRmt(int rid) {   
	// 	System.out.println("<<< Processor "+rid+" has actually gone");
    }


    public void startRmt(int rid) {
    }

    /** Run the simulation */
    public void runRmt(int rid, Time t) {
        if(debug)	
            System.out.println("BasicKernel::Run Rmt called");
        //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp); 
        compress();
        //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
	SimSystem.runPar(1, 1, getID(), t); 
	if (debug)	System.out.println("BasicKernel::Run Rmt done");
    }
    
    public void compress(){
        if(CompressedBeforeSimulation){
            //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
            //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp); 
            inports.compress_final(); 
            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
            System.out.println("compress_final");
            CompressedBeforeSimulation=false;
//            if(neosim.util.SmallLists.LinkedList.checkCompression(inports)){ 18/04/05
                int k=getNumEnts();
		int i=0;
	 	while(i<k){
                    Entity nlocal = SimSystem.getEntity(i);
                    if(nlocal instanceof EntityImpl){
                        //neosim.kernel.Deadlock_Message ecp2=new neosim.kernel.Deadlock_Message();
                        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(nlocal,ecp2);                         
                        ((EntityImpl)nlocal).compress_final();
                        //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp2);
                    }
                    i++;
                }               
                /*int k=getNumEnts();
                int i=0;
                while(i<k){
                    EntityImpl nlocal = getLocalEntity(i);
                    if(nlocal!=null)
                        nlocal.compress_final();
                    else
                        System.out.println("Entity "+i+" null");                    
                    i++;*/
//                }
//            }
        }        
    }

    /** Reset simulation time to 0.0 
     */
    public void resetRmt(int rid) {	
  	SimSystem.reset();
    }
    

    public void restartRmt(int rid) {

	//	System.out.println(id+" BARRIER RESTART ARRIVED "+rid);
        this.terminate();
        getSimSystem().initialise(this, id);
        this.kernelInit();
      
	bsem.semSignal();
    }

 

    /** Send event from particular port
     */
    public final void eventRmt(int rid, int srce, short pid, Event ev) {//EntityID srce//PortID pid
	if (pid == 0) {
            //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
            //neosim.kernel.DeadlockTracker.LockDeadlockTracker(ev,ecp);             
	    // Send to system entity, this is a system command
	    SystemEntity se = getSysEntity();
	    se.postEvent(ev);
            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
	    // Deal with it immediately if not running
            //neosim.kernel.Deadlock_Message ecp6=new neosim.kernel.Deadlock_Message();
            //neosim.kernel.DeadlockTracker.LockDeadlockTracker(ev,ecp6); 
            EventList el=se.getEventList();
            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp6);
	    if (!SimSystem.running()) { se.handleEvents(el, se.getLocalTime()); }
	} else {
              //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
              //neosim.kernel.DeadlockTracker.LockDeadlockTracker(ev,ecp);               
                inports.sendEvent(pid, srce, ev);
                //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);                
	}
	//	System.out.println("Done eventRmt");
    }


    /** Deal with Connection query event
     */
    protected boolean CompressedBeforeSimulation=false;
    public final void connectionQueryRmt(int rid, int srce, short pid, ConnectionQueryEvent ev) {//EntityID srce//PortID pid
	inports.connectionQueryRmt(rid, srce, pid, ev);    
        /*
		Vector dests = ip.getConnections();
		for (int i=0; i<dests.size(); i++) {
		    Connection c = (Connection)dests.elementAt(i);
		    ConnectionQueryEvent cqe = (ConnectionQueryEvent)ev.clone();
		    cqe.setQueryConnection(c);
		    eventRmt( rid, srce, pid, c.getDstEntityID(), PortID.SYSTEM_PORT, cqe );
		}
	    } else {
		System.out.println("Input port : "+rid+" "+ev.getOutputEntity()+" "+ev.getOutputPort()+" dangling");
	    }*/
    }

    /** Send event to particular input port
     */
    public final void eventRmt(int rid, int srce, short srcpid, int deste_id, short destpid, Event ev) {//EntityID srce//PortID srcpid,EntityID deste_id,//PortID destpid
	/* Send to Script Reader if dest is System */
	if (deste_id==0) {
                //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
                //neosim.kernel.DeadlockTracker.LockDeadlockTracker(ev,ecp);              
	    EntityCallbackEvent ece = (EntityCallbackEvent) ev;
	    ece.doOnEntity( getSysEntity() );
            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
	} else {	    
	    EntityImpl ne = (EntityImpl)(SimSystem.getEntity( new Integer(deste_id )));
	    if (ne==null) { 
                System.out.println("eventRmt to unknown entity "+deste_id); 
                neosim.kernel.CommandQueue.PrintObject(ev, "event_rmt_error.txt");
            }
	    else {
                //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
                //neosim.kernel.DeadlockTracker.LockDeadlockTracker(ev,ecp);              
		Connection c = new ConnectionImpl(  srce, srcpid, ne, destpid, Time.ZERO.t );
		ev.setConnection(c);
		ne.postEvent( ev );
		// Deal with it immediately if not running
          //      System.out.println("Doing "+ev.getClass().getName()+" "+ne.getClass().getName());
                //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
                //neosim.kernel.Deadlock_Message ecp6=new neosim.kernel.Deadlock_Message();
                //neosim.kernel.DeadlockTracker.LockDeadlockTracker(ev,ecp6);                 
                EventList el=ne.getEventList();
                //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp6);
                //neosim.kernel.Deadlock_Message ecp7=new neosim.kernel.Deadlock_Message();                
                //neosim.kernel.DeadlockTracker.LockDeadlockTracker(ev,ecp7);                                 
		if (!SimSystem.running()) { ne.handleEvents(el, ne.getLocalTime()); }
                //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp7);
          //      System.out.println("Done "+ev.getClass().getName());
	    }
	}
    }


    /** Send command to given dest 
     */ 
    public void sendCommand(int p, CommandObj co) {
	CommandQueue ocq = ((CommandQueue)othercqs.elementAt(p));

        
	if (ocq!=null) {
	    try {
                //neosim.kernel.Deadlock_Message ecp0=new neosim.kernel.Deadlock_Message();
                //neosim.kernel.DeadlockTracker.LockDeadlockTracker(co,ecp0);                 
		ocq.add(co);
                //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp0);
	    } catch (Exception e) { 
		System.out.println(" ... sendCommand exception "+e+" sleep 10 seconds");
                try{
                    Thread.sleep(10000);                
                }
                catch(Exception x){
                    x.printStackTrace();
                }
	    }
	}
    }

    /** Issue orders to system 
     */
    public void sendCommand(CommandObj co) {
	try { 
	    for (int i=0; i<othercqs.size(); i++) {
		//		System.out.println("Sending command to "+i);
		sendCommand(i, co);
	    }
	} catch (Exception e) {	   
	    System.out.println("Failed to construct "+e.getMessage());
 	}
    }

    /** Issue orders to system, return response as a vector
     * This could use the async output queues too.
     */
    public void syncCommand(QueryCommand qc) {
        qc.setQid(getQid());
	if (debug)	System.out.println("syncCommand #"+qid);

	sendCommand(qc);

	if (debug)	System.out.println("syncCommand #"+qid+"waiting for answers");

        /** wait for responses */
        Vector v = waitForAnswers( qc.getQid() );      
	if (debug)	System.out.println("syncCommand #"+qid+" done");
    }


    /** Utility functions to issue commands
     */
    public void initSim() {
	System.out.println("BasicKernel: issuing initSim command ");
	syncCommand( new InitCommand( id ) );
	//	System.out.println("BasicKernel: done initSim command ");
    }
 
    /** Mechanism for remote user commands to get access to user data 
     * Typical usage: at initialisation, userApp is set to a reference to
     * the user's application object; it can then be used in extended
     * commands to operate on that application.
     */
    Object userapp;
    public final Object getUserApp() { return userapp; }
    public final void   setUserApp(Object o) { userapp = o; };
 

    /** Run for t time units 
     */
    public final void runSim(Time t) {
	if (debug)	System.out.println("   BasicKernel: Running til "+t.s());
	syncCommand( new RunCommand( id, t ) );
	if (debug)	System.out.println("   BasicKernel: Finished running til "+t.s());
    }


    /** Deal with commands from outside 
     */
    public final void command(CommandObj co) { 
	if (debug)	System.out.println("Got a command "+co);
	try {
            //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
            //neosim.kernel.DeadlockTracker.LockDeadlockTracker(co,ecp); 
	    if (co.isBlocking()) {
                //neosim.kernel.Deadlock_Message ecp0=new neosim.kernel.Deadlock_Message();
                //neosim.kernel.DeadlockTracker.LockDeadlockTracker(co,ecp0);                 
		if (debug)System.out.println("Putting command on work Q");
		co.setBlocking(false);
                //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp0);
                //neosim.kernel.Deadlock_Message ecp1=new neosim.kernel.Deadlock_Message();
                //neosim.kernel.DeadlockTracker.LockDeadlockTracker(co,ecp1);                  
		workq.add(co);
                //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp1);
	    } else {
		if (debug) System.out.println("Doing a command");
                if(debug_doit)
                    neosim.kernel.CommandQueue.PrintObject(co,"DebugDoit.txt");
                //neosim.kernel.Deadlock_Message ecp0=new neosim.kernel.Deadlock_Message();
                //neosim.kernel.DeadlockTracker.LockDeadlockTracker(co,ecp0);                                     
		co.doit(this); 
                //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp0);                
	    }
            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
	} catch (Exception e) {
	    System.out.println(" Command exception "+co+" : "+e);
	    e.printStackTrace();
	}
    }

    public final void destroy() {
	if (debug)	System.out.println("BasicKernel: Sending DestroyKernel");
 	syncCommand( new DestroyKernelCommand( id ) );
	if (debug)	System.out.println("BasicKernel: Sent DestroyKernel");
    }

    public void destroyRmt(int rid) {
	// Called by DestroyKernelCommand
	terminate();
	if (debug)	System.out.println("BasicKernel: DestroyRmt called - killing threads");	
    }
	  
    public void init() {
	//	System.out.println("Basic Kernel initialising");
	id = 0;
	this.arraySpikes=new SpikeArray(id);
	/* Create new command queue */
	cq = new CommandQueue(this);
	cq.setQName("SEQQUEUE");
	cq.start();	
	addRmt(0,"seq",this);

	//	System.out.println("Basic Kernel done init()");

	/** Initialise local SimSystem ** fwh added oct 00
	 */
	getSimSystem().initialise(this, id);
    }

    /** get inports for a given processor 
     
    public final InPortsTable getInports(int rmtproc) {
	return (InPortsTable)inports.elementAt(rmtproc);
    }*/

    // Add an input port 
    
    /*public final void addInport(int rmtproc, InPortTable ip) {
	InPortsTable ips = (InPortsTable)inports.elementAt(rmtproc);
	ips.add(ip);
    }

    // Show inports 
     
    public final void dispInports() {	
	for (int i=0; i<inports.size(); i++) {
	    InPortsTable ips = (InPortsTable)inports.elementAt(i);
	    ips.disp();
	}
    } */
    
    /** Generate Vrml 
     */
    public final void genVrml(String fname) {
	SimSystem.genVrml(fname);
    }
     
    /** Send event from particular connection 
     */
    public final void sendEvent(int destp, int srce, short cid, Event ev) {//EntityID srce//PortID cid
	if (debug) System.out.println("Basic kernel sending event from "+srce+" port "+cid);
               
	CommandQueue cq = ((CommandQueue)othercqs.elementAt(destp));
	if (cq!=null) {
	    try {
		cq.add( new EventCommand(id,srce,cid,ev) );
	    } catch (Exception e) { 
		System.out.println(" ... sendCommand exception "+e);
	    }
	}
    }



    /** Set simulation time to 0.0 
     */
    public void reset() {  
	//JAG syncCommand( new ResetCommand( id ) );		
        sendCommand( new RestartCommand( id ) );
    }

    /* Performance monitoring commands
     */
    public final void monitorProcessors(PerfMonitor pmon, boolean realTime) {
	SimSystem.monitorProcessors(pmon, realTime);
    }

    /* Performance monitoring commands 
     */
    public final void monitorEntities(PerfMonitor pmon, boolean realTime) {
	SimSystem.monitorEntities(pmon, realTime);	
    }





    /** --------------- Methods from implementation.Kernel -----------------
     */
 

    PopulationTree popTree;

    NeosimClasses classes;
   
    Vector entityLocation;  
    int nextHandle = 1;//EntityID nextHandle = new EntityID(1);

    public final int getNbEntities(){ //tatalNb of entities JAG
        if(entityLocation==null)
            return 0;
        return entityLocation.size();
    }
    protected final void kernelInit() {
//        inports_1_1=null;
	otherprocs = new Vector(10);    // vector of RmtApps
	allprocs   = new ProcManager(); // vector of RmtProcs
	//inports    = new Vector();      // vector of InPorts
        inports= new InPortsTable(this); 
//	inConnections = new Hashtable(); JAG030205
	othercqs   = new Vector();      // vector of cqs
        
        inpQueues = new Vector();
        allgatherid = 1000;      
	workq      = new CommandQueue(this); // Async queue for running blockable jobs here
	workq.setQName("WORK");
	workq.start();

	/* Initialise callback queue */
	callbackQueue = new CallbackQueue();
        
        nextHandle = 1;//new EntityID(1);
        
	popTree = new PopulationTreeImpl(""); //SOCKET_ERROR
	classes = new NeosimClassesImpl();
	
	entityLocation = new Vector();    // Vector of processor # Integer() for all entities 
        
        /*force garbage collection*/
        try {
           System.gc();
           Thread.currentThread().sleep(20);
           System.runFinalization();
           Thread.currentThread().sleep(20);
        }
        catch (InterruptedException ex){
            ex.printStackTrace();
        }
        /*again*/
        try {
           System.gc();
           Thread.currentThread().sleep(20);
           System.runFinalization();
           Thread.currentThread().sleep(20);
        }
        catch (InterruptedException ex){
            ex.printStackTrace();
        }  
        
        
	addRemoteEntity( 0, 0); // *** Processor 0 for entity 0 *** //new EntityID(0),0
	SimSystem.setKernel(this);
        
        /*Thread truc[]=new Thread[Thread.activeCount()];
        Thread.enumerate(truc);
        System.out.println("1********");
        for(int i=0;i<truc.length;i++)
            System.out.println(truc[i].getName());
        System.out.println("NbThreads: "+truc.length);
        System.out.println("1********");*/
        System.out.println("**** BasicKernel  USE "+ Runtime.getRuntime().totalMemory()+" bytes ****");   
    }

    /** Allocate a block of entity handles
     */
    public final int allocateIDs( int size ) { //retrun EntityID
	int n = nextHandle;//EntityID n = new EntityID( nextHandle.id );
	nextHandle += size;
	return n;
    }

    /** Return processor given a handle
     */
    public final int getEntityProcessor(int e) {//EntityID e
	if (debug) {	System.out.println("Getting processor of remote entity "+e); }
	if (e >= entityLocation.size()) {
	    System.out.println("error - handle "+e+"  entityLocation size "+entityLocation.size());
	    return 0;
	}

	Integer p = (Integer)entityLocation.elementAt(e);
	if (debug) {	System.out.println("Got "+p); }

	return p.intValue();
    }

    /** Return a neosim entity given a handle
     */
    public final EntityImpl getLocalEntity( int e ) { //EntityID e
	return (EntityImpl)SimSystem.getEntity(new Integer(e));
    } 

    /** Add entity location info (for lookup of entity processor given handle)
     */
    public final void addRemoteEntity( int e, int proc ) {//EntityID e
	//	System.out.println("Adding remote entity "+e.id+" to proc "+proc);
	if (entityLocation.size() < e+1) {
	    entityLocation.setSize(e+1);
	}
	entityLocation.setElementAt( new Integer(proc), e );
    }

    

    /** Send an event from an output port of an entity 
     */
    public final void sendEvent( int handle, short pid, Event e ) {//EntityID handle//PortID pid
	if (pid==0) {
	    // A system command, send it to all processors' SysEntitys
	    //System.out.println("send system command event");
	    SysEntEvent se = (SysEntEvent) e;
	    se.setQid(getQid());
	    se.setProc(getRank());
 
	    //	    System.out.println("making callback handler");
	    CallbackHandler cbh = new CallbackHandler( se.getQid(), getRank(), e.getTime(), this,
						       handle, pid, se.getCallback() );
	    //	    System.out.println("adding cbh");
            //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
            //neosim.kernel.DeadlockTracker.LockDeadlockTracker(cbh,ecp);
	    callbackQueue.add(cbh);
            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
	    //	    System.out.println("starting cbh");
	    cbh.start();

	    //	    System.out.println("sending command");
	    sendCommand( new EventCommand( getRank(), handle, PortID.SYSTEM_PORT, e ) );
	    //	    System.out.println("sent event command");



	} else {
	    // A point-point send, send it to all destination processors of (handle,pid)
	    // lookup output connections of (handle,pid) [here, a list of processors]
	    // send an EventCommand to each processor
	    EntityImpl ne = getLocalEntity( handle );
	    if (ne!=null) {
		OutPortTable opt = ne.getOutConnection( pid );
		Vector ports = opt.getPorts();
		for (int i=0; i<ports.size(); i++) {
		    OutPort op = (OutPort)ports.elementAt(i);
		    int proc = op.getProc();
		    sendCommand( proc, new EventCommand( getRank(), handle, pid, e ) );
		}
	    }
	}
    }
 

    /** Send event to input of given entity
     */
    public final void sendEvent( int handle, short srcp, int deste, short destp, Event e ) { //EntityID handle,EntityID deste//PortID srcp//PortID destp
        //neosim.kernel.Deadlock_Message ecp0=new neosim.kernel.Deadlock_Message();
        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp0);  
	if (debug) { System.out.println("BasicKernel abt to get entity processor"); }
	int destproc = getEntityProcessor(deste);
	if (debug) { 
	    System.out.println("BasicKernel Send event to input "+deste+"."+destp+" proc "+destproc);
	    System.out.flush(); 
	}
        //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp0);    

        //neosim.kernel.Deadlock_Message ecp1=new neosim.kernel.Deadlock_Message();
        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp1);          
	sendCommand( destproc, new EventToCommand( getRank(), handle, PortID.SYSTEM_PORT, deste, destp, e ) );	
	if (debug) { System.out.println("BasicKernel Sent event to input "+deste+"."+destp);}
        //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp1); 
    }



    /** This is not in Kernel interface yet... should it be? - sends to port 0 */
    public final void bcastEvent( int handle, Population p, PopulationQueryEvent pqe ) {//EntityID handle
	// Set up population callback handler
	pqe.setQid(getQid());
	pqe.setProc(getRank());
 
	PopulationCallbackHandler cbh = new PopulationCallbackHandler( pqe.getQid(), getRank(), pqe.getTime(), 
								       p.getNumEnts(), this,
								       handle, PortID.SYSTEM_PORT, pqe.getCallback() );
        //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(cbh,ecp);
	callbackQueue.add(cbh);
        //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
	cbh.start();
	
	// For all members of p, sendEvent...
	// Basic algorithm: send event to each member of the population individually
	// A better approach would check which processors the population sits on
	// and send a SysEntEvent to each of those processors.
	for (int i=0; i<p.getNumEnts(); i++) {	    
	    sendEvent( handle, PortID.SYSTEM_PORT, p.getEntityID(i),  PortID.SYSTEM_PORT, pqe );
	}
    }


    // Deal with entity queries/inits/updates.
    public final void defaultEventHandler( Entity ent, Event e ) { 
	if (e.isEntityEvent()) {        
	    ((EntityEvent)e).doOnEntity( ent );
	} else {
	    System.out.println("default event handler called for "+ent.getName()+" but don't know what to do with it");
	}
    }

    // Deal with construct/connect commands	
    public final void systemEventHandler( Entity ent, Event e ) {
	if (e.isSystemEvent()) {
	    SystemEntity syse = SimSystem.getSysEntity();
	    ((SysEntEvent)e).doOnSysEntity( syse );
	}
    }
 
    /**  Returns the top level population tree.
     * @param t the time of the request (needed as the population tree
     * changes with time)
     * @return the top level population tree
     *
     */
    public final PopulationTree getPopulationTree(Time t) {
	return popTree;
    }

    /** Returns the set of user defined classes.
     * @return the set of user classes.
     *
     */
    public final NeosimClasses  getNeosimClasses() {
	return classes;
    }
 
    /** Looks up an EntityClass object given a name,
     * e.g. "CompartmentalEntity"
     * @param name The fully qualified name of the class
     * @return the entity class, if found, or null if not.
     */
    public final EntityClass    lookupEntityClass( String name ) {
	return (EntityClass)classes.getClassFromName(name);
    }

    public final int   lookupEntityID( Time t, String name ) {//return EntityID
	/*** The time is not yet included in the pop tree ***/
	return popTree.getEntityID(name);
    }

    public final String   lookupEntityName( Time t, int eid ) {//EntityID eid
	/*** The time is not yet included in the pop tree ***/
	return "lookupEntityName not yet implemented"; 
	// should be popTree.getEntityName(eid);
    }

    /** Population names are separated by "." or "/"
     */
    public final Population     lookupPopulation( Time t, String name ) {
	Vector popNames = splitString( name );
	PopulationTree pt = popTree;
	PopulationNode pn = null;
	Population ret = null;
	for (int level=0; level<popNames.size(); level++) {
	    pn = pt.getChild( (String)popNames.elementAt(level) );
	    if (pn!=null) {
		if (!pn.isLeaf()) {
		    pt = (PopulationTree)pn;
		} else if (level==popNames.size()-1) {
		    ret = (Population)pn;
		}
	    }
	}
	return ret;
    }
 
    /** Split a full population string into a vector 
     */
    final Vector splitString(String s) {
	Vector labels = new Vector();
	StringTokenizer st = new StringTokenizer(s, "./");
	while (st.hasMoreTokens())
	    labels.addElement(st.nextToken());  
	return labels;
    }

    /** Build a population at a given time 
     * @param e the originating entity
     * @param t the time to build the population (<b> must be >= localTime() + getMinOutputDelay()</b>)
     * @param p the population to build
     * @param c the callback to call when the population has been built.
     */
    public final void   construct ( int e, Time t, PopulationBuilder p, Callback c) {//EntityID e
	if (debug)	System.out.println("BasicKernel::construct population");		
	sendEvent( e, PortID.SYSTEM_PORT, new ConstructEvent(  t, e, p, c ) );
	if (debug)	System.out.println("BasicKernel::construct population done");
    }

    /** Destroy a population at a given time. Calls every member's destroy() method.
     * @param e the originating entity
     * @param t the time to build the population (<b> must be >= localTime() + getMinOutputDelay()</b>)
     * @param p the population to destroy
     * @param c the callback to call when the population has been destroyed	
     */
    public final void  destroy ( int e, Time t, Population p, Callback c ) {//EntityID e
	sendEvent( e, PortID.SYSTEM_PORT, new DestroyEvent(  t, e, p, c ) );	
    }

    /** Make connections at a given time
     * @param t the time to make the connections (<b> must be >= localTime() + getMinOutputDelay()</b>)
     * @param p the projection to make
     * @param c the callback to call when the connections have been made
     */
    public final void  connect ( int e, Time t, Projection p, Callback c ) {//EntityID e
	sendEvent( e, PortID.SYSTEM_PORT, new ConnectEvent(  t, e, p, c ) );	
    }

    /** Delete connections at a given time
     * @param t the time to delete the connections (<b> must be >= localTime() + getMinOutputDelay()</b>)
     * @param p the population to destroy
     * @param c the callback to call when the population has been destroyed	
     */
    public final void  disconnect ( int e, Time t, Projection p, Callback c ) {//EntityID e
	sendEvent( e, PortID.SYSTEM_PORT, new DisconnectEvent(  t, e, p, c ) );	
    }

    /** Update another entity at a given time
     * @param t the time to update the entity (<b> must be >= localTime() + getMinOutputDelay()</b>)
     * @param e the handle of the entity to update
     * @param eu the update to perform	
     * @param c the callback to call when the update has been done
     */
    public final void  updateEntity( int e,Time t, int de, EntityUpdate eu, Callback c ) {//EntityID e,//EntityID de
	//EntityUpdateEvent ue = new EntityUpdateEvent( t, de, PortID.SYSTEM_PORT, eu, c );
	EntityUpdateEvent ue = new EntityUpdateEvent( t, e, PortID.SYSTEM_PORT, eu, c );
	sendEvent( e, PortID.SYSTEM_PORT, de, PortID.SYSTEM_PORT, ue );
    }

    /** Query another entity at a given time
     * @param t the time to query the entity (<b> must be >= localTime() + getMinOutputDelay()</b>)
     * @param e the handle of the entity to query
     * @param eq the query to perform	
     * @param c the callback to call with the results of the query
     */
    public final void  queryEntity( int e, Time t, int de, EntityQuery eq, Callback c ) {//EntityID e,//EntityID de
	//	EntityQueryEvent qe = new EntityQueryEvent( t, de, PortID.SYSTEM_PORT, eq, c );
	EntityQueryEvent qe = new EntityQueryEvent( t, e, PortID.SYSTEM_PORT, eq, c );
	//	System.out.println("Sending query to entity "+de.id);
	sendEvent( e, PortID.SYSTEM_PORT, de, PortID.SYSTEM_PORT, qe );
    }
    
    /** Update a population of entities at a given time
     * @param t the time to update the entity (<b> must be >= localTime() + getMinOutputDelay()</b>)
     * @param p the population to update
     * @param eu the update to perform	
     * @param c the callback to call when the update has been done
     */
    public final void  updatePopulation( int e, Time t, Population p, EntityUpdate eu, Callback c ) {//EntityID e
	PopulationUpdateEvent pue = new PopulationUpdateEvent( t, e, PortID.SYSTEM_PORT, p, eu, c );
	bcastEvent( e,  p, pue );
    }

    /** Query a population of entities at a given time
     * @param t the time to query the population(<b> must be >= localTime() + getMinOutputDelay()</b>)
     * @param p the population to query
     * @param eq the query to perform	
     * @param c the callback to call with the results of the query. The results will
     * be in the form of a vector of objects, indexed by entties' index in the population.
     */
    public final void  queryPopulation( int e, Time t, Population p, EntityQuery eq, Callback c ) {////EntityID e
	PopulationQueryEvent pqe = new PopulationQueryEvent( t, e, PortID.SYSTEM_PORT, p, eq, c );
	bcastEvent( e, p, pqe );
    }


    /** Query output connections of a port.
     * @param oe the originating entity
     * @param t the time to query the connections(<b> must be >= localTime() + getMinOutputDelay()</b>)
     * @param srce the entity handle of the src entity to query
     * @param srcp the src output port
     * @param cq the query to run on each output connection
     * @param c the callback to call with the results of the query. The results will
     * be in the form of a vector of objects, ordered by entity handle/input connection id.
     */
    public final void queryOutputConnections( int oe, Time t, int srce, short srcp, ConnectionQuery cq, Callback cb ) {//EntityID oe,EntityID srce//PortID srcp
	ConnectionQueryEvent cqe = new ConnectionQueryEvent( t, oe, PortID.SYSTEM_PORT, cq, srce, srcp, cb );

	EntityImpl ne = getLocalEntity( srce );
	if (ne!=null) {
	    OutPortTable opt = ne.getOutConnection( srcp );
	    
	    // --- Get number of responses expected, set up handler.
	    int nresp = opt.numConnections();
	    int qid = getQid();
	    
	    ConnectionCallbackHandler cbh = new ConnectionCallbackHandler( qid, getRank(), t, nresp, this, oe, PortID.SYSTEM_PORT, cb );
            
            //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
            //neosim.kernel.DeadlockTracker.LockDeadlockTracker(cbh,ecp);
	    callbackQueue.add(cbh);
            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
	    cbh.start();
	    cqe.setQid(qid);
	    cqe.setProc(getRank());

	    // --- Send commands to all relevant processors
	    Vector ports = opt.getPorts();
	    for (int i=0; i<ports.size(); i++) {
		OutPort op = (OutPort)ports.elementAt(i);
		int proc = op.getProc();
		sendCommand( proc, new ConnectionQueryCommand( getRank(), oe, PortID.SYSTEM_PORT, cqe ) );
	    }
	}	
    }


    /** Return the number of input connections on a port
     * @param inport the input port id
     * @return the number of connections
     */
    public final int  getNumInConnections(int e, short inport) {//EntityID e// PortID inport
        
        EntityImpl ne = getLocalEntity( e );
        return ne.getNumInConnections(inport);
        /*Vector v = ne.getInConnections(inport);
	//Vector v = (Vector)(inConnections.get( InPortTable.getHashKey( e, inport )));//JAG030205
	if (v==null) return 0; else return v.size();*/
    }

    /** Return the input connections to a port
     * @param outport the output port id
     * @return  the number of connections
     */
    public final Vector getInConnections(int e, short inport) {//EntityID e//PortID inport
	//	System.out.println("Getting in connections from "+e.id+" . "+inport.id);
	//Object hk = InPortTable.getHashKey( e, inport ); //JAG030205
	//	System.out.println("Got hash "+hk);
        
	//Object ret = inConnections.get( hk );
	//	System.out.println("Got obj "+ret);
	EntityImpl ne = getLocalEntity( e );
        if(ne==null)
            return null;
        //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(ne,ecp);         
        int array[][]= ne.getInConnections(inport);
        //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
        Vector ret= inports.fillVector(e, inport, array, ne);
        
        
        if(ret==null)
            return null;        
	if (ret!= null) return ret; else return null;
    }
    
    /** Add an input connection from remote processor
     */
//    boolean test_addInConnection=true;
  //  static public int nbconns=0;
    public final synchronized int addInConnection( int rmtproc, Connection c, boolean BuildingConnection ) {
   //     nbconns++;
        CompressedBeforeSimulation=true;
        /*if(test_addInConnection){
            System.out.println("**** test_addInConnection  USE "+ Runtime.getRuntime().totalMemory()+" bytes ****");
            test_addInConnection=false;
        }*/
	// Add it to the dest entity
	int e = c.getDstEntityID();//EntityID e
	EntityImpl ne = getLocalEntity(e);
        int cid ;//ConnectionID
	if(BuildingConnection) 
        {
                //neosim.kernel.Deadlock_Message ecp2=new neosim.kernel.Deadlock_Message();
                //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp2);   
                if(neosim.util.SmallLists.LinkedList.checkCompression(inports)){
                    int k=getNumEnts();
		    int i=0;
	 	    while(i<k){
                        Entity nlocal = SimSystem.getEntity(i);
                        if(nlocal instanceof EntityImpl){
                            //neosim.kernel.Deadlock_Message ecp3=new neosim.kernel.Deadlock_Message();
                            //neosim.kernel.DeadlockTracker.LockDeadlockTracker(nlocal,ecp3);                             
                            ((EntityImpl)nlocal).compress();
                            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp3);
                        }
                        i++;
                    }
                }
                //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp2);
            
                cid = ne.addInConnection( c );
            // Add it to the input ports table.
                /*InPortTable ipt = new InPortTable( c.getSrcEntityID(), c.getSrcPortID(), rmtproc );//new PortID()
                ipt.addConnection( c );
                InPortsTable ipts = getInports(rmtproc);
                ipts.add( ipt );*/
                //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
                //neosim.kernel.DeadlockTracker.LockDeadlockTracker(c,ecp);                
                inports.addInputConnection(c.getSrcPortID(), c.getDstPortID(),c.getSrcEntityID(),e,c.getDelay(),cid);
                //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
	// Add it to the inConnections hash table
            /*Vector v = getInConnections( c.getSrcEntityID(), c.getSrcPortID() );//new PortID(

	//	System.out.println("Got connections vector ok");

            if (v!=null) { v.addElement( c ); } else {//JAG030205
                v = new Vector(); 
                v.addElement(c);
                inConnections.put( InPortTable.getHashKey( c.getSrcEntityID(), c.getSrcPortID() ), v );//new PortID(
            }*/
                return cid;
       }
        else{
            System.out.println("neosim.kernel.BasicKernel.addInConnection : should clean the connection in the entityImpl -JAG");
            //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
            //neosim.kernel.DeadlockTracker.LockDeadlockTracker(c,ecp);              
            inports.removeConnection(c.getSrcEntityID(), e, c.getSrcPortID(), c.getDstPortID());
            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);            
           /* InPortsTable ipts = getInports(rmtproc);
            InPortTable ipt=ipts.getport(c.getSrcEntityID(),c.getSrcPortID());//new PortID(
            Connection c2=ipt.removeConnection(c);*/
            //Vector v = getInConnections( c.getSrcEntityID(), c.getSrcPortID() );//new PortID(
            
            //if (v!=null) { v.remove( c2 ); }
            //ne.deleteInConnection(c2.getConnectionID());
            ne.deleteInConnection(c); 
            return c.getConnectionID();
            
        }
	//	System.out.println("Done addInConnection");
    }

    public final void trace(String msg) {
        //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(msg,ecp);  
	SimSystem.rawtrace(msg);
        //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
    } 
    
    /**RMI TESTS ONLY*/    
    byte[][] ByteArray=new byte[4][];
    int[][] IntArray=new int[4][];
    
    public void startTests() throws java.rmi.RemoteException{
        ByteArray[0]=new byte[10];
        ByteArray[1]=new byte[100];
        ByteArray[2]=new byte[1000];
        ByteArray[3]=new byte[10000];

        IntArray[0]=new int[10];
        IntArray[1]=new int[100];
        IntArray[2]=new int[1000];
        IntArray[3]=new int[10000];
    }
    public void call() throws java.rmi.RemoteException{
        //Holger_modules.module.I_ClampImpl.I_ClampImpl obj=new Holger_modules.module.I_ClampImpl.I_ClampImpl();
        Holger_modules.module.destmethod.SpikeDestMethod obj=new Holger_modules.module.destmethod.SpikeDestMethod();
    }  
    public void sendByte(byte[] array) throws java.rmi.RemoteException{
    }
    public byte[] receiveByte(byte size) throws java.rmi.RemoteException{
      return ByteArray[size];
    }
    public void sendInt(int[] array) throws java.rmi.RemoteException{
      
    }
    public int[] receiveInt(byte size) throws java.rmi.RemoteException{
      return IntArray[size];
    }
    public long sum(int[] array) throws java.rmi.RemoteException{
      short length=(short)array.length;
      long result=0;
      for(short i=0;i<length;i++)
        result+=array[i];
      return result;
    }
    public long sum(byte[] array) throws java.rmi.RemoteException{
      short length=(short)array.length;
      long result=0;
      for(short i=0;i<length;i++)
        result+=array[i];
      return result;
    }      
    
    protected SpikeArray arraySpikes; 
    protected Vector AllArraySpikes;
    /** Add a spike to the spike list
     */
    public void addSpike(int handle, short p, long t){
      //  System.out.println("BK:addSpike "+handle+" "+t+" "+p);
        arraySpikes.addSpike(handle, t, p);
    }
    
     /** "Working" Implementation. Need a better implementation for parallel kernels 
     */
    boolean exchangeSpikes_debug=true;
    protected final void proceedExchangeSpikes(){
        byte buffer[]=new byte[14];
        if(exchangeSpikes_debug){
            System.out.println("**** exchangeSpikes  USE "+ Runtime.getRuntime().totalMemory()+" bytes, nbconns = "+InportsTableOut.nbconn +"****");   
            exchangeSpikes_debug=false;
        }
        
        int nb_kernels=AllArraySpikes.size();
        SpikeArray temp=null;
        int current_portable;
        for(short i=0;i<nb_kernels;i++){
            try{
                temp=(SpikeArray)AllArraySpikes.elementAt(i);
            }catch(Exception except){
                if(temp!=null)
                    System.out.println("SpikeArray_Exception "+temp.getClass().getName());
                else
                    System.out.println("SpikeArray_Exception: temp==null ");
                except.printStackTrace();
            }            
            while((buffer=temp.readSpike())!=null){
                
                short port=(short)buffer[13];
                port<<=8;
                port|=0xFF&(short)buffer[12];
                
                int handle=(int)buffer[11];
                handle<<=8;
                handle|=0xFF&(int)buffer[10];                
                handle<<=8;
                handle|=0xFF&(int)buffer[9];                
                handle<<=8;
                handle|=0xFF&(int)buffer[8];                                

                long time=(long)buffer[7];
                time<<=8;
                time|=0xFF&(long)buffer[6];                
                time<<=8;
                time|=0xFF&(long)buffer[5]; 
                time<<=8;
                time|=0xFF&(long)buffer[4]; 
                time<<=8;
                time|=0xFF&(long)buffer[3]; 
                time<<=8;
                time|=0xFF&(long)buffer[2]; 
                time<<=8;
                time|=0xFF&(long)buffer[1]; 
                time<<=8;
                time|=0xFF&(long)buffer[0];    
               
                inports.addSpike(port,handle,time);
            }
            temp.ClearSpike();
        }
        arraySpikes.ClearSpike();
       // AllMeasureMemory();             
    }
    public void exchangeSpikes(){
        try{
            AllArraySpikes=allgather(arraySpikes);
        }catch(ResetException rsexcept){
            System.out.println("exchangeSpikes_allgather_ResetException");
            rsexcept.printStackTrace();
        }
        proceedExchangeSpikes();
    }
    
    final private void FlushMemory(){
      try{
        System.gc();
        Thread.currentThread().sleep(100);
        System.runFinalization();
        Thread.currentThread().sleep(100);
        System.gc();
        Thread.currentThread().sleep(100);
        System.runFinalization();
        Thread.currentThread().sleep(100);     
      }catch(Exception e){
          e.printStackTrace();
      }        
    }
    /**Used to cehck the amount of memory taken by the stuff cleared*/
    final private Long MeasureMemory(String str,Long lng){       
      FlushMemory();
      long totalNewMemory = Runtime.getRuntime().totalMemory();        
      
      System.out.println(str+": Freed Memory "+(lng.longValue()-totalNewMemory));
      return new Long(totalNewMemory);

      //System.exit(-1);
    }
    
    final private void AllMeasureMemory(){
         FlushMemory();
         Long lng=new Long(Runtime.getRuntime().totalMemory());
         this.AllArraySpikes=null;
         lng=MeasureMemory("AllArraySpikes",lng);
         this.allprocs=null;
         lng=MeasureMemory("allprocs",lng);           
 	 this.arraySpikes=null;
         lng=MeasureMemory("arraySpikes",lng); 
         this.asem=null;
         lng=MeasureMemory("asem",lng);                     
         this.bsem=null;
         lng=MeasureMemory("bsem",lng);                     
         this.ByteArray=null;
         lng=MeasureMemory("ByteArray",lng);
         this.callbackQueue=null;
         lng=MeasureMemory("callbackQueue",lng);
         this.classes=null;
         lng=MeasureMemory("classes",lng);
	 //MeasureMemory(this.cq,"cq",lng);
         this.entityLocation=null;
         lng=MeasureMemory("entityLocation",lng);
         this.inports=null;
         lng=MeasureMemory("inports",lng);
	 this.inpQueues=null;
         lng=MeasureMemory("inpQueues",lng);
	 this.IntArray=null;
         lng=MeasureMemory("IntArray",lng);
         this.othercqs=null;
         lng=MeasureMemory("othercqs",lng);//Thread
         this.otherprocs=null;
         lng=MeasureMemory("otherprocs",lng);//Thread
         this.popTree=null;
         lng=MeasureMemory("popTree",lng);
         this.rflag=null;
         lng=MeasureMemory("rflag",lng);
         this.ssem=null;
	 lng=MeasureMemory("rflag",lng);
         this.wflag=null;
         lng=MeasureMemory("wflag",lng);
         this.workq=null;
         MeasureMemory("workq",lng); //Thread        
/* this.inports=null;
        Vector v=this.popTree.getChildren();
        for(int i=0;i<v.size();i++){
            Vector v2=((PopulationImpl)v.elementAt(i)).getLocalMembers();
            for(int j=0;j<v2.size();j++){
                ((EntityImpl)v2.elementAt(j)).deleteInConnection(j+1);
            }
        }*/
        System.exit(-1);        
    }
    
    public synchronized void SendArray(Object[] Array)  throws java.rmi.RemoteException {
        System.out.println("BesicKernel.SendArray should not be called");
    }
    
    public void signal() throws java.rmi.RemoteException {
    }
    
    public void SendSpikesArray(byte[] Spikes) throws java.rmi.RemoteException {
    }
    
}