/* CallbackQueue.java
 */

package neosim.kernel;

import neosim.iface.*;

import java.util.Vector;
import java.util.Enumeration;

/**
 * This class implements a queue of <em>CallbackHandlers</em>. 
 * It works like a normal FIFO
 * queue, but during insertion entities are kept in order from the 
 * smallest left window time
 * stamp to the largest. This means the next entity to be activated
 * will be at the top of the queue. <P>
 * @see		neosim.SimSystem
 * @version     0.1, 1 July 1998
 * @author      Fred Howell
 */

public class CallbackQueue extends Vector {

  /**
   * Allocates a new CallbackQueue object.
   */
  public CallbackQueue() {
    super();
  }

  /**
   * Remove and return the event at the top of the queue.
   * @return           The next event.
   */
  public final synchronized CallbackHandler pop() {
    if (isEmpty()) return null; 
    else {
      CallbackHandler ent = (CallbackHandler)firstElement();
      removeElementAt(0);
      return ent;
    }
  }

    /**
     * Return the event at the top of the queue, without removing it.
     * @return	The next event.
     */
    public final CallbackHandler top() {
	return (CallbackHandler)firstElement();
    }
    
    /**
     * Return the minimum time.
     * @return	The min callback time.
     *//*
    public final synchronized Time getMinTime() {
	Time min = new Time(); min.t=-1;
	Enumeration e;
	CallbackHandler c;
	for(e = elements(); e.hasMoreElements();) {
	    c = (CallbackHandler)e.nextElement();
	    if (min.t == -1) { min = c.getTime(); }
	    else if (c.getTime().t < min.t) {
		min = c.getTime();
	    }
	}
	return min;
    }
    */
    /**
     * Remove an element given a qid
     */
    public final synchronized void removeCallback( int qid ) {
	Enumeration e;
	CallbackHandler c;
	
	int i = -1;
	for (e = elements(); e.hasMoreElements() && (i == -1);) {
	    c = (CallbackHandler)e.nextElement();
	    if(c.getQid() == qid)
		i = indexOf(c);
	}	
	if (i!=-1) removeElementAt(i);
    }        

    /**
     * Add a new callback handler to the queue, preserving the temporal order of the
     * times.
     * @param h	The callback handler to be put on the queue.
     */
    public final synchronized void add(CallbackHandler h) {
	int i;
	Enumeration e;
	CallbackHandler c;
	
	i = -1;
	for(e = elements(); e.hasMoreElements() && (i == -1);) {
	    c = (CallbackHandler)e.nextElement();
	    if(c.getTime().t > h.getTime().t)
		i = indexOf(c);
	}
	
	if (i == -1)
	    addElement((Object)h);
	else
	    insertElementAt((Object)h, i);
    }
}
