package neosim.kernel;

/** The super class for all commands
 */
public class CommandObj implements java.io.Serializable {
    /** id of sender */
    protected int id;	

    /** whether this command blocks */
    protected boolean blocking = false;

    /** Build a command object giving id of sending processor */
    public CommandObj(int id) {
	this.id = id; 
    }
    public CommandObj() {

    }
    public final void setID(int id) { this.id=id; }    
    /** Return the id of sending processor 
     * @return the id of sending processor 
     */
    public final int getID() { return id; }

    /** Return the type of this command (useful for debugging)
     * @return the type of this command
     */
    public String getType() { return "Basic"; }

    /** Return whether this command blocks the calling thread
     * @return whether this command blocks the calling thread
     */
    public final boolean isBlocking() { return blocking; }

    /** Set this command to block / not to
     * @param b  whether this command blocks the calling thread
     */
    public final void setBlocking(boolean b) { blocking = b; }

    /** Run this command 
     * @param hp the kernel to run the command on
     */
    public void doit(SimKernel hp) {
	System.out.println("Executing a basic command\n");
    }

    /** Run a query
     * @param hp the kernel to run the query on
     */
    public final Object doquery(SimKernel hp) {
	System.out.println("Executing a query command\n");
	return "results";
    }
    
    public boolean isDelayed() { return false; }
    
    
    //Essential to assess the size of the event queue in order to flush it (otherwise you might reach the maximal amount of memory available)
    public int getSize(){
        return 15;
    }
}

