package neosim.kernel;
import neosim.iface.*;

/** A DestroyEvent contains a doOnSysEntity() method which is executed
 * by the default event handler when an event arrives.
 * It deletes a population
 */
public class DestroyEvent extends EntityEvent {
    Population p;
    Callback c;

    public DestroyEvent(Time t, int srce, Population p, Callback c ) { //EntityID srce
	super( t, srce, PortID.SYSTEM_PORT );
	this.p = p;
	this.c = c;
    }

    /** Destroy local portion of population */
    public final void doOnEntity( Entity e ) {
	// ...
	e.sendEvent(PortID.SYSTEM_PORT, new EntityCallbackEvent( getTime(), EntityID.SYSTEM_ENTITY, 
								  PortID.SYSTEM_PORT,c, null ));
    }

    public final ClassID registerClass(NeosimClasses nc) { 
	cid = nc.registerClass("DestroyEvent", "EntityEvent", "neosim.kernel"); return cid; 
    }
    
    public void readExternal(java.io.ObjectInput objectInput) throws java.io.IOException, java.lang.ClassNotFoundException {
        super.readExternal(objectInput);
        p=(Population)objectInput.readObject();
        c=(Callback)objectInput.readObject();
    }
    
    public void writeExternal(java.io.ObjectOutput objectOutput) throws java.io.IOException {
        super.writeExternal(objectOutput);
        objectOutput.writeObject(p);
        objectOutput.writeObject(c);
    }   
    
    public int getSize(){
        return 120;
    }     
}
