package neosim.kernel;

import neosim.iface.*;
import java.util.Hashtable;
import java.util.Enumeration;
import neosim.util.SmallLists.*;
import neosim.implementation.*;


/** A table of input connections from a particular processor
 * Hash value is eid + cid * 10000
 * Note this only allows 10000 output connections from an entity.
 */
public final class InPortsTable extends LinkedList {
    InPortsTable next;
    LinkedListSrcPort_In Array[];
    BasicKernel sk;
    
    public InPortsTable(BasicKernel sk) {
        clear();
        this.sk=sk;
        Array=new LinkedListSrcPort_In[this.ListSize];
        LinkedList.TotalMemoryLinkedLists+=this.getSize();   
        
    }
    
  /*  public static void main(String[] args) {
        long a[]=new long[25];
        int b[]=new int[25];
        int c[]=new int[25];
        
        for(int i=0;i<25;i++){
            a[i]=5222-i;
            b[i]=5222-i;
            c[i]=5222-i;
        }
        LinkedListConnection lc=new LinkedListConnection(0);
        lc.heapsort(a,b,c,0,24);
    }*/
    
    public void compress() {
        for(int i=0;i<RealIndex_read;i++){
            if(Array[i]!=null){
                //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
                //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp); 
                Array[i].compress();
                //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
            }
        }
        if(next!=null)
            next.compress();        
    }
    
    public void compress_final() {
        
        InPortsTable stuff=this;
        short MaxPortIn=-1;
        short current;
        int TotalLength=0;
        do{
            for(int i=0;i<stuff.RealIndex_read;i++){
                LinkedListSrcPort_In muche=stuff.Array[i];
                if(muche!=null){
                    current=muche.getID();
                    if(current>MaxPortIn)
                        MaxPortIn=current;
                }
            }
            TotalLength+=stuff.getSize();
            stuff=stuff.next;
        }while(stuff!=null);
        
        LinkedListSrcPort_In[] NewArray=new LinkedListSrcPort_In[MaxPortIn+1];
        
        stuff=this;
        do{
            for(int i=0;i<stuff.RealIndex_read;i++){
                LinkedListSrcPort_In muche=stuff.Array[i];
                if(muche!=null){                
                    NewArray[muche.getID()]=muche;
                }
            }
            stuff=stuff.next;
        }while(stuff!=null);
        
        
        Array=NewArray;
        this.ListSize=MaxPortIn+1;
        LinkedList.TotalMemoryLinkedLists=getSize()+LinkedList.TotalMemoryLinkedLists-TotalLength;
        next=null;
        this.RealIndex_read=ListSize;

        for(int i=0;i<RealIndex_read;i++){
            if(Array[i]!=null)
                Array[i].compress_final();
        }   
        
        int nbents=sk.getNumEnts();
        for(int i=0;i<nbents;i++){
            EntityImpl truc=(EntityImpl)SimSystem.getEntity(new Integer(i));
            if(truc!=null){
                truc.compress_final();
            }
        }
    }
    
    public int getSize() {
        return 28+Array.length<<2;
    }
    
    public synchronized void addInputConnection(short srcPort,short destPort, int srcID, int destID, long delay, int cid){
          for(int i=0;i<RealIndex_read;i++){
              if(Array[i]!=null&&Array[i].getID()==destPort){
//                  System.out.println("Add "+srcPort+" "+destPort+" "+srcID+" "+destID+" "+delay+" "+cid);
                  Array[i].addInputConnection(destPort,srcID, destID, delay, cid);
                  return;
              }
          }
          if(RealIndex_read==this.ListSize){
              if(next==null)
                next=new InPortsTable(sk);
              next.addInputConnection(srcPort,destPort, srcID, destID, delay, cid); 
          }
          else{
              Array[RealIndex_read]=new LinkedListSrcPort_In(destPort);
              Array[RealIndex_read++].addInputConnection(destPort,srcID, destID, delay, cid);
          }        
    }
    
    public void sendEvent(short SrcPortID, int SrcID, Event ev){
        LinkedListSrcPort_In trucSrcPort=Array[SrcPortID];
        if(trucSrcPort==null)
            return;
        int length=trucSrcPort.Array.length;
        for(int i=0;i<length;i++){
            LinkedListDestPort_In trucDestPort=trucSrcPort.Array[i];
            if(trucDestPort!=null){
                //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
                //neosim.kernel.DeadlockTracker.LockDeadlockTracker(ev,ecp);                 
                trucDestPort.sendEvent(SrcPortID,SrcID,sk,ev);
                //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
            }
        }
    }
    
    public void addSpike(short SrcPortID, int SrcID, long time){
        LinkedListSrcPort_In trucSrcPort=Array[SrcPortID];
        if(trucSrcPort==null)
            return;
        int length=trucSrcPort.Array.length;
        for(int i=0;i<length;i++){
            LinkedListDestPort_In trucDestPort=trucSrcPort.Array[i];
            if(trucDestPort!=null)
                trucDestPort.addSpike(SrcPortID,SrcID,sk,time);
        }
    }    
    
    public final void connectionQueryRmt(int rid, int srce, short pid, ConnectionQueryEvent ev) {
      LinkedListSrcPort_In trucSrcPort=Array[ev.getOutputPort()];//pid
        if(trucSrcPort==null)
            return;
        int length=trucSrcPort.Array.length;
        for(int i=0;i<length;i++){
            LinkedListDestPort_In trucDestPort=trucSrcPort.Array[i];
            if(trucDestPort!=null)
                trucDestPort.connectionQueryRmt(rid,srce,pid,sk,ev);
        }        
    }
    
    public final java.util.Vector fillVector(int destID, short srcPortID, int[][] TempArray, EntityImpl destent){
        java.util.Vector ret=new java.util.Vector();

        for(int i=0;i<TempArray.length;i++){
            if(TempArray[i]!=null){//i=destPortID
                for(int j=0;j<TempArray[i].length;j++){
                    
                    LinkedListSrcPort_In chose=Array[srcPortID];
                    LinkedListDestPort_In stuff=chose.Array[i];
                    stuff.fillVector(TempArray[i][j],destID,srcPortID,ret,destent);
                }
            }
        }
        return ret;
    }
    
    public final void removeConnection(int srcID, int destID, short pid, short destpid) {
        LinkedListSrcPort_In trucSrcPort=Array[pid];
        if(trucSrcPort==null)
            return;
        LinkedListDestPort_In trucDestPort=trucSrcPort.Array[destpid];
        if(trucDestPort!=null)
                trucDestPort.removeConnection(srcID,destID);
    }
   
                    
   /* public final void add(InPortTable ip) {
	// If InPortTable already there, add synapses of this ip to the existing one.
	InPortTable oldp = (InPortTable)ht.get(ip.getHashKey());
	if (oldp!=null) {
	    // Add ip's synapses to oldp.
	    oldp.merge(ip);
	} else {
	    //	    System.out.println("Adding port to inportstable...");
	    // ip.disp();
	    ht.put(ip.getHashKey(), ip);
	}
    }
    public final InPortTable getport(int eid, short pid) {//EntityID eid, PortID pid
	return (InPortTable)ht.get(InPortTable.getHashKey(eid,pid));
    }
    public final void disp() {
	SimSystem.ptrace(-1,"=== Input Ports from "+srcp+" ---");
	Enumeration e;
	InPortTable ip;
	for (e = ht.elements(); e.hasMoreElements();) {
	    ip = (InPortTable)e.nextElement();
	    ip.disp();
	}
    }*/
    
}





final class LinkedListDestPort_In extends LinkedList {
    LinkedListConnection Array[];
    LinkedListDestPort_In next=null;    
    short destPortID;
    int min=-1;

    /** Creates a new instance of LinkedListDestPort */
    
    public LinkedListDestPort_In(short destPortID){
        this.destPortID=destPortID;
        Array=new LinkedListConnection[this.ListSize];        
        LinkedList.TotalMemoryLinkedLists+=this.getSize();
    }
    
    public synchronized void compress(){
        for(int i=0;i<RealIndex_read;i++){
            if(Array[i]!=null){
                //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
                //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp); 
	        Array[i].compress();
                //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp); 
            }
        }
        if(next!=null)
            next.compress();       
    }
    
    public synchronized void addInputConnection(int srcID, int destID, long delay, int cid){
          for(int i=0;i<RealIndex_read;i++){
              if(Array[i]!=null&&Array[i].getID()==srcID){
                  Array[i].addInputConnection(destID, delay, cid);
                  return;
              }
          }
          if(RealIndex_read==this.ListSize){
              if(next==null)
                next=new LinkedListDestPort_In(destPortID);
              next.addInputConnection(srcID, destID, delay, cid); 
          }
          else{
              Array[RealIndex_read]=new LinkedListConnection(srcID);
              Array[RealIndex_read++].addInputConnection(destID, delay, cid);
          }         
    }

    public short getID(){
        return destPortID;
    } 
    
    public int getSize() {
        return 28+Array.length<<2; //(LinkedListDestPort_In=4)+(destPortID=2)+(Array=4)+(Array.size*(LinkedListConnection*=4)+(Overhead=10))+8 in superclass
    }
    
    public void compress_final() {
        int max=-2000000000,min=2000000000,tmp;
	int TotalLength=0;
        int TotalSize=0;
        LinkedListDestPort_In stuff=this;
        
        do{
            for(int i=0;i<stuff.RealIndex_read;i++){
                if(stuff.Array[i]!=null){
                    stuff.Array[i].compress_final();
                    tmp=stuff.Array[i].getID();
                    if(tmp<min)
                        min=tmp;
                    if(tmp>max)
                        max=tmp;
                }
            }
            TotalLength+=stuff.getSize();
            TotalSize+=stuff.RealIndex_read;
            stuff=stuff.next;
            
        }while(stuff!=null);
                
        if((TotalSize!=0)&&((max-min)/TotalSize<10)){//basically if more than 10% of the entities connect to the entities locally described in this kernel, for these sourceport and destport, we make an array. The 10% number is arbitrarily set.
            LinkedListConnection[] NewArray=new LinkedListConnection[max-min+1];
        
            stuff=this;
            do{
                for(int i=0;i<stuff.RealIndex_read;i++){
                    if(stuff.Array[i]!=null)
                        NewArray[stuff.Array[i].getID()-min]=stuff.Array[i];
                }
                stuff=stuff.next;
            }while(stuff!=null);
        
            Array=NewArray;
            this.min=min;
            this.ListSize=max-min+1;
        }
        else{
            
            LinkedListConnection[] NewArray=new LinkedListConnection[TotalSize];
            stuff=this;
            int j=0;
            do{
                for(int i=0;i<stuff.RealIndex_read;i++,j++){
                    NewArray[j]=stuff.Array[i];
                }
                stuff=stuff.next;
            }while(stuff!=null);
            Array=NewArray;
            //we sort the guys so they can be accessed quicker
            sort(Array,0,Array.length-1);
            min=-1;
        }
        LinkedList.TotalMemoryLinkedLists=getSize()+LinkedList.TotalMemoryLinkedLists-TotalLength;
        next=null;
        this.RealIndex_read=ListSize;
    }
    //dichotomic search of complecity in ln(Array.length)/ln(2);
    public LinkedListConnection find(int srcID){
        LinkedListConnection stuff;

        if(this.min<0){
            int left=0;
            int right=Array.length-1;
            
            if(srcID<Array[left].getID()||srcID>Array[right].getID())
                return null;//not there            
            
            int middle=(left+right)>>>1;
            int id;
            while(true){
                id=Array[middle].getID();
                if(id==srcID)
                    return Array[middle];
                else if(id<srcID)
                    right=middle;
                else//id>srcID
                    left=middle;
                middle=(left+right)>>>1;
            }
        }
        else{
            int index = srcID-min;
             if(index<0||index>=Array.length)
                 return null;//the neuron spikes on the port but is not connected to anything
             stuff=Array[index];        
             return stuff; //actually no neuron locally there or on that port     
        }
    }
    
    public void fillVector(int srcID, int destID, short srcPortID, java.util.Vector ret, EntityImpl destent){
        LinkedListConnection stuff=find(srcID);
        int[] TempArray=stuff.getInputConnection();
        int i=0;
        long delay;
        int cid;
        ConnectionImpl c;
        while(i<TempArray.length){
            if(TempArray[i+3]==destID){
                delay=TempArray[i++];
                delay<<=32;
                delay|=(long)(TempArray[i++]&0xFFFFFFFF);
                cid=TempArray[i++];

                c=new ConnectionImpl(srcID,srcPortID, destent, this.destPortID, delay); 
                c.setConnectionID(cid);
                ret.add(c);
                i++;
            }
            else i+=4;
        }
    }

    public void  sendEvent(short srcportID, int srcID, BasicKernel sk,Event ev){
        LinkedListConnection stuff=null;

        if(this.min<0){
            int left=0;
            int right=Array.length-1;
            
            if(srcID<Array[left].getID()||srcID>Array[right].getID())
                return;//not there            
            
            int middle=(left+right)>>>1;
            int id;
            while(stuff==null){
                id=Array[middle].getID();
                if(id==srcID){
                    stuff=Array[middle];
                }
                else if(id<srcID)
                    right=middle;
                else//id>srcID
                    left=middle;
                middle=(left+right)>>>1;
            }
        }
        else{
            int index = srcID-min;
             if(index<0||index>=Array.length)
                 return;//the neuron spikes on the port but is not connected to anything
             stuff=Array[index];        
             if(stuff==null)
                 return; //actually no neuron locally there or on that port
        }
        //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(ev,ecp);         
        int[] Connections=stuff.getInputConnection();
        //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
        int length=Connections.length;
        int i=0;
        int destCid;
        int destID;
        long delay;
        while(i<length){
            delay=Connections[i++];
            delay<<=32;
            delay|=(long)(Connections[i++]&0xFFFFFFFF);
            destCid=Connections[i++];
            destID=Connections[i++];            
            
            neosim.iface.Event newe = (neosim.iface.Event)ev.clone();
            Time arrivalTime = new Time(); 
            arrivalTime.t =  ev.getTime().t + delay;
            newe.setTime( arrivalTime );
            neosim.implementation.EntityImpl destent=(EntityImpl)SimSystem.getEntity(new Integer(destID));
            ConnectionImpl impl=new ConnectionImpl(srcID, srcportID, destent, this.destPortID, delay );
            impl.setConnectionID(destCid);
            newe.setConnection(impl);
            //neosim.kernel.Deadlock_Message ecp2=new neosim.kernel.Deadlock_Message();
            //neosim.kernel.DeadlockTracker.LockDeadlockTracker(ev,ecp2);             
            destent.postEvent(newe);    
            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp2);            
        }
    }   
    
    public void  addSpike(short srcportID, int srcID, BasicKernel sk, long time){
        LinkedListConnection stuff=null;

        if(this.min<0){
            int left=0;
            int right=Array.length-1;
           
            if(srcID<Array[left].getID()||srcID>Array[right].getID())
                return;//not there            
           
            int middle=(left+right)>>>1;
            int id;
            while(stuff==null){
                id=Array[middle].getID();
                if(id==srcID){
                    stuff=Array[middle];
                }
                else if(id<srcID)
                    right=middle;
                else//id>srcID
                    left=middle;
                middle=(left+right)>>>1;
            }
        }
        else{
            int index = srcID-min;
             if(index<0||index>=Array.length)
                 return;//the neuron spikes on the port but is not connected to anything
             stuff=Array[index];        
             if(stuff==null)
                 return; //actually no neuron locally there or on that port
             //do your 
        }
        int[] Connections=stuff.getInputConnection();
        int length=Connections.length;
        int i=0;
        int destCid;
        int destID;
        long delay;
        while(i<length){
            

            delay=Connections[i++];
            delay<<=32;
            delay|=(long)(Connections[i++]&0xFFFFFFFF);
            delay+=time;
            destCid=Connections[i++];
            destID=Connections[i++];
            
            neosim.implementation.EntityImpl destent=(EntityImpl)SimSystem.getEntity(new Integer(destID));
            //18/04/05->
            //destent.addSpike(destCid, delay);
            //<-18/04/05
            destent.addSpike(srcID, delay);
            
        }
    }        
    
    
   void sort(LinkedListConnection a[], int lo0, int hi0) { 
        int lo = lo0;
        int hi = hi0;
        if (lo >= hi) {
            return;
        }
        int mid = a[(lo + hi) >>>1].getID();
        while (lo < hi) {
            while (lo<hi && a[lo].getID() < mid) {
                lo++;
            }
            while (lo<hi && a[hi].getID() >= mid) {
                hi--;
            }
            if (lo < hi) {
                LinkedListConnection T = a[lo];
                a[lo] = a[hi];
                a[hi] = T;                           }
        }
        if (hi < lo) {
            int T = hi;
            hi = lo;
            lo = T;
        }
        sort(a, lo0, lo);
        sort(a, lo == lo0 ? lo+1 : lo, hi0);
    }      
   
    public final void connectionQueryRmt(int rid, int srcID, short pid, BasicKernel sk, ConnectionQueryEvent ev) {
        LinkedListConnection stuff=null;

        if(this.min<0){
            int left=0;
            int right=Array.length-1;
            
            if(srcID<Array[left].getID()||srcID>Array[right].getID())
                return;//not there            
            
            int middle=(left+right)>>>1;
            int id;
            while(stuff==null){
                id=Array[middle].getID();
                if(id==srcID){
                    stuff=Array[middle];
                }
                else if(id<srcID)
                    right=middle;
                else//id>srcID
                    left=middle;
                middle=(left+right)>>>1;
            }
        }
        else{
            int index = srcID-min;
             if(index<0||index>=Array.length)
                 return;//the neuron spikes on the port but is not connected to anything
             stuff=Array[index];        
             if(stuff==null)
                 return; //actually no neuron locally there or on that port       
        }
        int[] Connections=stuff.getInputConnection();
        int length=Connections.length;
        int i=0;
        int destCid;
        int destID;
        long delay;
        while(i<length){
            delay=Connections[i++];
            delay<<=32;
            delay|=(long)(Connections[i++]&0xFFFFFFFF);
            destCid=Connections[i++];
            destID=Connections[i++];
            neosim.implementation.EntityImpl destent=(EntityImpl)SimSystem.getEntity(new Integer(destID));
            ConnectionImpl impl=new ConnectionImpl(srcID, pid, destent, this.destPortID, delay );
            impl.setConnectionID(destCid);
	    ConnectionQueryEvent cqe = (ConnectionQueryEvent)ev.clone();
            cqe.setQueryConnection(impl);
	    sk.eventRmt( rid, srcID, pid, destID, PortID.SYSTEM_PORT, cqe ); 
       }          
    }
    
    
    
    
    public final void removeConnection(int srcID, int destID) {
        LinkedListConnection stuff=null;

        if(this.min<0){
            int left=0;
            int right=Array.length-1;
            int middle=(left+right)>>>1;
            
            if(srcID<Array[left].getID()||srcID>Array[right].getID())
                return;//not there
            
            int id;
            while(stuff==null){
                id=Array[middle].getID();
                if(id==srcID){
                    stuff=Array[middle];
                }
                else if(id<srcID)
                    right=middle;
                else//id>srcID
                    left=middle;
                middle=(left+right)>>>1;
            }
        }
        else{
             int index = srcID-min;
             if(index<0||index>=Array.length)
                 return;//the neuron spikes on the port but is not connected to anything
             stuff=Array[index];        
             if(stuff==null)
                 return; //actually no neuron locally there or on that port      
        }
        stuff.removeInputConnection(destID); 
    }
}



final class LinkedListSrcPort_In extends LinkedList {
    protected LinkedListSrcPort_In next=null;       
    /** Creates a new instance of LinkedListSrcPort */
    short srcPortID;
    LinkedListDestPort_In Array[];
    
    public LinkedListSrcPort_In(short srcPortID) {
        this.srcPortID=srcPortID;
        Array=new LinkedListDestPort_In[this.ListSize];
        LinkedList.TotalMemoryLinkedLists+=this.getSize();
    }
    
    public synchronized void compress() {
        for(int i=0;i<RealIndex_read;i++){
            if(Array[i]!=null){
                //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
                //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp);                 
                Array[i].compress();
                //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp); 
            }
        }
        if(next!=null)
            next.compress();
    }
    public  void compress_final() {
        //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp);        
        synchronized(this){
        LinkedListSrcPort_In stuff=this;
        short MaxPortIn=-1;
        short current;
        int TotalLength=0;
        do{
            for(int i=0;i<stuff.RealIndex_read;i++){
                if(stuff.Array[i]!=null){
                    current=stuff.Array[i].getID();
                    if(current>MaxPortIn)
                        MaxPortIn=current;
                }
            }
            TotalLength+=stuff.getSize();
            stuff=stuff.next;
        }while(stuff!=null);
        
        LinkedListDestPort_In[] NewArray=new LinkedListDestPort_In[MaxPortIn+1];
        
        stuff=this;
        do{
            for(int i=0;i<stuff.RealIndex_read;i++){
                if(stuff.Array[i]!=null)
                    NewArray[stuff.Array[i].getID()]=stuff.Array[i];
            }
            stuff=stuff.next;
        }while(stuff!=null);
        
        
        Array=NewArray;
        this.ListSize=MaxPortIn+1;
        LinkedList.TotalMemoryLinkedLists=getSize()+LinkedList.TotalMemoryLinkedLists-TotalLength;
        next=null;
        this.RealIndex_read=ListSize;

        for(int i=0;i<RealIndex_read;i++){
            if(Array[i]!=null)
                Array[i].compress_final();
        }
        
        }
        //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);         
    }    
    
    public synchronized void addInputConnection(short destPort, int srcID, int destID, long delay, int cid){
          for(int i=0;i<RealIndex_read;i++){
              if(Array[i]!=null&&Array[i].getID()==destPort){
                  Array[i].addInputConnection(srcID, destID, delay, cid);
                  return;
              }
          }
          if(RealIndex_read==this.ListSize){
              if(next==null)
                next=new LinkedListSrcPort_In(srcPortID);
              next.addInputConnection(destPort, srcID, destID, delay, cid); 
          }
          else{
              Array[RealIndex_read]=new LinkedListDestPort_In(destPort);
              Array[RealIndex_read++].addInputConnection(srcID, destID, delay, cid);
          }        
    }

    public short getID(){
        return srcPortID;
    } 
    
    public int getSize() {
        return 28+Array.length<<2; //(LinkedListSrcPort_In=4)+(srcPortID=2)+(Array=4)+(Array.size*(LinkedListDestPort_In*=4)+(Overhead=10))+8in superclass
    }
}


final class LinkedListConnection extends LinkedList {
    LinkedListConnection next=null;   
    int srceID;
    byte Array[];
    boolean compressed=false;    
    
    /** Creates a new instance of LinkedListConnection */
    public LinkedListConnection(int srceID) {
        this.srceID=srceID;
        Array=new byte[this.ListSize<<4];//*16 : 2 int 1 long   (4+4+8=16)     
        LinkedList.TotalMemoryLinkedLists+=this.getSize();
    }
    
    public synchronized void compress() {
        LinkedListConnection stuff=this;
        int i=0;
        
        int TotalMem=0;
        do{
            i+=stuff.RealIndex_read;
            TotalMem+=stuff.getSize();
            stuff=stuff.next;
        }while(stuff!=null);
        
        long delay_temp[]=new long[i];
        int cid_temp[]=new int[i];        
        int destid_temp[]=new int[i];        
        
        stuff=this;
        int j,k, current_cid, current_destid, previous_cid, previous_destid;
        long previous_delay,current_delay;
        i=0;
        previous_delay=0;
        previous_cid=0;
        previous_destid=0;
        
        
        
        byte b; 
        do{
            j=0;
            if(stuff.compressed){
                k=0;                
                while(j<stuff.RealIndex_read){
                    b=stuff.Array[k++];
                    current_delay = b & 0x7F;
                    for(int shift=7;(b & 0x80) != 0; shift += 7){
                        b=stuff.Array[k++];
                        current_delay |= (long)((b & 0x7F) << shift);
                    }
		    current_delay+=previous_delay;
		    previous_delay=current_delay;
                    
                    b=stuff.Array[k++];
                    current_cid = b & 0x7F;
                    for(int shift=7;(b & 0x80) != 0; shift += 7){
                        b=stuff.Array[k++];
                        current_cid |= ((b & 0x7F) << shift);
                    }
                    current_cid+=previous_cid;
                    previous_cid=current_cid;

                    b=stuff.Array[k++];
                    current_destid = b & 0x7F;
                    for(int shift=7;(b & 0x80) != 0; shift += 7){
                        b=stuff.Array[k++];
                        current_destid |= ((b & 0x7F) << shift);
                    }   
                    current_destid+=previous_destid;
                    previous_destid=current_destid;                    
                    
                    delay_temp[i]=current_delay;
                    cid_temp[i]=current_cid;
                    destid_temp[i]=current_destid;
                    i++;
                    j++;
                }
            }
            else{
                int last=stuff.RealIndex_read<<4;
                while(j<last){
                    j+=15;
                    current_destid=stuff.Array[j--];
                    current_destid<<=8;
                    current_destid|=(int)(0xFF&stuff.Array[j--]);
                    current_destid<<=8;
                    current_destid|=(int)(0xFF&stuff.Array[j--]);                    
                    current_destid<<=8;
                    current_destid|=(int)(0xFF&stuff.Array[j--]);                    

                    current_cid=stuff.Array[j--];
                    current_cid<<=8;
                    current_cid|=(int)(0xFF&stuff.Array[j--]);
                    current_cid<<=8;
                    current_cid|=(int)(0xFF&stuff.Array[j--]);                    
                    current_cid<<=8;
                    current_cid|=(int)(0xFF&stuff.Array[j--]);  
                    
                    current_delay=stuff.Array[j--];
                    current_delay<<=8;
                    current_delay|=(long)(0xFF&stuff.Array[j--]);
                    current_delay<<=8;
                    current_delay|=(long)(0xFF&stuff.Array[j--]);                    
                    current_delay<<=8;
                    current_delay|=(long)(0xFF&stuff.Array[j--]);
                    current_delay<<=8;
                    current_delay|=(long)(0xFF&stuff.Array[j--]);                    
                    current_delay<<=8;
                    current_delay|=(long)(0xFF&stuff.Array[j--]);                    
                    current_delay<<=8;
                    current_delay|=(long)(0xFF&stuff.Array[j--]);                    
                    current_delay<<=8;
                    current_delay|=(long)(0xFF&stuff.Array[j]);
                    
                    delay_temp[i]=current_delay;
                    cid_temp[i]=current_cid;
                    destid_temp[i++]=current_destid;                    
                    j+=16;
                }
            }
            stuff=stuff.next;
        }while(stuff!=null);
        //sort
        try{
            sort(delay_temp,cid_temp,destid_temp,0,delay_temp.length-1,0);
        }
        catch(Exception e){
            try{
                java.io.PrintWriter trcstream=new java.io.PrintWriter(new java.io.FileOutputStream("ExceptionInportsTable.txt"));
                e.printStackTrace(trcstream);
                trcstream.close();
            }
            catch(Exception e_){                
                e_.printStackTrace();
            }
        }        
     //Compression
        byte NewArray[]=new byte[delay_temp.length<<4];
        previous_delay=0;
        previous_cid=0;
        previous_destid=0;
        k=0;
        for(i=0;i<delay_temp.length;i++){
            current_delay=delay_temp[i]-previous_delay;
            previous_delay=delay_temp[i];
            
            while ((current_delay & ~0x7F) != 0) {
                b = (byte)((current_delay & 0x7f) | 0x80);
                NewArray[k++]=b;	    
                current_delay >>>= 7;
            }
            NewArray[k++]=(byte)current_delay;   
            
            current_cid=cid_temp[i]-previous_cid;
            previous_cid=cid_temp[i];
            
            while ((current_cid & ~0x7F) != 0) {
                b = (byte)((current_cid & 0x7f) | 0x80);
                NewArray[k++]=b;	    
                current_cid >>>= 7;
            }
            NewArray[k++]=(byte)current_cid; 
            
            current_destid=destid_temp[i]-previous_destid;
            previous_destid=destid_temp[i];
            
            while ((current_destid & ~0x7F) != 0) {
                b = (byte)((current_destid & 0x7f) | 0x80);
                NewArray[k++]=b;	    
                current_destid >>>= 7;
            }
            NewArray[k++]=(byte)current_destid;             
        }
        
        //Clean
        Array=new byte[k];
        System.arraycopy(NewArray,0, Array, 0, k);
        RealIndex_read=delay_temp.length;
        ListSize=RealIndex_read;
        this.compressed=true;
        this.next=null;  
        
        LinkedList.TotalMemoryLinkedLists+=getSize()-TotalMem;
    }
    
    public synchronized int[] getInputConnection(){
        LinkedListConnection stuff=this;
        int i=0;
        
        int TotalMem=0;
        do{
            i+=stuff.RealIndex_read;
            TotalMem+=stuff.getSize();
            stuff=stuff.next;
        }while(stuff!=null);
        
        int temp[]=new int[i<<2];
        
        stuff=this;
        int j,k, current_cid, current_destid, previous_cid, previous_destid, current_delay2, current_delay3;
        long previous_delay,current_delay;
        i=0;
        previous_delay=0;
        previous_cid=0;
        previous_destid=0;
        
        byte b; 
        do{
            j=0;
            if(stuff.compressed){
                k=0;                
                while(j<stuff.RealIndex_read){
                    b=stuff.Array[k++];
                    current_delay = b & 0x7F;
                    for(int shift=7;(b & 0x80) != 0; shift += 7){
                        b=stuff.Array[k++];
                        current_delay |= (long)((b & 0x7F) << shift);
                    }
		    current_delay+=previous_delay;
		    previous_delay=current_delay;
                    
                    b=stuff.Array[k++];
                    current_cid = b & 0x7F;
                    for(int shift=7;(b & 0x80) != 0; shift += 7){
                        b=stuff.Array[k++];
                        current_cid |= ((b & 0x7F) << shift);
                    }
                    current_cid+=previous_cid;
                    previous_cid=current_cid;

                    b=stuff.Array[k++];
                    current_destid = b & 0x7F;
                    for(int shift=7;(b & 0x80) != 0; shift += 7){
                        b=stuff.Array[k++];
                        current_destid |= ((b & 0x7F) << shift);
                    }   
                    current_destid+=previous_destid;
                    previous_destid=current_destid;                    
                    
                    temp[i++]=(int)(current_delay>>32);
                    temp[i++]=(int)(current_delay&0xFFFFFFFF);
                    temp[i++]=current_cid;
                    temp[i++]=current_destid;
                    
                    j++;
                }
            }
            else{
                int last=stuff.RealIndex_read<<4;
                while(j<last){
                    j+=15;
                    current_destid=stuff.Array[j--];
                    current_destid<<=8;
                    current_destid|=(int)(0xFF&stuff.Array[j--]);
                    current_destid<<=8;
                    current_destid|=(int)(0xFF&stuff.Array[j--]);                    
                    current_destid<<=8;
                    current_destid|=(int)(0xFF&stuff.Array[j--]);                    

                    current_cid=stuff.Array[j--];
                    current_cid<<=8;
                    current_cid|=(int)(0xFF&stuff.Array[j--]);
                    current_cid<<=8;
                    current_cid|=(int)(0xFF&stuff.Array[j--]);                    
                    current_cid<<=8;
                    current_cid|=(int)(0xFF&stuff.Array[j--]);  
                    
                    current_delay2=stuff.Array[j--];
                    current_delay2<<=8;
                    current_delay2|=(int)(0xFF&stuff.Array[j--]);
                    current_delay2<<=8;
                    current_delay2|=(int)(0xFF&stuff.Array[j--]);                    
                    current_delay2<<=8;
                    current_delay2|=(int)(0xFF&stuff.Array[j--]);
                    

                    current_delay3=stuff.Array[j--];                    
                    current_delay3<<=8;
                    current_delay3|=(int)(0xFF&stuff.Array[j--]);                    
                    current_delay3<<=8;
                    current_delay3|=(int)(0xFF&stuff.Array[j--]);                    
                    current_delay3<<=8;
                    current_delay3|=(int)(0xFF&stuff.Array[j]);
                    
                    temp[i++]=current_delay2;
                    temp[i++]=current_delay3;
                    temp[i++]=current_cid;
                    temp[i++]=current_destid;                    
                    j+=16;
                }
            }
            stuff=stuff.next;
        }while(stuff!=null);     
        return temp;
    }

    public synchronized void removeInputConnection(int destID){
       LinkedListConnection stuff=this;
        int i=0;
        
        int TotalMem=0;
        do{
            i+=stuff.RealIndex_read;
            TotalMem+=stuff.getSize();
            stuff=stuff.next;
        }while(stuff!=null);
        
        long delay_temp[]=new long[i];
        int cid_temp[]=new int[i];        
        int destid_temp[]=new int[i];        
        
        stuff=this;
        int j,k, current_cid, current_destid, previous_cid, previous_destid;
        long previous_delay,current_delay;
        i=0;
        previous_delay=0;
        previous_cid=0;
        previous_destid=0;
        
        byte b; 
        do{
            j=0;
            if(stuff.compressed){
                k=0;                
                while(j<stuff.RealIndex_read){
                    b=stuff.Array[k++];
                    current_delay = b & 0x7F;
                    for(int shift=7;(b & 0x80) != 0; shift += 7){
                        b=stuff.Array[k++];
                        current_delay |= (long)((b & 0x7F) << shift);
                    }
		    current_delay+=previous_delay;
		    previous_delay=current_delay;
                    
                    b=stuff.Array[k++];
                    current_cid = b & 0x7F;
                    for(int shift=7;(b & 0x80) != 0; shift += 7){
                        b=stuff.Array[k++];
                        current_cid |= ((b & 0x7F) << shift);
                    }
                    current_cid+=previous_cid;
                    previous_cid=current_cid;

                    b=stuff.Array[k++];
                    current_destid = b & 0x7F;
                    for(int shift=7;(b & 0x80) != 0; shift += 7){
                        b=stuff.Array[k++];
                        current_destid |= ((b & 0x7F) << shift);
                    }   
                    current_destid+=previous_destid;
                    previous_destid=current_destid;  
                    
                    if(destID!=current_destid){                    
                        delay_temp[i]=current_delay;
                        cid_temp[i]=current_cid;
                        destid_temp[i]=current_destid;
                        i++;
                    }
                    j++;
                }
            }
            else{
                int last=stuff.RealIndex_read<<4;
                while(j<last){
                    j+=15;
                    current_destid=stuff.Array[j--];
                    current_destid<<=8;
                    current_destid|=(int)(0xFF&stuff.Array[j--]);
                    current_destid<<=8;
                    current_destid|=(int)(0xFF&stuff.Array[j--]);                    
                    current_destid<<=8;
                    current_destid|=(int)(0xFF&stuff.Array[j--]);                    

                    current_cid=stuff.Array[j--];
                    current_cid<<=8;
                    current_cid|=(int)(0xFF&stuff.Array[j--]);
                    current_cid<<=8;
                    current_cid|=(int)(0xFF&stuff.Array[j--]);                    
                    current_cid<<=8;
                    current_cid|=(int)(0xFF&stuff.Array[j--]);  
                    
                    current_delay=stuff.Array[j--];
                    current_delay<<=8;
                    current_delay|=(long)(0xFF&stuff.Array[j--]);
                    current_delay<<=8;
                    current_delay|=(long)(0xFF&stuff.Array[j--]);                    
                    current_delay<<=8;
                    current_delay|=(long)(0xFF&stuff.Array[j--]);
                    current_delay<<=8;
                    current_delay|=(long)(0xFF&stuff.Array[j--]);                    
                    current_delay<<=8;
                    current_delay|=(long)(0xFF&stuff.Array[j--]);                    
                    current_delay<<=8;
                    current_delay|=(long)(0xFF&stuff.Array[j--]);                    
                    current_delay<<=8;
                    current_delay|=(long)(0xFF&stuff.Array[j]);
                    
                    if(destID!=current_destid){
	                    delay_temp[i]=current_delay;
        	            cid_temp[i]=current_cid;
                	    destid_temp[i++]=current_destid;   
                    }
                    j+=16;
                }
            }
            stuff=stuff.next;
        }while(stuff!=null);
        
        long delay_temp2[]=new long[i];
        int cid_tem2p[]=new int[i], destid_temp2[]=new int[i]; 
        
        System.arraycopy(delay_temp,0, delay_temp2,0,i);
        System.arraycopy(cid_temp,0, cid_tem2p,0,i);
        System.arraycopy(destid_temp,0, destid_temp2,0,i);
        
        delay_temp=delay_temp2;
        cid_temp=cid_tem2p;
        destid_temp=destid_temp2;
        
        //sort
        try{
            sort(delay_temp,cid_temp,destid_temp,0,delay_temp.length-1,0);
        }
        catch(Exception e){
            try{
                java.io.PrintWriter trcstream=new java.io.PrintWriter(new java.io.FileOutputStream("ExceptionInportsTable.txt"));
                e.printStackTrace(trcstream);
                trcstream.close();
            }
            catch(Exception e_){                
                e_.printStackTrace();
            }
        }
     //Compression
        byte NewArray[]=new byte[delay_temp.length<<4];
        previous_delay=0;
        previous_cid=0;
        previous_destid=0;
        k=0;
        for(i=0;i<delay_temp.length;i++){
            current_delay=delay_temp[i]-previous_delay;
            previous_delay=delay_temp[i];
            
            while ((current_delay & ~0x7F) != 0) {
                b = (byte)((current_delay & 0x7f) | 0x80);
                NewArray[k++]=b;	    
                current_delay >>>= 7;
            }
            NewArray[k++]=(byte)current_delay;   
            
            current_cid=cid_temp[i]-previous_cid;
            previous_cid=cid_temp[i];
            
            while ((current_cid & ~0x7F) != 0) {
                b = (byte)((current_cid & 0x7f) | 0x80);
                NewArray[k++]=b;	    
                current_cid >>>= 7;
            }
            NewArray[k++]=(byte)current_cid; 
            
            current_destid=destid_temp[i]-previous_destid;
            previous_destid=destid_temp[i];
            
            while ((current_destid & ~0x7F) != 0) {
                b = (byte)((current_destid & 0x7f) | 0x80);
                NewArray[k++]=b;	    
                current_destid >>>= 7;
            }
            NewArray[k++]=(byte)current_destid;             
        }
        
        //Clean
        Array=new byte[k];
        System.arraycopy(NewArray,0, Array, 0, k);
        RealIndex_read=delay_temp.length;
        ListSize=RealIndex_read;
        this.compressed=true;
        this.next=null;  
        
        LinkedList.TotalMemoryLinkedLists+=getSize()-TotalMem;        
    }    
    
    public synchronized void addInputConnection(int destID, long delay, int cid){
        if(RealIndex_read==this.ListSize){
           if(next==null)
              next=new LinkedListConnection(srceID);
           next.addInputConnection(destID,delay,cid); 
        }
        else{
           int current=RealIndex_read<<4;

           Array[current++]=(byte)(delay&0xFF);
           delay>>=8;
           Array[current++]=(byte)(delay&0xFF);
           delay>>=8;
           Array[current++]=(byte)(delay&0xFF);
           delay>>=8;
           Array[current++]=(byte)(delay&0xFF);
           delay>>=8;
           Array[current++]=(byte)(delay&0xFF);
           delay>>=8;
           Array[current++]=(byte)(delay&0xFF);           
           delay>>=8;
           Array[current++]=(byte)(delay&0xFF);
           delay>>=8;
           Array[current++]=(byte)(delay&0xFF);

           Array[current++]=(byte)(cid&0xFF);
           cid>>=8;
           Array[current++]=(byte)(cid&0xFF);
           cid>>=8;
           Array[current++]=(byte)(cid&0xFF);
           cid>>=8;
           Array[current++]=(byte)(cid&0xFF);
           
           Array[current++]=(byte)(destID&0xFF);
           destID>>=8;
           Array[current++]=(byte)(destID&0xFF);
           destID>>=8;
           Array[current++]=(byte)(destID&0xFF);
           destID>>=8;
           Array[current++]=(byte)(destID&0xFF);
           
           RealIndex_read++;
        }         
    }   
    
    public int getID(){
        return srceID;
    }    
    
    public int getSize() {
        return 28+Array.length;
    }
    
    public void compress_final() {
        this.compress();
    }
    
   void sort(long a[], int c[], int d[], int lo0, int hi0, int k) { 
        int lo = lo0;
        int hi = hi0;
        int i;
        if (lo >= hi) {
            return;
        }
        long mid = a[(lo + hi) >>>1];
        while (lo < hi) {
            while (lo<hi && a[lo] < mid) {
                lo++;
            }
            while (lo<hi && a[hi] >= mid) {
                hi--;
            }
            if (lo < hi) {
                long T = a[lo];
                a[lo] = a[hi];
                a[hi] = T;               
                               
                i = c[lo];
                c[lo] = c[hi];
                c[hi] = i;
                
                i = d[lo];
                d[lo] = d[hi];
                d[hi] = i;                
            }
        }
        if (hi < lo) {
            int T = hi;
            hi = lo;
            lo = T;
        }
        k++;
        if(k<16384){//in case the thread statck is too big
	        sort(a, c, d, lo0, lo,k);
        	sort(a, c, d, lo == lo0 ? lo+1 : lo, hi0,k);
        }
        else{
            heapsort(a,c,d,lo0,lo);
            heapsort(a,c,d,lo == lo0 ? lo+1 : lo,hi0);
        }
    }        
   
    void heapsort(long a[], int c[], int d[], int lo0, int hi0)
    {
        int n = hi0-lo0+1, i = n>>1, j, w,tc,td;
        long ta;

        for (;;) {
            if (i>0){
                ta = a[--i+lo0];
                tc = c[i+lo0]; 
                td = d[i+lo0]; 
            }
            else {
                if (--n<=0) 
                    return;
                ta = a[n+lo0]; a[n+lo0] = a[lo0];
                tc = c[n+lo0]; c[n+lo0] = c[lo0];
                td = d[n+lo0]; d[n+lo0] = d[lo0];
            }

            j = i;
            w = (i<<1)+1;

            while (w<n) {
                if (w+1<n && a[lo0+w+1]>a[lo0+w])
                    ++w;
                if (a[lo0+w]>ta) {
                    a[lo0+j] = a[lo0+w];
                    c[lo0+j] = c[lo0+w];
                    d[lo0+j] = d[lo0+w];
                    j = w; w = (j<<1)+1;
                }
                else
                    break;
            }
            a[lo0+j] = ta;
            c[lo0+j] = tc;
            d[lo0+j] = td;            
        }
    }   
    

}


