/* InputQueue.java
 */

package neosim.kernel;

import java.util.Vector;
import java.util.Enumeration;

/**
 * This class implements a queue of objects - it is a simple FIFO queue.
 * with a semaphore which gets signalled on each arrival
 * @see		neosim.SimSystem
 * @version     0.1, 25 Nov 1998
 * @author      Fred Howell
 */

public class InputQueue extends Vector {
    public Semaphore sem;
    Vector qids = new Vector();
//    transient DeadlockTracker tracker=new DeadlockTracker();    
    private boolean debug = false;

    /**
     * Allocates a new InputQueue object.
     */
    public InputQueue() {
	super();
	sem = new Semaphore(0);
    }

    /**
     * Remove and return the event at the top of the queue.
     * If queue empty, waits on semaphore.
     * @return           The next event.
     */
    public final Object pop() {
	sem.sem_wait(this);
	if (isEmpty()) return null; 
	else {
	    Object o = firstElement();
	    removeElementAt(0);
	    return o;
	}
    }

    /**
     * Return the event at the top of the queue, without removing it.
     * @return	The next event.
     */
    public final Object top() {
	return firstElement();
    }

    /** Return index of element qid. This should probably be a hashtable,
     * but it is expected that the queues will only be a few elements long
     */
    public final int getIndexByQid( int qid ) {
	int ret = -1;
	for (int i=0; i<qids.size() && ret==-1; i++) {
	    Integer it = (Integer)qids.elementAt(i);
	    if (it.intValue() == qid) { ret = i; }
	}
	return ret;
    }
    
    /** Get object of given id from queues */
    public final synchronized Object popSync(int qid) {
	/* Check for qid on q */
	int oid=-1;
	while (oid==-1) {
	    oid = getIndexByQid(qid);
	    if (oid == -1) {
		if (debug)		System.out.println("IQ: waiting for "+qid+" to come in to "+this);
		try { 
//                    System.out.println("B. Pop "+qid);
                    //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
                    //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp);
                    wait(); 
                    //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);                                                        
//                    System.out.println("Y. Pop "+qid);
                } catch (Exception e) {}
	    } else {
		if (debug)		System.out.println("IQ: Got "+qid+"!");
	    }
	}
	Object o = elementAt(oid);
	removeElementAt(oid);
	qids.removeElementAt(oid);
	return o;
    }

    /** Add object + id to queues */
    public final synchronized void addSync(Object new_obj, int qid) {
	addElement(new_obj);
	qids.addElement(new Integer(qid));
	if (debug)	System.out.println("IQ: Adding object "+qid+" to input queue"+this);
	notifyAll();
    }
    
    /**
     * Add a new object to the queue
     * @param new_event	The event to be put on the queue.
     */
    public final void add(Object new_obj, int qid) {
	addElement(new_obj);
	// Check that objects arrive in order (... tbd ...)
	sem.sem_signal();
    }
}
