package neosim.kernel;
import neosim.iface.*;

/** The event which executes an EntityQuery on
 * all members of a population
 */
public class PopulationQueryEvent extends EntityEvent {
    EntityQuery eq;
    String pname;
    Callback cb;

    protected int proc; // originating proc
    int qid;
    
    static boolean pdebug = false;
    public PopulationQueryEvent(Time t, int srce, short srcport, //EntityID srce//PortID srcport
				Population p, EntityQuery eq, Callback cb ) { 
	super( t, srce, srcport );
	this.pname  = p.getName();
	this.eq = eq; 
	this.cb = cb;
    }
    public final void setQid(int qid) { this.qid = qid; }
    public final int getQid() { return qid; } 
    public final void setProc(int proc) { this.proc = proc; }
    public final int getProc() { return proc; } 
    public final Callback getCallback() { return cb; }

    public void doOnEntity( Entity e ) {
	if (pdebug)	System.out.println("Population query event " + e.getName() + " "+pname);
	int idx = e.getPopulation().getIndex( e.getID() );
	PopCBObject pco = new PopCBObject( eq.query(e), idx );
	sendCallback( pco );	
	if (pdebug)	System.out.println("Population query event sent callback");
    }

    /** Send a callback to originator, containing the answer, 
     * + the entity's index in the population
     * Note this is ripe for an optimisation based on 
     * running population query on all local entities.
     */
    public final void sendCallback( PopCBObject ans ) {
	SimKernel sk = SimSystem.getKernel();
	sk.sendCommand( proc, new QueryResponse( proc, qid, ans ) );
    }

    public ClassID registerClass(NeosimClasses nc) { 
	cid = nc.registerClass("PopulationQueryEvent", "EntityEvent", "neosim.kernel"); 
	return cid;
    }
    
    public void readExternal(java.io.ObjectInput objectInput) throws java.io.IOException, java.lang.ClassNotFoundException {
        super.readExternal(objectInput);
        proc=objectInput.readInt();
        qid=objectInput.readInt();
        pname=objectInput.readUTF();
        eq=(EntityQuery)objectInput.readObject();
        cb=(Callback)objectInput.readObject();
    }
    
    public PopulationQueryEvent(){
        super();
    }
    
    public void writeExternal(java.io.ObjectOutput objectOutput) throws java.io.IOException {
        super.writeExternal(objectOutput);
        objectOutput.writeInt(proc);
        objectOutput.writeInt(qid);        
        objectOutput.writeUTF(pname);
        objectOutput.writeObject(eq);
        objectOutput.writeObject(cb);
    }  
    
    public int getSize(){
        return 125;
    }    
}
