/* Semaphore.java
 */

package neosim.kernel;

/**
 * This is a counting semaphore class. It is used internally by the
 * Sim_system to
 * synchronise the running of the threaded Sim_entities. It should not
 * be needed in user simulations.
 * @see		eduni.simjava.Sim_system
 * @version     1.0, 4 September 1996
 * @author      Ross McNab
 */

public final class Semaphore {
  private int count;
//  transient DeadlockTracker tracker=new DeadlockTracker();
  // Constructors
  /**
   * Allocate a new Semaphore object, with an initial count of zero.
   */
  public Semaphore() {
    count = 1;
  }
 public int getCount(){//debug purpose
     return count;
 }
  /**
   * Allocates a new Semaphore object, with a given initial count.
   * @param count	The initial count of the semaphore.
   */
  public Semaphore(int count) {
    this.count = count;
  }

  /**
   * Try to obtain the Semaphore. If the count is above zero, the
   * function decrements it, then return so the calling thread can
   * continue. If the count is zero then the calling thread is suspended
   * until it becomes non-zero.
   */
  public final synchronized void p(Object obj) {
    while (count == 0) {
      try { 
          //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
          //neosim.kernel.DeadlockTracker.LockDeadlockTracker(obj,ecp);
          wait(); 
          //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);          
      } catch (InterruptedException e) { }
    }
    count--;
  }

  /**
   * Free the Semaphore, by incrementing the internal count.
   */
  public final synchronized void v() {
    count++;
    notify();
  }

  public final void semWait(Object obj) { p(obj); }
  public final void semSignal() { v(); }
  public final void sem_wait(Object obj) { p(obj); }
  public final void sem_signal() { v(); }


}
