package neosim.kernel;

import java.util.Vector;

import neosim.perf.*;

import neosim.iface.*;
import neosim.implementation.*;

/** Kernel interface for commands */
public interface SimKernel extends Collective, Kernel {
    public void eventRmt(int rid, int srce, short pid, Event ev);//EntityID srce//PortID pid
    public void eventRmt(int rid, int srce, short srcpid, int deste, short destpid, Event ev);// EntityID srce, PortID srcpid, EntityID deste, PortID destpid,//PortID srcpid//PortID destpid
    public void connectionQueryRmt(int rid, int srce, short pid, ConnectionQueryEvent ev);//EntityID srce, PortID pid
    public void command(CommandObj co);
    public void syncCommand(QueryCommand qc);
    public void sendCommand(int p, CommandObj co);
    public void sendCommand(CommandObj co);
 
    public void genVrml(String url);

    /* Local simulation manager */
    public SimSystem getSimSystem();
    public SystemEntity getSysEntity();
    public Object getUserApp();
    public void   setUserApp(Object o);

    public void   barrier();
    public void   flush(boolean FlushMeasurement);
    public int    getID();
    public int    getRank();
    public int    getGroupsize();
    public int    getNumEnts();
    public Time   getMinDelay();

    /* EntityIDs */
    public int allocateIDs( int size );//return EntityID

    /* Connections */
    //public InPortsTable getInports(int rmtproc);
  //  public void addInport(int rmtproc, InPortTable ip);

    /* Running (Called by SimSystem) */
    public void setRunning(Time endt);
    public void setStopped();
    public void setSleeping();
    public Time getMinTime();

    /* Remote delivery of info */
    public void qresponseRmt(int id,int qid,Object res);
    public void restartRmt(int rid);
    public void addRmt(int rid, String n, RmtApp obj);
    public void removeRmt(int rid);
    public void stoppedRmt(int rid);
    public void runningRmt(int rid, Time rwt);
    public void runRmt(int rid, Time rwt);
    public void resetRmt(int rid);
    public void destroyRmt(int rid);


    /* Dynamic comms functions */
    public Vector gatherQresponses(int qid);
    public int getQid();

    /* Performance monitoring commands */
    public void monitorProcessors(PerfMonitor pmon, boolean realTime);
    public void monitorEntities(PerfMonitor pmon, boolean realTime);

    /** Get the total number of entities*/
    public int getNbEntities();//JAG
} 
