/* SimProcessor.java
 */

package neosim.kernel;

import java.util.Vector;
import java.util.Enumeration;

import neosim.iface.Time;

/**
 * This class represents a parallel process running in the system.
 * The SimProcessor has a number of local SimEntities under its
 * control.
 * @see         SimSystem
 * @version     0.1, 2 July 1998
 * @author      Fred Howell
 */

public class SimProcessor extends Thread implements RmtProc {

    private boolean debug = false;

    private int me;            // Unique id
    private Semaphore restart; // Used by SimSystem to schedule us
    private Time right_time = new Time(0); // Time it's safe to run until.
    private Time endTime= new Time(0);    // Simulation run time
    
    /** The standard constructor.
     * @param id The index of this processor.
     * @return A new instance of the class SimProcessor
     */
    public SimProcessor(int id) {
	this.me = id;
	restart = new Semaphore(0);
         setName(getName()+" "+this.getClass().getName()+ " "+me);
    }
    
    public static final int SLEEPING = 101; // Waiting for someone to finish.
    public static final int RUNNING  = 102; // Running an entity.
    public static final int STOPPED  = 103; // Finished running an entity.
    
    int state = STOPPED;
    
    public int getID() { return me; }
    
    
    public void setState(int i) { 
	state = i;
	if (debug) {     	System.out.println(me+" Sim Processor state set to "+i); }
    }
    public int  getState_() { return state; }
    public void restart() { restart.v(); }
    public void pauseme() { restart.p(this); }
    public void setGoing() { restart.v(); } 
    public void setID(int id) { me = id; }
    public void poison() { restart.v(); }
    public Time getRightWindowTime() { return right_time; }
    public void setRightWindowTime(Time t) { right_time.t = t.t; }
    public void setEndTime(Time t) { endTime.t = t.t; }
    
    public final void run() {
	if (debug) { System.out.println(me+" Sim processor run() called"); }
	SimSystem.paused(); // Tell the system we're up and running
	pauseme();
	
	if (debug) {     	System.out.println(me+" Sim processor restarted"); }

	while (SimSystem.runTickPar(me,this)) { 

	    	if (debug) System.out.println(me+" Sim processor run a tick");

	}

	if (debug) {      	System.out.println(me+" Sim processor stopping"); }
	
	SimSystem.paused();

	if (debug) {      	System.out.println(me+" Sim processor exiting"); }
    } 
}
