/* SimThread.java
 */

package neosim.kernel;

import java.util.Vector;
import java.util.Enumeration;

import neosim.iface.Time;

/**
 * This class represents a parallel simulation thread running in the system.
 * It calles SimSystem.runEntities
 *  
 * @see         SimSystem
 * @version     1.0, 4 Oct 2000
 * @author      Fred Howell
 */

public final class SimThread extends Thread {

    private boolean debug = false;
    private boolean poisoned = false;

    private int me;            // Unique id
    private Semaphore restart; // Used by SimSystem to schedule us
    private ThreadManager tm;
    private SimProcessor sp;
    
    /** The standard constructor.
     * @param id The index of this processor.
     * @return A new instance of the class SimThread
     */
    public SimThread(int id, ThreadManager tm, SimProcessor sp) {
	this.me = id;
	this.tm = tm;
	this.sp = sp;
	restart = new Semaphore(0);
        setName(getName()+" "+this.getClass().getName()+ " "+me);
            }
         
    public int getID() { return me; }
         
    public void restart() { restart.v(); }
    public void pauseme() { restart.p(this); }
  
    public void setID(int id) { me = id; }

    /** Called externally to end this thread */
    public void poison() { poisoned = true; restart.v(); }
    
    public final void run() {
 
	if (debug) {     	System.out.println(me+" Sim thread starting"); }

	while (!poisoned) {
	    // wait for restart
	    pauseme();
	    // do entity updates
	    Time st = tm.getSafeTime();
	    if (debug) {     	System.out.println(me+" run entities to "+st); }

	    SimSystem.runEntities(sp, st);
	    // signal done
	    tm.paused();
	} 

	if (debug) {     	System.out.println(me+" Sim thread exiting"); }
    }    
}
