/** ThreadManager.java
 * Maintains the status of each simulation thread.
 */

package neosim.kernel;

import java.util.Vector;
import java.util.Enumeration;
import neosim.iface.Time;

/**
 * This class implements a vector of SimThreads
 * 
 * @see		neosim.SimSystem, neosim.SimThread
 * @version     0.1, 4 Oct 2000
 * @author      Fred Howell
 */

public final class ThreadManager  {

    Vector threads;

    Semaphore tds = new Semaphore(0);
    Time safeTime = new Time(0);

    private boolean debug = false;
    
    /** SimProcessor to which these threads are attached 
     */
    SimProcessor sp;
  
    /**
     * Allocates a new ThreadManager object.
     */
    public ThreadManager(SimProcessor sp, int num) {
	threads = new Vector();
	this.sp = sp;
	for (int i=0; i<num; i++) {
	    threads.add( new SimThread( i, this, sp ) );
	}
	startThreads();
    }  
 
    /** 
     * Start
     */
    public void startThreads() {
	for (int i=0; i<threads.size(); i++) {
	    getThread(i).start();
	}
    }

    /** 
     * Kill
     */
    public void killThreads() {
	if (debug) { System.out.println("thread manager killing threads"); }
	for (int i=0; i<threads.size(); i++) {
	    getThread(i).poison();
	}
    }

    /** 
     * Run threads (blocks until all completed)
     */
    public void runThreads(Time safeTime) {
	this.safeTime = safeTime;
	if (debug) { System.out.println("run Threads setting "+
					threads.size()+" threads going to "+safeTime.s()); }
	for (int i=0; i<threads.size(); i++) {
	    getThread(i).restart();	    
	}
	if (debug) { System.out.println("run Threads waiting for restart"); }
	for (int i=0; i<threads.size(); i++) {
	    tds.semWait(this);
	}
	if (debug) { System.out.println("run Threads done"); }
    }

    /**
     * Return thread i
     */  
    public SimThread getThread(int i) {
	return (SimThread)threads.elementAt(i);
    }
     
    /** Return SimProcessor
     */
    public SimProcessor getSimProcessor() { return sp; }

    /** Return Safe time to advance entities to
     */
    public Time getSafeTime() { return safeTime; }
 
    /** A Thread has paused 
     */
    public void paused() { tds.semSignal(); }
}
