package neosim.neuroml;

import neosim.implementation.*;
import neosim.iface.*;
import neosim.kernel.*;
import java.util.Vector;
import java.util.Hashtable;
import neuroml.sim.run.*;

/**
 * Neosim implementation of a NeuroML connection maker.
 * Gives DestMethod a makeConnection() function.
 * @author Paul Rogister
 * @date Sep 2001
 */
public final class NeuromlConnectionMaker implements neuroml.sim.run.ConnectionMaker {
    //static public int nbconns=0;
    Entity deste;
    neuroml.sim.run.ConnectionRequest cr;
    neuroml.sim.run.DestMethod dm;
//    transient neosim.kernel.DeadlockTracker tracker=new DeadlockTracker();
    public NeuromlConnectionMaker() {}

    /** Initialise connection maker to a given entity
     */
    public NeuromlConnectionMaker(Entity deste, neuroml.sim.run.ConnectionRequest cr, 
				  neuroml.sim.run.DestMethod dm) {
	this.deste = deste;
	this.cr = cr;
	this.dm = dm;
    }
    
    /** Return the ID of the target entity
     */
    public int getDstID() {
	return deste.getID();
    }


    // Keep a tab of the delay for the connection made
    // currently doesn't work if >1 connection :(
    private double delay = 1.0;

    /** 
     * This method is called by the considerRequest method to 
     * create a connection.
     * It could actually get all this info (except maybe delay)
     * from the connection request...
     */
    public int makeConnection(int srcID, short srcPortID, int dstID, short dstPortID, double delay, boolean BuildingConnection) {//EntityID srcID, PortID srcPortID, EntityID dstID, PortID dstPortID
        //nbconns++;
	this.delay = delay;
	//	NeosimConnectionImpl Connection = new NeosimConnectionImpl(c);
	// This is the Neosim connection object at target end...
	ConnectionImpl ci =  new ConnectionImpl(srcID, srcPortID,//new EntityID(srcID)
						(EntityImpl)deste, dstPortID, new Time(delay).t);
	
	// This method of adding connections seems clumsy...
	BasicKernel bk    = (BasicKernel) SimSystem.getKernel();			       
	int srcproc  = bk.getEntityProcessor(srcID);	//new EntityID(srcID)
        //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(ci,ecp);   
	int cid=bk.addInConnection(srcproc, ci,BuildingConnection);
	//neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
	// Setting of dest entity parameters done by the DestMethod, not here.
	// c.setDstParams(deste, ci.getConnectionID());	
	return cid;
    }   


    /** Make the connection using info from cr, dm
     */
    public int makeConnection(short dstPortID, double delay, boolean BuildingConnection) {//PortID dstPortID ???
	return makeConnection( cr.getSrcID(), 
			       cr.getSrcPortID(), 
			       getDstID(), 
			       dstPortID, 
			       delay, 
                               BuildingConnection);
    }

    /** Return the delay
     */
    public double getDelay() { return delay; }
}
