package neosim.neuroml;

import neuroml.core.*;
import neuroml.util.*;
import neuroml.sim.state.*;
import neosim.iface.*;
import java.util.Hashtable;
import java.util.Vector;

import neuroml.sim.run.*;

/** The NeuromlSimulator class provides the neuroml interface to 
 * neosim.
 */
public final class NeuromlSimulator implements neuroml.sim.run.Simulator {
  
    neosim.iface.ScriptReader sc;

    private static boolean debug = false;

    public NeuromlSimulator( neosim.iface.ScriptReader c ) { 
        this.sc = c; 
    }

    public Object getReader(){
        return sc;
    }
    /** Returns the model.
     */
    public Object getModel() {
	return null; // no neuroml model from ScriptReader (yet?)
    }
 
    /** Initialise the Neosim kernel.
     */
    public void bootstrap(String args[]) {
	sc.bootstrap(args);
    }

    /** Initialise kernel - 
     * defaults (seq/thr/rmi) provided by ScriptReaderImpl
     */
    public void initKernel(String args[]) { 
	sc.initKernel(args);
    }

    /** Initialise the simulation - called automatically
     */
    public void initSim(){
	sc.initSim();
    }

    /** Run the simulation until the given time
     * @param t the time to run for.
     * @deprecated
     */
    public void runSim(double t) { 
	run(t);
    }
    
    /** Run the simulation until the given time
     * @param t the time to run for.
     */
    public void run(double t) { 
	if (debug) System.out.println("NeuromlSimulator.runSim called to "+t);
	sc.runSim(new Time(t));
    }
    
    /** Build a model from a NeuroML object description
     */
    public void buildModel( Object model, Namespace ns ) {
	if (debug) System.out.println("NeuromlSimulator.buildModel not yet implemented"); 
    }

    /** Launch the simulation when loaded from a jarbean, with parameters
     */
    public void launch(String args[]) {
	// nop
    }

    /** Parameterize the simulation 
     * @deprecated
     */
    public void setParameters(String args[]) {
	//nop
    }

    /** Launch the simulation when loaded from a bean, 
    * parameters are the simulator public fields set beforehand
     */
    public void launch() {
	// nop
    }

    /** Reset the simulation to time 0
     */
    public void reset() { 
	sc.reset();
    }

    /** Terminate the kernel.
     */
    public void terminate() { 
	sc.terminate();
    }

    /** Construct a population of entities
     * @param p the population to build
     */
    //public void construct ( PopulationFactory p){ 
    //	NeosimPopulationBuilderImpl pbuilder = new NeosimPopulationBuilderImpl(p);
    //	sc.construct(pbuilder);
    //}

    /** Destroy a population of entities
     * @param p the population to destroy
     */
    public void destroy(String p) { 
	Population pop = (Population)(sc.getPopulationTree().getChild(p));
	//NeosimPopulationImpl pop = new NeosimPopulationImpl(p);
	sc.destroy(pop);
    }

    /** Connect up a projection
     * @param p the projection to make
     */
    /*public void connect ( ProjectionState p ){ 
	NeosimProjectionImpl proj = new NeosimProjectionImpl(p);
	sc.connect(proj);

    }
    */
    /** Disconnect a projection
     * @param p the projection to break
     */
    /*public void disconnect ( ProjectionState p ){ 
	NeosimProjectionImpl proj = new NeosimProjectionImpl(p);
	sc.disconnect(proj);
    }
    */

    /** Perform a generic update on an entity
     * @param e the id of the entity to update
     * @param eu the update to perform
     */   
    /*public void  updateElement( int e, CellStateUpdate eu ){ 
	NeosimEntityUpdate neu = new NeosimEntityUpdate(eu);
	sc.updateEntity(new EntityID(e),neu);
    }
    */

    /**  
     * @deprecated use queryElement(e,Query) instead
     */       
    public Object queryElement(int e, String eq) { 
	NeosimEntityQueryOld neq = new NeosimEntityQueryOld(eq);
	return sc.queryEntity(e,neq);//new EntityID e
    }

    public Object queryElement(int e, Query q) { 
	NeosimEntityQuery neq = new NeosimEntityQuery(q);
	return sc.queryEntity(e,neq);//new EntityID e
    }

    /** Perform an update on a population
     * @param p the population (by its name) to query
     * @param eu the update to perform
     */       
    /*public void   updatePopulation( String p, String eu ){ 
	NeosimEntityUpdate neu = new NeosimEntityUpdate(eu);
	Population pop = (Population)(sc.getPopulationTree().getChild(p));
	//NeosimPopulationImpl pop = new NeosimPopulationImpl(p);
	sc.updatePopulation(pop,neu);
    }
    */
  
    /** @deprecated use queryPopulation(p,Query) instead
     */
    public Vector queryPopulation(String p, String eq) { 
	NeosimEntityQueryOld neq = new NeosimEntityQueryOld(eq);
	Population pop = (Population)(sc.getPopulationTree().getChild(p));
	if (debug) System.out.println("NeuromlSimulator queryPopulation "+pop);
	//NeosimPopulationImpl pop = new NeosimPopulationImpl(p);
	return sc.queryPopulation(pop,neq);
    }
    
    public java.util.List queryPopulation(String p, Query q) { 
	NeosimEntityQuery neq = new NeosimEntityQuery(q);
	Population pop = (Population)(sc.getPopulationTree().getChild(p));
	if (debug) System.out.println("NeuromlSimulator queryPopulation "+pop);
	//NeosimPopulationImpl pop = new NeosimPopulationImpl(p);
	return sc.queryPopulation(pop,neq);
    }

    /** Perform an query on the output connections of an entity
     * @param e the source entity
     * @param p the source output port ID
     * @param cq the query to perform
     * @return the result of the query (as a Vector of Objects)
     */       
    /*public Vector queryOutputConnections( int e, int p, ConnectionStateQuery cq ){
	EntityID eid = new EntityID(e);
	PortID pid = new PortID(p);
	NeosimConnectionQuery ncq = new NeosimConnectionQuery(cq);
	return sc.queryOutputConnections(eid,pid,ncq);
	
     }
    */

    /** Lookup an entity ID given its name
     * @param name the name of the entity
     * @return the entity ID
     */
    public int lookupElementID( String name ) {
	return sc.lookupEntityID(name);
    }
    
    /** Lookup a Population given its name
     * @param name the name of the population
     * @return the population state
     */   
    public PopulationState lookupPopulation( String name ) { 
	Population p = sc.lookupPopulation(name);
	if (p!=null) {
	    return new NeuromlPopulationState(p);
	} else {
	    return null; // Not found !!!
	}
    }    

    /** Lookup a Element given its name
     * @param name the name of the element
     * @return the element state
     */   
    public ElementState lookupElement( String name ) { 
	// Get the id
	// Do a query
	System.out.println("lookupElement not yet implemented");
	return null; // new NeuromlElementState(sc.lookupPopulation(name));
    }  

    /** Get root of the population tree
     * @return the population state of the root. Should be NetworkState.
     */   
    public PopulationState getRootElement() {
	return new NeuromlPopulationTree(sc.getPopulationTree());
    }  
}
