package neosim.perf;

import neosim.util.*;
 

import java.util.Vector;

/** Performance monitoring logger.<p>
 *
 *  Implementation of PerfMonitor that writes to a file.
 *  Format:- 
<pre>
 $types
 &lt;typename> &lt;values>
 $bars
 &lt;entitystatename> &lt;typename>
 $events
 &lt;t> &lt;entstate> &lt;val>
</pre>
 */
public final class PerfLogger implements PerfMonitor, Filecmds {
    
    Types  paramTypes;
    Vector states;
    FileListener fs;
    String fname;
    double t0;
    PerfMonitor next = null;

    public PerfLogger(String fname) {
	this.fname = fname;
	resetAll();
    }

    public void daisyChain( PerfMonitor next ) {
	this.next = next;
    }

    public int registerType(String typename, String values) {
	if (next!=null) { int dummy = next.registerType(typename,values); }
	paramTypes.add(new TypeParam( typename, values ));
	return paramTypes.size()-1;
    }

    public int registerEntityState(String name, String typename) {
	if (next!=null) { int dummy = next.registerEntityState(name,typename); }
	TypeParam tp = paramTypes.find(typename);
	if (tp==null) { 
	    System.out.println("Type "+typename+" not registered");
	    return -1;
	}
	states.addElement( new EntityState(name, tp) );
	return states.size()-1;
    }
    public void fprint(String l) {
	fs.handleFile( new FileEventObject(this,l) );
    }

    /** Write header before events */
    public void resetEvents() {
	if (next!=null) { next.resetEvents(); }
	int i,j;
	String line;
	fs.handleFile( new FileEventObject(this,START) );
	fprint("$types");
	for (i=0; i<paramTypes.size(); i++) {
	    TypeParam tp = paramTypes.get(i);
	    line = tp.getTypename();
	    for (j=0; j<tp.getNumLabels(); j++) {
		line = line + " " + tp.getLabel(j);
	    }
	    fprint(line);
	}
	fprint("$bars");
	for (i=0; i<states.size(); i++) {
	    EntityState es = (EntityState)(states.elementAt(i));
	    fprint( es.getName() + " " + es.getType().getTypename());
	}
	fprint("$events");
    }

    public void addEvent(int estateId, double t, String statename) {
	if (next!=null) { next.addEvent(estateId,t,statename); }
	EntityState es = (EntityState)(states.elementAt(estateId));
	String line =  "u:"+es.getName()+" at "+t+": P "+statename;
	fs.handleFile(new FileEventObject(this,line));
    }

    public void resetAll() {
	if (next!=null) { next.resetAll(); }
	paramTypes  = new Types();
	states = new Vector(); // of EntityState	
	fs = new FileSaver(fname);
    }

    public void finished() {
	if (next!=null) { next.finished(); }
	fs.handleFile(new FileEventObject(this,EOF));
    }
}
