package neosim.renderer;

import java.awt.*;
import java.util.*;
import java.io.*;
import java.net.*;
import java.applet.Applet;
import java.awt.event.*;
import neosim.iface.*;
import neosim.implementation.Shape;
import neosim.kernel.Pos;


/** A Java3D diagram in a separate window (and thread).
 *  
 * Usage (from a class which generates GraphObjects)
 * <pre>
 *   AnimWindow gw = new AnimWindow();
 *   gw.start();
 * </pre>
 */
public final class AnimWindow extends Thread {
    Frame f;
    AnimApp td;
    Pausable p; // Thing to restart
    Netvis nv;
    static Panel RenderingPanel;
    public static void setPanel(Panel thatPanel){
        RenderingPanel=thatPanel;
    }
    
    public AnimWindow(Pausable p, Netvis nv) { 
        this.p = p;
	this.nv = nv;
        //f = new Frame("Java3D Display");
        String[] args = {"Java3D Display"};
        td = new AnimApp(args, p,nv);
        setName(getName()+" "+this.getClass().getName()+ " "+nv.getID());
    }
   /* public void sendEvents(EventList el) {
        System.out.println("J3D Window got "+el.getEvents().size()+" events");
    }*/

    public void startRenderer() {
        td.startRenderer();
    }
    public void continuousRender() {
        td.continuousRender();
    }

    /** Add entity */
    public void addEntityToVis( int eid, String name, double x, double y, double z, Shape s ) {//EntityID  eid
	td.addEntityToVis(eid,name,x,y,z,s,null);
    }
    
    public void addEntityToVis( int eid, String name, double x, double y, double z, Shape s, Vector v ) {//EntityID eid
	td.addEntityToVis(eid,name,x,y,z,s,v);
    }

    /** Add connection */
    public void addConnToVis( Pos p1, Pos p2 ) {
	td.addConnToVis(p1, p2);
    }

    /** Add connection */
    public void highlightDest( int eid ) {//EntityID eid
	td.highlightDest(eid);
    }

    /** Displays the window */
    public void run() {
	//        System.out.println("AnimWindow starting");
        //f.add(td);
        RenderingPanel.removeAll();
        
        RenderingPanel.setLayout(new BorderLayout());
        RenderingPanel.add(td,BorderLayout.CENTER);
        RenderingPanel.repaint();
        RenderingPanel.getParent().repaint();
        //td.setSize(400,400);
        //f.pack();
        //f.setVisible(true);
	//        System.out.println("AnimWindow starting - releasing pause");
        p.restart();
    }

    /** Returns the graph diagram for this window */
    public AnimApp getDiag() { return td; }
    public void clean(){
        td.clean();
    }
    
     public void flash( int eid ) {//EntityID
	new FlashTread(td, eid);
    }
    
   class FlashTread implements Runnable{
        Thread t;
        int dest;//EntityID
        AnimApp anim;
        public FlashTread(AnimApp anim, int dest){//EntityID
            this.anim=anim;
            this.dest=dest;
            t=new Thread(this);
            t.start();
        }
        
        public void run() {
            anim.highlightDest(dest);
            try{
                Thread.sleep(500);
            }
            catch(InterruptedException e){
            }
            anim.highlightDest(dest);
        }
        
    }    
}
