/**
 * Java3D netvis .jar file
 */
 
package neosim.renderer;

import neosim.iface.*;
import neosim.implementation.*; 
import neosim.util.BasicApp; 

import neosim.kernel.*;
import neosim.util.*;
import neosim.util.SmallLists.*;

import java.util.Vector;
import java.beans.*;

import neuroml.sim.state.*;
import neuroml.sim.run.*;
import neosim.neuroml.*;
 
import java.util.*;
import java.lang.Math;

  

/** Network Visualiser class.  
 */
public final class NetvisClass extends EntityClassImpl {

    private AnimWindow app;
    transient private ScriptReader sr;
    private Netvis myNetvis;
    short in;//PortID
    

    public NetvisClass() {
	// Bean Constructor!!! - 
	// JavaBeans need a zero argument constructor
        in = addInPort(SpikeEvent.inst.getEventClass(),"in");
    }   
    public NetvisClass(ScriptReader sr) {
        in = addInPort(SpikeEvent.inst.getEventClass(),"in");
	this.sr = sr;
    }   
    void setApp( AnimWindow app ) { this.app = app; }
    ScriptReader getScriptReader() { return sr; } // *** This is a nasty way to call runSim()
    public NetvisClass(String className) {
        super(className);
	// This not used from bean!!!
        in = addInPort(SpikeEvent.inst.getEventClass(),"in");
    }
    public Entity makeEntity(Population p, int i) {
        myNetvis=new Netvis(this, p, i );
        return myNetvis;
    }
    //18/04/05 necessary ?
   /* public void clean()
    {
        myNetvis.CheckConnection();
    }*/
    public boolean IsInParent() {
        return true;
    }
}

/** class for shape data extracted from entity
 */
final class ShapeData {
    String n;
    Pos p;
    neosim.implementation.Shape s;
    int eid;//EntityID eid;
    ShapeData( int eid, String n, Pos p, neosim.implementation.Shape s ) { //EntityID eid
	this.eid = eid; 
	this.n = n; this.p = p; this.s = s;  
    }
    int getEntityID() { return eid; }//rerurn EntityID 
    String getName() { return n; }
    Pos getPos() { return p; }
    neosim.implementation.Shape getShape() { return s; }
}

/** class for connection queries
 */
class ConnData {
    int destID; // ID of dest entity //EntityID
    String name; // Name of dest entity
    Pos p;       // Pos of dest entity
    // Space for pos of connection here, weight, anything else
    ConnData( int dest, String name, Pos p ) { this.destID = dest; this.name = name; this.p = p;   }////EntityID
    int getDestID() { return destID; } //EntityID
    String getName() { return name; }
    Pos getPos() { return p; }
}

/** Callback for entity queries
 */
final class AddEntityToVis implements Callback {
    public AddEntityToVis(){
    }
    
    public void callback( Entity e, Object o ) {
	try {
	    //System.out.println(e.getName()+" AddEntityToVis Callback called ");
	    Netvis nv= (Netvis)e;
	    ShapeData sd = (ShapeData) o;
	    nv.addEntityToVis( sd.getEntityID(), sd.getName(), sd.getPos(), sd.getShape() );
	} catch ( ClassCastException ex) { 
	    System.out.println("AddEntityToVis - exception "+ex);
	}
    }
    public void readExternal(java.io.ObjectInput objectInput) throws java.io.IOException, java.lang.ClassNotFoundException { 
    }
    
    public void writeExternal(java.io.ObjectOutput objectOutput) throws java.io.IOException {
    }       
}
 
/** Get entity position...
 */
final class GetPos extends EntityQuery {
    public Object query( Entity e ) {
	try {
	    //Entity3D e2 = (Entity3D) e;
            PositionState e2 = (PositionState) e;
           // System.out.println(e.getName()+" :GetPos: Got an entity3D "+e2.getXPos()+ " "+e2.getYPos()+" "+ e2.getZPos());
            if(e2.getShape()==null){
               // System.out.println("No shape");
                e2.setShape(new neosim.implementation.Shape("red","sphere",0.3));
            }
	    return new ShapeData(e.getID(), e.getName(),
				  new Pos( e2.getXPos(), e2.getYPos(), e2.getZPos()),
				  (neosim.implementation.Shape)(e2.getShape()));
	} catch (ClassCastException ex) { 
	    //System.out.println("GetPos: Not an entity3D but a "+e.getClass().getName() + " name: "+e.getName()+" id: "+e.getID() ); 
	}
        return new ShapeData( e.getID(), e.getName(), new Pos(-e.getID(),0,0), null );
    }
}


/** Netvis - directs incoming events to J3D display
 */
final class Netvis extends EntityImpl implements Pausable  {
    transient static AnimWindow app; // static as a hack
    transient Semaphore s = new Semaphore();

    transient short in;//PortID
    transient Hashtable bars = new Hashtable(); // Table of names/erc entity IDs.
    transient SpikeIndexTable spTable;
    transient SpikeFlasher rc=new SpikeFlasher();
        
 /*   public int addInConnection(int srcID, short srcPort, short destPort) {//ConnectionID return
	// Add to local vector of incoming connections.
	//inConnections.addElement(c);
        this.LinkedListCon.addOutConnection(srcPort, destPort, srcID);
	 //new ConnectionID( NumberConnection++ );//inConnections.size() ConnectionID cid = new ConnectionID
	//c.setConnectionID(cid);
	return NumberConnection++;
    }    */
    
    public Netvis(EntityClass ec, Population p, int i) {
        super(ec,p,i);
        spTable=new SpikeIndexTable(); 
        /* Make a default input port (for spikes) */
        in = 1;//new PortID(
        //System.out.println("Starting Java3D applet...");
        app = new AnimWindow( this,this ); // Init the AnimWindow with this entity's restart() method
        ((NetvisClass)ec).setApp(app);
        app.start();
        // Wait for restart...
        pauseme(); 
    }
   
  public void addSpike(int index,long time){
         spTable.addSpike(index,time);
    }
    void addEntityToVis(int eid, String name, Pos p, neosim.implementation.Shape s ) {////EntityID eid
	app.addEntityToVis(eid, name, p.x,p.y,p.z, s);
    }
    
    static void addEntityToVis(int eid, String name, Pos p, neosim.implementation.Shape s, Vector v ) {//EntityID eid
	app.addEntityToVis(eid, name, p.x,p.y,p.z, s, v);
    }

    static void addConnToVis(  Pos sp, Pos dp ) {
	app.addConnToVis(   sp,  dp );
    }

    static void highlightDest(  int dest ) {//EntityID dest
	app.highlightDest(  dest );
    }
   
    static void flash(  int dest ) {//EntityID dest
	//app.highlightDest(  dest );
        app.flash(dest);
        
    }
    
 

    /** Called when user clicks on an entity
     */
    void clickConnections(int eid) {//EntityID eid
	//System.out.println("Netvis clockconnections called = "+eid);
 
	
	SimSystem.getKernel().queryOutputConnections( eid, getLocalTime(), eid, (short)1,//new PortID(
	     new ConnectionQuery() {
		     public Object query( Entity deste, Connection c ) {
			 if (false) System.out.println("Connection query running on "+deste.getName());
			 String name = deste.getName();
			 Pos p = new Pos(0,0,0);
			 try {
			     PositionState e3d = (PositionState)deste;
			     p = new Pos(e3d.getXPos(), e3d.getYPos(),e3d.getZPos());
			 } catch (ClassCastException e ) {
			     // not an e3d 
			 }
			 return new ConnData( deste.getID(), name, p );
		     }
		 }, 
	     new Callback() {		      
		     public void callback( Entity e, Object o ) {
			 Vector n = (Vector) o;

		 Pos srcp = new Pos(0,0,0);
			 try {
			     Entity3D e3d = (Entity3D)e;
			     srcp = e3d.getPos();
			 } catch (ClassCastException ex) {
			 }

			 for (int i=0; i<n.size(); i++) {
			     ConnData cd = (ConnData) n.elementAt(i);
			     if (false) 
				 System.out.println(e.getName()+" "+srcp+"  Connected to: "+cd.getName()+" Pos:"+cd.getPos());
			     if (!cd.getName().equals("netvis0")) {
				 //				 Netvis.addConnToVis( srcp, cd.getPos());
				 Netvis.highlightDest( cd.getDestID() );
			     }
			 }
		     }
                    public void readExternal(java.io.ObjectInput objectInput) throws java.io.IOException, java.lang.ClassNotFoundException { 
                    }
    
                    public void writeExternal(java.io.ObjectOutput objectOutput) throws java.io.IOException {
                    }                       
		 });
	//System.out.println("Netvis clockconnections sent out the query");

    }

    /** Implem of Pausable iface */
    public void restart() {
        s.semSignal();
    }

    void pauseme() { s.semWait(this); }

    

    boolean doneQuery = false;

    public void advance( Time endt ) {
        //JAN05 CHANGE 
        //rc.advance(endt.s());	    
        spTable.advance(rc, endt.t);
	
    }    
    
    public Time handleEvents(EventList el, Time endt) {
       //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
       //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp); 

        boolean passed = false;


        boolean goon=true;
        
	while (goon && !passed) { 
            Event e = null;
            Event e2 = null;
            synchronized(el){
                goon=el.hasMoreElements();
                if(goon){
                    e = el.top();
                    if (e.getTime().t <= endt.t) {
                        e2 = el.pop();
                    } else { 
                        passed = true; 
                    }
                }
            }
            if(goon&&!passed){
                advance( e2.getTime() );
		if (e2.getConnection().getDstPortID()==in) { // This is a bug!
		    Time outTime = e.getTime();
		    //trace("received spike event at "+outTime.s());
		    if (!doneQuery) {
			doneQuery = true;		
                    }
                } else {
                    if (e.isEntityEvent()) {
                        defaultEventHandler( e2 );
                    }                      
                }
            }
            advance( endt );
       }
       //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp); 
       //<-18/04/05
        return endt;
        
    }
    
    public synchronized int addInConnection(int srcID, short srcPort, short destPort) {
        int chose=super.addInConnection(srcID, srcPort, destPort);
        //System.out.println("NETVIS ADD CONN"); //ASSERT
        return chose;
    }
    
    public synchronized void CheckConnection(int TotalEntityToCall){
        while(this.NumberConnection!=TotalEntityToCall){
            try{
                //ASSERT
                System.out.println("Wait with "+NumberConnection+" connections out of "+TotalEntityToCall); 
                //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
                //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp);                
                wait();
                //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);                                
            }
            catch(Exception e){
                e.printStackTrace();
            }
        }
        app.clean();
        //neosim.kernel.CommandQueue.PrintObject(this.LinkedListCon, "NetvisConnections_before.txt");
        ((BasicKernel)this.sk).compress();        
      //  neosim.kernel.CommandQueue.PrintObject(this.LinkedListCon, "NetvisConnections_after.txt");
        //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp);          
        int[][] v = getInConnections(in);
        //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
 //       System.out.println("DEBUG Netvis: connected to "+v.size()+" entites");
        //Time t=new Time(getLocalTime());
        //t.add(new Time(0.1));        
	if (v==null){
            System.out.println("No input connections!!!");
	}
        else{
            Vector IdVector=new Vector();
/*            for(int i=0;i<v.size();i++){
                Connection c = (Connection)v.elementAt(i);
                if(c!=null&&c.getDstPortID()==1){
                    int myID=c.getSrcEntityID();//EntityID myID
                    if(!IdVector.contains(myID))
                        IdVector.add(myID);
                }
            }*/
           /* for(int i=0;i<v[0].length;i++)
                System.out.println("V[0]["+i+"] "+(v[0][i]==null?"":v[0][i]));*/

            if(v[1]!=null){//port 1
                 for(int j=0;j<v[1].length;j++){
                    int myID=v[1][j];
                    //ASSERT
                    //System.out.println("myID "+myID);
                    if(!IdVector.contains(myID))
                        IdVector.add(myID);
                 }
            }
            int TotalToReceive=0;
            for(int k=0;k<v.length;k++){
                if(v[k]!=null){//port 1
                    for (int i=0; i<v[k].length; i++) {
                        TotalToReceive++;
                    }
                }
            }  
            TreatPosition machin=new TreatPosition(TotalToReceive,IdVector);
            for(int k=0;k<v.length;k++){
                if(v[k]!=null){//port 1
                    for (int i=0; i<v[k].length; i++) {
                        int srce = v[k][i];//EntityID srce
                    //Parallel 01: 18/04/05->
                      //  queryEntity(getLocalTime(), srce, new OutputQuery(IdVector,getID()),new NullCallback());//was not commented before
                        //<-Parallel 01: 18/04/05->
                        queryEntity(getLocalTime(), srce, new OutputQuery(),new OutputCallback(machin));
                    }
                }
            }  
            //BuildingThread chose=new BuildingThread(machin);                        
        }

    }
}
final class SpikeFlasher implements neuroml.sim.run.RunnableCell{
    
    public SpikeFlasher(){
        
    }
    
    public void advance(double t) {
    }
    
    public Object getModel() {
        return null;
    }
    
    public ElementState getState() {
        return null;
    }
    
    public void handle(Object o, double t, int inputConnection) {
    }
    
    public void handleSpike(double t, int inputConnection) {
        //int srce = e2.getConnection().getSrcEntityID();//EntityID srce
        
        Netvis.flash(inputConnection);
    }
    
    public void reset() {
    }
    
    public void setInputConnectionParameters(int cid, neuroml.sim.run.DestMethod dm, neuroml.sim.run.ConnectionRequest cr) {
    }
    
    public void setMeasured(neuroml.model.MeasurementTool.MeasurementCollection MeasurementList) {
    }
    
    public void setMeasured(int SrcId, neuroml.model.MeasurementTool.MeasurementCollection MeasurementList) {
    }
    
}

final class Couple implements java.io.Serializable {
    public int[][] array;
    public Pos myPos;
    public int id;
    public String Name;
    public Couple(Pos myPos,int[][] array,int id,String Name){
        this.myPos=myPos;
        this.array=array;
        this.id=id;
        this.Name=Name;
    }
}



final class OutputQuery extends EntityQuery implements java.io.Externalizable {
    //transient neosim.kernel.DeadlockTracker tracker=new neosim.kernel.DeadlockTracker();
    public OutputQuery(){
    }
    
    public Object query( Entity deste) {
        //System.out.println("queryOutputConnections " +deste.getID());
        PositionState e2 = (PositionState)deste;
        Pos obj1=new Pos( e2.getXPos(), e2.getYPos(), e2.getZPos());
       // ((EntityImpl)deste).check_compressed();//compress_final();
        //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(deste,ecp);         
        int[][] TempArray=deste.getInConnections((short)1);
        //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp); 
        //neosim.kernel.CommandQueue.PrintObject(((EntityImpl)deste).LinkedListCon,"LinkedListCon_"+deste.getID()+".txt");
        return new Couple(obj1,TempArray,deste.getID(),deste.getName());
	////return null;
    }
    
    
    public void readExternal(java.io.ObjectInput objectInput) throws java.io.IOException, java.lang.ClassNotFoundException {      
    }
    
    public void writeExternal(java.io.ObjectOutput objectOutput) throws java.io.IOException {
    }    
}

final class TreatPosition implements java.io.Serializable{
    static int Total=0;
    //TreatPosition is somehow multiplied so you need the static (kind of bug)
    static int CountDown=0;
    static Vector v;
    static Hashtable array;
    static Vector IdVector;

    public TreatPosition(int Total, Vector IdVector){
        TreatPosition.IdVector=IdVector;
        TreatPosition.Total=Total;

        CountDown=Total;
        array=new Hashtable();
    }
    public synchronized void decount(Couple truc){
        CountDown--;
        //ASSERT
        //System.out.println(CountDown);
        array.put(new Integer(truc.id),truc);;
        if(CountDown==0){
            for (Enumeration enum_ = array.elements() ; enum_.hasMoreElements() ;) {
                Couple temp=(Couple)enum_.nextElement();
                int[][] v=temp.array;
                Vector PosConnections=new Vector();
                for(int k=0;k<v.length;k++){
                    if(v[k]!=null){//port 1
                        for (int j=0; j<v[k].length; j++) {
                            int srce = v[k][j];      
                            Integer intSrce=new Integer(srce);
                            if(IdVector.contains(intSrce)){
                                PosConnections.add(((Couple)(array.get(intSrce))).myPos);
                            }
                        }
                    }
                }
                Netvis.addEntityToVis(temp.id, temp.Name, temp.myPos, new neosim.implementation.Shape("red","sphere",0.3),(Vector) PosConnections );
            } 
            //ASSERT
            //System.out.println("CountDown Finished"); 
            //notifyAll();//notifty should be enough but since it is
        }
    }
}

final class OutputCallback implements Callback{
    TreatPosition stuff;
    public OutputCallback(TreatPosition stuff){
        this.stuff=stuff;
    }
    
    public void callback( Entity e, Object o ){
        stuff.decount((Couple)o);
      //  System.out.println("NullCallback " +e.getID());
    } 
}



final class NullCallback implements Callback{
    public NullCallback(){}
    
    public void callback( Entity e, Object o ){
      //  System.out.println("NullCallback " +e.getID());
    }
    public void readExternal(java.io.ObjectInput objectInput) throws java.io.IOException, java.lang.ClassNotFoundException { 
    }
    
    public void writeExternal(java.io.ObjectOutput objectOutput) throws java.io.IOException {
    }     
}



/*
final class OutputQuery extends EntityQuery implements java.io.Externalizable {
    Vector IdVector;
    int Id;
    public OutputQuery(){
        IdVector=new Vector(10,10);
    }
    
    public OutputQuery(Vector IdVector, int Id){
        this.IdVector=IdVector;
        this.Id=Id;
    }
    public Object query( Entity deste) {
        //System.out.println("queryOutputConnections " +deste.getID());
        deste.queryOutputConnections(deste.getLocalTime(),(short)1,new QueryBuildFullneuron(IdVector), new CallbackBuildFullneuron(Id));//new PortID(1)
	return null;
    }
    
    public void readExternal(java.io.ObjectInput objectInput) throws java.io.IOException, java.lang.ClassNotFoundException {
        Id=objectInput.readInt();
        int size=objectInput.readInt();
        for(int i=0;i<size;i++){
            IdVector.add(objectInput.readInt());//new EntityID(
        }        
    }
    
    public void writeExternal(java.io.ObjectOutput objectOutput) throws java.io.IOException {
        objectOutput.writeInt(Id);
        int size=IdVector.size();
        objectOutput.writeInt(size);
        for(int i=0;i<size;i++){
            objectOutput.writeInt(((Integer)IdVector.elementAt(i)).intValue());//(EntityID)IdVector.elementAt(i)).id
        }
    }
    
}

final class NullCallback implements Callback{
    public NullCallback(){}
    
    public void callback( Entity e, Object o ){
      //  System.out.println("NullCallback " +e.getID());
    }
    public void readExternal(java.io.ObjectInput objectInput) throws java.io.IOException, java.lang.ClassNotFoundException { 
    }
    
    public void writeExternal(java.io.ObjectOutput objectOutput) throws java.io.IOException {
    }     
}



final class QueryBuildFullneuron extends ConnectionQuery{
    Vector ListConnections;
    public QueryBuildFullneuron(Vector v){
        ListConnections=v;
    }
    
    public Object query( Entity deste, Connection c ) {
        //System.out.println("DEBUG QueryBuildFullneuron: "+c.getSrcEntityID().id+" "+deste.getID().id);
        int j=ListConnections.size();
        boolean continue_=true;
        int desteId=deste.getID();
        for(int i=0;i<j&&continue_;i++){
            if(((Integer)(ListConnections.elementAt(i))).intValue()==desteId )//c.getSrcEntityID() CHANGE_MARCH desteId //((EntityID)(ListConnections.elementAt(i))).id 
                continue_=false;
        }
	if(!continue_){
            try{
                RunnableCell myCell=((NeosimEntityImpl)deste).getRunnableCell();
                if(myCell instanceof PositionState){
                   // System.out.println("DEBUG QueryBuildFullneuron: return myCell");
                    return myCell;
                }
                else{
                  //  System.out.println("DEBUG QueryBuildFullneuron: return null 1");
                    return null;
                }
            }
            catch(Exception e){
              // System.out.println("DEBUG QueryBuildFullneuron: return null 2");
                return null;
            }
        }
        else {
           // System.out.println("DEBUG QueryBuildFullneuron: return null 3");
            return null;
        }
    }
}

final class CallbackBuildFullneuron implements Callback {
    int ToCall;
    public CallbackBuildFullneuron(){}
    
    public CallbackBuildFullneuron(int ToCall){
        this.ToCall=ToCall;
    }
    public void callback( Entity e, Object o ) {
	try {
            PositionState e2 = (PositionState) e;

          //  System.out.println(e.getName()+" :GetPos: Got an entity3D "+e2.getXPos()+ " "+e2.getYPos()+" "+ e2.getZPos());
            if(e2.getShape()==null){
          //      System.out.println("No shape");
                e2.setShape(new neosim.implementation.Shape("red","sphere",0.3));
            }
            //Netvis.addEntityToVis( e.getID(), e.getName(), new Pos( e2.getXPos(), e2.getYPos(), e2.getZPos()), (neosim.implementation.Shape)(e2.getShape()),(Vector) o );
            
  //Time t, EntityID e, EntityQuery eq, Callback c
            e.queryEntity(e.getLocalTime(), ToCall, new  NetvisQuery( e.getName(),e.getID(),new Pos( e2.getXPos(), e2.getYPos(), e2.getZPos()),(neosim.implementation.Shape)(e2.getShape()),(Vector) o), new NullCallback());//new EntityID(ToCall)
	} catch ( ClassCastException ex) { 
	    System.out.println("CallbackBuildFullneuron - exception "+ex);
	}
    }
    public void readExternal(java.io.ObjectInput objectInput) throws java.io.IOException, java.lang.ClassNotFoundException { 
        ToCall=objectInput.readInt();
    }
    
    public void writeExternal(java.io.ObjectOutput objectOutput) throws java.io.IOException {
        objectOutput.writeInt(ToCall);
    }     
}
*/

  
