/** 
 * Netvis control module
 * @author Fred Howell
 * @date Oct 2000
 * @version 1.0
 */

 
package neosim.renderer; 

import neosim.implementation.*;
import neosim.neuroml.*;
import neosim.iface.*;
import neosim.kernel.*;
import neosim.util.*;
import java.util.*;
import neuroml.sim.run.*;

import java.lang.reflect.*;

 

/** A 3D network visualiser for Neosim
 */	
public final class NetvisCtrl implements RunnableControl { //extends ControlModuleImpl {
  
    String msg="";
    Vector ToDisconnect=new Vector();
 //   PopulationImpl ToDestroy = null;
    NetvisClass netvisStuff=null;
    int NetvisId=0;
    static boolean VisualizationOnOff=true;
    Vector PopulationsToMonitor;
    Vector PopulationInSuperLayer;
    Vector PopulationInUnderLayer;
    
    /** This jar file requires ./SimpleProjection.jar to be present
     * We need a better way of doing this...  zip:./SimpleProjection.jar ???
     */
    //    String locn="http://www.anc.ed.ac.uk/~fwh/jars/SimpleProjection.jar";
    //String locn="file:./SimpleProjection.jar";

    String locn="file:C:/Zip_05_01/jars/NML_PLAY.jar"; //"file:C:/nvc/netvisctrl/clean/jars/SimpleProjection.jar";
    public NetvisCtrl() { } 
    
    public NetvisCtrl(Vector PopulationsToMonitor,Vector PopulationInSuperLayer,Vector PopulationInUnderLayer){
         setVectors(PopulationsToMonitor,PopulationInSuperLayer,PopulationInUnderLayer);
    } 
    
    public void setVectors(Vector PopulationsToMonitor,Vector PopulationInSuperLayer,Vector PopulationInUnderLayer){
         this.PopulationsToMonitor=PopulationsToMonitor;
         this.PopulationInUnderLayer=PopulationInUnderLayer;
         this.PopulationInSuperLayer=PopulationInSuperLayer;
    }
     
     
   
    /** Set initial parameters
     */
    public void init(Hashtable params) {
	//System.out.println("NetvisCtrl init");
	String m = (String)params.get("message");
	if (m!=null) msg = m;
    }
    public void run(neuroml.sim.run.Simulator simul) {
        if(!NetvisCtrl.VisualizationOnOff)
            return;
        System.out.println("=== Netvis Ctrl Manager ===");       
        ScriptReader sr=(ScriptReader)(simul.getReader());  
       PopulationTree root = sr.getPopulationTree();
       PopulationImpl ToDestroy = (PopulationImpl)root.getChild("netvis");
     //  System.out.println(this.getClass().getName()+" 01");
       
        if(ToDestroy==null){
         //   System.out.println(this.getClass().getName()+" 02a");
            netvisStuff=new NetvisClass(sr);           
            ToDestroy=new PopulationBuilderImpl( "netvis", 1,netvisStuff,null,null);//SOCKET_ERROR , root
            NetvisId=SimSystem.getKernel().getNbEntities();// getNumEntities();
            sr.construct( (PopulationBuilderImpl)ToDestroy);
        }
        else{
         //   System.out.println(this.getClass().getName()+" 02b");
            for(int i=0;i<ToDisconnect.size();i++)
            {
                Projection pr=(Projection)(ToDisconnect.elementAt(i));
                sr.disconnect(pr);
            }
        }
        ToDisconnect=new Vector();
  	
	Population nv = (Population)root.getChild( "netvis" );

	Vector nodes = root.getChildren();
        
        //((ScriptReaderImpl)sr).pauseme();
        //ASSERT
         //System.out.println("Add pauseseme to prevent(Semaphore) the application to go on without the connections being established");           
         int TotalEntityToConnect=0;
	for (int i=0; i<nodes.size(); i++) {
	    Population p = (Population)nodes.elementAt(i);
	    String name = p.getName();
	    if (!name.equals("netvis")) {
             
              //  System.out.println(this.getClass().getName()+" 03 "+i);
                if(PopulationsToMonitor.contains(name)){//PopulationInSuperLayer.contains(name)||PopulationsToMonitor.contains(name)||PopulationInUnderLayer.contains(name) //
                  //  System.out.println("Monitoring "+name);
                    Projection pr=new SimpleProjImpl( p.getName(), (short)1, nv.getName(), (short)1, new Time(RefreshVisualization) );//0.1?//new PortID(1)//new PortID(1)
                    sr.connect(pr);
                    TotalEntityToConnect+=p.getNumEnts();
                    ToDisconnect.add(pr);
                }
                if(PopulationInSuperLayer.contains(name)||PopulationInUnderLayer.contains(name)){//PopulationInSuperLayer.contains(name)||PopulationsToMonitor.contains(name)||PopulationInUnderLayer.contains(name) //
                 //   System.out.println("Monitoring "+name);
                    Projection pr=new SimpleProjImpl( p.getName(),(short)1, nv.getName(), (short)2, new Time(RefreshVisualization) );//0.1?//new PortID(1)//new PortID(2) //port=2?
                    sr.connect(pr);
                    TotalEntityToConnect+=p.getNumEnts();
                    ToDisconnect.add(pr);
                }
	    }
	}
        try{
            //Thread.sleep(100);//18/04/05 not required since it waits the right number of connections
            
     /** Query another entity at a given time
     * @param e the originating entity
     * @param t the time to query the entity (<b> must be >= localTime() + getMinOutputDelay()</b>)
     * @param d the handle of the entity to query
     * @param eq the query to perform	
     * @param c the callback to call with the results of the query
     */
            SimSystem.getKernel().queryEntity(SimSystem.getKernel().getID(),SimSystem.getKernel().getSysEntity().getLocalTime(), NetvisId,  new CleanQuery(TotalEntityToConnect) , new CleanCallback());//new EntityID(SimSystem.getKernel().getID()), new EntityID(NetvisId)
           // netvisStuff.clean();
        }
        catch(Exception excp){
            //System.out.println(excp);
            excp.printStackTrace();
        }
	//System.out.println("=== Netvis Ctrl Manager - running simulation... ");  
	//System.out.println("=== Netvis Ctrl Manager - waiting on a semphore... ");  
	//System.out.println("=== Netvis Ctrl Manager - leaving now. ===");  
    }

   
    /** Bootstrap - runs in main thread
     */
    public void run(ScriptReader sr) {
	System.out.println("=== Netvis Ctrl Manager ===");  
	Semaphore s = new Semaphore(0);
	

	// Make a netvis entity
  	PopulationTree root = sr.getPopulationTree();

	// Nasty way to pass script reader to netvis.
	sr.construct( new PopulationBuilderImpl( "netvis",  1,new NetvisClass(sr),null,null));//SOCKET_ERROR root,
	Population nv = (Population)root.getChild( "netvis" );
	
	// Connect all other populations to this netvis
	Population sp = (Population)root.getChild( "spikepop" );

	/**	Projection pr = ModuleLoader.getProjection(
						   "http://www.anc.ed.ac.uk/~fwh/jars/SimpleProjection.jar",
						   "fred",
						   sp, new PortID(1), 
						   nv, new PortID(1), 
						   new Time(2), new Hashtable());
						   sr.connect(pr);
	*/

	Vector nodes = root.getChildren();
	for (int i=0; i<nodes.size(); i++) {
	    Population p = (Population)nodes.elementAt(i);
	    String name = p.getName();
	    if (!name.equals("netvis")) {
                if(PopulationsToMonitor.contains(name)){
                    //System.out.println("Monitoring "+name);
                    Projection pr = ModuleLoader.getProjection(
							   locn,
							   "fred",
							   p.getName(), (short)1, //new PortID(1)
							   nv.getName(), (short)1, //new PortID(1)
							   new Time(2), new Hashtable());
                    sr.connect(pr);
                }
	    }
	}

	System.out.println("=== Netvis Ctrl Manager - running simulation... ");  
	sr.runSim(new Time(10.0));
	System.out.println("=== Netvis Ctrl Manager - waiting on a semphore... ");  
	s.semWait(this);
	System.out.println("=== Netvis Ctrl Manager - leaving now. ===");  	     
    }
    
    static double RefreshVisualization=0.1;
    
    static public void setRefreshVisualization(double RefreshVisualization,boolean bool){
        NetvisCtrl.RefreshVisualization=RefreshVisualization;
        NetvisCtrl.VisualizationOnOff=bool;
        
    }
    
    static public double getRefreshVisualization(){
        return RefreshVisualization;
    }
}

final class CleanQuery extends EntityQuery {
    int TotalEntityToCall;
    public CleanQuery(int truc){
        super();
        TotalEntityToCall=truc;
    }
    public Object query( Entity deste) {
        //System.out.println(deste.getClass().getName());
        try{
            //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
            //neosim.kernel.DeadlockTracker.LockDeadlockTracker(deste,ecp);             
            ((Netvis)deste).CheckConnection(TotalEntityToCall);
            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);  
        }catch(ClassCastException excp){
            neosim.kernel.CommandQueue.PrintObject(deste,"ClassCastException.txt");
            excp.printStackTrace();
        }
	return new Integer(1);
    }
}

final class CleanCallback implements Callback{
    public CleanCallback(){
    }
    
    public void callback( Entity e, Object o ){
        //System.out.println("Clean Callback Done");
    }
    public void readExternal(java.io.ObjectInput objectInput) throws java.io.IOException, java.lang.ClassNotFoundException { 
    }
    
    public void writeExternal(java.io.ObjectOutput objectOutput) throws java.io.IOException {
    }  
}