/** RMI based SimKernel
 */

package neosim.rmikernel;

import neosim.kernel.*;
import neosim.iface.*;
import neosim.implementation.*;
 
 
import java.rmi.*;
import java.net.*;
import java.io.*;
import java.util.Vector;
import java.rmi.server.*;
import java.util.Random;
import java.util.Hashtable;
import java.util.Enumeration;
 

/** RMI Simulation Kernel
 */
public final class RmiKernel extends BasicKernel {
  //  transient DeadlockTracker tracker=new DeadlockTracker();
    
    static int NbKernels=0;
    
    public static void main(String args[]){
        RmiKernel chose=new RmiKernel(0,10);
        chose.TestCompressionSpikes();
    }
    private RmiKernel(int kernel, int nprocs){
        //for test only
        super();
        
        this.nprocs=nprocs;
        AllgatherObject=new Object[nprocs];
        this.id=kernel;
    }
    
    double runt = 0.0;
    
    private boolean debug = false;
 
    /** Name of user, server */
    String username, servername;

    int nprocs;
    int nthreads;
    int AllgatherArray[];
    Object AllgatherObject[];
    RmtServer  server;

    public RmiKernel(String sname, String uname, int nprocs, int nthreads) {
	super();
        
        NbKernels++;
        System.out.println("Nb Kernels = "+NbKernels);
        
	username   = uname;
	servername = sname;
	this.nprocs = nprocs;
	this.nthreads = nthreads;
        if(neosim.util.loaders.nmllauncher.NeosimNeuroMLLauncher.Player!=null)
                neosim.util.loaders.nmllauncher.NeosimNeuroMLLauncher.Player.println("RMI Kernel: "+sname+" "+nprocs+" procs, "+nthreads+" threads");
        if (debug) System.out.println("RMI Kernel: "+sname+" "+nprocs+" procs, "+nthreads+" threads");
 	System.setSecurityManager(new RMISecurityManager());
	System.out.println("RmiKernel:  set security manager");
         if(neosim.util.loaders.nmllauncher.NeosimNeuroMLLauncher.Player!=null)
                neosim.util.loaders.nmllauncher.NeosimNeuroMLLauncher.Player.println("Security manager set");       
        try{
                UnicastRemoteObject.exportObject(this);
	              
                server = (RmtServer)Naming.lookup("//" + servername + "/RmiServer");
                pmgr   = (RmtProcMgr)Naming.lookup("//" + servername + "/RmiServer");    
                if(neosim.util.loaders.nmllauncher.NeosimNeuroMLLauncher.Player!=null)
                    neosim.util.loaders.nmllauncher.NeosimNeuroMLLauncher.Player.println("Found Server "+servername);                       
        } catch (Exception e) {
	    System.out.println("RmiKernel exception: " + e.getMessage());
	    e.printStackTrace();
	}        
	// SimSystem.setKernel( this ); // needed?
    }

    public final RmtServer getServer()    { return server; }
  
   
    /** Kill all threads... */
    public final void terminate() {	
	super.terminate();
        if(cq!=null)
                cq.terminate();
        
    }

    Integer BarrierInteger=new Integer(id);
    /** Barrier  - using allgather */
    int CounterServer=0;   
    public final void barrier()  {
        //Huge BUG in RMI : it loses the track of the server when the server is not called for a long period... So let's call it sometimes.
      /*  CounterServer++;
        if(CounterServer==100){
            CounterServer=0;
            try{
                server.call();
            }catch(Exception e){
                e.printStackTrace();
            }
        }*/
            
        //try{
            this.barrierSocket();
            //allgather(BarrierInteger);
        //}catch(neosim.kernel.ResetException e){  e.printStackTrace(); }
    }
    
    public final void barrier_server(){
        //to be used when we don't know each other
        try {
            server.barrier();
        } catch (RemoteException e) {
            e.printStackTrace();
        }        
    }
 

    /** Run the simulation */
    public final void runRmt(int rid, Time t) {
	//if (debug) System.out.println("Run Rmt starting, "+getGroupsize()+" procs, "+nthreads+" local threads");
        compress();       
	getSimSystem().runPar(getGroupsize(),nthreads,getID(),t);
	//if (debug) System.out.println("Run Rmt done");
    }
 
 
    /** Send command to given dest 
     */ 
    public final void sendCommand(int p, CommandObj co) {
	CommandQueue cq = ((CommandQueue)othercqs.elementAt(p));
      /*  if(co instanceof EventToCommand){
            Event ev=((EventToCommand)co).getEvent();
            if(ev instanceof EntityCallbackEvent){
                new Exception().printStackTrace();
                System.out.println(getClass().getName()+" "+ev.getClass().getName()+" ... Just saying I am there since the stack is SO big ");
            }
        }       */ 

	if (cq!=null) {
            boolean goon=true;
            while(goon){
                try {
                    goon=false ;
                    //neosim.kernel.Deadlock_Message ecp1=new neosim.kernel.Deadlock_Message();
                    //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp1);                 
                    cq.add(co);
                    //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp1);

                } catch (Exception e) { 
                    try{
                        System.out.println(" ... sendCommand exception "+e);
                        neosim.kernel.CommandQueue.PrintObject(co,"ErrorsCommanqueueAdd_"+getID()+".txt");
                        java.io.PrintWriter trcstream = new java.io.PrintWriter(new java.io.FileOutputStream("CommandQueueAdd_1_Error.txt"));
                        e.printStackTrace(trcstream);
                        trcstream.close();
                        Thread.sleep(15000);                
                    }
                    catch(Exception x){
                        x.printStackTrace();
                    }
                }
	    }
	}
    }

    /** Issue orders to system 
     */
    public final void sendCommand(CommandObj co) {
	try { 
	    for (int p=0; p<otherprocs.size(); p++) {
		sendCommand(p, co);
	    }    
	    //	    getServer().command( co );
	} catch (Exception e) {	   
	    System.out.println("Failed to construct "+e.getMessage());
 	}
    }

    /** Issue orders to system, return response as a vector
     */
    public final void syncCommand(QueryCommand qc) {
        qc.setQid(qid++);
        try { 
	    for (int p=0; p<otherprocs.size(); p++) {
		sendCommand(p, qc);
	    }    
	    // server based version: getServer().command( qc );
        } catch (Exception e) {    
            System.out.println("Failed to construct "+e.getMessage());
        }
        /** wait for responses */
        Vector v = waitForAnswers( qc.getQid() );      
    }

 
   
    // Called by a destroyKernel command
    public final void destroyRmt(int rid) {
        //commented 18/04/05->
	/*try { server.done(id,this); } catch (Exception e) {
	    System.out.println("Exception in server.done :"+e);
	}

	// Wait for done signal from all others
	for (int i=0; i<otherprocs.size(); i++) {
	    if (debug)  System.out.println("Waiting for kernel done :"+i);
	    kernelDone.semWait();
	}*/
        //<-commented18/04/05
        neosim.kernel.DeadlockTracker.tracker.kill();
	barrier();
	if (debug) 	System.out.println("Kernels all done, killing threads");
     
	try { server.remove(id,this); } catch (Exception e) {
            e.printStackTrace();
	    // This will throw an exception if the server suicides.
	}
	if(rid!=-1){
            try {
	    //boolean success = true;
                boolean success = UnicastRemoteObject.unexportObject(this, true);
                if (success) { 
                    if (debug) System.out.println("Unexported RMIKernel OK"); 
                } else {
                    System.out.println("Failed to unexport RMIKernel"); 
                }
            } catch (Exception e) { 
                System.out.println("Unexport error " +e);
            }
        }
        if(bserverSocket!=null){
            try {
                for(int i=0;i<bArraySocket.length;i++){
                    if(bArraySocket[i]!=null){
                        //ArraySocket[i].close();
                        bArraySocket[i].close();
                        //osArray[i].close();
                        //isArray[i].close();
                        bosArray[i].close();
                        bisArray[i].close();
                    }
                }
                bserverSocket.close();                
//                serverSocket.close();
           }catch (IOException e) {
               e.printStackTrace();
           }
        }        

	// Kill local threads
	terminate();
    }

    /** Version of addRmt which 
     * signals a semaphore when someone is added
     */
    Semaphore kernelAdded = new Semaphore(0);
    public final void addRmt(int rid, String n, RmtApp obj) {
	if (debug) System.out.println("["+id+"] RmiKernel: addRmt called");
	super.addRmt(rid,n,obj);
	kernelAdded.semSignal();
    }
	  
    /** Some processor has finished its tasks
     */
    Semaphore kernelDone = new Semaphore(0);
    public final void doneRmt(int rid) {
	super.doneRmt(rid);
	kernelDone.semSignal();
    }

    /** Version of removeRmt which signals a 
     * semaphore when someone leaves.
     */
    Semaphore kernelRemoved = new Semaphore(0);
    public final void removeRmt(int rid) {   
	super.removeRmt(rid);
	kernelRemoved.semSignal();	
    }



    public final void init() {
          
	if (debug) System.out.println("RmiKernel: connecting to server");
	    // Create and install a security manager
        
	try {
	    
	    if (debug) System.out.println("RmiKernel: calling server add");
            
            java.net.InetAddress idr = java.net.InetAddress.getLocalHost();
            String str=idr.getHostAddress();
            String str2=str.substring(0,str.indexOf('.'));
            byte a0=(byte)Integer.parseInt(str2);
            str=str.substring(str.indexOf('.')+1);
            str2=str.substring(0,str.indexOf('.'));
            byte a1=(byte)Integer.parseInt(str2);
            str=str.substring(str.indexOf('.')+1);
            str2=str.substring(0,str.indexOf('.'));
            byte a2=(byte)Integer.parseInt(str2);
            str=str.substring(str.indexOf('.')+1);
            byte a3=(byte)Integer.parseInt(str2);
            int IPint=a0;
            IPint<<=8;
            IPint|=(int)(a1&0xFF);
            IPint<<=8;
            IPint|=(int)(a2&0xFF);
            IPint<<=8;
            IPint|=(int)(a3&0xFF);
            
	    id = server.add(this,username,IPint);
            this.arraySpikes=new SpikeArray(id);
	    //	    getState(id).name = username;

	    /* Create new command queue */

	    cq = new CommandQueue(server);
	    cq.setQName("TO_SERVER");
	    cq.start();

	    /* Initialise local SimSystem */
            if(neosim.util.loaders.nmllauncher.NeosimNeuroMLLauncher.Player!=null)
                neosim.util.loaders.nmllauncher.NeosimNeuroMLLauncher.Player.println("Initialise local SimSystem");            
	    getSimSystem().initialise(this, id);

	    /* wait for all kernels to add */
	    if (debug) System.out.println("["+id+"]--- waiting for processors...");
	    for (int i=0; i<nprocs; i++) {
		kernelAdded.semWait(this);
		if (debug) 	System.out.print(" "+ (nprocs - i));
	    }
	    if (debug) 	System.out.println("["+id+"]--- Got all processors, trying a barrier");
       
            if(neosim.util.loaders.nmllauncher.NeosimNeuroMLLauncher.Player!=null)
                neosim.util.loaders.nmllauncher.NeosimNeuroMLLauncher.Player.println("Waiting on barrier");

	    //barrier_server();            
            AllgatherArray=server.sortIPs(id);
            AllgatherObject=new Object[nprocs];
            this.CreateSockets();
            
            if(neosim.util.loaders.nmllauncher.NeosimNeuroMLLauncher.Player!=null)
                neosim.util.loaders.nmllauncher.NeosimNeuroMLLauncher.Player.println("Done barrier");            
            System.out.println("Done barrier");
            TestsSocket();
            //ASSERT
           // TestAllgatherSpeed();
            //0, the starting application,othercqs.size(): the last added Kernel, so to speak the kernel running on the server
            //tests(0,this.othercqs.size()-1);
            // EventTests();
            //tests_Library(0,this.othercqs.size()-1);
	    if (debug) 	System.out.println("["+id+"]--- Done barrier");
 
	} catch (Exception e) {
	    System.out.println("RmiKernel exception: " + e.getMessage());
	    e.printStackTrace();
	}
       // neosim.kernel.DeadlockTracker.tracker.start();        
    }

   
    /** Extra commands from SimKernel */
    public final void reset() {  
        sendCommand( new RestartCommand( id ) );
        
         /*barrier();*/
        System.out.println("Reset Done");
        //sendCommand( new RestartCommand( -1) );//id   
    }
    /*public void resetRmt(int rid) {	
        System.out.println("Before barrier");
        barrier();
        System.out.println("After barrier");
  	super.resetRmt(rid);
    }*/
    
    public final void restartRmt(int rid) {
        if(id!=nprocs-1){ 
            destroyRmt(-1);    
            barrier_server();
            getSimSystem().initialise(this, id);
            this.kernelInit();
            System.out.println("REBUILDING");
            init();
        }
        else{
            destroyRmt(0);
            barrier_server();            
        }
       

    }
    //JAG DANGEROUS OPTIMIZATION
    public final Vector allgather_(Object o)  throws ResetException {
        Vector v;
        try{
            v=server.allgather(o);
        }
        catch(Exception e){
            e.printStackTrace();
            return null;
        }
        return v;
    }  

    public final Vector allgather(Object o) throws ResetException {
            return this.AllGather_obj(o);
    }    
    
    //These test are performed after the RmtApp have been added and exchanged; they are mainly designed to see how quick exchanges between the local Application and a remote object;

    void EventTests(){
        long start, end;
       
           
        
        System.out.println("");
        System.out.println("*************************************");
        System.out.println("         Event tests");
        System.out.println("*************************************");
        System.out.println("");       
        Integer EventMessage=new Integer(1000000);
        Vector VecMessage=new Vector(2);
        start=System.currentTimeMillis();

        for(int i=0;i<10000;i++){
            try{
                VecMessage=super.allgather(EventMessage);
            }
            catch(neosim.kernel.ResetException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("Test 1: "+(end-start)+" ms");            
        start=System.currentTimeMillis();

        for(int i=0;i<10000;i++){
            try{
                VecMessage=allgather(EventMessage);
            }
            catch(neosim.kernel.ResetException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("Test 2: "+(end-start)+" ms");  
        System.out.println("tests done. bye.");
        barrier();
    }
    
    void TestAllgatherSpeed(){
        long start=System.currentTimeMillis();
        Integer truc=new Integer(0);
        for(int i=0;i<100;i++){
            try{
                allgather(truc);
            }
            catch(Exception e){
                e.printStackTrace();
            }
        }
        long end=System.currentTimeMillis();
        System.out.println("AllgatherSpeed "+(end-start)+" ms");
        int array[]=new int[10000000];
        for(int i=0;i<10000000;i++)
            array[i]=i+1;
        
        start=System.currentTimeMillis();
        for(int j=0;j<10;j++){
            for(int i=1;i<10000000;i++){
                array[i]=(int)array[i-1]*array[i];
            }   
        }
        end=System.currentTimeMillis();
        
        System.out.println("Multi "+(end-start)+" ms");
            

    }
    
    void tests(int idTested,int RemoteObjectId){
        if(this.id!=idTested){
//Only the client whose id=idTested make the tests, others wait for client 0 (the starting, remote client) to make the test            
            barrier();
            System.out.println("tests done. bye.");
            System.exit(0);
        }
        CommandQueue cq=(CommandQueue)this.othercqs.elementAt(RemoteObjectId);
        RmtApp remoteKernel=(RmtApp)cq.getDest();
        long start,end;
        MiscTests.SerializedClass obj;
        
/**        
 *          Serialization tests
 */
        System.out.println("");
        System.out.println("*************************************");
        System.out.println("         Serialization tests");
        System.out.println("*************************************");
        System.out.println("");
        obj=new MiscTests.Serialized_Class();
        start=System.currentTimeMillis();

        for(int i=0;i<10000;i++){
            try{
                server.sendObject(obj);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println(obj.getClass().getName()+" "+(end-start)+" ms");

        obj=new MiscTests.ExternalizedClass();
        start=System.currentTimeMillis();

        for(int i=0;i<10000;i++){
            try{
                server.sendObject(obj);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println(obj.getClass().getName()+" "+(end-start)+" ms");        

        obj=new MiscTests.ExternalizedOptimalClass(true);
        start=System.currentTimeMillis();

        for(int i=0;i<10000;i++){
            try{
                server.sendObject(obj);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println(obj.getClass().getName()+" "+(end-start)+" ms");         

        MiscTests.ExternalizedOptimalClass serializedObject=new MiscTests.ExternalizedOptimalClass();
        start=System.currentTimeMillis();


        serializedObject.compression();
        end=System.currentTimeMillis();
        System.out.println("COMPRESSION "+(end-start)+" ms");  
        
        
        start=System.currentTimeMillis();

        for(int i=0;i<10000;i++){
            try{
                server.call();
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("CALL "+(end-start)+" ms");            
        
/**        
 *          RMI tests
 */        
        System.out.println("");
        System.out.println("*************************************");
        System.out.println("         RMI tests");
        System.out.println("*************************************");
        System.out.println("");        
//TEST_1        
        start=System.currentTimeMillis();

        for(int i=0;i<100000;i++){
            try{
                server.call();
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=100000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 1: "+(end-start)+" ms");   

//TEST_2        
        start=System.currentTimeMillis();

        for(int i=0;i<100000;i++){
            try{
                remoteKernel.call();
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=100000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 2: "+(end-start)+" ms");         
        
//TEST_3(1)        
        byte[] arrayByte=new byte[10];  
        for(int i=0;i<10;i++){
            arrayByte[i]=(byte)(i&0xFF);
        }        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                server.sendByte(arrayByte);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 3(1): "+(end-start)+" ms");
//TEST_3(2)        
        arrayByte=new byte[100];  
        for(int i=0;i<100;i++){
            arrayByte[i]=(byte)(i&0xFF);
        }        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                server.sendByte(arrayByte);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 3(2): "+(end-start)+" ms");   
//TEST_3(3)        
        arrayByte=new byte[1000];
        for(int i=0;i<1000;i++){
            arrayByte[i]=(byte)(i&0xFF);
        }        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                server.sendByte(arrayByte);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 3(3): "+(end-start)+" ms");   
//TEST_3(4)        
        arrayByte=new byte[10000];  
        for(int i=0;i<10000;i++){
            arrayByte[i]=(byte)(i&0xFF);
        }        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                server.sendByte(arrayByte);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 3(4): "+(end-start)+" ms");          

        
//TEST_4(1)        
       arrayByte=new byte[10];  
        for(int i=0;i<10;i++){
            arrayByte[i]=(byte)(i&0xFF);
        }        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                remoteKernel.sendByte(arrayByte);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 4(1): "+(end-start)+" ms");
//TEST_4(2)        
        arrayByte=new byte[100];  
        for(int i=0;i<100;i++){
            arrayByte[i]=(byte)(i&0xFF);
        }        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                remoteKernel.sendByte(arrayByte);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 4(2): "+(end-start)+" ms");   
//TEST_4(3)        
        arrayByte=new byte[1000];
        for(int i=0;i<1000;i++){
            arrayByte[i]=(byte)(i&0xFF);
        }        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                remoteKernel.sendByte(arrayByte);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 3(3): "+(end-start)+" ms");   
//TEST_4(4)        
        arrayByte=new byte[10000];  
        for(int i=0;i<10000;i++){
            arrayByte[i]=(byte)(i&0xFF);
        }        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                remoteKernel.sendByte(arrayByte);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 4(3): "+(end-start)+" ms");           
        
//TEST_5(1)        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                arrayByte=server.receiveByte((byte)0);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 5(1): "+(end-start)+" ms");    
//TEST_5(2)        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                arrayByte=server.receiveByte((byte)1);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 5(2): "+(end-start)+" ms"); 
//TEST_5(3)        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                arrayByte=server.receiveByte((byte)2);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 5(3): "+(end-start)+" ms");
//TEST_5(4)        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                arrayByte=server.receiveByte((byte)3);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 5(4): "+(end-start)+" ms");   

        arrayByte=new byte[1000];
        for(int i=0;i<1000;i++){
            arrayByte[i]=(byte)(i&0xFF);
        }
//TEST_6(1)        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                arrayByte=remoteKernel.receiveByte((byte)0);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 6(1): "+(end-start)+" ms");    
//TEST_6(2)        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                arrayByte=remoteKernel.receiveByte((byte)1);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 6(2): "+(end-start)+" ms"); 
//TEST_6(3)        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                arrayByte=server.receiveByte((byte)2);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 6(3): "+(end-start)+" ms");
//TEST_6(4)        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                arrayByte=remoteKernel.receiveByte((byte)3);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 6(4): "+(end-start)+" ms");   

        arrayByte=new byte[1000];
        for(int i=0;i<1000;i++){
            arrayByte[i]=(byte)(i&0xFF);
        }        
//TEST_7        
        arrayByte=new byte[1000];
        for(int i=0;i<1000;i++){
            arrayByte[i]=(byte)(i&0xFF);
        }       
        long result;
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                result=server.sum(arrayByte);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 7: "+(end-start)+" ms");  
    
//TEST_8(1)        
        int[] arrayInt=new int[10];  
        for(int i=0;i<10;i++){
            arrayInt[i]=(int)(i&0xFF);
        }        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                server.sendInt(arrayInt);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 8(1): "+(end-start)+" ms");
//TEST_8(2)        
        arrayInt=new int[100];  
        for(int i=0;i<100;i++){
            arrayInt[i]=(int)(i&0xFF);
        }        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                server.sendInt(arrayInt);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 8(2): "+(end-start)+" ms");   
//TEST_8(3)        
        arrayInt=new int[1000];
        for(int i=0;i<1000;i++){
            arrayInt[i]=(int)(i&0xFF);
        }        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                server.sendInt(arrayInt);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 8(3): "+(end-start)+" ms");   
//TEST_8(4)        
        arrayInt=new int[10000];  
        for(int i=0;i<10000;i++){
            arrayInt[i]=(int)(i&0xFF);
        }        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                server.sendInt(arrayInt);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 8(3): "+(end-start)+" ms");          

        
//TEST_9(1)        
       arrayInt=new int[10];  
        for(int i=0;i<10;i++){
            arrayInt[i]=(int)(i&0xFF);
        }        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                remoteKernel.sendInt(arrayInt);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 9(1): "+(end-start)+" ms");
//TEST_9(2)        
        arrayInt=new int[100];  
        for(int i=0;i<100;i++){
            arrayInt[i]=(int)(i&0xFF);
        }        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                remoteKernel.sendInt(arrayInt);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 9(2): "+(end-start)+" ms");   
//TEST_9(3)        
        arrayInt=new int[1000];
        for(int i=0;i<1000;i++){
            arrayInt[i]=(int)(i&0xFF);
        }        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                remoteKernel.sendInt(arrayInt);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 9(3): "+(end-start)+" ms");   
//TEST_9(4)        
        arrayInt=new int[10000];  
        for(int i=0;i<10000;i++){
            arrayInt[i]=(int)(i&0xFF);
        }        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                remoteKernel.sendInt(arrayInt);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 9(4): "+(end-start)+" ms");           
        
//TEST_10(1)        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                arrayInt=server.receiveInt((byte)0);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 10(1): "+(end-start)+" ms");    
//TEST_10(2)        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                arrayInt=server.receiveInt((byte)1);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 10(2): "+(end-start)+" ms"); 
//TEST_10(3)        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                arrayInt=server.receiveInt((byte)2);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 10(3): "+(end-start)+" ms");
//TEST_10(4)        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                arrayInt=server.receiveInt((byte)3);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 10(4): "+(end-start)+" ms");   

        arrayInt=new int[1000];
        for(int i=0;i<1000;i++){
            arrayInt[i]=(int)(i&0xFF);
        }
//TEST_11(1)        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                arrayInt=remoteKernel.receiveInt((byte)0);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 11(1): "+(end-start)+" ms");    
//TEST_11(2)        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                arrayInt=remoteKernel.receiveInt((byte)1);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 11(2): "+(end-start)+" ms"); 
//TEST_11(3)        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                arrayInt=server.receiveInt((byte)2);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 11(3): "+(end-start)+" ms");
//TEST_11(4)        
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                arrayInt=remoteKernel.receiveInt((byte)3);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 11(4): "+(end-start)+" ms");   

        arrayInt=new int[1000];
        for(int i=0;i<1000;i++){
            arrayInt[i]=(int)(i&0xFF);
        }        
//TEST_12        
        arrayInt=new int[1000];
        for(int i=0;i<1000;i++){
            arrayInt[i]=i;
        }       
        start=System.currentTimeMillis();
        for(int i=0;i<10000;i++){
            try{
                result=server.sum(arrayInt);
            }
            catch(java.rmi.RemoteException e){
                System.out.println("Reached "+i);
                e.printStackTrace();
                i=10000;
            }
        }
        end=System.currentTimeMillis();
        System.out.println("TEST 12: "+(end-start)+" ms");  

        //barrier with others, after all RmtApp have been added and exchanged
        barrier();        
        System.out.println("tests done. bye.");
        System.exit(0);
    }       
    
    void tests_Library(int idTested,int RemoteObjectId){
        if(this.id!=idTested){
//Only the client whose id=idTested make the tests, others wait for client 0 (the starting, remote client) to make the test            
            barrier();
            System.out.println("tests done. bye.");
        
        }
        CommandQueue cq=(CommandQueue)this.othercqs.elementAt(RemoteObjectId);
        RmtApp remoteKernel=(RmtApp)cq.getDest();        
        System.out.println("ID: "+idTested+" trying to build Holger_modules.module.destmethod.SpikeDestMethod on "+idTested);
        //Holger_modules.module.I_ClampImpl.I_ClampImpl obj=new Holger_modules.module.I_ClampImpl.I_ClampImpl();
        Holger_modules.module.destmethod.SpikeDestMethod obj=new Holger_modules.module.destmethod.SpikeDestMethod();
        System.out.println("ID: "+idTested+" trying to build Holger_modules.module.destmethod.SpikeDestMethod on "+RemoteObjectId);
        try{
            remoteKernel.call();
            barrier();        
            System.out.println("tests done. bye.");            
        }
        catch(Exception e){
            e.printStackTrace();
        }      
    }
    public final void SendArray(Object[] Array)  throws java.rmi.RemoteException {
        try{
            //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
            //neosim.kernel.DeadlockTracker.LockDeadlockTracker(Array,ecp);
            synchronized(AllgatherObject){
                //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
                for(int i=0;i<Array.length;i++){
                    if(Array[i]!=null&&AllgatherObject[i]==null){
                        AllgatherObject[i]=Array[i];
                    }
                }
                AllgatherObject.notify();
            }
        }
        catch(Exception e){
            e.printStackTrace();
        }
    }

    protected Vector AllGather_obj(Object obj){

        int length=AllgatherArray.length;
        Vector v;
        
        //gather
   /*     for(int i=0;i<length;i++)
            System.out.println(id+" : "+AllgatherArray[i]); */
        //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(obj,ecp); 
        synchronized(AllgatherObject){
            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
            AllgatherObject[id]=obj;
            for(int i=0;i<length;i++){
                if(AllgatherArray[i]<0){
                    CommandQueue cq=(CommandQueue)this.othercqs.elementAt(-AllgatherArray[i]-1);
                    RmtApp remoteKernel=(RmtApp)cq.getDest();
                    try{
                        remoteKernel.SendArray(AllgatherObject);
                    }
                    catch(java.lang.Exception e){
                        e.printStackTrace();
                    }      
                }
                else if(AllgatherArray[i]>0){
                    while(AllgatherObject[AllgatherArray[i]-1]==null)//if
                        try{
                            //neosim.kernel.Deadlock_Message ecp2=new neosim.kernel.Deadlock_Message();
                            //neosim.kernel.DeadlockTracker.LockDeadlockTracker(obj,ecp2);           
                            AllgatherObject.wait();                       
                            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp2);
                        } catch(Exception e){
                            e.printStackTrace();
                        }
                }
            }
        //distribute
            for(int i=length-1;i>=0;i--){
                if(AllgatherArray[i]>0){
                    CommandQueue cq=(CommandQueue)this.othercqs.elementAt(AllgatherArray[i]-1);
                    RmtApp remoteKernel=(RmtApp)cq.getDest(); 
                    try{
                        remoteKernel.SendArray(AllgatherObject);
                    }
                    catch(java.lang.Exception e){
                        e.printStackTrace();
                    }                    
                //send my guy
                //notify
                }
                else if(AllgatherArray[i]<0){
                    while(AllgatherObject[-AllgatherArray[i]-1]==null)//if
                        try{
                            //neosim.kernel.Deadlock_Message ecp3=new neosim.kernel.Deadlock_Message();
                            //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp3);                              
                            AllgatherObject.wait();    
                            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp3);                            
                        }
                        catch(Exception e){
                            e.printStackTrace();
                        }                                              
                }
            }
            v=new Vector();
            for(int i=0;i<nprocs;i++){
                v.addElement(AllgatherObject[i]);
                AllgatherObject[i]=null;
            }   
        }
        return v;
    }
    
    public void signal() throws java.rmi.RemoteException {
    }
    
    String IPs[];
    java.net.InetAddress idr;
 //   ServerSocket serverSocket;
    ServerSocket bserverSocket;
//    Socket ArraySocket[];
    Socket bArraySocket[];    
//    OutputStream osArray[];
//    InputStream isArray[];
    
    BufferedInputStream bisArray[];
    BufferedOutputStream bosArray[];
    
    /* Socket Communication-conversion + compression to byte arrays */
    
    public final void CreateSockets(){       
        try {
 //           serverSocket = new ServerSocket(4500+this.id);
            bserverSocket = new ServerSocket(4500+this.id+nprocs);
        }catch (IOException e) {
            
            System.err.println("Could not listen on port: "+(4500+this.id));
            e.printStackTrace();
            System.exit(1);
        }        
        try{
            idr = java.net.InetAddress.getLocalHost();
        }
        catch(java.net.UnknownHostException e){
            e.printStackTrace();
        }
        String str=idr.getHostAddress();
        Vector v=AllGather_obj(str);
        IPs=new String[v.size()];
        for(int i=0;i<IPs.length;i++){
            IPs[i]=(String)(v.elementAt(i));
        }
 //       ArraySocket=new Socket[AllgatherArray.length];
 //       osArray=new OutputStream[AllgatherArray.length];
 //       isArray=new InputStream[AllgatherArray.length];
        bArraySocket=new Socket[AllgatherArray.length];
        bosArray=new BufferedOutputStream[AllgatherArray.length];
        bisArray=new BufferedInputStream[AllgatherArray.length];
        
        for(int i=0;i<AllgatherArray.length;i++){
            try {
                if(AllgatherArray[i]>0){
 //                   ArraySocket[i] = new Socket(IPs[AllgatherArray[i]-1],4500+AllgatherArray[i]-1);
 //                   osArray[i]=ArraySocket[i].getOutputStream();
 //                   isArray[i]=ArraySocket[i].getInputStream();
                    
                    bArraySocket[i] = new Socket(IPs[AllgatherArray[i]-1],4500+AllgatherArray[i]-1+nprocs);
                    bosArray[i]=new BufferedOutputStream(bArraySocket[i].getOutputStream(),100);
                    bisArray[i]=new BufferedInputStream(bArraySocket[i].getInputStream(),100);                    
                }
                else if(AllgatherArray[i]<0){
//                    ArraySocket[i] = serverSocket.accept(); 
//                    osArray[i]=ArraySocket[i].getOutputStream();
//                    isArray[i]=ArraySocket[i].getInputStream(); 
                    
                    bArraySocket[i] = bserverSocket.accept();  
                    bosArray[i]=new BufferedOutputStream(bArraySocket[i].getOutputStream(),100);
                    bisArray[i]=new BufferedInputStream(bArraySocket[i].getInputStream(),100);                        
                }
            } catch (java.net.UnknownHostException e) {
               e.printStackTrace();
            } catch (IOException e) {
               e.printStackTrace();
            }
            //ensure they are build in proper order
            //the allgather type of barrier is the fastest available at this stage of the kernel construction
            this.AllGather_obj(this.BarrierInteger);
            
        }
        arrayBarrierSocket=new boolean[nprocs];     
        for(int i=0;i<nprocs;i++)
            arrayBarrierSocket[i]=false;            
    }
    
    byte ArrayOfSpikes[]=new byte[0];
    
    private final void convert(SpikeArray spikes){
        int size=0;//1 at least per SpikeArray, for the description of the size of the SpikeArray (0 if unknown)
        int comp_size,compsize2;

        comp_size=spikes.getSize();
	size+=comp_size;
        compsize2=comp_size;
        while((comp_size&~0x7F)!=0){ //count bytes necessary to express comp_size if comp_size>128
            size++;
            comp_size>>>=7;
        }
        ArrayOfSpikes=new byte[size+nprocs];
        int inc=id; //fill with 0 before and after the bytes describing "spikes" 
        while ((compsize2 & ~0x7F) != 0) {
	    byte b = (byte)((compsize2 & 0x7f) | 0x80);
	    ArrayOfSpikes[inc++]=b;	    
	    compsize2 >>>= 7;
	}
	ArrayOfSpikes[inc++]=(byte)compsize2;        
        
        spikes.fill(ArrayOfSpikes,inc);
    }
   
    
    private final void include(byte byteArray[]){
        byte[] TempArray;
        int inc1=0,inc2=0,former_inc=0;
        byte b1,b2;
        int i1,i2;
        int totaltoAdd=0;  
        //resizing
        for(int k=0;k<nprocs;k++){
            former_inc=inc1;
           // try{
                b1 = byteArray[inc1++];
           /* }
            catch(Exception e){
                System.out.println("BYTE_ARRAY length = "+byteArray.length+"ArrayOfSpikes length = "+ArrayOfSpikes.length);
                
              //  neosim.kernel.CommandQueue.PrintObject(byteArray, "BYTE_ARRAY.txt");
              //  neosim.kernel.CommandQueue.PrintObject(ArrayOfSpikes, "ArrayOfSpikes.txt");
                e.printStackTrace();
                return;
            }*/
	
            i1 = b1 & 0x7F;
            for (int shift = 7; (b1 & 0x80) != 0; shift += 7) {
                b1 = byteArray[inc1++];
                i1 |= (b1 & 0x7F) << shift;
            }
        
            b2=ArrayOfSpikes[inc2++];
            i2 = b2 & 0x7F;
            for (int shift = 7; (b2 & 0x80) != 0; shift += 7) {
                b2 = ArrayOfSpikes[inc2++];
                i2 |= (b2 & 0x7F) << shift;
            }    
            inc1+=i1;
            inc2+=i2;      
            if(i1!=0&&i2==0){
                totaltoAdd+=inc1-former_inc-1; //size for the length of the real array-1 because of the previously existing (byte 0) for nothing known;
            }

        }//for(k...)
        TempArray=new byte[ArrayOfSpikes.length+totaltoAdd];
        for(int i=0;i<TempArray.length;i++)
            TempArray[i]=0;        
        //now copying the missing info 
        int kernelID1,kernelID2;
        long Time1,Time2;
        short port1,port2;
        i1=i2=inc1=inc2=former_inc=0;
        b1=b2=0;
        int inc3=0;
        int former_inc2;
        for(int k=0;k<nprocs;k++){
            former_inc=inc1;
            b1 = byteArray[inc1++];
	
            i1 = b1 & 0x7F;
            for (int shift = 7; (b1 & 0x80) != 0; shift += 7) {
                b1 = byteArray[inc1++];
                i1 |= (b1 & 0x7F) << shift;
            }
            former_inc2=inc2;
            b2=ArrayOfSpikes[inc2++];
            i2 = b2 & 0x7F;
            for (int shift = 7; (b2 & 0x80) != 0; shift += 7) {
                b2 = ArrayOfSpikes[inc2++];
                i2 |= (b2 & 0x7F) << shift;
            }    
            inc1+=i1;
            inc2+=i2;        
            if(i1!=0&&i2==0){
                System.arraycopy(byteArray, former_inc, TempArray, inc3, inc1-former_inc);
                inc3+=inc1-former_inc;
            }
            else{
                System.arraycopy(ArrayOfSpikes, former_inc2, TempArray, inc3, inc2-former_inc2);
                inc3+=inc2-former_inc2;
            }

        }  //for k...      
        ArrayOfSpikes=TempArray;
    }
    
    public Vector decode(){
        Vector v=new Vector();
        int inc=0;
   //     try{
            for(int k=0;k<nprocs;k++){
                SpikeArray spk=new SpikeArray();
                inc=spk.decode(ArrayOfSpikes,inc);
                v.addElement(spk);
            }
 //        }catch(Exception e){
 //            neosim.kernel.CommandQueue.PrintObject(ArrayOfSpikes, "ERROR_DECODE.txt");
 //            e.printStackTrace();
 //        }

        ArrayOfSpikes=new byte[nprocs];
        for(int i=0;i<nprocs;i++)
            ArrayOfSpikes[i]=0;
        return v;
    }
    
    private void TestCompressionSpikes(){
        byte ByteArray[];
        byte TempArray[];
        SpikeArray spikes=new SpikeArray(0);
        spikes.addSpike(99,(long)(9999), (short)(9)); 
        //convert your own
        this.convert(spikes);        
        

        for(int i=1;i<nprocs;i++){
            //that should come from other procs            
            spikes=new SpikeArray(i);
            if(i!=1) //test in case no spikes
                spikes.addSpike(i,(long)(i*i), (short)i); 
            this.id=i;
            ByteArray=ArrayOfSpikes;
            this.convert(spikes);
            this.id=0;
            TempArray=ArrayOfSpikes;
            ArrayOfSpikes=ByteArray;
            //Include the coming bytes Array
            this.include(TempArray);
        }
        //once you get it all
        decode();
//        neosim.kernel.CommandQueue.PrintObject(AllgatherObject, "TestCompressionSpikes.txt");
    }
    
    public synchronized final void SendSpikesArray(byte[] Spikes) throws java.rmi.RemoteException {
        //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this.BarrierInteger,ecp);    
        synchronized(AllgatherObject){
            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
            try{
                include(Spikes);
                AllgatherObject.notify();
            }
            catch(Exception e){
                 e.printStackTrace();
            }
        }
    } 
    
    final boolean CheckSpikesArrayEmpty(int i){
         byte b;
         int finalinc;
         int handle;
         long time;
         short port;
         int counter;
         int inc=0;
         while(i-->0){
            b = ArrayOfSpikes[inc++];
            finalinc = b & 0x7F;
            for (int shift = 7; (b & 0x80) != 0; shift += 7){
                b = ArrayOfSpikes[inc++];
                finalinc |= (b & 0x7F) << shift;
            }
            inc+=finalinc;        
         }
         if(ArrayOfSpikes[inc]==0)
            return true;
         else
            return false;
    }
        
    public final Vector allgatherSpikes(SpikeArray spikes) {
       /*java.io.File f=new java.io.File("ARRAY_SPIKES_DECODE_"+id+".txt");
       if(f!=null)
           f.delete();
       neosim.kernel.CommandQueue.PrintObject(new StrMessage("SpikeArray"), "ARRAY_SPIKES_DECODE_"+id+".txt");
       neosim.kernel.CommandQueue.PrintObject(spikes, "ARRAY_SPIKES_DECODE_"+id+".txt");*/
       convert(spikes);
       /*neosim.kernel.CommandQueue.PrintObject(new StrMessage("ArrayOfSpikes"), "ARRAY_SPIKES_DECODE_"+id+".txt");
       neosim.kernel.CommandQueue.PrintObject(ArrayOfSpikes, "ARRAY_SPIKES_DECODE_"+id+".txt");*/
       byte[] TempArray=new byte[0];
       int j,length;
       byte b;
       for(int i=0;i<AllgatherArray.length;i++){
           if(AllgatherArray[i]<0){
               try{
                   //wait to be ready
                  // bisArray[i].read(); 
                   length=ArrayOfSpikes.length;
                   //write on several bytes because sending int does not seem to work (sometimes some weird 255 offsets happen).
                    while ((length & ~0x7F) != 0) {
                        b = (byte)((length & 0x7f) | 0x80);
                        bosArray[i].write(b);	    
                        length >>>= 7;
                    }
                    bosArray[i].write((byte)length);	    
                   // bosArray[i].flush(); 
                   
                   //send packets.
                   length=ArrayOfSpikes.length-100;
                   for(j=0;j<length;j+=100){
                      bosArray[i].write(ArrayOfSpikes,j,100);    
                   //   bosArray[i].flush();    
                   }
                   bosArray[i].write(ArrayOfSpikes,j,ArrayOfSpikes.length-j);    
                   bosArray[i].flush();                                             
               }catch(java.io.IOException e){ e.printStackTrace(); }
            }
            else if(AllgatherArray[i]>0){
             // while(CheckSpikesArrayEmpty(AllgatherArray[i]-1)){
                   try{
                      // bosArray[i].write(0); 
                     //  bosArray[i].flush();
                      //decode on several bytes because sending int does not seem to work (sometimes some weird 255 offsets happen).
                      b=(byte)bisArray[i].read();                      
                      length = b & 0x7F;
                      for (int shift = 7; (b & 0x80) != 0; shift += 7) {
                        b = (byte)bisArray[i].read(); 
                        length |= (b & 0x7F) << shift;
                      }
                       //read packets.
                      TempArray=new byte[length];
                      
                        
                      int bytesRead = 0; // in a single iteration
                      int offset    = 0; // current offset in buffer
                      int BUFFER_SIZE = 100;
                      while ((bytesRead != -1) && (offset < length)) {
                         int requestSize = ((int) length - offset) < BUFFER_SIZE ? ((int) length - offset) : BUFFER_SIZE;
                         bytesRead = bisArray[i].read(TempArray, offset, requestSize);
                         offset    += bytesRead;
                      }
                     /* for(int m=0;m<length;m++){
                          TempArray[m]=(byte)bisArray[i].read();
                      }*/
                    
                   }catch(java.io.IOException e){ e.printStackTrace(); }   
                   /*neosim.kernel.CommandQueue.PrintObject(new StrMessage("TempArray"), "ARRAY_SPIKES_DECODE_"+id+".txt");
                   neosim.kernel.CommandQueue.PrintObject(TempArray, "ARRAY_SPIKES_DECODE_"+id+".txt");*/
                   include(TempArray);
                   //neosim.kernel.CommandQueue.PrintObject(new StrMessage("ArrayOfSpikes"), "ARRAY_SPIKES_DECODE_"+id+".txt");
            //    }
            }
         }
     //   neosim.kernel.CommandQueue.PrintObject(new StrMessage("Passed"), "ARRAY_SPIKES_DECODE_"+id+".txt");
        //distribute
         for(int i=AllgatherArray.length-1;i>=0;i--){
            if(AllgatherArray[i]>0){
                try{
                    //wait to be ready
                   // bisArray[i].read(); 
                    length=ArrayOfSpikes.length; 
                    //write on several bytes because sending int does not seem to work (sometimes some weird 255 offsets happen).
                    while ((length & ~0x7F) != 0) {
                        b = (byte)((length & 0x7f) | 0x80);
                        bosArray[i].write(b);	    
                        length >>>= 7;
                    }
                    bosArray[i].write((byte)length);	    
                  //  bosArray[i].flush(); 
                     //send packets.
                    length=ArrayOfSpikes.length-100;                    
                    for(j=0;j<length;j+=100){
                      bosArray[i].write(ArrayOfSpikes,j,100);    
                   //   bosArray[i].flush();   
                    }
                    bosArray[i].write(ArrayOfSpikes,j,ArrayOfSpikes.length-j);    
                    bosArray[i].flush(); 
                }catch(java.io.IOException e){ e.printStackTrace(); }                        
            }
            else if(AllgatherArray[i]<0){
              //  while(CheckSpikesArrayEmpty(-AllgatherArray[i]-1)){//if
                    try{
                        //say you are ready
                      //  bosArray[i].write(0);
                      //  bosArray[i].flush();                          
                        //decode on several bytes because sending int does not seem to work (sometimes some weird 255 offsets happen).
                        b=(byte)bisArray[i].read();                      
                        length = b & 0x7F;
                        for (int shift = 7; (b & 0x80) != 0; shift += 7) {
                            b = (byte)bisArray[i].read(); 
                            length |= (b & 0x7F) << shift;
                        }
                        //read packets.
                        TempArray=new byte[length];
                        /*length-=512;
                        for(j=0;j<length;j+=512){
                            bisArray[i].read(TempArray,j,512);
                        }                    
                        bisArray[i].read(TempArray,j,TempArray.length-j);*/
                       
                        int bytesRead = 0; // in a single iteration
                        int offset    = 0; // current offset in buffer
                        int BUFFER_SIZE = 100;
                        while ((bytesRead!=-1)&&(offset<length)){
                          int requestSize = ((int) length - offset) < BUFFER_SIZE ? ((int) length - offset) : BUFFER_SIZE;
                          bytesRead = bisArray[i].read(TempArray, offset, requestSize);
                          offset    += bytesRead;
                        }
                      /*for(int m=0;m<length;m++){
                          TempArray[m]=(byte)bisArray[i].read();
                      }    */                    
                        
                    }catch(java.io.IOException e){ e.printStackTrace(); } 
                   /* neosim.kernel.CommandQueue.PrintObject(new StrMessage("TempArray"), "ARRAY_SPIKES_DECODE_"+id+".txt");
                    neosim.kernel.CommandQueue.PrintObject(TempArray, "ARRAY_SPIKES_DECODE_"+id+".txt");*/
                    include(TempArray); 
                   // neosim.kernel.CommandQueue.PrintObject(new StrMessage("ArrayOfSpikes"), "ARRAY_SPIKES_DECODE_"+id+".txt");                    
               // }
            } 
         }
        //neosim.kernel.CommandQueue.PrintObject(new StrMessage("Exchanged/to decode"), "ARRAY_SPIKES_DECODE_"+id+".txt");
         return decode();     
    }    
    boolean[] arrayBarrierSocket;
    
    
    private final void barrierSocket() {
       int length;
       int idread=-1;
       byte b;
       for(int i=0;i<AllgatherArray.length;i++){
           if(AllgatherArray[i]<0){
               try{    

                   //osArray[i].write(id+1);  
                    bisArray[i].read(); 
                    int ID_=id+1;
                    //write on several bytes because sending int does not seem to work (sometimes some weird 255 offsets happen).
                    while ((ID_ & ~0x7F) != 0) {
                        b = (byte)((ID_ & 0x7f) | 0x80);
                        bosArray[i].write(b);	    
                        ID_ >>>= 7;
                    }
                    bosArray[i].write((byte)ID_);	    
                    bosArray[i].flush();                                        
               }catch(java.io.IOException e){ e.printStackTrace(); }
            }
            else if(AllgatherArray[i]>0){
               while(!arrayBarrierSocket[AllgatherArray[i]-1]){
                   try{
                      
                       bosArray[i].write(0);   
                       bosArray[i].flush();   
                      //decode on several bytes because sending int does not seem to work (sometimes some weird 255 offsets happen).
                       b=(byte)bisArray[i].read();                      
                       idread = b & 0x7F;
                       for (int shift = 7; (b & 0x80) != 0; shift += 7) {
                          b = (byte)bisArray[i].read(); 
                         idread |= (b & 0x7F) << shift;
                       }
                     //idread=isArray[i].read();
                   }catch(java.io.IOException e){ e.printStackTrace(); }        
                   arrayBarrierSocket[idread-1]=true;                   
                }
            }
         }
       
       
       
         /*
          //no need for distribution
          for(int i=AllgatherArray.length-1;i>=0;i--){
            if(AllgatherArray[i]>0){
                try{
                    osArray[i].write(id+1);
                }catch(java.io.IOException e){ e.printStackTrace(); }                        
            }
            else if(AllgatherArray[i]<0){
                while(!arrayBarrierSocket[-AllgatherArray[i]-1]){//if
                    try{
                        idread=isArray[i].read();
                    }catch(java.io.IOException e){ e.printStackTrace(); } 
                    arrayBarrierSocket[idread-1]=true;                                                            
                }
            } 
         }*/
       for(int i=0;i<nprocs;i++){
           arrayBarrierSocket[i]=false;
       }
    }    
    
    
   /* public final void exchangeSpikes(){
        AllArraySpikes=allgatherSpikes(arraySpikes);
        proceedExchangeSpikes();
    }*/    
    
    
    private final void TestsSocket(){
        long start,end;
        start=System.currentTimeMillis();

      /*  for(int i=0;i<10000;i++){
            this.barrierSocket();
        }
        end=System.currentTimeMillis();
        System.out.println("barrierSocket : "+(end-start)+" ms");    
          
        System.out.println("100/nprocs : "+(100/nprocs));  
        for(int i=0;i<1000/nprocs;i++)
            this.addSpike(5,(short)1, 50000000);        
        start=System.currentTimeMillis();

        for(int i=0;i<1000;i++){
            
            this.allgatherSpikes(arraySpikes);
        }
        end=System.currentTimeMillis();

        System.out.println("allgatherSpikes : "+(end-start)+" ms");
        
        this.barrierSocket();       
        
        start=System.currentTimeMillis();
        try{
            for(int i=0;i<1000;i++){
                this.AllGather_obj(arraySpikes);
            }
        }
        catch(Exception e){
            e.printStackTrace();
        }
        end=System.currentTimeMillis();

        System.out.println("allgather : "+(end-start)+" ms");        
        
                
        Vector v=new Vector();
        for(int i=0;i<1000/nprocs;i++)
            v.add(new neosim.iface.SpikeEvent(new neosim.iface.Time(5.0),5,(short)5));
        this.barrierSocket();        
        start=System.currentTimeMillis();
        try{       
            for(int i=0;i<1000;i++){
                this.allgather(v);
            }
        }
        catch(Exception e){
            e.printStackTrace();
        }
        end=System.currentTimeMillis();
        System.out.println("allgather RMI : "+(end-start)+" ms");        
   */
    }
}

       /*
       for(int i=0;i<v.size();i++)
           include(((byte[])v.elementAt(i)));
       return decode(); */     
         /*
        int length=AllgatherArray.length;
        Vector v;
        synchronized(AllgatherObject){
            convert(spikes);
            for(int i=0;i<length;i++){
                if(AllgatherArray[i]<0){
                    CommandQueue cq=(CommandQueue)this.othercqs.elementAt(-AllgatherArray[i]-1);
                    RmtApp remoteKernel=(RmtApp)cq.getDest();
                    try{
                        remoteKernel.SendSpikesArray(ArrayOfSpikes);
                    }
                    catch(java.lang.Exception e){
                        e.printStackTrace();
                    }      
                }
                else if(AllgatherArray[i]>0){
                    while(CheckSpikesArrayEmpty(AllgatherArray[i]-1))//if
                        try{
                            AllgatherObject.wait();                       
                        } catch(Exception e){
                            e.printStackTrace();
                        }
                }
            }
        //distribute
            for(int i=length-1;i>=0;i--){
                if(AllgatherArray[i]>0){
                    CommandQueue cq=(CommandQueue)this.othercqs.elementAt(AllgatherArray[i]-1);
                    RmtApp remoteKernel=(RmtApp)cq.getDest(); 
                    try{
                        remoteKernel.SendSpikesArray(ArrayOfSpikes);
                    }
                    catch(java.lang.Exception e){
                        e.printStackTrace();
                    }                    
                //send my guy
                //notify
                }
                else if(AllgatherArray[i]<0){
                    while(CheckSpikesArrayEmpty(-AllgatherArray[i]-1))//if
                        try{
                            AllgatherObject.wait();                       
                        }
                        catch(Exception e){
                            e.printStackTrace();
*/