
package neosim.util;

import java.awt.*;
import java.util.*;
import java.io.*;
import java.net.*;
import java.applet.Applet;
import java.awt.event.*;


/**
 * FileLoader 
 * Converts ascii file from URL to FileEventListener format
 */
 
public class FileLoader implements Filecmds, Runnable {

  transient Thread thread;
  String urlName;


  /** Create a file loader with the given url */
  public FileLoader(String urlName) {
    this.urlName = urlName;
    thread = new Thread(this);
  }


  /** Called after output has been wired */
  public void startLoading() {
    System.out.println("StartLoading");
    thread.start();
  }

  protected String nextLine(BufferedReader r) {
    try {
      String l = r.readLine();
      return l;
    } catch (Exception e) {
      return null;
    }
  }

  /** Starts reading the trace in from the URL. */
  public void run() {
    URL u;
    try {
      u = new URL(urlName);
    } catch (Exception e) {
      System.out.println("Can't open URL "+urlName);
      return;
    }

    DataInputStream file;
    try {
      file = new DataInputStream(u.openStream());
    } catch (IOException ioe) {
      System.out.println("Couldn't open URL\n");
      return;
    }
    BufferedReader in = new BufferedReader(new InputStreamReader(file));
    String l;

    int numevents = 0;
 
    forwardFile( new FileEventObject(this,START) );

    while ((l = nextLine(in))!=null) {
      forwardFile( new FileEventObject(this,l) );
      System.out.println(numevents+":"+l);
      numevents += 1;
    }    
    
    forwardFile( new FileEventObject(this,EOF) );

    System.out.println("Read in "+numevents+" lines");
  }

  private Vector fileListeners = new Vector();
  /** Javabeans file event output */
  public synchronized void addFileListener(FileListener l) {
    fileListeners.addElement(l); 
  }
  /** Javabeans file event output */
  public synchronized void removeFileListener(FileListener l) {
    fileListeners.removeElement(l); 
  }
  /** Sends file event onwards to any listeners */
  public void forwardFile(FileEventObject e) {
    Vector l;
    FileEventObject weo = new FileEventObject(this,e);
    synchronized(this) { l = (Vector)fileListeners.clone(); }
    for (int i=0; i<l.size(); i++) {
      FileListener wl = (FileListener) l.elementAt(i);
      wl.handleFile(weo);
    }
  }  
}
