//package neosim.neuroml.nmllauncher;
package neosim.util.loaders.nmllauncher;

import neuroml.util.*;
import neuroml.model.network.*;
import neuroml.model.cell.*;
import neuroml.environment.*;
import neuroml.environment.io.*;
import neuroml.environment.schedule.*;
import neuroml.core.ModuleRef;

import neosim.kernel.*;
import neosim.neuroml.*;
import neosim.iface.*;
import neosim.implementation.*;
import neosim.util.*;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.beans.*;
import java.lang.Integer;
import java.io.*;
import neuroml.model.I_Clamp.*;
import neuroml.model.MeasurementTool.*;


/**
 * Launch a Neosim simulation from the NeuroML description of the Simulation 
 * (NeuromlPkg class) description generated by XMLIn from a NeuroML description file
 *
 * @author Paul Rogister      
 *
 * @version 1.1, 27 Aug 2001   
 *                
 */
public class NeosimNeuroMLLauncher {
    static public nmlplay.PlayerCallBack Player;
    Namespace namespace;
    
    protected ScriptReader reader;
    protected neuroml.NeuromlPkg neuromlDesc;
    protected boolean cons = false; 
    protected Hashtable entityTable = new Hashtable() ;
    protected Hashtable populationTable = new Hashtable() ;
    protected Hashtable popHasEntities = new Hashtable() ;
    protected Hashtable networkAvailable = new Hashtable() ; 
    protected Hashtable sourceMethodsAvailable = new Hashtable() ; 
    protected Hashtable destMethodsAvailable = new Hashtable() ; 

    PopulationTree root;

    static private boolean debug = false;

    BufferedWriter errorsReport;
    FileOutputStream errorsReportFile;
    String errorsReportFileName = "errors.log";
    boolean errorsOn = false;
    
    Vector networkVect=new Vector(1,1);//false

    /**
     * Setups NeosimLauncher and builds a list of Networks available
     *  
     * @param xr XML_reader : link to Neosim scriptreader
     * @param sd Simulation description generated from a xml file
     *    
     */
    public NeosimNeuroMLLauncher( XML_Reader xr , Object sd ) {
	startNeosimNeuroMLLauncher(xr,sd,null);
    }

    public NeosimNeuroMLLauncher( XML_Reader xr , Object sd, Namespace namespace ) {
	startNeosimNeuroMLLauncher(xr,sd,namespace);
    }

    public void startNeosimNeuroMLLauncher( XML_Reader xr, Object sd, Namespace namespace ) {
	createErrorFile();
	if (namespace!=null) {this.namespace = namespace;}
	reader = xr.ba; // Hack to get at NEOSIM script reader
	try {
	    neuromlDesc = (neuroml.NeuromlPkg)sd;
	} catch (Exception ex) {
	    System.out.println("\n\nstartNeosimNeuroMLLauncher: Error - The "+
			       "second parameter must be a NeuromlPkg class "+ex);
	    System.out.println("Please verify that your NeuroML description "+
			       "begins with the NeuromlPkg tag");
	}
	root = reader.getPopulationTree(); 
    }
    
    /**
     * Check the version, not implemented yet
     *
     */
    public void checkVersion() {	
    }

    /**
     * Starts the building of the main model
     * <p>Call buildnetwork for each network of the model</p>
     *
     * @see #buildnetwork
     * @see #buildprobesnetwork
     */
    public void buildModel(){
        if(Player!=null){
            Player.println("");
            Player.println("NEOSIM: Start building model "+this.neuromlDesc.getName());
            Player.println("-----------------------------------------------------------");
        }

	// errors log
	if(errorsReportFile==null)
	    createErrorFile();

	//Vector networkVect = new Vector();
       // networkVect = new Vector();

	Hashtable refs = namespace.getReferences();
	Hashtable objs = (Hashtable) refs.get("NetworkPkg");
	if (debug) System.out.println("objs list : "+objs.toString());
        
        
        
	if (objs!=null) {
	    for (Enumeration enum_ = objs.keys() ; enum_.hasMoreElements() ;) {
		String k = (String)enum_.nextElement();
		NetworkPkg netpkg = (NetworkPkg)objs.get(k);
		networkVect.addAll(netpkg.networks.getElements());
                for(int i=0;i<netpkg.networks.size();i++){
                    Network net=(neuroml.model.network.Network)networkVect.elementAt(i);
                    if(netpkg.MainNetwork==null) System.out.println("No main Network in NetworkPackage "+netpkg.getName());
                    String stuff=netpkg.MainNetwork.getTargetName();
                    if(stuff==null) System.out.println("You need a main network, this will crash");
                    if(stuff.endsWith(netpkg.getName()+".networks."+ net.getName()))
                        buildNetwork(net);	
                    //if(net.getName())
                }
	    }
	}
        
        

	// add probes network
	Vector probesVect = new Vector();

	objs = (Hashtable) refs.get("Probe");

	if (objs!=null) {
	    if (debug) System.out.println("objs list : "+objs.toString());

	    for (Enumeration enum_ = objs.keys() ; enum_.hasMoreElements() ;) {
		String k = (String)enum_.nextElement();
		Probe probe = (Probe)objs.get(k);
		probesVect.add(probe);
	    }
	}

	objs = (Hashtable) refs.get("Attachment");
	Vector attachVect = new Vector();

	if (objs!=null) {
	    if (debug) System.out.println("objs list : "+objs.toString());

	    for (Enumeration enum_ = objs.keys() ; enum_.hasMoreElements() ;) {
		String k = (String)enum_.nextElement();
		Attachment att = (Attachment)objs.get(k);
		attachVect.add(att);
	    }
	}
	buildProbesNetwork(probesVect,attachVect);

	// end of buildmodel : close error log
	closeErrorFile();

    }

    /**
     * Launch the simulation
     * <p>Call to XML_Reader.runSim</p>
     */ 
    public void runSimulation() {
	// look in SimDesc for Control options and run sim
	// if control module from a jar file
	
	if (neuromlDesc.environment.schedule instanceof  ControlledSchedule) {
	    ControlledSchedule controlledSchedule = (ControlledSchedule)neuromlDesc.environment.schedule;
	    neosim.iface.ControlModule cm = jarLoadControlModule(controlledSchedule);
	    if (debug) {
		System.out.println("Run simulation : Begin Simulation from Control Module");
	    }
	    cm.run(reader);
	} else { // standard launch
	    Time simtime = new Time();
	    simtime.set(neuromlDesc.environment.schedule.runtime);
	    if (debug) {
		System.out.println("Run simulation : Simulation Time : "+simtime.s());
		//removed for test 
		System.out.println("Run simulation : Begin Simulation");
	    }
	    reader.runSim(simtime);
	}	
    }
    
    /**
     * Converts string to int
     *              
     * @param  s String to be converted      
     * @return int conversion of String          
     */
    protected static int atoi(String s) { return (Integer.valueOf(s)).intValue(); }

    /* Remove 1st string from array */
    protected static String[] trunc(String[] a1) {
	int newlength = a1.length-1; if (newlength<0) return null;
	String[] answer = new String[newlength];
	for (int i=0; i<newlength; i++) { answer[i] = a1[i+1]; }
	return answer;
    }

    /** 
     * Construct a Neosim entity 
     *
     * @param entity Entity Description used to build the Neosim entity
     */   
    public void makeEntity(neuroml.model.network.Element entitydesc ) {
	String jar;
	String name;
	Hashtable params = new Hashtable();

	neuroml.model.cell.ModuleNeuron entity;
	neuroml.model.network.ModuleProbe probe;
	EntityClass ec = null;

	try {	    
	    // Module Neuron entity
	    if (entitydesc instanceof neuroml.model.cell.ModuleNeuron) {

		entity = (neuroml.model.cell.ModuleNeuron)entitydesc;
				
		if (debug) System.out.println("entity.module.parameters "+entity.module.parameters);
		if (entity.module.parameters instanceof neuroml.sim.run.RunnableCellFactory)
		    ec = NeuromlConverter.convertEntityClass((neuroml.sim.run.RunnableCellFactory)entity.module.parameters);
		else {
		    System.out.println("Error : makeEntity : module is not a RunnableCellFactory");
		    writeErrors("- Entity "+entity.getName()+" : module is not a RunnableCellFactory");
		}
		//EntityClass ec = neosim.util.ModuleLoader.getEntityClass(jar, name, params); 
	    
		// adding (entityname,entityclass) to a Hashtable
        		entityTable.put(entity.getName(),ec);

		} else if (entitydesc instanceof neuroml.model.network.ModuleProbe) {

		    probe = (neuroml.model.network.ModuleProbe)entitydesc;
		    
		    if (debug) System.out.println("probe.module.parameters "+probe.module.parameters);
		    if (probe.module.parameters instanceof neuroml.sim.run.RunnableCellFactory) {
			ec = NeuromlConverter.convertEntityClass((neuroml.sim.run.RunnableCellFactory)probe.module.parameters);
			// + init model of factory with the whole model for probe
			((NeosimEntityClassImpl)ec).init(neuromlDesc);
		    
		    } else {
			System.out.println("Error : makeEntity Probe : module is not a RunnableCellFactory");
			writeErrors("Probe "+probe.getName()+" : module is not a RunnableCellFactory");
		    }
		    		    
		    // adding (entityname,entityclass) to a Hashtable
		    entityTable.put(probe.getName(),ec);

		    if (debug) System.out.println(" makeEntity Probe : probe.getName() "+probe.getName());		    
		} else {
		    System.out.println("Error - unsupported class for entity "+entitydesc);
		    writeErrors("- Entity "+entitydesc.getName()+" : unsupported class");
		}

	} catch (Exception ex) {
	    System.out.println("NeosimNeuroMLLauncher Error - At this stage of development, the entity must "+
			       "be defined as ModuleNeuron "+ex);
	    writeErrors("- Entity "+entitydesc.getName()+" must be defined as ModuleNeuron");
	}
    }
    
    
    /** 
     * Load a control module from a jar file
     *
     * @param Control module to run the simulation
     */
    public  neosim.iface.ControlModule jarLoadControlModule( ControlledSchedule cdesc ) {
	String jar;
	Hashtable params = new Hashtable();

	if (debug) {
	    System.out.println("NeuroMLLauncher jarLoadControlModule");
	    System.out.println("jarLoadControlModule cdesc "+cdesc);
	    System.out.println("jarLoadControlModule cdesc.module "+cdesc.module);
//MODULE--JAG	    System.out.println("jarLoadControlModule cdesc.module "+cdesc.module.module.getTargetName());
	}
	    
	neosim.iface.ControlModule cm = null;
	if (cdesc.module.parameters instanceof neuroml.sim.run.RunnableControl)
	    cm = NeuromlConverter.convertControlModule((neuroml.sim.run.RunnableControl)cdesc.module.parameters);

	else {
	    System.out.println("Error : jarLoadControlModule : module is not a RunnableControl");
	    writeErrors("- ControlModule "+cdesc.module.parameters+" : module is not a RunnableControl");
	    
	}

	//ControlModule cm = neosim.util.ModuleLoader.getControlModule(jar, params);
	//ControlModule cm;
	return cm;
    }

    /** 
     * Return the size of 
     * <p>Call to XML_Reader.construct</p>
     *
     * @param  dimNum dimension number (between 1 and 3)
     * @param  struct Indexed Structure of a population
     */
    public int getIndexedDim(int dimNum, neuroml.model.network.IndexedStructure struct ) {
	if ((struct.ndims < dimNum)||(dimNum == 0)){ // the dimension requested is out of bounds
	    return 1;
	}
	else {
	    try {
		neuroml.model.network.IndexedDimension dim = (neuroml.model.network.IndexedDimension)
		    struct.dimensions.getElements().elementAt(dimNum-1);
		return dim.size;

	    } catch (Exception ex) {
		System.out.println("Error - The dimension of the IndexedStructure is"+
				   "not an IndexedDimension"+ex);
		writeErrors("- The dimension of the IndexedStructure "+struct+" is"+
			    "not an IndexedDimension");
		return 1;
	    }		    
	}
    }	
        
    /** 
     * Construct a Neosim population 
     * <p>Call to XML_Reader.construct</p>
     *
     * @param  pop Population Description used to build the Neosim population
     */
    public void makePopulation( neuroml.model.network.ElementRef elem, String RootFromNetwork) {	//, String ID_Network
	String entityname;
	String popname;
	String popType;
	int number;

	Vector entitieslist = new Vector();
	Hashtable params;

	entityname = elem.elementClass.getTargetName();
	// for some reasons the target names seems to be sometimes 
	// changed to the long path name (when using editObjectPanel), 
	// with name of packages , 
	// we have to keep only the "last" name
	int dotindex = entityname.lastIndexOf('.');
	if (dotindex!=-1)
	    entityname = entityname.substring(dotindex+1);
	
	//	entityname = pop.getElementName();
        popname=new String();
        
        if(RootFromNetwork!=null)
            popname=new String(RootFromNetwork + ".");

	popname = new String(popname+elem.getName()); //ID_Network+

	//popType = pop.getPopType(); replaced by a cast on the structure
	//params = pop.getparameters(); for jar population only
        if(Player!=null)
            Player.println("Build node "+entityname+" "+popname);
	if (debug) { System.out.println("Make population "+entityname+" "+popname); }

	// two possibilities : population of cells or population of networks
	if (debug) System.out.println(entityTable.toString());
        
        
        
        Class expClass_ =  NeuromlSys.getClassFromJar( entityname );
        if (expClass_ == null) {
	//if ((baseClass != null)&&(expClass == null)) {
                try{
                    expClass_ =  NeuromlSys.getClassFromShortName( entityname );
                }
                catch(Exception e){
                    e=e;
                }
        }
        

        if(expClass_!=null)
                entityname=expClass_.getName();
        
	if (entityTable.containsKey(entityname)){
	    //population of cells
	    // Get the entity class given the name
	    EntityClass ec = (EntityClass)entityTable.get(entityname);
	    /** If element ref is ref to single element... */
	    if (! (elem instanceof neuroml.model.network.Population)) {
 		PopulationBuilder sg = new PopulationBuilderImpl(popname, 1, ec, null, null ); //SOCKET_ERROR	 , root
 		reader.construct( sg );
		populationTable.put(popname,sg); // *** ??? Is this required??? ***
	    } else { // Element is a Population!
                
                if(elem instanceof neuroml.model.network.RealPopulation){ //JAG
                    neuroml.model.network.RealPopulation  pop = (neuroml.model.network.RealPopulation)elem;		
                    I_Clamp IntensityClamp=null;
                    try{
                       IntensityClamp=(I_Clamp) (pop.ExternalIntensity.module.parameters);
                    }
                    catch(Exception escve){
                        
                    }
                    
                    if (pop.structure instanceof neuroml.model.network.Grid1DStructure) {//default one
                        // if number == 0 then maybe the structure is a derivated class
                        //  Grid1DPopulation
                        neuroml.model.network.Grid1DStructure grid1d = (neuroml.model.network.Grid1DStructure)pop.structure;
                        number = grid1d.size;
                        PopulationBuilder sg = new PopulationBuilderImpl(popname, number, ec, (EntityInit)null,IntensityClamp);	//SOCKET_ERROR  , root
                        reader.construct( sg );
                        populationTable.put(popname,sg);
                    }
                    else if(pop.structure instanceof neuroml.model.network.Grid3DStructure) {
		    //  Grid3DPopulation
                        neuroml.model.network.Grid3DStructure grid3d = (neuroml.model.network.Grid3DStructure)pop.structure;
		    // should be a more general one
                        int xdim,ydim,zdim;
                        double xorig,yorig,zorig;
                        double xdelta,ydelta,zdelta;
		    
                        xdim = grid3d.xsize;
                        ydim = grid3d.ysize;
                        zdim = grid3d.zsize;
                        xorig = 0; yorig = 0; zorig = 0;
                        xdelta = 1; ydelta = 1; zdelta = 1;
		    //System.out.println("Build 3DGrid Population");
                        Grid3DPopulation sg = new Grid3DPopulation(popname,xdim,ydim,zdim,xorig,yorig,zorig,xdelta,ydelta,zdelta,ec,IntensityClamp); //SOCKET_ERROR ,root
		    // Building of the population
                        reader.construct( sg );
                   
                        populationTable.put(popname,sg);
                    } else if(pop.structure instanceof neuroml.model.network.ModulePosition) {
		    //  Grid3DPopulation
                        neuroml.model.network.ModulePosition grid3d = (neuroml.model.network.ModulePosition)pop.structure;
                        ModuleRef muche=grid3d.module;
                        PositionModule truc=(PositionModule)(muche.parameters);
			Position3DPopulation sg = new Position3DPopulation(popname,truc.getSize(),ec,truc,IntensityClamp);//SOCKET_ERROR ,root
		    // Building of the population
                        reader.construct( sg );
                        populationTable.put(popname,sg);
                    }  else if (pop.structure instanceof neuroml.model.network.IndexedStructure) {
		    // hacking, try Grid3DPopulation
		    // should be a more general one
                        int xdim,ydim,zdim;
                        double xorig,yorig,zorig;
                        double xdelta,ydelta,zdelta;
		    
                        xdim = getIndexedDim(1,(neuroml.model.network.IndexedStructure)pop.structure);
                        ydim = getIndexedDim(2,(neuroml.model.network.IndexedStructure)pop.structure);
                        zdim = getIndexedDim(3,(neuroml.model.network.IndexedStructure)pop.structure);
                        xorig = 0; yorig = 0; zorig = 0;
                        xdelta = 1; ydelta = 1; zdelta = 1;
		    //System.out.println("Build 3DGrid Population");
                        Grid3DPopulation sg = new Grid3DPopulation(popname,xdim,ydim,zdim,xorig,yorig,zorig,xdelta,ydelta,zdelta,ec,IntensityClamp); //SOCKET_ERROR ,root
		    // Building of the population
                        reader.construct( sg );
                        populationTable.put(popname,sg);
		    
                    }  else if (true) {
                        PopulationBuilder sg = new PopulationBuilderImpl(popname, 5, ec, null,IntensityClamp );//SOCKET_ERROR , root
		    // Building of the population
                        reader.construct( sg );
                        populationTable.put(popname,sg);
                    }	else {
                        System.out.println("Error : Population : structure unknown");
                        writeErrors("-Population "+popname+" : structure unknown");
                    }

                }else{
                     System.out.println("Error: Can't build population "+elem.getClass().getName()+" JAG");
                }
	    }
 
	    entitieslist.addElement(entityname);
	    
	    popHasEntities.put(popname,entitieslist);
	}                       
        else if(elem instanceof neuroml.model.network.SubNetwork){
              String NameNet=((neuroml.model.network.SubNetwork)elem).elementClass.getTargetName();
              boolean found=false;
              int i;
              for (i=0; i<networkVect.size()&&!found; i++) {
                   if(NameNet.endsWith(  ((neuroml.model.network.Network)(networkVect.elementAt(i))).getName()))
                       found=true;
              }
              if(found){
                  neuroml.model.network.Network net=(neuroml.model.network.Network)(networkVect.elementAt(i-1));
                  Vector popVect = net.elements.getElements(); 
                  for(int j=0; j<popVect.size(); j++) {	
                      ElementRef e = (ElementRef)popVect.elementAt(j);
                      makePopulation(e,popname);//,ID_Network
                  }
	// network projections
                  Vector projVect = net.projections.getElements();
                  for (int k=0; k<projVect.size(); k++) {	
                      makeProjection( (neuroml.model.network.Projection)projVect.elementAt(k), popname ); //, ID_Network
                  }
                  if(((SubNetwork)elem).Probes){
                     Vector probesVect = net.probes.getElements();
                     for(int j=0; j<probesVect.size(); j++) {	
                        Probe e = (Probe)probesVect.elementAt(j);
                        neuroml.model.network.Population probePop = new neuroml.model.network.RealPopulation(e.getName(),e.getName()); // new neuroml.model.network.Population(e.getName(),e.getName()); JAG
                        makePopulation( probePop , popname ); //should be network name //,ID_Network
                     }
                     Vector attachVect = net.attachments.getElements();
	// probes projections
                     for (int k=0; k<attachVect.size(); k++) {
                        makeAttachment( (neuroml.model.network.Attachment)attachVect.elementAt(k), popname ); //, ID_Network
                    }
                  }
              }     
        } else { 
	    System.out.println("Error : Population : entity unknown : "+entityname);
	    writeErrors("- Population "+popname+" : entity unknown : "+entityname);
	}
	//population of networks
	// construct the networks
	//    for (int i=0; i<number; i++) {
	//! watch the Integer class
	//	buildnetwork((int)networkAvailable.get(entityname));
	//  }	
	//	}
    }

    
    /** 
     * Construct a Neosim connection 
     * 
     * @param  connection Connection Description used to build the Neosim connection
     * @param spop PopulationBuilder of the source population
     * @param srcec Entity class of the source population
     * @param dpop PopulationBuilder of the destination population
     * @param destec Entity class of the destination population
     */
    public ConnectionSpec makeConnection(neuroml.model.network.Connection connection, 
					 PopulationBuilder spop, EntityClass srcec, PopulationBuilder dpop, EntityClass destec) {
	if (debug) {	System.out.println("Make Connection"); }
	short srcport = srcec.getOutPort(connection.sourcePort);//PortID srcport
	short destport = destec.getInPort(connection.destPort);	//PortID destport
	

	/** For now, the entity IDs have to be indices in the parent population
	 * This should however be a full XPath style lookup into any neuron
	 */
	int srce  = spop.getEntityID(atoi(connection.source.getTargetName()));//EntityID srce
	int deste = dpop.getEntityID(atoi(connection.dest.getTargetName()));//EntityID deste

	Time delay = new Time(connection.delay.value);
	// prepare the connection for the Neosim level
	Hashtable params = new Hashtable();
	params.put("weight",Double.toString(connection.weight.value));
	ParamConnectionSpec neosimconnection = new ParamConnectionSpec(srce,srcport,deste,destport,delay.t,params);

	// return it to be included in the projection
	return (neosimconnection);
    } 

    /** 
     * Construct a Neosim connection fpr probes
     * 
     * @param  connection ProbeConnection Description used to build the Neosim connection
     * @param spop PopulationBuilder of the source population
     * @param srcec Entity class of the source population
     * @param dpop PopulationBuilder of the destination population
     * @param destec Entity class of the destination population
     */
    public ConnectionSpec makeProbeConnection(neuroml.model.network.ProbeConnection connection, PopulationBuilder spop, 
					      EntityClass srcec, PopulationBuilder dpop, EntityClass destec) {
	if (debug) {	System.out.println("Make Probe Connection"); }
	short srcport = 1; // default?//PortID srcport
	short destport = 1;	//PortID destport
				     

	/** For now, the entity IDs have to be indices in the parent population
	 * This should however be a full XPath style lookup into any neuron
	 */
	int srce  = spop.getEntityID(atoi(connection.source.getTargetName()));//EntityID srce
	int deste = dpop.getEntityID(0); //always only one target : the probe//EntityID deste

	Time delay = new Time(connection.delay.value);
	// prepare the connection for the Neosim level
	Hashtable params = new Hashtable();
	//params.put("weight",Double.toString(connection.weight.value));
	ParamConnectionSpec neosimconnection = new ParamConnectionSpec(srce,srcport,deste,destport,delay.t,params);

	// return it to be included in the projection
	return (neosimconnection);
    } 

    /** 
     * Construct a Neosim projection 
     * <p>Call to XML_Reader.connect</p>
     *
     * @param  proj Projection Description used to build the Neosim projection
     */
    public void makeProjection(neuroml.model.network.Projection proj, String RootFromNetwork) {	 //, String ID_Network
	String srcpop=new String();   // name of the src population
	String destpop=new String();  // name of the dest population
	// neosim projection builder
	neosim.iface.Projection neosimprojection;
        if(Player!=null)
            Player.println("Make Projection "+proj.getName());
	if (debug) { System.out.println("Make Projection"); }	
        
        if(RootFromNetwork!=null)
            RootFromNetwork=new String(RootFromNetwork + ".");
        else
            RootFromNetwork=new String();
        
       
	srcpop =proj.source.getTargetName();
	destpop=proj.dest.getTargetName();

	// for some reason the target names seems to be sometimes 
	// changed to the long path name, with name of packages, 
	// we have to keep only the "last" name
	// *** NB This will change with hierarchical connections !!! ***
	int dotindex = srcpop.lastIndexOf("elements.");
	if (dotindex!=-1)
	    srcpop = srcpop.substring(dotindex+9);
	dotindex = destpop.lastIndexOf("elements.");
	if (dotindex!=-1)
	    destpop = destpop.substring(dotindex+9);
        
        srcpop =new String(RootFromNetwork+srcpop ); //ID_Network+
        destpop=new String(RootFromNetwork+destpop); //ID_Network+

	// standard connections
	if (proj instanceof neuroml.model.network.RawProjection) {
	    if (debug) { System.out.println("Make Raw Projection"); }
	    
	    Vector Connections = new Vector();
	    String srcname;  // name of the src entity
	    String destname; // name of the dest entity
	    EntityClass srcec;
	    EntityClass destec;	
	    // get the entity class from the population name
	    if (debug) System.out.println("NeuromlLauncher : popHasEntities "+popHasEntities.toString());
	    Vector srcentities  = (Vector)popHasEntities.get(srcpop);
	    Vector destentities  = (Vector)popHasEntities.get(destpop);
	    
	    if (srcentities==null) {
		System.out.println("Neuroml File Error : wrong population name ("+
				   srcpop+") as projection source for projection named "+proj.getName()); 
		writeErrors("- Wrong population name ("+srcpop+
			    ") as projection source for projection named "+proj.getName()+"\n"); 
		return;
	    }
	    if (destentities==null) {
		System.out.println("Neuroml File Error : wrong population name ("+
				   srcpop+") as projection dest for projection named "+proj.getName());
		writeErrors("- Wrong population name ("+srcpop+
			    ") as projection dest for projection named "+proj.getName());
		return;
	    }
	    // temporary : as there is only one entity source class in a population, and the same for dest
	    srcname = (String)srcentities.elementAt(0);	
	    destname = (String)destentities.elementAt(0);
	    srcec = (EntityClass)entityTable.get(srcname);
	    destec = (EntityClass)entityTable.get(destname);
	    
	    neuroml.model.network.RawProjection rawproj = (neuroml.model.network.RawProjection)proj;
	    // get the population class from the population name	 
	    PopulationBuilder spop = (PopulationBuilder)populationTable.get(srcpop);
	    PopulationBuilder dpop = (PopulationBuilder)populationTable.get(destpop);

	    for (int k=0; k<(rawproj.connections.getElements().size()); k++) {
		Connections.addElement( 
		     makeConnection((neuroml.model.network.Connection)
				    rawproj.connections.getElements().elementAt(k),
				    spop,srcec,dpop,destec)); 	
	    }	    
	    // create the Neosim projection // move that upper is jar is treated
	    neosimprojection = new ProjectionImpl(Connections);	    
	    // register the projection at Neosim level
	    reader.connect(neosimprojection);

	}  else if (proj instanceof neuroml.model.network.GeneralProjection) {
	    // GeneralProjection was module projection...
	    if (debug) { System.out.println("Make General Projection"); }
	    neuroml.model.network.GeneralProjection modproj = (neuroml.model.network.GeneralProjection)proj;	     
	    neosim.iface.SourceMethod sourceMethod = null;
	    neosim.iface.DestMethod destMethod = null;
	    
	    if (modproj.sourceMethod.parameters instanceof neuroml.sim.run.SourceMethod) {
		sourceMethod = NeuromlConverter.convertSourceMethod(
		    (neuroml.sim.run.SourceMethod)modproj.sourceMethod.parameters);
	    } else { 
		System.out.println("Error : makeProjection : module is not a SourceMethod");
		writeErrors("- Projection "+proj.getName()+" : module is not a SourceMethod");
	    }
	    
	    if (modproj.destMethod.parameters instanceof neuroml.sim.run.DestMethod) {
		destMethod = NeuromlConverter.convertDestMethod((neuroml.sim.run.DestMethod)modproj.destMethod.parameters);
	    }  else {
		System.out.println("Error : makeProjection : module is not a DestMethod");     
		writeErrors("- Projection "+proj.getName()+" : module is not a DestMethod");	
	    }
	    
	    neosimprojection = new GeneralProjectionImpl(srcpop,destpop,sourceMethod,destMethod); 
	    reader.connect(neosimprojection);	    	    
	}  
    }	
	
    
    
    public void makeAttachment(neuroml.model.network.Attachment proj, String RootFromNetwork) {	//, String ID_Network
	String srcpop=new String();   // name of the src population
	String destpop=new String();  // name of the dest population
	// neosim projection builder
	neosim.iface.Projection neosimprojection;
        if(Player!=null)
            Player.println("Make Attachment "+proj.getName());
	if (debug) { System.out.println("Make Attachment"); }	
        
        if(RootFromNetwork!=null)
            RootFromNetwork=new String(RootFromNetwork + ".");
        else
            RootFromNetwork=new String();
        
	srcpop =proj.source.getTargetName();
	destpop=proj.dest.getTargetName();

	// for some reason the target names seems to be sometimes 
	// changed to the long path name, with name of packages, 
	// we have to keep only the "last" name
	// *** NB This will change with hierarchical connections !!! ***
	int dotindex = srcpop.lastIndexOf("elements.");
	if (dotindex!=-1)
	    srcpop = srcpop.substring(dotindex+9);
	dotindex = destpop.lastIndexOf("probes.");
	if (dotindex!=-1)
	    destpop = destpop.substring(dotindex+7);
        
        srcpop =new String(RootFromNetwork+srcpop ); //ID_Network+
        destpop=new String(RootFromNetwork+destpop); //ID_Network+

	// standard connections
	if (proj instanceof neuroml.model.network.RawAttachment) {
	    if (debug) { System.out.println("Make Raw Projection"); }
	    
	    Vector Connections = new Vector();
	    String srcname;  // name of the src entity
	    String destname; // name of the dest entity
	    EntityClass srcec;
	    EntityClass destec;	
	    // get the entity class from the population name
	    if (debug) System.out.println("NeuromlLauncher : popHasEntities "+popHasEntities.toString());
	    Vector srcentities  = (Vector)popHasEntities.get(srcpop);
	    Vector destentities  = (Vector)popHasEntities.get(destpop);
	    
	    if (srcentities==null) {
		System.out.println("Neuroml File Error : wrong population name ("+
				   srcpop+") as projection source for projection named "+proj.getName()); 
		writeErrors("- Wrong population name ("+srcpop+
			    ") as projection source for projection named "+proj.getName()+"\n"); 
		return;
	    }
	    if (destentities==null) {
		System.out.println("Neuroml File Error : wrong population name ("+
				   srcpop+") as attachment dest for attachment named "+proj.getName());
		writeErrors("- Wrong probe name ("+srcpop+
			    ") as projection dest for attachment named "+proj.getName());
		return;
	    }
	    // temporary : as there is only one entity source class in a population, and the same for dest
	    srcname = (String)srcentities.elementAt(0);	
	    destname = (String)destentities.elementAt(0);
	    srcec = (EntityClass)entityTable.get(srcname);
	    destec = (EntityClass)entityTable.get(destname);
	    
	    neuroml.model.network.RawAttachment rawproj = (neuroml.model.network.RawAttachment)proj;
	    // get the population class from the population name	 
	    PopulationBuilder spop = (PopulationBuilder)populationTable.get(srcpop);
	    PopulationBuilder dpop = (PopulationBuilder)populationTable.get(destpop);

	    for (int k=0; k<(rawproj.connections.getElements().size()); k++) {
		Connections.addElement( 
		     makeProbeConnection((neuroml.model.network.ProbeConnection)rawproj.connections.getElements().elementAt(k),
				    spop,srcec,dpop,destec)); 	
	    }	    
	    // create the Neosim projection // move that upper is jar is treated
	    neosimprojection = new ProjectionImpl(Connections);	    
	    // register the projection at Neosim level
	    reader.connect(neosimprojection);

	}  else if (proj instanceof neuroml.model.network.ModuleAttachment) {
	    // GeneralProjection was module projection...
	    if (debug) { System.out.println("Make Module Attachment"); }
	    neuroml.model.network.ModuleAttachment modproj = (neuroml.model.network.ModuleAttachment)proj;	     
	    neosim.iface.SourceMethod sourceMethod = null;
	    neosim.iface.DestMethod destMethod = null;
	    short srcId=0;
	    if (modproj.sourceMethod.parameters instanceof neuroml.sim.run.SourceMethod) {
		sourceMethod = NeuromlConverter.convertSourceMethod(
		    (neuroml.sim.run.SourceMethod)modproj.sourceMethod.parameters);
                srcId=((neuroml.sim.run.SourceMethod)modproj.sourceMethod.parameters).getSrcId();
	    } else { 
		System.out.println("Error : makeAttachment : module is not a SourceMethod");
		writeErrors("- Attachment "+proj.getName()+" : module is not a SourceMethod");
	    }
	    
	    if (modproj.destMethod.parameters instanceof neuroml.sim.run.DestMethod) {
		destMethod = NeuromlConverter.convertDestMethod((neuroml.sim.run.DestMethod)modproj.destMethod.parameters);
	    }  else {
		System.out.println("Error : makeAttachment : module is not a DestMethod");     
		writeErrors("- Attachment "+proj.getName()+" : module is not a DestMethod");	
	    }
            
            if (modproj.destMethod.parameters instanceof MeasurementMethod){
                ((MeasurementMethod)modproj.destMethod.parameters).SetParent(modproj,srcId);
            }
	    
	    neosimprojection = new GeneralProjectionImpl(srcpop,destpop,sourceMethod,destMethod); 
	    reader.connect(neosimprojection);	    	    
	} else { // default attachment

	    if (debug) { System.out.println("Make default Attachment"); }
	  
	    // use defaults
	    neosim.iface.SourceMethod sourceMethod = new DefaultSourceMethodToAll((short)1);//new PortID(1
	    neosim.iface.DestMethod destMethod = new DefaultDestMethodFromAll((short)1,new Time(10));    //new PortID(1
	    neosimprojection = new GeneralProjectionImpl(srcpop,destpop,sourceMethod,destMethod); 
	    reader.connect(neosimprojection);	    
    	}
    }
	
    /** 
     * Construct a Neosim projection 
     * <p>Call to XML_Reader.connect</p>
     *
     * @param  attach Attachement (Probe's projection) Description used to build the Neosim projection
     */
    public void makeAttachment( neuroml.model.network.Attachment attach) {	
	String srcpop;   // name of the src population
	String destpop;  // name of the dest population
	// neosim projection builder
	neosim.iface.Projection neosimprojection;
	
	if (debug) { System.out.println("Make Attachment"); }
	
	srcpop = attach.source.getTargetName();
	// for some reasons the target names seems to be sometimes 
	// changed to the long path name, with name of packages , 
	// we have to keep only the "last" name
	int dotindex = srcpop.lastIndexOf('.');
	if (dotindex!=-1)
	    srcpop = srcpop.substring(dotindex+1);

	//destpop = attach.probe+"s";
	destpop = attach.dest.getTargetName();
        dotindex = destpop.lastIndexOf('.');
	if (dotindex!=-1)
            destpop=destpop.substring(dotindex+1);

	// standard connections
	if (attach instanceof neuroml.model.network.RawAttachment) {
	    if (debug) { System.out.println("Make Raw Attachment"); }

	    Vector Connections = new Vector();
	    String srcname;  // name of the src entity
	    String destname; // name of the dest entity
	    EntityClass srcec;
	    EntityClass destec;

	    // get the entity class from the population name
	    Vector srcentities  = (Vector)popHasEntities.get(srcpop);
	    Vector destentities  = (Vector)popHasEntities.get(destpop);

	    if (srcentities==null) {
		System.out.println("Neuroml File Error : wrong population name ("+
				   srcpop+")as attachment location for projection named "+
				   attach.getName()); 
		writeErrors("- Wrong population name ("+srcpop+
			    ")as attachment location for projection named "+attach.getName());
		return;
	    }
	    if (destentities==null) {
		System.out.println("Neuroml File Error : wrong population name ("+
				   srcpop+")as attachment probe for projection named "+attach.getName());
		writeErrors("- Wrong population name ("+srcpop+
			    ")as attachment probe for projection named "+attach.getName());
		return;
	    }

	    // temporary : as there is only one entity source class in a population, and the same for dest
	    srcname = (String)srcentities.elementAt(0);	
	    destname = (String)destentities.elementAt(0);
	    srcec = (EntityClass)entityTable.get(srcname);
	    destec = (EntityClass)entityTable.get(destname);

	    neuroml.model.network.RawAttachment rawproj = (neuroml.model.network.RawAttachment)attach;
	    // get the population class from the population name	 
	    PopulationBuilder spop = (PopulationBuilder)populationTable.get(srcpop);
	    PopulationBuilder dpop = (PopulationBuilder)populationTable.get(destpop);

	  	
		for(int k=0; k<(rawproj.connections.getElements().size()); k++) {
		    Connections.addElement( makeProbeConnection(
		       (neuroml.model.network.ProbeConnection)rawproj.connections.getElements().elementAt(k),
		       spop,srcec,dpop,destec)); 	
		}
		
		// create the Neosim projection // move that upper is jar is treated
		neosimprojection = new ProjectionImpl(Connections);
		
		// register the projection at Neosim level
		reader.connect(neosimprojection);

	} else if (attach instanceof neuroml.model.network.ModuleAttachment) {
	    if (debug) { System.out.println("Make Module Attachment"); }
	    neuroml.model.network.ModuleAttachment modproj = (neuroml.model.network.ModuleAttachment)attach;
	    //if (debug) { System.out.println("Make Module Attachment 1"); }

	    // use default
            
	    //JAG neosim.iface.SourceMethod sourceMethod = new DefaultSourceMethodToAll(new PortID(1));
	    //if (debug) { System.out.println("Make Module Attachment 2"); }

	    neosim.iface.DestMethod destMethod = null;
            neosim.iface.SourceMethod sourceMethod = null;

	    //if (debug) { System.out.println("Make Module Attachment 3"); }
            /*JAG*******************************************************************
             *
             *
             *
	    if (modproj.method.parameters instanceof neuroml.sim.run.DestMethod){
		destMethod = NeuromlConverter.convertDestMethod((neuroml.sim.run.DestMethod)modproj.method.parameters);	 
		neosimprojection = new GeneralProjectionImpl(srcpop,destpop,sourceMethod,destMethod); 
		reader.connect(neosimprojection);
	    } else {
		System.out.println("Error : makeAttachment : module is not a DestMethod");     
		writeErrors("- Attachment "+attach.getName()+" : module is not a DestMethod");
	    }
            *
            *
            *
            *
            *///**********************************************************************
            
            if (modproj.destMethod.parameters instanceof neuroml.sim.run.DestMethod) 
                destMethod = NeuromlConverter.convertDestMethod((neuroml.sim.run.DestMethod)modproj.destMethod.parameters);	 
            else {
		System.out.println("Error : makeAttachment : module is not a DestMethod");     
		writeErrors("- Attachment "+attach.getName()+" : module is not a DestMethod");
	    }
            if(modproj.sourceMethod==null)
                sourceMethod = new DefaultSourceMethodToAll((short)1);//new PortID(
            else if (modproj.sourceMethod.parameters == null)
                sourceMethod = new DefaultSourceMethodToAll((short)1);//new PortID(
            else if (modproj.sourceMethod.parameters instanceof neuroml.sim.run.SourceMethod) 
                sourceMethod = NeuromlConverter.convertSourceMethod((neuroml.sim.run.SourceMethod)modproj.sourceMethod.parameters);	 
            else
            {
                System.out.println("Error : makeAttachment : module is not a SourceMethod");     
		writeErrors("- Attachment "+attach.getName()+" : module is not a SourceMethod");
            }
            if(sourceMethod!=null&&destMethod!=null)
            {
                neosimprojection = new GeneralProjectionImpl(srcpop,destpop,sourceMethod,destMethod); 
                reader.connect(neosimprojection);
            }
            
	   // neosimprojection = new GeneralProjectionImpl(srcpop,destpop,sourceMethod,destMethod); 
	    //if (debug) { System.out.println("Make Module Attachment 5 "+srcpop+" "+destpop+" "+sourceMethod+" "+destMethod);}

	    //reader.connect(neosimprojection);
	    //if (debug) { System.out.println("Make Module Attachment 6");}
    	}  else { // default attachment

	    if (debug) { System.out.println("Make default Attachment"); }
	  
	    // use defaults
	    neosim.iface.SourceMethod sourceMethod = new DefaultSourceMethodToAll((short)1);//new PortID(
	    neosim.iface.DestMethod destMethod = new DefaultDestMethodFromAll((short)1,new Time(10));    //new PortID(
	    neosimprojection = new GeneralProjectionImpl(srcpop,destpop,sourceMethod,destMethod); 
	    reader.connect(neosimprojection);	    
    	}
    }
    
    /**
     * Starts the building of entity and loading of the modules (.jar or .lib)
     * <p>Call makeEntity for each entity of the simulation</p>
     *
     * @see #makeEntity
     */
    public void loadPrototypes(){
	// we assume that only the entities actually used for the simulation are in that list.
        if(Player!=null)
            Player.println("Load the prototypes");
	// errors log
	if(errorsReportFile==null)
	    createErrorFile();

	Vector entitiesVect = new Vector();

	Hashtable refs = namespace.getReferences();
	// get entities from cell pkg

	Hashtable objs = (Hashtable) refs.get("CellPkg");
	if (debug) System.out.println("objs list : "+objs.toString());

	if (objs!=null) {
	    for (Enumeration enum_ = objs.keys() ; enum_.hasMoreElements() ;) {
		String k = (String)enum_.nextElement();
		CellPkg cellpkg = (CellPkg)objs.get(k);
		entitiesVect.addAll(cellpkg.neurons.getElements());
	    }
	}
	
	// get probe entities from io package
	// there is currently only one io pkg but as well doing as if there were more
	//System.out.println("refs list : "+refs.toString());

	objs = (Hashtable) refs.get("Probe");
	if (debug) if (objs!=null) System.out.println("objs list : "+objs.toString());
	if (objs!=null) {
	    for (Enumeration enum_ = objs.keys() ; enum_.hasMoreElements() ;) {
		String k = (String)enum_.nextElement();
		Probe probe = (Probe)objs.get(k);
		entitiesVect.add(probe);
	    }
	}
 
	for (int i=0; i<entitiesVect.size(); i++) {
	    makeEntity((neuroml.model.network.Element)entitiesVect.elementAt(i));
	}
    }

    /**
     * Starts the building of a network 
     * <p>Call makePopulation for each population of the network</p>
     * <p>Call makeProjection for each projection of the network</p>
     *
     * @param net network 
     * @see #makePopulation
     * @see #makeProjection
     */
    public void buildNetwork(neuroml.model.network.Network net) {
	// network populations
        if(Player!=null)
            Player.println("Build Network "+net.getName());
	Vector popVect = net.elements.getElements(); 

	for(int j=0; j<popVect.size(); j++) {	
	    ElementRef e = (ElementRef)popVect.elementAt(j);
	    //makePopulation( e , null , new String(net.getName()+"."));
            makePopulation( e , net.getName());
	}
	// network projections
	Vector projVect = net.projections.getElements();

	for (int k=0; k<projVect.size(); k++) {	
	    //makeProjection( (neuroml.model.network.Projection)projVect.elementAt(k),null, new String(net.getName()+".") );
            makeProjection( (neuroml.model.network.Projection)projVect.elementAt(k),net.getName());
	}
        
        Vector probesVect = net.probes.getElements();
        
        for(int j=0; j<probesVect.size(); j++) {	
             Probe e = (Probe)probesVect.elementAt(j);
             neuroml.model.network.Population probePop = new neuroml.model.network.RealPopulation(e.getName(),e.getName()); // new neuroml.model.network.Population(e.getName(),e.getName()); JAG
             //makePopulation( probePop , null, new String(net.getName()+".") ); //should be network name
             makePopulation( probePop , net.getName());
        }
        
        Vector attachVect = net.attachments.getElements();
	// probes projections
        for (int k=0; k<attachVect.size(); k++) {
             //makeAttachment( (neuroml.model.network.Attachment)attachVect.elementAt(k), null, new String(net.getName()+".")  );
            makeAttachment( (neuroml.model.network.Attachment)attachVect.elementAt(k), net.getName());
        }
    }

    /**
     * Starts the building of the network of probes
     * <p>Call makePopulation for each population of probes</p>
     * <p>Call makeAttachment for each attachment of probes </p>
     *
     * @param probesVect vector of probes
     * @param attachVect vector of attachments
     * @see #makePopulation
     * @see #makeAttachment
     */
    public void buildProbesNetwork(Vector probesVect, Vector attachVect) {
/*
	// for all probes, build population	
	for(int j=0; j<probesVect.size(); j++) {	
	    Probe e = (Probe)probesVect.elementAt(j);
            neuroml.model.network.Population probePop = new neuroml.model.network.RealPopulation(e.getName(),e.getName()); // new neuroml.model.network.Population(e.getName(),e.getName()); JAG
	    makePopulation( probePop , null , new String() ); //should be network name
	}

	// probes projections
	for (int k=0; k<attachVect.size(); k++) {
	    System.out.println("attach "+attachVect.elementAt(k) );	
	    makeAttachment( (neuroml.model.network.Attachment)attachVect.elementAt(k) );
	}	*/
    }

    /** Error log opening method
     * 
     */
    void createErrorFile(){
	try {
	    errorsReportFile = new FileOutputStream(errorsReportFileName);
	    errorsReport  = new BufferedWriter(new OutputStreamWriter(errorsReportFile));
	    errorsOn = true;
	} catch ( FileNotFoundException fnfe ){
	    errorsOn = false;
	}
    }

    /** Error log closing method
     * 
     */
    void closeErrorFile(){
	if (errorsReportFile!=null) {
	    try {
		errorsReport.flush();
		errorsReport.close();
		errorsReportFile.close();
		errorsReportFile = null;
	    } catch (Exception e ){	
		System.out.println(e);
	    }	    
	}
    }

    /** Error log writing method
     * 
     */
    void writeErrors(String s) {
	if (errorsOn)
	try {
	    errorsReport.write(s);
	    errorsReport.newLine();
	} catch (Exception e) { System.out.println(e); }
    }

    /**
     *   DefaultSourceMethodToAll connects the cell to a all dest cells
     */
    class DefaultSourceMethodToAll extends SourceMethodImpl {       
	/**
	 * ConnectToAll
	 * @param srcPort outpout port of the source cell
	 *
	 */
        
	DefaultSourceMethodToAll(short srcPort) { //PortID
	    super( srcPort );	
	}    
	
	/**
	 * sendRequests to the dest cells
	 */
	public final void sendRequests( Entity srce, neosim.iface.Population dPop, neosim.iface.DestMethod dm ) {   
	    if (debug) System.out.println("DefaultSourceMethodToAll sendRequests dPop "+dPop);
	    for (int destIndex=0;destIndex<dPop.getNumEnts();destIndex++) {
		if (debug) System.out.println("DefaultSourceMethodToAll sendRequest destIndex "+destIndex);		
		sendRequest( srce, dPop.getEntityID(destIndex),
			     new ConnectionRequestImpl( srce.getIndex(), srce.getID(), srcPort, dm ),true );
	    }
	}
    }
    
    /**
     * DefaultDestMethodFromAll : default dest method : accept all conenction requests
     */
    class DefaultDestMethodFromAll extends DestMethodImpl {
	short destPort;//PortID
	Time delay;
	Hashtable params = new Hashtable();

	public DefaultDestMethodFromAll(short destPort, Time delay) {//PortID
	    super(destPort,delay);
	    this.destPort = destPort; 
	    this.delay    = delay;
	}
	
	/** The considerRequest method checks the source entity's details 
	 * against the dest entity's details, and returns True or False
	 * depending on whether or not the connection will be made. 
	 * @param deste the destination entity
	 * @param cr the connection request from the source entity
	 * @return connection if a connection made, null otherwise.
	 */
	public neosim.iface.Connection considerRequest( Entity deste, ConnectionRequest cr ) {	    
	    ParamConnectionSpec c = new ParamConnectionSpec( cr.getSrcEntityID(), cr.getSrcPortID(), 
							     deste.getID(), destPort, delay.t, params);
	    int cid = this.makeConnection(deste, c,true );//ConnectionID cid
	    c.setDstParams(deste,cid);	    
	    return c;
	}		
    }        
}
