package neuroml;
 
import neuroml.core.*;
import neuroml.model.ModelPkg;
import neuroml.model.cell.CellPkg;
import neuroml.model.channel.ChannelPkg;
import neuroml.model.network.NetworkPkg;
import neuroml.environment.EnvironmentPkg;
import neuroml.environment.io.IOPkg;
import neuroml.environment.schedule.SchedulePkg;
import neuroml.sim.run.SimRunPkg;
import neuroml.util.module.UtilModulePkg;

import neuroml.model.I_Clamp.I_ClampNmlPkg;
import neuroml.model.Synapse_nml.synapse_nmlPkg;
import neuroml.model.ions_channel.Ions_channelPkg;
import neuroml.model.MeasurementTool.MeasurementToolPkg;

import java.util.Vector;

/** Top level class for NeuroML.<p>
 * This maps to a top level XML tag for neuroml:
 *<pre>
 *  &lt;neuroml class="NeuromlPkg" description="..." ...>
 *     &lt;... here be all contents of any neuroml file ...>
 *  &lt;/neuroml>
 *</pre><p>
 *
 * It also defines a static string which lists all neuroml model
 * packages. This is used by the XMLIn parser to look up class names.
 */
public class NeuromlPkg extends Pkg {    

    /** Description
     */
    public String description = "";

    /** Author
     */
    public String author = "";

    /** Space separated list of keywords
     */
    public String keywords = "";
    
    /**List of references to ModuleRef IIRC JAG */
    //public PermanentSet ImplementionModules=new PermanentSet("Modules Implementation");
    public Set ImplementionModules=new PermanentSet("Modules Implementation");

    /** The model definitions */
    public Set models = new Set("Model");

    /** The environment for the models - input, output, schedule */
    public EnvironmentPkg environment = new EnvironmentPkg();

    /** The code modules referred to in the model definition */
    //public Set modules = new Set("Module");
    public Set modules = new Set("Module");

    /** A list of neuroml model packages.
     * Used by the XMLIn
     * @see neuroml.util.XMLIn
     */

    public static String[] getPackageList() { 
	String[] ret = {
	    "neuroml",
	    "neuroml.core",
	    "neuroml.model",
	    "neuroml.model.cell",
	    "neuroml.model.channel",
	    "neuroml.model.network",
            "neuroml.model.I_Clamp",
            "neuroml.model.Synapse_nml",
            "neuroml.model.ions_channel",
            "neuroml.model.MeasurementTool",            
	    "neuroml.data",
	    "neuroml.environment.io",
	    "neuroml.environment.schedule",
	    "neuroml.environment",
            "neuroml.sim.run",
            "neuroml.sim.state",
            "neuroml.util",
            "neuroml.util.module"

        };
	return ret;
    }


    /** utility to add array of classnames 
     * to vector, prepending with full package name
     */
    static void appendArray(Vector v, String pkg, String[]a) {
	for (int i=0; i<a.length; i++) v.addElement(pkg+"."+a[i]);
    }

    /** Vector of string -> String[] */
    public static String[] vectorToArray(Vector v) {//jag public
	String[] a = new String[v.size()];
	for (int i=0; i<a.length; i++) a[i] = (String)v.elementAt(i);
	return a;	
    }
    
    /** Return a list of all classes 
     * Used by the Namespace.  This seems a klunky way to do this,
     * but java.lang.Package doesn't include an equivalent.
     *
     * @see neuroml.util.Namespace
     */
    public static String[] getClassList() {
	Vector ret = new Vector();
	appendArray( ret, "neuroml.core", Pkg.getClassList() );
	appendArray( ret, "neuroml.model.channel", ChannelPkg.getClassList() );
	appendArray( ret, "neuroml.model.cell", CellPkg.getClassList() );
	appendArray( ret, "neuroml.model.network", NetworkPkg.getClassList() );
	appendArray( ret, "neuroml.environment", EnvironmentPkg.getClassList() );
	appendArray( ret, "neuroml.environment.io", IOPkg.getClassList() );
	appendArray( ret, "neuroml.environment.schedule", SchedulePkg.getClassList() );
        appendArray(ret, "neuroml.sim.run", SimRunPkg.getClassList());
        appendArray(ret, "neuroml.util.module", UtilModulePkg.getClassList());

        appendArray(ret, "neuroml.model.I_Clamp", I_ClampNmlPkg.getClassList());
        appendArray(ret, "neuroml.model.Synapse_nml", synapse_nmlPkg.getClassList());
        appendArray(ret, "neuroml.model.ions_channel", Ions_channelPkg.getClassList());
        appendArray( ret, "neuroml.model.MeasurementTool", MeasurementToolPkg.getClassList() );
	return vectorToArray( ret );
    }
}
