package neuroml.core;

/**
 * A superclass for enumerated values.
 * This needs to be subclassed:
 * <pre>
 *   public MyState extends Enum {
 *     static string[] options = {"STATE1", "STATE2", "STATE3"};
 *     public String[] getOptions() { return options; }
 *   }
 *
 *   example usage:-
 *     public Test {
 *       public MyState mystate = new MyState("STATE1");
 *     }
 *
 *   example NeuroML:-
 *     &lt;neuroml class="Test" mystate="STATE1"/>
 * </pre>
 */
public class Enum {  

    protected String value="";

    public Enum() { 
    }

    /** Construct an enumerated value
     */
    public Enum(String v) { 
	value = v;
    }

    /** Return the range of options of this ENUM
     * @return the array of options of this ENUM
     */
    public String[] getOptions() { String[] opts = {"true","false"};  return opts; }

    /** Returns the value of this ENUM
     * @return the value of this ENUM as a string
     */
    public String getValue() { return value; }

    /** Checks if the set value is valid
     * @param  p the value to check against getOptions()
     * @return true is value is valid
     */
    public boolean isValid(String p) { 
	boolean valid = false; 
	String[] opts = getOptions();
	for (int i=0; i<opts.length; i++) {
	    if (p.equals(opts[i])) { valid = true; }
	}
	return valid;
    }

    /** Returns the index of a given value
     * @param  p the value to check against getOptions()
     * @return true is value is valid
     */
    public int indexOf(String p) { 
	int found = -1;
	String[] opts = getOptions();
	for (int i=0; i<opts.length; i++) {
	    if (p.equals(opts[i])) { found=i; }
	}
	return found;
    }

    /** Returns the value of this ENUM
     * @return the value of this ENUM as a string
     */
    public void setValue(String v) { value=v; } 

    /** Converts this enum to a string
     * @return the value of this ENUM as a string
     */
    public String toString() { return value; }
}
