package neuroml.core;

import java.util.Vector;
import java.util.Collection;
import java.util.Iterator;

/**
 * Core support for collections of objects.
 * This should be changed to java.util.Collection
 * so user classes don't need a dependency on this one.
 * 
 * This one could become a "collection with default type"
 * and implement java.util.Collection
 */
public class Set extends NamedObject implements ContentRestricted, java.util.Collection  {
    protected String type;
    protected Vector<Object> elements; // to become Set in future

    /** Construct an empty Set
     */
    public Set() {
	elements=new Vector<Object>();
	name = new String();
	type = new String();
    }
    /** Construct an empty Set of a default class
     */
    public Set(String type) { 
	this.type = type;	
	elements=new Vector<Object>(); 
	name = new String();
    }
  
    /** @Deprecated use add() instead */
    public final void addElement( Object o ) { elements.addElement(o); }
    /** @Deprecated use remove() instead */
    public final void removeElement( Object o ) { elements.removeElement(o); }
    /** @Deprecated use getContentClassName() instead */
    public String getElementType() { return type; }

    /** @Deprecated use getContentClassName() instead */
    public final String getType() { return type; }
    /** @Deprecated use setContentClassName() instead */
    public final void setType(String n) { type = n; }    
   
    /** @Deprecated - use the java.util.Collection interface to 
     * this set instead
     */
    public final Vector<Object> getElements() { return elements; 
}
    /** @Deprecated - use the java.util.Collection interface clear() addAll() instead.
     */
    public final void setElements(Vector<Object> elems) { elements = elems; }

    // Implementation of java.util.Collection - delegated to elements.
    public final boolean add(Object o) { return elements.add(o); }
    public final boolean addAll(Collection c) { return elements.addAll(c); }
    public final void clear() { elements.clear(); }
    public final boolean contains(Object o) { return elements.contains(o); }
    public final boolean containsAll(Collection c) { return elements.containsAll(c); }
    public final boolean equals(Object o) { return elements.equals(o); }
    public final int hashCode() { return elements.hashCode(); }
    public final boolean isEmpty() { return elements.isEmpty(); }
    public final Iterator iterator() { return elements.iterator(); }
    public final boolean remove(Object o) { return elements.remove(o); }
    public final boolean removeAll(Collection c) { return elements.removeAll(c); }
    public final boolean retainAll(Collection c) { return elements.retainAll(c); }
    public final int size() { return elements.size(); }
    public final Object[] toArray() {return elements.toArray(); }
    public final Object[] toArray(Object[] a) { return elements.toArray(a); }    

    // Implementation of ContentRestricted
    public final String getContentClassName() { return type; }
    public final void setContentClassName(String s) { type = s; }
    
    public final Object elementAt(int i)
    {
        return elements.elementAt(i);
    }
    
}
