/*
 * TreeRef.java
 *
 * Created on 13 January 2004, 16:41
 */

package neuroml.core;
import java.util.Vector;
import neuroml.util.*;
/**
 *
 * @author  Jean-Alain Grunchec
 * JAG for network
 *
 * Elements refers to the "elements" attribute of a certain set. 
 *
 * This set is supposed to contain objects of a certain class (for instance "population").
 * This class can be subdivided in objects that needs tracking (for instance "RealPopulation" and "SubNetwork").
 * (String)Name of classes which need to be tracked must be either in leaves or nodes. 
 * If it is in leaves, then the object is a leaf object (no editable subobjects) (i.e. RealPopulation).
 * If it is in nodes, then the object is a node object (subobjects can be edited) (i.e. SubNetwork).
 * For GUI implementation, see guikit.EditTreeRefPanel
 * 
 *
 */
public class TreeRef extends Ref {
   
    Vector<String> leaves=new Vector<String>(1,1);
    Vector<String> nodes=new Vector<String>(1,1);
    Vector<Object> Elements;
    /** Creates a new instance of TreeRef */
    public TreeRef(Set ReferedSet) {
        this(null,ReferedSet);
    }
    
    public TreeRef(String string,Set ReferedSet) {
        super(string);
        Elements=ReferedSet.getElements();
    }
    
    public void addLeaf(String Leaf)
    {
        if(!leaves.contains(Leaf))
            leaves.add(Leaf);
    }
    
    public void removeLeaf(String Leaf)
    {
        if(leaves.contains(Leaf))
            leaves.remove(Leaf);
    }
    
    public void addNode(String Node)
    {
        if(!nodes.contains(Node))
            nodes.add(Node);
    }
    
    public void removeNode(String Node)
    {
         if(nodes.contains(Node))
            nodes.remove(Node);
    }  
    
    public TreeRef(String string) {
	super(string);
    }
    
    public TreeRef(String string, String string_0_) {
        super(string,string_0_);
    } 
    
    public Vector<String> getLeaves()
    {
        return leaves;
    }
    
    public Vector<String> getNodes()
    {
        return nodes;
    }
    
    public Vector<Object> getElements()
    {
        return Elements;
    }
    
    public void setElements(Vector<Object> Elem)
    {
        Elements=Elem;
    }
    public void setTargetName(String n) { 
        try{
            NeuromlSys.getNewestNamespace().UpdateRefered(targetName,n,this);
        }
        catch(Exception e){
            
        }
        targetName = n;
    }
}
