package neuroml.model.channel;

import neuroml.core.*;

/** May be used instead of ParameterizedHHVGate or TabulatedHHGate to
 *  provide more flexibility in the choice of what Rate classes are used
 *  for forwardRate and backwardRate.  This is done by casting them to
 *  The appropriate subclass of HHRate. It also provides additional fields
 *  to describe the gate.  
 * @author David Beeman, University of Colorado, Boulder
 */
public class HHVGate extends HHGate {

    public HHRate forwardRate = new HHRate();
    public HHRate backwardRate= new HHRate();

    /** Time unit used with the rate variables. e.g. "sec" or "msec".
     */
    public Ref timeUnit = new Ref("Unit");

    /** Voltage unit used for the rate variables. e.g. "V" or "mV".
     */
    public Ref voltageUnit = new Ref("Unit");

    /** Minimum voltage expected to be applied to the gate
    */
    public double vmin;

    /** Maximum voltage expected to be applied to the gate
    */
    public double vmax;


/** If true, perform an instantaneous calculation of the gate activation
 * variable, setting it equal to forwardRate/(forwardRate + backwardRate).
 * The default (false) is to solve the Hodgkin-Huxley DE's.
 */
    public boolean instantCalculation = false;

/** If true, use forwardRate, backwardRate for state variables tau, minf
 *  instead of the default of using rate variables alpha, beta.
*/
    public boolean useState = false;

    public HHVGate() {
    }

    public HHVGate(String name) {
	super(name);
    }

}
