package neuroml.sim.run;
 
import neuroml.sim.state.*;

/** Interface for a post-synaptic terminal.
 */
public interface PostSynapse {

    /** Handle a spike of neurotransmitter. 
     * This will be called by the runnable cell using the results of the
     * corresponding PreSynapse.handleSpike()
     *
     * @param t the spike arrival time
     * @param o a data object for (e.g.) amount of neuroxmitter release
     */
    public void handleNeurotransmitter(double t, Object o);

    /** Handle backpropgating action potential
     * This will be called by the runnable cell when it fires.
     *
     * @param t the backpropgating spike arrival time at this synapse.
     * @param data a data object for (e.g.) the strength of backpropogating spike
     */
    public void handleBackpropogatingSpike(double t, Object data);

    /** Advance internal state of this pre synaptic terminal
     */
    public void advance(double t);

    /** Return state of this post synaptic terminal.
     */
    public SynapseState getState();
}
