package neuroml.sim.run;
 
import neuroml.sim.state.*;
import neuroml.model.MeasurementTool.*;
/** A runnable cell. This interface is used for 
 * modules which can send and receive events.
 */
public interface RunnableCell extends java.io.Serializable {
 
    /** Return current cell state 
     */
    public ElementState getState();

    /** Return neuroml model which produced this 
     */
    public Object getModel();
    
    /** Handle a spike on a given input 	
     */
    public void handleSpike(double t, int inputConnection);  

    /** Handle any other kind of event
     */
    public void handle(Object o, double t, int inputConnection);

    /** Advance time to t secs
     */
    public void advance(double t);

    /** reset to time 0 
     */
    public void reset();


    // --- temporary ---

    /** Set internal connection parameters. Can be used to 
     * set extra synapse parameters used locally in the runnable
     * cell, e.g. weights.   The values for these weights may
     * be extracted from the DestMethod
     * 
     * @param cid the connection ID     
     * @param dm the destination method used (way to get delay/dst port ID, possibly weight if that's
                    part of the destmethod)
     * @param cr The connection request sent from the source
     */
    public void setInputConnectionParameters(int cid, DestMethod dm, ConnectionRequest cr);
    
    /**
     * Give Feedback to the neuron if it does have to send measures
     */
    public void setMeasured(MeasurementCollection MeasurementList);
    public void setMeasured(int SrcId, MeasurementCollection MeasurementList);
}
