package neuroml.sim.run;
 
import neuroml.sim.state.*;
import java.util.Vector;
import neuroml.util.*;
import neuroml.core.*;


/** The interface a Simulator provides to give access to control 
 * simulation runs, and access to instantiated state.
 * NOTE This interface is subject to change as it includes
 * too many methods.
 */
public interface Simulator {
     
    /** Initialise the simulator.
     * @Deprecated as I'm not convinced this is always needed
     */
    public void bootstrap(String args[]);

    /** Initialise kernel - 
     * defaults (seq/thr/rmi) provided by ScriptReaderImpl
     */
    public void initKernel(String args[]);

    /** Initialise the simulation 
     * @Deprecated as I'm not convinced this is always needed
     */
    public void initSim();

    /** Run the simulation until the given time
     * @param t the time to run until.
     */
    public void run(double t);    

    /** Run the simulation until the given time
     * @param t the time to run until.
     * @Deprecated use run() instead
     */
    public void runSim(double t);    

    /** Terminate the kernel.
     * @Deprecated as I'm not convinced this is always needed
     */
    public void terminate();

    /** Parameterise the simulation 
     * @Deprecated as I'm not convinced this is always needed
     */
    public void setParameters(String args[]);

    /** Launch the simulation when loaded from a module
     * @Deprecated ? or at least improve the docs
     */
    public void launch(String args[]);
    
    /** Launch the simulation when loaded from a module
     * using public parameter fields of the simulator
     * @Deprecated ? or at least improve the docs
     */
    public void launch();

    /** Build a Neuroml model specified in the argument
     * @param model the neuroml model to build
     * @param ns the namespace
     */
    public void buildModel(Object model, Namespace ns);

    /** Returns the model.
     */
    public Object getModel();

    /** Reset the simulation to time 0
     */
    public void reset();

    /** Construct a population of entities
     * @param p the population to build
     */
    //public void construct ( PopulationFactory p);

    /** Destroy a population of entities
     * @param p the population to destroy
     */
    //public void destroy ( String p );

    /** Connect up a projection
     * @param p the projection to make
     */
    //public void connect ( ProjectionState p );

    /** Disconnect a projection
     * @param p the projection to break
     */
    //public void disconnect ( ProjectionState p );


    /** Perform a generic update on an entity
     * @param e the id of the entity to update
     * @param eu the update to perform
     */   
    //public void   updateElement( int e, String eu );


    /** Perform an query on an entity
     * @param e the id of the entity to update
     * @param eq the query to perform
     * @return the result of the query (as an Object)
     * @Deprecated
     */       
    public Object queryElement( int e, String eq );

    /** Query an element and return the object
     * @param e the id of the element to query
     * @param q the query to run
     * @return  the reponse to the query
     */
    public Object queryElement( int e, Query q );

    /** Perform an update on a population
     * @param p the population (by its name)to query
     * @param eu the name of the update to perform
     */       
    //public void   updatePopulation( String p, String eu );

    /** Perform an query on a population
     * @param p the population (by its name) to query
     * @param eq the name of the query to perform
     * @return the result of the query (as a list of objects)
     */       
    public Vector queryPopulation( String p, String eq );

    /** Perform an query on a population
     * @param p the population's name to query
     * @param q the query to perform
     * @return the result of the query (as a list of Objects)
     */       
    public java.util.List queryPopulation( String p, Query q );

    /** Perform an query on the output connections of an entity
     * @param e the source entity
     * @param p the source output port ID
     * @param cq the query to perform
     * @return the result of the query (as a Vector of Objects)
     */       
    //public Vector queryOutputConnections( int e, int p, ConnectionStateQuery cq );

    /** Lookup an entity ID given its name
     * @param name the name of the element
     * @return the element ID
     */
    public int lookupElementID(String name);

    /** Lookup an element given its name
     * @param name the name of the element
     * @return the element's state
     */   
    public ElementState lookupElement(String name);

    /** Lookup a population given its name
     * @param name the name of the element
     * @return the element's state
     */   
    public PopulationState lookupPopulation(String name);

    /** Get root of the population tree
     * @return the population state of the root. Should be NetworkState.
     */   
    public PopulationState getRootElement();
    
    public Object getReader();

}
