package neuroml.util;
import java.util.Vector;
 
/** Information about a neuroml class -
 * Includes "derivedFrom" info which you don't get from java.lang.Class
 *
 * @see Namespace
 */
public class ClassInfo {

    /** Name of class */
    String className; 

    /** Parent's info */
    ClassInfo parentClassInfo;

    /** Sub classes - vector of classinfo */
    Vector<ClassInfo> subClasses = new Vector<ClassInfo>();

    public ClassInfo(String cn, ClassInfo p) {
	className = cn;
	parentClassInfo = p;
    }

    public String getClassName() { return className; }

    /** Add a subclass */
    public void addSubClass( ClassInfo subclass ) {
	subClasses.addElement( subclass );
    }

    /** Remove a subclass */
    public void removeSubClass( ClassInfo subclass ) {
	subClasses.remove( subclass );
    }

    /** Get subclasses (direct descendents) */
    public Vector getSubClasses() { return subClasses; }

    /** Get all subclasses (incl descendents of descendents) */
    public Vector<ClassInfo> getAllSubClasses() {
	Vector<ClassInfo> ret = new Vector<ClassInfo>();
	ret.addAll(subClasses);
	for (int i=0; i<subClasses.size(); i++) {
	    ret.addAll( (subClasses.elementAt(i)).getAllSubClasses() );
	}	
	return ret;
    }
    

    public String toString() { return className; }
}  
