package neuroml.util;

import neuroml.sim.run.*;

import java.util.Hashtable;
import java.net.URL;
import java.util.zip.*;

import java.io.*;
import java.util.*;
import java.util.zip.*;
import java.beans.*;
import java.net.*;
import java.awt.*;

/**
 * Neuroml Module Loader - 
 * utility functions to create objects from 
 * jar files. <br>
 *
 * Usage   : <tt>Object o = ModuleLoader.getObject( {url of jar file}, {object name} )</tt><p>
 * Usage II: <tt>Class c = ModuleLoader.getBeanClass( {url of jar file}, {class name} )</tt>
 *
 * Missing features: <br>
 * * get an image
 * * get appropriate parameters
 * 
 *
 * @author Fred Howell
 * @date Nov 2000
 */
public class ModuleLoader {

    /** Manifest &c of previously loaded jar
     */
    static JarLoader jl = null;
    static ObjectBean ob = null;

    /** Call JarLoader.loadJarDoOnBean and catch any exceptions
     */
    static void loadJar( String ename, DoOnBean dob ) {
	try {      
	    jl = JarLoader.loadJarDoOnBean(ename, dob);
	} catch (Exception e) {
	    System.out.println("Module Loader exception loading"+ename+" "+e);
	    e.printStackTrace();
	}      	
    }

    /** Return an object from a jar file
     */
    public static Object getObject(String ename, String name) {
        ob = new ObjectBean(name);
	loadJar(ename,ob);
        return ob.getObject();
    }

    /** Return an object from a jar file
     */
    public static Simulator getSimulator(String ename, String name) {
        SimulatorBean os = new SimulatorBean(name);
	loadJar(ename,os);
        return os.getSimulator();
    }

    /** Return class name
     */
    public static String getObjectName() {
	return ob.getBeanName();
    }

    /** Return manifest for previously loaded jar file
     * Note we are using our own implementation of Manifest.java,
     * not the one in java.util.jar
     */
    public static String getManifestValue(String name) {
	Manifest mf = null;
	MessageHeader mh = null;
	if (jl!=null) {
	    try {
		// --- Restricted to first manifest entry ! ---
		mf = jl.getManifest();
		mh = mf.entryAt(0);
		return mh.findValue( name );
	    } catch (Exception e) {
		System.out.println("Couldn't find manifest "+name+" "+mf+" "+mh);
	    }
	    return null;
	}
	return null;
    }

    /** Return a resource from a jar file (previously loaded)
     */
    public static URL getResource(String ename, String name) {
	if (name==null) { return null; }
	else { return JarLoader.getLoader().getResource ( name ); }
    }
 
    /** Return a class from a jar file
     */
    public static Class getBeanClass(String ename, String name) {
        ClassBean eb = new ClassBean(name);
	loadJar(ename,eb);
        return eb.getBeanClass();
    }
 
}
