/**
 * Neuroml runnable cell factory
 */

package neuroml.util.module;

import neuroml.sim.run.*;
import neuroml.sim.state.*;
import neuroml.core.*;
import neuroml.model.MeasurementTool.MeasurementSet;
 
/** Produces instances of runnable cells 
 */
public class BasicProbeFactory implements RunnableCellFactory, Cloneable {
      
    /** The NeuroML model which will be used to produce
     * instances
     */
    Object model;

    /** There has to be a null constructor, as this class
     * will go in a loadable module
     */
    public BasicProbeFactory() {	
    }

    /** Initialise this factory
     */
    public void init(Object model) {
	this.model = model;
    }
    

    int inPort=1, outPort=1;

    /** Sets the port values (*** Warning - this function may change ***)
     */
    public void setInPort(int inport){ inPort = inport; }
    public void setOutPort(int outport){ outPort = outport; }

    /** Gets the port values
     */
    public int getInPort() { return inPort;}
    public int getOutPort() { return outPort;}

    /** This is the important method of this class - it makes new
     * instances
     * @param parent The parent population
     * @param index The index in the parent population
     * @param sps The target of the cell's calls to send a spike
     */
    public RunnableCell makeRunnableCell(PopulationState parent,int id, int index, 
					 SpikeSender sps) {
	return new BasicRunnableCell(this, parent,id, index, sps);
    }
   
    /** Set the model of this cell - called by the module loader
     */
    public void setModel(Object model) {
	this.model = model;
    }

    /** Get the model of this cell
     */
    public Object getModel(){
	return model;
    }
    
    public void setFactory(MeasurementSet MeasurementList, String str) {
    }  
    
    public boolean IsProbe() {
        return true;
    }
    
}
