/**
 * Neuroml runnable cell factory
 */

package neuroml.util.module;

import neuroml.sim.run.*;
import neuroml.sim.state.*;
import neuroml.core.*;
import neuroml.model.network.*;
import neuroml.model.MeasurementTool.*;
import java.awt.*;
import java.awt.geom.*;
import guikit.*;
/*
 * Produces instances of runnable cells 
 */

public class BasicRunnableCellFactory implements RunnableCellFactory, Cloneable, java.io.Serializable {
      
    /*
     * The NeuroML model which will be used to produce
     * instances
     */
    transient Object model; //required ???
    


    /*
     * There has to be a null constructor, as this class
     * will go in a loadable module
     */
    
    public BasicRunnableCellFactory() {	

    }

    /** Initialise this factory
     */
    public void init(Object model) {
	this.model = model;
    }
    

    int inPort=1, outPort=1;

    /** Sets the port values (*** Warning - this function may change ***)
     */
    public final void setInPort(int inport){ inPort = inport; }
    public final void setOutPort(int outport){ outPort = outport; }

    /** Gets the port values
     */
    public final int getInPort() { return inPort;}
    public final int getOutPort() { return outPort;}

    /** This is the important method of this class - it makes new
     * instances
     * @param parent The parent population
     * @param index The index in the parent population
     * @param sps The target of the cell's calls to send a spike
     */
    public RunnableCell makeRunnableCell(PopulationState p, int id, int index, SpikeSender sps) {
	return new BasicRunnableCell(this, p, id, index, sps);
    }
   
    /** Set the model of this cell - called by the module loader
     */
    public final void setModel(Object model) {
	this.model = model;
    }

    /** Get the model of this cell
     */
    public final Object getModel(){
	return model;
    }
    
    public void draw(Graphics2D g, Object pop, double x, double y, double width, double height, String msg, boolean selected, Font f){
        BasicStroke stroke = new BasicStroke(2.0f);
        BasicStroke wideStroke = new BasicStroke(3.0f);
        BasicStroke thickStroke = new BasicStroke(3.0f);
        int sw = g.getFontMetrics(f).stringWidth(msg);
	int sh = g.getFontMetrics(f).getHeight();
	g.setFont(f);
	int shadow = 4;
        Color colour;
               
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        
        double SIZE=height;
        if(SIZE>width) SIZE=width;
        
        g.setStroke(wideStroke);
        g.setColor(Color.BLUE);
        g.draw(new Line2D.Double(x,y,x+SIZE*0.5,y));
        g.draw(new Line2D.Double(x,y,x+SIZE*0.5/1.4142,y+SIZE*0.5/1.4142));
        g.draw(new Line2D.Double(x,y,x,y+SIZE*0.5));
        g.setStroke(thickStroke);
        g.setColor(Color.RED);
        g.setStroke(stroke);

        g.draw(new Line2D.Double(x,y,x-width*0.5,y-height*0.5));
        g.setColor(Color.white);  
        
        if(selected){
            g.draw( new RoundRectangle2D.Double( x-width*0.5, y-height*0.5, width, height,width*0.2, height*0.2) );
        }
        else{
            Stroke drawingStroke3 = new BasicStroke(1,BasicStroke.CAP_BUTT,BasicStroke.JOIN_ROUND,0, new float[] {3.0f, 6.0f}, 0);
            g.setColor(Color.darkGray);
            g.setStroke(drawingStroke3);
            g.draw( new RoundRectangle2D.Double( x-width*0.5, y-height*0.5, width, height,width*0.2, height*0.2) );
        }

                       // g.draw(new Arc2D.Double(x-width*0.5, y, width, height, 90.0,90.0,Arc2D.OPEN));g.draw(new Arc2D.Double(x-width*0.5, y, width, height, 90.0,90.0,Arc2D.OPEN));
        g.setStroke(stroke);
        g.setColor(Color.DARK_GRAY);
        g.fill(new Ellipse2D.Double(x-SIZE*0.4,  y-SIZE*0.4, SIZE*0.8,SIZE*0.8));
        g.setColor(getColorSoma());
        g.fill(new Ellipse2D.Double(x-SIZE*0.3,  y-SIZE*0.3, SIZE*0.6,SIZE*0.6));
        
        DrawExtra(g, pop, x, y, width, height, msg, selected, f);
        
	g.setColor(Color.white); // Should be foreground!!
	g.drawString(msg, (float)(x - sw/2), (float)(y - height*0.25));
        //g.setColor(Color.BLACK);
        
        if(pop instanceof RealPopulation)
        {
            PopulationStructure ps = ((RealPopulation)pop).structure;
            if (ps instanceof Grid1DStructure) {
                  int sz = ((Grid1DStructure)ps).size;
                  g.drawString("size="+sz,(float)x - sw/2, (float)y +sh*1.5f);
                 
              } else if (ps instanceof Grid3DStructure) {
                  int xs = ((Grid3DStructure)ps).xsize;
                  int ys = ((Grid3DStructure)ps).ysize;
                  int zs = ((Grid3DStructure)ps).zsize;
                  g.drawString("size=["+
			((Grid3DStructure)ps).xsize+","+
			((Grid3DStructure)ps).ysize+","+
			((Grid3DStructure)ps).zsize+"]",
			(float)x - sw/2, (float)y +sh*1.5f);
                } else if (ps instanceof IndexedStructure) {
                    g.drawString("index ndims=["+
				 ((IndexedStructure)ps).ndims+"]",
				 (float)x - sw/2, (float)y +sh*1.5f);
                } else if (ps instanceof ModulePosition) {
                    Object o=((ModulePosition)ps).module.parameters;
                    if(o instanceof PositionModule){
                            g.drawString("Size: "+
				 ((PositionModule)o).getSize(),
				 (float)x - sw/2, (float)y +sh*1.5f);
                    }
                }
            }//RealPopulation           
    }
    public Color getColorSoma(){
        return null;
    }
    
    public void DrawExtra(Graphics2D g, Object pop, double x, double y, double width, double height, String msg, boolean selected, Font f){
    }
       
    public void setFactory(MeasurementSet MeasurementList, String str) {
        ModuleMeasurement msTool=new ModuleMeasurement();
	msTool.module.parameters=new CellExternalIntensity();
        MeasurementList.add(msTool);
        
        msTool=new ModuleMeasurement();
	msTool.module.parameters=new CellInternalIntensity();
        MeasurementList.add(msTool);
        
        msTool=new ModuleMeasurement();
	msTool.module.parameters=new CellIntensity();
        MeasurementList.add(msTool);
        
        msTool=new ModuleMeasurement();
	msTool.module.parameters=new CellVoltage();
        MeasurementList.add(msTool);       
    }
    
    public boolean IsProbe() {
        return false;
    }
    
}
