package neuroml.util.schema;

import neuroml.core.*; 
import neuroml.util.*; 
import java.util.Vector;
import java.util.Iterator;
import java.util.Hashtable;
import java.lang.reflect.*;
import java.io.*;
import java.net.URL;
import java.io.*;

import java.io.InputStream;
import java.io.FileInputStream;
import java.io.File;
import java.net.URL;
import java.util.zip.*;
import java.util.jar.*;


/** 
 * The top level NeuromlSchema class.
 *
 *<p>
 * <em>... Is this better than choosing java classes/.jar files as the standard?
 * - possibly as you can edit XML with a standard editor (w/o being a programmer)
 * - and maybe convert it into/from XML-Schema using XSLT...</em>
 * 
 * @author Fred Howell
 * @date 28/3/01
 */
public class NeuromlSchema {

    //  public String schemaLocation="http://www.anc.ed.ac.uk/~fwh/repository/nmlschema/nmlschema.jar";
    /** The namespace of this schema. This could point to a NeuromlSchema definition 
     * of this document
     */
    public String xmlns="http://www.neuroml.org/neuroml/util/nmlschemaschema.nml";


    /** The set of packages in this schema
     */
    public Set packages = new Set("neuroml.util.schema.NeuromlPackage");
    
    /** Converts a java package to instance of NeuromlPackage / NeuromlClass
     * Works by instantiating all the class names, reflecting and producing
     * instances of NeuromlPackage and NeuromlClass.
     * @param jarfile the url of the .jar file
     * @param pkgname the fully qualified package name
     * @param classNames the class names
     */
    public static NeuromlPackage toNeuromlPackage(String jarfile, String pkgname, String[] classNames) {
	try { 
	    URL u = new URL(jarfile);   
	    NeuromlSys.registerClassesFromJar(u);
	} catch (Exception e) {}
	return toNeuromlPackage(pkgname,classNames);
    }
    public static NeuromlPackage toNeuromlPackage( String pkgname, String[] classNames) {
	NeuromlPackage ret = new NeuromlPackage(pkgname);
	for (int i=0; i<classNames.length; i++) {
	    NeuromlClass nc = toNeuromlClass( pkgname, classNames[i] );
	    if (nc!=null) ret.classes.addElement( nc );
	}
	return ret;
    }

    /** Convert a string class name to a NeuromlClass by instantiating
     * the class using reflection.
     */
    public static NeuromlClass toNeuromlClass(String pkgname, String sname) {
	NeuromlClass ret = new NeuromlClass(sname);
	try {
	    Class c = NeuromlSys.getClassFromJar( pkgname+"."+sname );
	    Class sc = c.getSuperclass();
	    if (!sc.getName().equals("java.lang.Object")) {
		ret.base = sc.getName();
	    }
	    Object o = c.newInstance();
	    Field[] flds = c.getDeclaredFields(); // not getFields
	    for (int i=0; i<flds.length; i++) {
		int modif = flds[i].getModifiers();		
		if (Modifier.isPublic( modif ) && !Modifier.isStatic( modif )) {	
		    Object fob = flds[i].get(o);
		    NeuromlField nf = new NeuromlField( flds[i].getName(),  
							XMLOut.chopClass(flds[i].getType().getName()) );
		    // Check if field has elementType
		    try {
			if (fob==null) {
			    fob = flds[i].getType().newInstance();
			}
			if (fob instanceof ContentRestricted) {
			    nf.elementType = XMLOut.chopClass(((ContentRestricted)fob).getContentClassName());
			}
		    } catch (Exception e) {}

		    // Not doing defaults yet...
		    
		    ret.fields.addElement( nf );
		}
	    }
	    return ret;
	} catch (Exception e) {
	    System.out.println("Couldn't make "+pkgname+"."+sname+" "+e);
	    return null;
	}
    }

    /** This method converts a NeuromlSchema to java source. Example use:
<code>
	Hashtable jsrc = toJava(ns);
	java.util.Set s = jsrc.keySet();
        for (Iterator i1 = s.iterator(); i1.hasNext(); ) {
            String pn = (String)i1.next();
            Hashtable jsrcs = (Hashtable)jsrc.get(pn);
	    for (Iterator i2 = jsrcs.keySet().iterator(); i2.hasNext(); ) {
		String cn = (String)i2.next();
		String java = jsrcs.get(cn);
		System.out.println("\n\nGenerating "+pn+"/"+cn+".java :-");
		System.out.println(java);
	    }
	    
        }
</code>
     * @return Hashtable [package name] of hashtable [class name] of string [java source]
     */
    public static Hashtable toJavaSource(NeuromlSchema ns) {
	Hashtable<String,Hashtable<String,String>> pkg = new Hashtable<String,Hashtable<String,String>>();

	Vector v = ns.packages.getElements();
	for (int p=0; p<v.size(); p++) {
	    NeuromlPackage np = (NeuromlPackage)v.elementAt(p);
	    Hashtable<String,String> pc = new Hashtable<String,String>(); // Hash of classnames -> java sources
	    pkg.put(np.getName(), pc);
	 
	    Vector cv = np.classes.getElements();
	    for (int c=0; c<cv.size(); c++) {
		NeuromlClass nc = (NeuromlClass)cv.elementAt(c);
		StringBuffer js = new StringBuffer(); // Java source...
		js.append("\n// New file :"+nc.getName()+".java\n");   
		js.append("package "+np.getName()+";\n");   
		js.append("\nimport neuroml.core.*;\n\n");
		if (nc.desc!=null) {
		    js.append("\n/** "+nc.desc+"\n");
		    js.append(" */"+"\n");		    
		}
		js.append("public class "+nc.getName());
		if (nc.base!=null) {
		    js.append(" extends "+nc.base);
		}
		js.append(" {\n");
		Vector fv = nc.fields.getElements();
		for (int f=0; f<fv.size(); f++) {
		    NeuromlField nf = (NeuromlField)fv.elementAt(f);   
		    
		    // *** Types to add here:-
		    // String + quote
		    // Set + defaultType
		    // Ref + defaultType
		    // = null;
		    // EnumClass;
		    if (nf.desc!=null) {
			js.append("\n    /** "+nf.desc+"\n");
			js.append("     */"+"\n");		    
		    }
		    js.append("    public "+nf.type+" "+nf.getName());
		    if (nf.defaultValue != null) {
			js.append(" = "+nf.defaultValue);
		    }
		    js.append(";\n");		    
		}
		js.append("}\n");
		pc.put(nc.getName(), js.toString());
	    }
	}
	return pkg;
    }

    /** Convert schema coded in a .jar file to a schema in NeuromlSchema
     */
    public static NeuromlSchema jarToSchema(String jarname) {
	NeuromlSchema ret=new NeuromlSchema();
	InputStream inf=null;
	JarInputStream jinf=null;
	try { 
	    URL u = new URL(jarname);   
 	    inf = u.openStream();
	    jinf = new JarInputStream(inf);
	} catch (Exception e) {
	    System.out.println("jarToSchema couldn't open "+jarname+" "+e);
	    return null;		
	}

	Hashtable<String,Vector<String>> packageNames = new Hashtable<String,Vector<String>>();

	ZipEntry ze;
	try {
	    Vector cl = new Vector();
	    do {
		ze = jinf.getNextEntry();
		if (ze!=null) {
		    if (ze.getName().endsWith(".class")) {
			String zc  = ze.getName().substring(0,ze.getName().length()-6);
			zc = zc.replace('/','.');
			String sn = zc.substring(zc.lastIndexOf(".")+1);
			String pkn = zc.substring( 0, zc.lastIndexOf(".") );
			Vector<String> v = (packageNames.get(pkn));			
			if (v==null) {
			    v = new Vector<String>();
			    packageNames.put(pkn,v);
			}
			v.addElement( sn );
			System.out.println("Found class "+pkn+" "+sn);
		    }
		}
	    } while (ze != null);
	} catch (Exception e) {
	    System.out.println(e);	    
	    e.printStackTrace();
	}	

	// OK, convert it to NeuromlSchema...
	java.util.Set s = packageNames.keySet();
        for (Iterator i2 = s.iterator(); i2.hasNext(); ) {
            String pn = (String)i2.next();
            Vector<String> pv = packageNames.get(pn);
	    String[] cns = (String[])pv.toArray(new String[0]);
	    ret.packages.addElement( toNeuromlPackage( pn, cns ) );
        }
	return ret;
    }

    
    /** The main method tests the NeuromlSchema classes
     * by constructing both XML-Schema and NeuroML schema versions
     * of a couple cf clases, and writing them out in both formats
     * as well in Java source.
     */
    public static void main(String[] args) {

	/** NeuromlSchema style... */
	NeuromlSchema ns = new NeuromlSchema();
	ns.xmlns="http://www.neuroml.org/neuroml.model.network.nml";
	NeuromlPackage np = new NeuromlPackage("neuroml.model.network");
	NeuromlClass nc = new NeuromlClass("Grid3DStructure");
	nc.desc="A PopulationStructure for a 3D mesh";
	nc.fields.addElement( new NeuromlField("xsize", "int") );
	nc.fields.addElement( new NeuromlField("ysize", "int") );
	nc.fields.addElement( new NeuromlField("zsize", "int") );
	np.classes.addElement(nc);
	NeuromlClass nc2 = new NeuromlClass("Grid4DStructure");
	nc2.base="Grid3DStructure";
	NeuromlField wf = new NeuromlField("wsize","double");
	wf.desc="The fourth dimension";
	nc2.fields.addElement( wf );
	np.classes.addElement(nc2);
	ns.packages.addElement(np);

	XMLOut.addKnownPackage("neuroml.util.schema"); 
	//	XMLOut.setUseTypeNameInSet( true );
	XMLOut.setUseSetName(true);
	System.out.println(XMLOut.toString( ns ));

	// Write out schema for schema
	XMLOut.addKnownPackage("java.lang"); 
	XMLOut.addKnownPackage("neuroml.core"); 
	NeuromlSchema ns2 = new NeuromlSchema();
	String[] cnames = { "NeuromlPackage", "NeuromlField", "NeuromlSchema", "NeuromlClass" };
	NeuromlPackage np2 = toNeuromlPackage("neuroml.util.schema", cnames);
	ns2.packages.add(np2);
	System.out.println("\n\nGenerating schema for neuromlschema");
	System.out.println(XMLOut.toString(ns2));
		

	// Java source
	Hashtable jsrc = toJavaSource(ns);
	java.util.Set s = jsrc.keySet();
        for (Iterator i1 = s.iterator(); i1.hasNext(); ) {
            String pn = (String)i1.next();
            Hashtable jsrcs = (Hashtable)jsrc.get(pn);
	    for (Iterator i2 = jsrcs.keySet().iterator(); i2.hasNext(); ) {
		String cn = (String)i2.next();
		String java = (String)jsrcs.get(cn);
		System.out.println("\n\nGenerating "+pn+"/"+cn+".java :-");
		System.out.println(java);
	    }
	    
        }
    }
}

