package nmlplay;

import neuroml.sim.run.*;

/**
 * A helper class for running NEOSIM in its own thread.
 *
 * @author Holger Bringmann (hcb@lambdabase.de)
 *
 */
public class RunSim extends Thread {
  /** Simulation start time.*/
  protected double from;
  /** Simulation end time.*/
  protected double to;
  /** Callback frequency. The callback class will be called each step.*/
  protected double step;
  /** The callback class, in this case the NMLPlay GUI.*/
  protected NMLPlay_2 callback;
  /** The NeuroML simulator.*/
  protected Simulator sim;

 // transient neosim.kernel.DeadlockTracker tracker=new neosim.kernel.DeadlockTracker();
   
  /** See field reference for information on parameters.*/
  public RunSim(Simulator sim, double from, double to, double step, NMLPlay_2 callback) {
    this.from = from;
    this.to   = to;
    this.step = step;
    this.callback = callback;
    this.sim = sim;
    setName(getName()+" "+this.getClass().getName()+ " "+from+ " "+to+ " "+step);

  }
  public void updateParameters(double to, double step){
     this.from = from;
     this.to   = to;
     this.step = step;     
  }
  
  /** Runs the simulator until paused or runtime expired. Will
   *  call NMLPlay.showProgress(time) each step and NMLPlay.runFinished(time)
   *  when simulation is over
   */
  public void run() {
      System.gc();
      System.runFinalization();        
      try{
        Thread.currentThread().sleep(1);
      }catch(Exception ecpt){
          
      }
    
    
    while(!callback.isStopped()){
        long start = System.currentTimeMillis();
        while ((to > from+step) && (!callback.isStopped())) {
            //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
            //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp);
            synchronized(callback.state){
                //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
                if(callback.isRunning()){
                    from+=step;
                    sim.runSim(step);
                    callback.showProgress(from);
                }  
                else{
                    //let other tasks do something
                    try{
                        Thread.sleep(100);
                    }
                    catch(java.lang.InterruptedException e){
                        e.printStackTrace();
                    }                  
                }
            }
        }
        long end = System.currentTimeMillis();
        //neosim.kernel.Deadlock_Message ecp2=new neosim.kernel.Deadlock_Message();
        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp2);
        synchronized(callback.state){
            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp2);            
            if(!callback.isStopped()){
                callback.pause(to);
            }
        }
        if(end-start>5) 
            System.out.println("Total duration = " +(end-start)+" ms");
        try{
            Thread.sleep(100);
        }
        catch(java.lang.InterruptedException e){
            e.printStackTrace();
        }
    }
     
     
    /*if (callback.isRunning()) {
      sim.runSim(to-from);
      callback.showProgress(to);
      callback.runFinished(to);
    } else if (callback.isPaused()) callback.pause(from);
    else if (callback.isStopped()) callback.stop();*/
  }
}

/*
             
             for(int i=0;i<10000;i++){
                 SerializationTest st = new SerializationTest();
                st.writeObject();
                st.readObject();
             }
             long end = System.currentTimeMillis();
 
 */
